/* // include header files!!!!!!!!!! */
#include "mpi.h"
#include <cstdio>
#include <cmath>
#include <math.h>
#include <string>
#include <cstdlib>
#include <ctime>
//#include "cstdlib.h"
//#include "nrutil.h"

//define constants
#define PI 3.14159265359
//#define par_dt (dx/23.0)
#define TINY 1e-20                        // a tiny value in order to avoid divided by 0
// define variable types.
#define float double
#define VISCOUS_DIFFUSION VISCOUS_DIFFUSION_CN(E_S_renew,&Visc_Flag, domain_cellnumber[1], &deta_x, cell_info, source)
#define SPECIES_DIFFUSION SPECIES_DIFFUSION_CN(E_S_renew, &SpeDiff_Flag, domain_cellnumber[1], &deta_x, cell_info, source)
#define MFR_DIFFUSION MFR_DIFFUSION_CN(E_S_renew, &MFRDiff_Flag, domain_cellnumber[1], deta_x, cell_info, source)
#define HEAT_TRANSFER  HEAT_TRANSFER_CN_TEM(E_S_renew, &Tem_Flag, domain_cellnumber[1], &deta_x, cell_info, source)



/*************************************************
the thermodynamic constants and the state constants
************************************************/
//const static double g_ux = -9.81;                        // gravity acceleration (m/s2)
const static double Pressure = 122000;                   // ambient pressure, Pa
const static double gravity_ac = -9.81;                   // gravity acceleration, the negative sign means the flame is downflow!!
const static double Tem_Reference = 293;                // the reference temperature is 273 K
const static double Sigma = 5.669e-8;                 // stifen-bolzman constant
const static double Tem_vap = 290.15;                 // vaporization point
const static double Tem_boiling = 374.0;             // boiling point
const static double Tem_devol = 450;                  // devolatilization start point
const static double Tem_Coflow = 1520;              // coflow temperature
const static double Tem_OP = 367;           // temperature of primary oxidizer
const static double Tem_OS = 450;                     // temperature of secondary oxidizer
const static double Tem_Wall = Tem_Coflow;                // heating wall temeprature
const static double Tem_Rad_BG = Tem_Wall;         // radiation background temperature=wall temperature

//define geometry parameters and small increments for meshes
const static double domain_y = 4.24e-1;                  // widht od ODT line
const static double domain_x = 13.e-1;                   // height of ODT simulation domain
const static double jet_width_OP = (0.6e-2);                  // jet width of primary oxidizer
const static double jet_width_OS = 2.4e-2;               // jet width of secondary oxidizer+primary oxidizer
const static double dy_cen = 0.6e-4;                     // cell size in the jet flow
const static double dy_co = 2.0e-4;                      // cell size in the co-flow
const static double dy_bg = 4.0e-4;                      // cell size of the backgrid cell
const static int cellnumber = (int)(((jet_width_OS+1e-10)/dy_cen)+(domain_y-jet_width_OS+1e-10)/dy_co);   // define initial cell numbers in r direction
const static int center_inicell = 200;                    // how many data points in uprofile.dat
//const static int out_inicell = 252; //how many data points of ou
const static int totalnumber = 5*cellnumber;
const static int gridnumber = totalnumber+1;
const static double dx_stir = 1.e-7;             // define space step of a stir action (m)
const static double dx_max = 4.0e-5;	                 // the maximum diffusion step
const static double dx_min = dx_stir;                      // the minimum diffusion step
const static double print_distance = 5e-3;             // to print each line every such distance
//const static int stirnumber = int((dx+1e-20)/dx_stir);		         // define space step in x direction (m)

const static int print_step = int (print_distance/dx_stir);                      // to print information every 500 step and every 500 numbers
const static int xstep_number = (int)((domain_x+1e-10)/dx_stir);      // define number of steps in x direction (m)
const static int record_xstep = int(5e-3/dx_stir);                      // sample frequence for x step
const static int equilibrium_step = int(5e-4/dx_stir);
const static int record_ystep = 2;                       // sample frequence for y step
const static int ysample_number = 600;		         // sample points in odt line
const static double ysample_width_ratio = (domain_y/jet_width_OP);                     // the ratio of the width of the average region to the jet width

const static int incells = 0;  //the cellnumber inside the jet region whose u should be initialized by S*(x/x0)^(-1/2) 

 // in drag law, 
//vg=v_latcoe*cell.v_lateral+v_tripcoe*cell.v_trip if there is no eddy
//vg=v_eddycoe*eddy.v+triplet_v if there is an eddy
const static double v_latcoe=1.0;                        
const static double v_eddycoe=0.5;
const static double v_tripcoe=0.;

//define initial conditions
const static int REALI = 1;                              // realization number
const static double covelocity = 2.;
const static double turb_intensity_center = 0.0;        // turbulent intensity in the jet
// in order to decrease the negative effect on eddy selection caused by random process, a constant coflow is prefered.
const static double turb_intensity_coflow = 0.0;        // turbulent intensity in the coflow


/********************************************************************************/
/*  The Following definitions are for particles!!!                              */
/********************************************************************************/
const static double par_massflow = 0.28332;               // total particle mass flow rate(kg/s) in jet slot with the depth of 1m
const static int par_tracknumber = 9000;                 // particle tracking number 
//const static double par_dt = (dx/23.0);                      // to define the particle time step(s)
const static int Par_SN=4;                               // how many sizes of the particle. for eg. if there r 3 kinds of particle: 10, 70 and 130(macron), then Par_SN=4, 0, 1, 2 are for 10, 70 and 130, 3 is for the total particles;

// to initial a particle sizee distribution
const static double par_size_dist[Par_SN-1]={2.4e-5, 4.2e-5, 7.8e-5};
const static double par_Tem_dist[Par_SN-1]={367,367,367};
// particle size profile. in this work, to regard every 18 tracks as a group, so that a maximum of 18 different particle size can be set as initial condition
//const static double par_size[18]={1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5};
const static double par_size[18]={par_size_dist[0],par_size_dist[0],par_size_dist[0],par_size_dist[0],par_size_dist[1],par_size_dist[1],par_size_dist[1],par_size_dist[1],par_size_dist[1],par_size_dist[1],par_size_dist[1],par_size_dist[1],par_size_dist[1],par_size_dist[1],par_size_dist[1],par_size_dist[1],par_size_dist[2],par_size_dist[2]};
// const static double par_size[18]={1.0e-5,3.4e-5,3.4e-5,4.4e-5,4.4e-5,4.4e-5,6.0e-5,6.0e-5,6.0e-5,6.0e-5,6.0e-5,6.0e-5,7.6e-5,7.6e-5,7.6e-5,8.6e-5,8.6e-5,1.1e-4};

//const static double Particle_Tem = 303;   // particle temperature, K

// coal primary analysis, they are: vfr_water, mfr_volatile, mfr_solidcarbon, mfr_ash, based on (as received basis)
//const static double coal_primary_mfr[4] = {0.0190 /*water*/, 0.4913 /*volatile*/, 0.4042 /*solid carbon*/, 0.0855 /*ash*/};

const static double coal_primary_mfr[4] = {0.001 /*water*/, 0.46 /*volatile*/, 0.4655 /*solid carbon*/, 0.0735 /*ash*/};
//const static double coal_primary_mfr[4] = {0.0 /*water*/, 0.4184 /*volatile*/, 0.4644 /*solid carbon*/, 0.1172 /*ash*/};

// coal ultimate analysis, they are mfr_C, mfr_H, mfr_O, mfr_N, mfr_S, any basis will be ok, but in initialize procedure(function), only (dry ash free basis is the standard case)
//const static double coal_ultimate_mfr[5] = {0.7170 /*C*/, 0.0505 /*H*/, 0.1117 /*O*/, 0.0129 /*N*/, 0.0034 /*S*/};  // here the basis is (as received)

const static double coal_ultimate_mfr[5] = {0.731 /*C*/, 0.0545 /*H*/, 0.1205 /*O*/, 0.014 /*N*/, 0.0055 /*S*/};  // here the basis is (as received)

const static double coal_HHV =  (29.4e6*(1-coal_primary_mfr[0]));  //Higher heat value, J/kg
const static double coal_ash_cp = 2800;  // heat capacity of the ash in coal, J/(kg.K)

const static double epsilon_p = 0.95;  //particle absorption coefficient in radiation heat transfer
/**********************end of particle information definition********************/





/********************************************************************************/
/*  The Following definitions are for Eddies!!!                                 */
/********************************************************************************/
const static double origin = (0.5*jet_width_OP);                // define the origin of the jet. no eddy will occur when height is less than origin
const static double eddy_min = (3.0*dy_cen);               // define the minimum eddies
const static double domain_factor = 0.6;                 // this factor shows the boundary of the eddies, which means the eddies only occurs at (domain_y*domain_factor/2, -domain_y*domain_facotr/2), this factor is introduced to decrease the useless randum numbers for eddy selection, so that eddy selection efficiency can be improved!  
const static double eddy_large_umin = (covelocity/10.);               // if the minimum value of the u differences among the three small eddies < eddy_large_umin, the large eddy would be suspended
const static double C_odt = 10.2;                         // determines the strength of turbulence: C=6.9
//since parameter Z in eddy selection will suppress not only small eddies but also large eddies. two parameters are adopted separately to consider the viscous 
// turn over effect according to viscous
const static double Z_large = 2.0;                       // large eddy suspension constant
const static double Z_small = 3.0;                       // small eddy suppression constant
const static double alfa = (2.0/3.0);                    // distribution rate of energy of velocity
const static double umin_eddy = (covelocity/10.);                     // if u after triplet map less than this value, this triplet map will be abandoned
const static int eddy_number = 5000;                     // the maximum eddy occurrence times per realization

const static double beta = 0.08;                          // the relation ship between eddy lifetime and eddy time scale
                                                         // eddy_lifetime=beta*t; t=C/(l^2*lemda)
const static double eddy_factor = 0.8;                   // eddy shape factor: the ratio of eddy height and eddy length




/*****************************************************
!!!!!!!!!!!!!!!!!!!for reaction mixture!!!!!!!!!!!!!!
******************************************************/
const static int Element_Number = 5;                     // C H O N Ar
const static int Phase_Number = 1;                       // Gas phase
const static double R = 8.314;                           // gas constant J/(mol*K)
const static int Species_Number = 19;                    // species number for gas mixtures

const static double Nondim_Le = 1.0;           //Lewis number   alfa/D
const static double Nondim_Pr = 1.0;          //Prandtl number v/alfa
const static double Nondim_Sc = 1.0;           // Schmidt number v/D 

// the composition of the centerflow(primary oxidizer (Mole fraction)
const static double Species_Mole_Center[Species_Number] = {0.0/*N2*/, 0.0/*C2H2*/, 0.0/*CH2*/, 0.0/*CH3*/, 0.00/*CH4*/, 0.00/*CO*/, 0./*CO2*/, 0.0/*H*/, 0.00/*H2*/, 0.0/*HCO*/, 0.042/*H2O*/, 0.0/*HO2*/, 0.102/*Ar*/, 0.0/*C_gas*/, 0.0/*NO*/, 0.0/*NH3*/, 0.0/*o*/, 0.856/*O2*/, 0.0/*OH*/};

// the composition of the secondary oxidizer (Mole fraction)
const static double Species_Mole_Center_1[Species_Number] = {0.00/*N2*/, 0.0/*C2H2*/, 0.0/*CH2*/, 0.0/*CH3*/, 0.00/*CH4*/, 0.00/*CO*/, 0./*CO2*/, 0.0/*H*/, 0.00/*H2*/, 0.0/*HCO*/, 1.0/*H2O*/, 0.0/*HO2*/, 0.000/*Ar*/, 0.0/*C_gas*/, 0.0/*NO*/, 0.0/*NH3*/, 0.0/*o*/, 0.0/*O2*/, 0.0/*OH*/};


// the composition of the coflow (mole fraction of recirculation materials)
const static double Species_Mole_Coflow[Species_Number] = {0.0073/*N2*/, 0.0/*C2H2*/, 0.0/*CH2*/, 0.0/*CH3*/, 0.0/*CH4*/, 0.36/*CO*/, 0.17/*CO2*/,0.0/*H*/,0.19/*H2*/,0.0/*HCO*/,0.236/*H2O*/,0.0/*HO2*/,0.0367/*Ar*/,0.0/*C_gas*/,0./*NO*/,0.0/*NH3*/,0.0/*O*/,0.0/*O2*/,0/*OH*/};
// the name of each species

// to give the initial mixture fraction value
const static double Mixfr_Center[5] = {0.0 /*moisture*/,0.0 /*volatile*/, 0.0 /*char*/, 1.0 /*primary*/, 0.0 /*secondary*/};
const static double Mixfr_Center_1[5] = {0.0 /*moisture*/,0.0 /*volatile*/, 0.0 /*char*/, 0.0 /*primary*/, 1.0 /*secondary*/};
const static double Mixfr_Coflow[5] = {0.0006 /*moisture*/,0.214 /*volatile*/, 0.270 /*char*/, 0.357 /*primary*/,0.158 /*secondary*/};



/********************************************************************************/
/*  The Following definitions are for species! 
/******************************************************************************/
// Molecular Weight for each Element {C, H, O, N}
const static double MW_Element[Element_Number]={12.0112,1.0080,15.9994,14.0067, 39.9480};
//const static double MW_Element[Element_Number]={12.0,1.0,16.0,14.0};


// I->species_number, J->Element_Number
const static double N_IJ[Species_Number+2][Element_Number]={/*for N2*/0,0,0,2,0,/*for C2H2*/2,2,0,0,0,/*for CH2*/1,2,0,0,0,/*for CH3*/1,3,0,0,0,/*for CH4*/1,4,0,0,0,/*for CO*/1,0,1,0,0,/*for CO2*/1,0,2,0,0,/*H*/0,1,0,0,0,/*H2*/0,2,0,0,0,/*HCO*/1,1,1,0,0,/*H2O*/0,2,1,0,0,/*HO2*/0,1,2,0,0,/*Ar*/0,0,0,0,1,/*C_gas*/1,0,0,0,0,/*NO*/0,0,1,1,0,/*NH3*/0,3,0,1,0,/*O*/0,0,1,0,0,/*O2*/0,0,2,0,0,/*OH*/0,1,1,0,0,/*H2O_L*/0,2,1,0,0,/*C_S*/1,0,0,0,0};


const static char Species_Name[Species_Number+2][7]={"N2    ","C2H2 ","CH2  ","CH3  ","CH4  ","CO   ","CO2  ","H    ","H2   ","HCO  ","H2O  ","HO2  ","Ar   ","C_gas ","NO    ","NH3   ","O    ","O2   ","OH   ","H2O_L ","C_S   "};
const static char CoalElement_Name[6]={"CHONS"};


// to set an initial lemda. if non-physicial lemda appears, initial lemda will be used.
const static double INI_Lambda_Fuerich[Element_Number] = { 1.1,-11.1243,-35.1,-10.5, -27.33};
const static double INI_Lambda_Oxrich[Element_Number] = {-5.0,-11.3,-12.5,-12.0, -27.33};

const static double INI_Lambda_Other[Element_Number] = {-0.3,-11.2,-11.5,-12.8, -27.33};

const static double Tem_Standard = 298.15;
// double Molecular_Weight[Species_Number+2];
// double Density_Standard[Species_Number+2];

//const static int spe_C = 0;
//const static int C_S = 1;
// in plumes, there r only 2 species: He and Air.
//const static int Air = 0;
//const static int He = 1; 

// for primary analysis
const static int Pry_M = 0;
const static int Pry_V = 1;
const static int Pry_C = 2;
const static int Pry_A = 3;

// for ultimate analysis
const static int Ult_C = 0;
const static int Ult_H = 1;
const static int Ult_O = 2;
const static int Ult_N = 3;
const static int Ult_S = 4;

// for element
const static int Ele_C = 0;
const static int Ele_H = 1;
const static int Ele_O = 2;
const static int Ele_N = 3;
const static int Ele_Ar = 4;

// those are for combustion cases.
const static int N2 = 0;
const static int C2H2 = 1;
const static int CH2 = 2;
const static int CH3 = 3;
const static int CH4 = 4;
const static int CO = 5;
const static int CO2 = 6;
const static int H = 7;
const static int H2 = 8;
const static int He = 8;
const static int HCO = 9;
const static int H2O = 10;
const static int HO2 = 11;
const static int Ar = 12;
const static int C_gas = 13;
const static int NO = 14;
const static int NH3 = 15;
const static int O = 16;
const static int O2 = 17;
const static int OH = 18;

// these two species are not considered in gas mixture since they are liquid and solid
const static int H2O_L = 19;
const static int C_S = 20; 




//**********These structures need redesign!**********************//
// this struct defines the property information for each species 
struct property_info
{
    //  double Molecular_Weight[Species_Number+2];
    double Heat_Capacity[Species_Number+2];
    double Density[Species_Number+2];
    double Enthalpy[Species_Number+2];                           // unit: J/kmol?
    double Entropy[Species_Number+2];
    double Gibbs_Energy[Species_Number+2];
    double Spe_Diff_Coeff[Species_Number+2];                    // diffusion coefficient of each species
    double Kinetic_Viscosity_Coeff[Species_Number+2];            // kinetic viscosity coefficient of each species
    double Conduction_Coeff[Species_Number+2];                   // conduction heat transfer coefficient or each species.  or flow? need check!
};


// this struct defines mixture information ls
struct mixture_information
{
    double Massfr[Species_Number];
    double Molefr[Species_Number];
    double Mole_Total;               // this parameter is used to make sure if sum(Molefr[i])==1?
    double Mixture_Fraction[5];   //C/H/O/N/Ar                            // mixture mixture fraction, for plumes, the M_F=Massfr[He]
    double Mixture_Fraction_Var[5];                               // the variance of mixture fraction M_F_V= <M_F^2>-<M_F>^2
    double MixtureFr_Coal[5];  //for mixture fraction of H2O/Vol/Carbon/Primary/Secondary
    double MixtureFr_Coal_Var[5];  //mixture fraction variance
    double Mole_Number_Flowrate;                               // Mole numbers flow rate of this cell   unit:mole/s
    double Mole_Number_Flux;                                   // mixture mole number flux in this cell , or mole flux number?  unit:mole/(m2s)
    double Mass_Flowrate;                                      // mass flow rate in a cell, unit:kg/s
    double Mass_Flux;                                          // mixture mass flux in this cell unit:kg(m2s)  
    double Tem;                                                // mixture temperature
    double Density;                                            // mixture density 
    double Molecular_Weight;                                   // mixture molecular weight
    double Heat_Capacity;                                      // mixture heat capacity in constant pressure
    double Enthalpy;                                           // mixture Enthalpy, J/mol
    double T_Enthalpy;       //mixture total enthalpy J
    double Entropy;                                            // mixture Entropy
    double Gibbs_Energy;                                       // mixture Gibbs energy
    double Kinetic_Viscosity_Coeff;                            // mixture kinetic viscosity coeff, kg/(m.s)
    double Spe_Diff_Coeff[Species_Number];                                     // mixture diffusion coefficient  (m2/s)
    double MFR_Diff_Coeff[5];                                  // mixture fraction diffusion coefficient
    double Spe_Diff_Coeff_mix;   // the average Diffusion coefficient
    double Conduction_Coeff;                                   // mixutre conduction coefficient w/(m.k)
    double Lambda[Element_Number];
    int EQU_FLAG;  // if an equilibrium calculation has been done on this cell
};
//************************************************************//





//*******************structures of gas phase*****************//

//cell information of gas phase and eddy state
struct cell_information
{
    int	eddystate;			   // if there is an eddy occupies the cell, eddystate=1, an eddy is a rectangle
    int eddystate_instant;                 // the eddy is a line, when this line occupy the cell, the cell state is 1.
    int eddynumber;			   // which eddy occupies this cell
    int abseddynumber;                     // absolute eddy number
    double y;				   // cell y position
    double y_old;                          // old cell position, this value is used to calculate lateral velocity
    double size;			   // cell size
    double size_old;                       // origional cell size before viscous process in nth step
    double u;				   // cell u velocity
    double u_old;                          // velocity at the beginning of the ODT step.
    double u_lastold;                      // u_old in last step, used to calculate the lateral velocity
    double v_trip;                         // cell v velocity according to the scrambling model
    double v_lateral;			   // cell v lateral velocity calculated by displacement of cells
    double w;				   // cell w velocity
    double pressure;                       // cell pressure, constant pressure at any time!!! non-compressible
    double Tem_old;			   // cell temperature at the beginning of the ODT step
    double Density_old;                    //density at the beginning of the ODT step
    double MFR_old[5];                     // old MFR
    double Massfr_old[Species_Number];
    double loading_rate[2];                  // loading_rate=par_massflux/cell_massflux
    //double viscosity;                      // kinetic viscosity
    struct mixture_information mixture;
};
//**********************end of gas phase*************************//
//***************************************************************//



//************structures of particles****************************//

//this structure is used to acquire numberdensity and its 1st and 2nd derivative
struct back_grid  
{
    int Flag_eddy;                  // flag which shows if this cell is occupied by an eddy, Flag_eddy=1 means that the cell was occupied by an eddy
                                    // Then the lateral velocity for this cell is equal to the one of the  nearest cell which is not occupied by an eddy. 
    double y;
    double mass0;                   // mass flux of gas phase (for variable density case)
    double mass1;
    double ugas0;		    // u velocity of gas phase of last ODT step
    double ugas1;                   // u velocity of gas phase of this ODT step
    double vgas0;                   // v velocity of gas phase of last ODT step
    double vgas1;                   // v velocity of gas phase of this ODT step
   
    double par_surface;                    // particle surface per m3,   m2/m3  =sum(pi*d^2*(par_numberdensity per track))
    double par_volume;		           // particle volume per m3 ( or volume fraction), m3/m3 =sum(pi*d^3/6*(par_numberdensity per track))
    double par_size;                       // particle size per m3 (1st moments=par_size/par_numberdensity), m/m3=sum(d*par_numberdensity per track)
    double par_d32;                        // particle sauter diameter in a cell
    double par_d10;			   // particle mass weighted mean diameter in a cell
    double par_number[Par_SN];             // particle tracks for each size
    double par_massflux[Par_SN];           // particle mass flux for each size (kg/(s*m^2))
    double par_u[Par_SN];                  // particle u velocity (m/s)
    double par_v[Par_SN];                  // particle v velocity (m/s)
    double par_Re[Par_SN];                 // particle Reynolds number
    double par_numberdensity[Par_SN];      // particle numberdensity

    double Tem[Par_SN];                    // particle temperature
    double Mass_P[5][Par_SN];              // each composition weight for each particle size, Mass_P[i]:Pry_M, Pry_V, Pry_C, Pry_A, total weight; kg/#
    double density[Par_SN];                // kg/m3

    //the following data were used to calculate the diffusion coefficient of all moments
    //the calculation method is according to: Di=(((par_ndith_new)-(par_ndith_old))/dx)/(2th differental of par_ithmom)
    double par_nd0th_old[Par_SN];
    double par_nd0th_new[Par_SN];
    double par_ddnd0th[Par_SN];   // 2nd derivative of 0th moment for each different size of particle
    double par_0thdiffcoe[Par_SN];

    //for a certain size of particle, its diffusion coefficient of all momentum are the same!!!, so there is no need to 
    //allocate the space to calculate their diffusion coefficient. The following variables are for the integral particles.
    double par_nd1st_old;
    double par_nd1st_new;
    double par_ddnd1st;      //2nd derivative of 1st moment for the total particle
    double par_1stdiffcoe;

    double par_nd2nd_old;
    double par_nd2nd_new;
    double par_ddnd2nd;      //2nd derivative of 2nd moment for the total particle
    double par_2nddiffcoe;

    double par_nd3rd_old;
    double par_nd3rd_new;
    double par_ddnd3rd;      //2nd derivative of 3rd moment for the total particle
    double par_3rddiffcoe;
};

		
/*********************************************************************
for oxy-coal combustion, a new struction name coal_information was
introduced. it is used to store the main properties of the coal, for
eg. the elementary species, the enthalpy and the heat value, et al
*********************************************************************/	
struct coal_information
{
    double mfr_primary[4];   //primary analysis, mfr of: water, volatile, solid carbon and ash, based on (As Received basis), this struct will store the renewed mfr of M/V/C/A to ORIGINAL coal!!!
    double mfr_ultimate[5];  //ultimate analysis, mfr of: C, H, O, N, S, based on (Dry_Ash_Free basis) //this struct stores the renewed mfr of C/H/O/N to ORIGINAL coal!!!!
    double mfr_species[Species_Number+2];  //to regard coal as a mixture, As Received basis sum(mfr_species) = 1-mfr_primary[3](mfr_ash)
    double coeff_ultimate[5];  //to regard coal as a material as: C(x)H(y)O(z)N(a)S(b) [b=0], with the molecular weight of 1000g/mole, the coeff_ultimate will store the value of x,y,z,a,and b
    double enthalpy;                      // J/kg
    double heat_value_estimate[2];           // J/kg
    double heat_value;                    // J/kg
    double density[5];                    // for water, volatile, carbon, ash, and the average value of the coal, kg/m3
    double heat_release[6];               // 0:latent heat for water, 1:latent heat for volatile, 2: heat release for carbon combustion(C+O2), 3: heat release for (C+CO2), 4: heat release for C+H2O,  5: heat release for C+H2
    double diameter;                      //m
    double Tem;
    double Mass[2];                       // mass[0] for original particle mass, mp0, mass[1] for latest particle mass, mp
    double Mass_P[4];                    // mass of liquid water, volatile, carbon and ash
    double heat_capacity[5];              //for water, volatile, carbon, ash, and the average value of the coal,  J/(kg.K)
};



/**************************************************************************************
struct eddy_information restroes each particle information, 
**************************************************************************************/
struct part_infor
{
    int cellin;  // which cell the particle is located
    int eddy_on;  // which eddy the particle is in
    int eddy_trip; // which eddy the particle has been tripletted in
    int eddy_trip_former; // which eddy the particle has been tripletted in before the latest triplet map.
    int boundary_type;	//particle boundary type, 0:in the domain; 1: escape;

    double x;    // x position of the particle (m)
    double y;
    double z;
    double eddy_displacement;  //the displacement during particle moving in the eddy(m)
    double u;	// particle u velocity (m/s)
    double v;
    double w;
    double vm;	//velocity magnitude (m/s) =sqrt(u^2+v^2+w^2);
    double diameter;  //particle initial diameter  (m)
    double surface;   // particle surface (m2)
    double volume;
    double density;   //particle initial density   (kg/m3)
    double mass;      //particle mass (kg)=.coal.Mass[1]
    double massflowrate;  // particle mass flow rate in this track (kg/s)  ini_mfr=total_massflowrate/total_tracks
    double particle_number;  // particle numbers in this track  (#/track)  ===.ini_massflorate/.ini_mass
    double Re_p;		// particle Reynold number =fabs(ug-up)*dp/ken_vis
    double tao;			// particle relaxition time s
    double temperature;  //particle temperature (K)
    struct coal_information coal;  //coal properties
};

// This structure defines the particle source term to cells
struct source_information
{
    double mass;             // actually it is mass_flux, kg/s
    double mom_u;            // the momentum flux, (kg.m)/s^2
    double mom_v;
    double mom_w;
    double Enth_h;           // the enthalpy flux source term for enthalpy equation
    double enthalpy;         // the enthalpy flux source term for temperature equation W or J/s
    double massflux_species[Species_Number];  // massflux of each species for the gas phase
    double mixture_fr[3];  //steam, volatile and carbon
};

//********************* end of particle structure*********************************//



/**************************************************************************************
struct eddy_information restroes each eddy information, ->xstart: x position(jth step), 
					->xend: x_position(jth step end); ->ystart: eddy_start, 
					->yend: eddy_end(the last cell occupied by this eddy), 
					->lifetime: eddy_lifetime; ->u eddy_u, ->v: eddy_v, ->w: eddy_w;
********************************************************************************************/
struct eddy_information
{
    int ser_number;         // the serial number of the eddy, show this eddy is the ser_number(th) eddy from the begin of this realization 
    int xcellstart;			// xth step that eddy start ( step number is based on the eddy sampling step)
    //    int xcellstart1;
    int xcellend;			// xth step that eddy end
    int ycellstart;             // cell number that eddy starts(the left cell the eddy occupies)
    int ycellend;
    double ystart;	        // y position that eddy starts(the left cell the eddy occupies)
    double yend;		// y position that eddy ends(the right cell the eddy occupies)
    double lifetime;	        // eddy life time (s)
    double length;              // original eddy length(m)
    double length1;             // eddy length at corrent odt line height
    double height;              // eddy height(m), which is proportional to the eddy length
    double height_judge;        // eddy height used to judge if this eddy is still have effects on the particle, which is proportional to the eddy life time!
    double u;			// eddy average u velocity =[integral of (u+u_trip)/2]/eddy_l
    double v;			// eddy average v velocity =[integral of (v+v_trip)/2]/eddy_l
    double w;			// eddy average w velocity =[integral of (w+w_trip)/2]/eddy_l
    double vlateral;		// eddy average v diffusion velocity
};






// notes: par_size_10/70/130 or par_surface_10/70/130 and par_volume_10/70/130 can be acquired 
// by par_size_10=par_numberdensity_10*10e-5, par_surface_10=par_numberdensity_10*10e-5^2*pi, et al.
struct cell_average
{

    int par_ndnumber[Par_SN];              // number of realizations that particle will appear at this cell
    int par_nd0thcoe_number[Par_SN];       // number of realizations that particle nubmer density flux is not zero at this cell
    int par_nd1stcoe_number;
    int par_nd2ndcoe_number;
    int par_nd3rdcoe_number;
 
    double x;
    double y;
    double u;
    double v_lateral;
    double v_trip;
    double v_ave;
    double w;
    //double density;
    double T;
    double eddy_density;
    double eddy_density_instant;
    double uu;  //Reynolds stress!
    double deta_u;
    double uv;
    double vv;
    double ww;

    double massfr_rms[Species_Number];

    struct mixture_information mixture;
    struct source_information source;
    struct back_grid  particle;

    //these value are just temperaty value 
    double par_0thdiffcoe[Par_SN];
    double par_1stdiffcoe;
    double par_2nddiffcoe;
    double par_3rddiffcoe;

};


// this struct is used to record important average scalar values at each sampled odt line(at each height)
struct axis_average
{
    double x;		           // odt line height 
    double u;		           // axial velocity along the axis(center line)
    double v_trip;                  // diffusion velocity along the axis (center line)
    double v_lateral;
    double Restress[4];            // Reuu, Revv, Reww and Reuv
    //    double density;
    double half_width_p;	   // half span width of the planar jet at positive position. half_width is the y location where is velocity is 1/2 of axial velocity at the center line
    double half_width_n;           // half span width of the planar jet at negative position.

    double half_width_fp[5];          // half width of the mixture fraction at positive position
    double half_width_fn[5];          // half width of the mixture fraction at negative position
    double half_width_fmrp[5];       // half width of the mixture fraction(according to transport equation) at positive position
    double half_width_fmrn[5];        // half width of the mixture fraction(according to transport equation) at negative position
    double par_ndmax[Par_SN];     // maximum particle number density at a certain height
    double par_ndmax_y[Par_SN];   // position where maximum particle number density appears
    double par_ndhalfmax_yn[Par_SN];
    double par_ndhalfmax_yp[Par_SN];
    double par_ndaxis[Par_SN];		// particle number density at the center line
    double par_density[Par_SN];        // particle density on the center line
    double par_Tem[Par_SN];            // particle temperature on the center line

    double eddy_instden_max;                // maximum eddy instant density
    double eddy_instden_max_y;              // location of maximum eddy instant density
    double eddy_den_max;		    // maximum eddy density
    double eddy_den_max_y;		    // y location where eddy_den_max appears
    double eddy_den_axis;
    struct mixture_information mixture;
};

/******************************end of average structure*****************************/




//this function is used to check the mass conservation of the flow, it is only used to debug the code!
void CHECK_CONVERSATION(int domain_cellnumber, struct cell_information *cell, struct source_information *source, char *step);



/*****************************************************************************************
below are sub-functions used in this program, the functions of each are listed 
as follows:
******************************************************************************************/
//**************************************************************************************//
//**************************************functions about matrix**************************//

// Thomas algorithm used to solve tridiagonal matrix algebraic equations.
// that is trig(a,b,c)*u=r, and u can be solved
void TRIDIAG(double *a,double *b,double *c, double *r,double *u, int domain_cellnumber);
// LU decomposion
void LUDCMP(double a[][Element_Number], int n, int *indx, double *d);
void LUBKSB(double a[][Element_Number], int n, int *indx, double *b);
// to find inverse matrix
void INVERSMATRIX(double a[][Element_Number], double a_inv[][Element_Number],int n, int *indx, double *d);




//**************************************************************************************//
//*********************************functions about initialization***********************//
//this function is used to initialize all average variables
void INI_AVE(int *avenumber, struct cell_average cell_ave[][ysample_number]);

// to initialize cell information in the centerflow except cell.size, cell.y, cell.u cell.v, cell.w
void INI_CENTERFLOW_CELL(int i, struct cell_information *cell);
// to initialize cell information in the secondary oxidizer region.
void INI_OS_CELL(int i, struct cell_information *cell);

//to initialize all initial information of the cell in the coflow region, this function is used in INITIALIze
void INI_COFLOW_CELL(int i, double S, double ini_halfjetwidth, struct cell_information *cell);

//to initialize all information of the cell in the coflow region (NOT INITIAL VALUE), this function is used in RENEW_GRID
void INI_BOUNDARY_CELL(int i, double U, struct cell_information *cell);

// this function is used to initialize all variables
void INITIALIZE(int *core_bdcell, double *grid_y, struct cell_information *cell, struct eddy_information *eddy, struct part_infor *par_info_ini,  struct part_infor *par_info_new, int *domain_cellnumber, struct back_grid *backgrid_nd);

// to initialize the particle terms for this simulation
void INI_PARTICLE_TRACK(struct cell_information *cell, struct part_infor *par_info_ini, struct part_infor *par_info_new);

// to preprocess the coal properties according to the primary analysis and the ultimate analysis
void INIT_COAL_PARTICLE(struct coal_information *coal);

//to identify the initial composition of the volatile of the coal
void COAL_INI_COMPOSITION(struct coal_information *coal);

// to set source term and particle information in each cell to 0
void INI_SOU_PARCELL(int domain_cellnumber, struct source_information *source, struct back_grid *backgrid_nd);

// this function is used to initialize particle numberdensity for each cell
//void INI_PARND(struct part_infor *par_info, struct back_grid *backgrid_nd);




//***************************************************************************************//
//********************************functions about diffusion and  meshes *****************//
// TO calculate the maximum diffusion step according to Fourier condition
double CALCU_DIFFDX_MAX(int domain_cellnumber, struct cell_information *cell);
// forward-Euler method to solve viscous diffusion process
void DIFFUSION_FE(int E_S_number, int *Visc_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);
// 2nd improved forward-Euler method to solve viscous diffusion process
void DIFFUSION_2NDFE(int E_S_number, int *Visc_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);
void DIFFUSION_4THRK(int *Visc_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell);
// to solve diffusion process using Crank-Necolson method
void VISCOUS_DIFFUSION_CNONLY(int E_S_number, int *Visc_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);
// to solve diffusion process using Crank-Nickolson method twice.
void VISCOUS_DIFFUSION_CN(int E_S_number, int *Visc_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);
// to change/decrease diffusion step when negative u velocity occurs
void MODIFY_DIFFDX(double *deta_x, double u_old, double u_new);

// Forward-Euler explicit method to solve species diffusion process
void SPECIES_DIFFUSION_FE(int E_S_number, int *SpeDiff_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);
// improved 2nd Forward-Euler explicit method to solve species diffusion process
void SPECIES_DIFFUSION_2NDFE(int E_S_number, int *SpeDiff_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);


// Crank-Nicolson implicit method to solve species diffusion process
void SPECIES_DIFFUSION_CNONLY(int E_S_number, int *SpeDiff_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);

// Crank-Nicolson implicit method followed by an explicit method to solve species diffusion process
void SPECIES_DIFFUSION_CN(int E_S_number, int *SpeDiff_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);
// to change/decrease diffusion step when negative mass fraction occurs
void MODIFY_SPEDIFFDX(double *deta_x, double Y_old, double Y_new);

// Forward-Euler explicit method to solve species diffusion process
void MFR_DIFFUSION_FE(int E_S_number, int *MFRDiff_Flag, int domain_cellnumber, double deta_x, struct cell_information *cell, struct source_information *source);
// improved 2nd Forward-Euler explicit method to solve species diffusion process
void MFR_DIFFUSION_2NDFE(int E_S_number, int *MFRDiff_Flag, int domain_cellnumber, double deta_x, struct cell_information *cell, struct source_information *source);


// Crank-Nicolson implicit method followed by an explicit method to solve species diffusion process
void MFR_DIFFUSION_CN(int E_S_number, int *MFRDiff_Flag, int domain_cellnumber, double deta_x, struct cell_information *cell, struct source_information *source);


//this function is used to calculate the final temperature according to the given enthalpy(J/mol) of a cell
double CALCULATE_CELL_TEM(struct cell_information cell);

// Forward-Euler explict method to solve the enthalpy equation
void HEAT_TRANSFER_FE_ENTHALPY(int E_S_number, int *Tem_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);

// Forward-Euler explict method to solve the enthalpy equation
void HEAT_TRANSFER_2NDFE_ENTHALPY(int E_S_number, int *Tem_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);

// Forward-Eular explicit method to solve Temperature Equations!!!
void HEAT_TRANSFER_FE_TEM(int E_S_number, int *Tem_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);

// Crank-Nicolson implicit method to solve Temperature Equations
void HEAT_TRANSFER_CN_TEM(int E_S_number, int *Tem_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source);

// to change/decrease diffusion step when negative mass fraction occurs
void MODIFY_TEMDIFFDX(double *deta_x, double T_old, double T_new);

// to calculate the buoyancy source term for the momentum equation of the gas phase
double SOURCE_BUOYANCY(struct cell_information cell, double deta_x);


// to calculate the radiation source term for the heat transfer equation
double SOURCE_RADIATION(struct cell_information cell, double deta_x);


// to calculate v diffusion velocity based on displacement of each cell
// TO CALCULATE lateral velocity according to lagrangian parcel displacement 
void CAL_VLATERAL_DISP(int E_S_number, int domain_cellnumber, double deta_x, struct cell_information *cell);

// to calculate lateral velocity based on continuous equation
void CAL_VLATERAL(int eddies, int eddynumber, int j, int domain_cellnumber, double deta_x, struct eddy_information *eddy, struct cell_information *cell, struct back_grid *backgrid_nd);

void CAL_VLATERAL1(int j, int domain_cellnumber, double deta_x, struct back_grid *backgrid_nd, struct cell_information *cell);



// those functions can be found in regrid.cpp. 
//these functions have relationship with grid manipulations
//to recalculate cell.y based on different cell.size
void RENEW_Y(int domain_cellnumber, double *grid_y, struct cell_information *cell);
// this function is used to regrid the domain to keep a constant domain widt
//to add or decrease grids according to the constant domain width
void REGRID_KEEPCONSTANT(int *core_bdcell, int *domain_cellnumber, int eddy_occur, double *grid_y, struct cell_information *cell, struct part_infor *par_info, struct eddy_information *eddy_info);
// to merge cell_1 and cell_2 according to the mixing law
void  MERGECELL(struct cell_information *cell, struct cell_information cell_1, struct cell_information cell_2);

//to split or merge cells whose size is too large or small
void MESH_MANAGEMENT_OLD(int E_S_number, int *core_bdcell, int *domain_cellnumber, double *grid_y,struct cell_information *cell);
void MESH_MANAGEMENT(int E_S_number, int *core_bdcell, int *domain_cellnumber, double *grid_y,struct cell_information *cell);

//these functions can be found in material.cpp
//these functions are used to calculate mixture thermal properties and particle properties!

void DEFINE_MFR(struct cell_information *cell);


// for source term renew!
void TWO_WAY_COUPLING_SOURCE(int domain_cellnumber, struct cell_information *cell, struct source_information *source);
// the dilatation model
void DILATATION_MODLE(double density_ratio, struct cell_information *cell);

// this function is used to calculate the THERMAL DYNAMIC properties for each species in gas phase
void CALCU_GASPROPERTY_THERMAL(double Tem, double Pre, struct property_info *species_property);

// this function is used to calculate the THERMAL DYNAMIC properties for H2O & C_S
void CALCU_WCPROPERTY_THERMAL(double Tem, double Pre, struct property_info *species_property);

// this function is used to calculate the density for each species
void CALCU_PROPERTY_DENSITY(double Tem, double Pre, struct property_info *species_property);

// to calculate density of the mixture
void CALCU_MIXTURE_DENSITY(int i, struct cell_information *cell);

// this function is used to calculate the diffussion coefficient, viscosity and heat conduction coefficients, et al, for each species
void CALCU_PROPERTY_COEFF(double Tem, double Pre, struct property_info *species_property);
// this function is used to calculate the total properties for each species
//void CALCU_PROPERTY(double Tem, double Pre, struct property_info *species_property);

// this function is used to calculate the mixture fraction coefficient according to the mixing law
void CALCU_MFRPROPERTY_COEFF(struct property_info species_property, struct cell_information *cell);



// this function is used to calculate enthalpy for a certain species based on a given temperature
void CALCULATE_ENTHALPY(int domain_cellnumber, double *Tem, double Enthalpy[][Species_Number]);
// to calculate enthalpy for each species for ith cell, J/kg.
void CALCULATE_ENTHALPY_ONECELL(double T, double *Enthalpy);

// this function is used to calculate enthalpy for a certain species based on a given temperature for all cell, J/(kg.K)
void CALCULATE_Heat_Cp(int domain_cellnumber, double *Tem, double Heat_Cp[][Species_Number]);
// this function is used to calculate enthalpy for a certain species based on a given temperature for one cell,  J/(kg.K)!!!
void CALCULATE_Heat_Cp_ONECELL(double T, double *Heat_Cp);


// to calculate the properties for each species in the mixture and to renew the mixture thermal properties BASED ON KNOWN MOLE FRACTION OF EACH SPECIES
void RENEW_MIXPRO_MOLEX(int i, struct cell_information *cell);
// to calculate the properties for each species in the mixture and to renew the mixture thermal properties BASED ON KNOWN MASS FRACTION OF EACH SPECIES
void RENEW_MIXPRO_MASSY(int i, struct cell_information *cell);
// // this function was used to renew the mixture property based on given cell[i].Mixture.Density*cell[i].Mixture.Massfr[k] and T, P
void RENEW_MIXPRO_RMASSYCONTAINER(int i, struct cell_information *cell);

// to calculate ash density in the coal
double CAL_ASH_DENSITY(double Tem);

 // to calculate dvolatile density in the coal
double CAL_VOL_DENSITY(double Tem);

 // to calculate liquid water density in the coal
double CAL_WATER_DENSITY(double Tem);

// to calculate coal density
void CAL_COAL_DENSITY(struct coal_information *coal);

// this function is used to calculate coal particle heat capacity for any time given .coal.Tem and its composition
void CAL_COAL_HEATCP(struct coal_information *coal);


// during coal particle combusiton process, its diameter and shape would change, 
// in this function, the particle density, capacity et al. would be renewed 
void RENEW_COAL_PROPERTY(struct part_infor *par_info);


// to calculate coal heating value (HHV AND LHV), enthalpy  and heat capacity et al.
void CAL_COAL_THERMAL(struct coal_information *coal);

// to calculate latent heat release in water evaporation process     J/kg
double CAL_LATENT_HEAT(double *Tem, double Pre);

void CAL_LATENT_HEAT_H(double *Tem, double Pre, double *latent_heat, double *Enthalpy_steam);

// to calculate the enthalpy of solid carbon at Tp, J/kg
double CAL_ENTHALPY_CS(double Tem, double Pre);
// to calculate heat release in C+O2->CO2        J/kg
double CAL_HEAT_CARBON_OXYGEN(double *Tem, double Pre);

// to calculate heat release in C+CO2->2CO        J/kg
double CAL_HEAT_CARBON_CO2(double *Tem, double Pre);

// to calculate heat release in C+H2O->CO+H2        J/kg
double CAL_HEAT_CARBON_STEAM(double *Tem, double Pre);

// to calculate heat release in C+2H2->CH4        J/kg
double CAL_HEAT_CARBON_CH4(double *Tem, double Pre);

/********************************************************************************************
//                        functions about triplet maps and eddies                           //
//These functions can be found in tripletmap.cpp
//these functions are used to judge if an eddy will occur, if an eddy occurs, a triplet map
//process will be implemented to represent the turbulent mixing process
*********************************************************************************************/

void TRIPLET_MAP(int E_S_number, int E_S_old, int domain_cellnumber, int *core_bdcell, double *grid_y, struct cell_information *cell, struct eddy_information *eddy_info, struct source_information *source, struct cell_average cell_ave[][ysample_number], int *eddy_occur, int *eddy_totalnumber, int *Trip_Flag, double *eddy_lp);

void SMALL_EDDY_ENERGY(int eddy_start, int eddy_cell, struct cell_information *cell, double *Energy_eddy1, double *Energy_eddy2, double *Energy_eddy3);

double CAL_SMALLEDDY_ENERGY(int eddy_start, int eddy_cell, struct cell_information *cell);

double CAL_LARGEEDDY_ENERGY(int eddy_start, int eddy_cell, struct cell_information *cell);

double CAL_LEMDA(int eddy_start, int eddy_cell, struct cell_information *cell);

void EDDY_TRIPLET(int xstep, int *FLAG, int eddy_start, int eddy_cell, struct cell_information *cell, struct source_information *source, struct cell_average cell_ave[][ysample_number]);

void EDDY_JUDGESTATE(int E_S_number, int E_S_old, int *eddy_occur, int domain_cellnumber, double *grid_y, struct cell_information *cell_info, struct eddy_information *eddy_info);

void EDDY_JUDGECELL(int eddy_occur, int domain_cellnumber, double *grid_y, struct eddy_information *eddy_info);

// TO identify eddy_info.ycellstart and .ycellend according to .ystart & .yend
void EDDY_JUDGECELL(int eddy_occur, int domain_cellnumber, double *grid_y, struct eddy_information *eddy_info);


/********************************************************************************************
//                        functions about particle tracking and reactions                   //
// These functions can be found in particle_interaction.cpp file.
// These functions r used to calculate particle position, velocity et al.
// particle tracking method and particle-eddy interaction model was implemented in these functions
//partile reactions was also considered in these functions
*********************************************************************************************/

void PARTICLE_JUDGECELL(int doamin_cellnumber, double *grid_y, struct cell_information *cell, struct part_infor *par_info);

void PARTICLE_TRACKING(int E_S_number, int domain_cellnumber, double deta_x, double *grid_y, struct cell_information *cell,struct eddy_information *eddy_info, struct part_infor *par_info, struct source_information *source);

void PARTICLE_TRIPLET(int i, double *grid_y, struct cell_information *cell, struct eddy_information eddy_info, struct part_infor *par_info,  struct source_information *source);
	
void PARTICLE_TRACK_ONESTEP(int i, int E_S_number, int domain_cellnumber, double deta_x, double *grid_y, struct cell_information *cell, struct part_infor *par_info,  struct source_information *source);
	

// to judge which law should be considered, source.enthalpy and source.massflux_species should be renewed in this function
void COAL_LAW_SWITCH(int i, double mod_par_dt, struct cell_information *cell, struct part_infor *par_info, struct part_infor *par_info_new, struct source_information *source);


// the first law of the coal particles: inert particle heating
// in this function, the partice will be heated according to convection, conduction and radiation,
void LAW_INERT_HEATING(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source);

// the second law of the coal particles: vaporization process
// in this function, the partice will be heated according to convection, conduction and radiation, the steam would be release according to vaporization process
void LAW_VAPORIZATION(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source);

// the third law of the coal particles: boiling process
// in this function, the water boiling process makes the water become steam due to convection, conduction and radiation heat transfer
void LAW_BOILING(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source);

// the fourth law of the coal particles: devolatilization process
// in this function, the devolatilizaiton process was predicted by the 1st order rules.
void LAW_DEVOLATILIZATION(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source);

// to calculate devolatilization rate according to the first order rules, the final particle weight after mod_par_dt time will be returned
double DEVOL_FIRST_ORDER(double mod_par_dt, double Tp, double Pre, struct part_infor *par_info);


// the fifth law of the coal particles: heterogeneous char reaction process
// in this function, the heterogeneous char reaction model is based on the empirical nth law to consider the pressure effect.
void LAW_CHAR_REACTION(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source);


// this function is used to calculate carbon consumption during time step mod_par_dt according to the empirical nth law.
void CHAR_REACTION_EMPIRICAL(double mod_par_dt, double Tem, double Pre, double *Char_consume, struct part_infor *par_info, struct cell_information *cell);

// the sixth law of the coal particles: devolatilization with heterogeneous char reaction process
// in this function, the heterogeneous char reaction model is based on the empirical nth law to consider the pressure effect. the devolatilization law is based on 1st order model
void LAW_DEVOL_CHAR_REACTION(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source);

// this function is used to define fp_reaction
double CALCU_FP_REACTION(struct part_infor *par_info, struct cell_information *cell, double heat_release);
// this function is used to define fp_devol
double CALCU_FP_DEVOL(struct part_infor *par_info, struct cell_information *cell);


// this function is uses to get the numberdenstiy value and its derivatives under the uniform size cell.
void RENEW_BACKGRID(int domain_cellnumber, double deta_x, double *grid_y, struct cell_information *cell, struct part_infor *par_info, struct back_grid *backgrid_nd);
/********************************************************************************************
//                        functions about postprocess:print data                            //
these function can be found in postprocess.cpp
*********************************************************************************************/

// this function is used to print the information of particle dispersion of selected points to a certain file
void PRINT_ALEX(int E_S_number, int domain_cellnumber, double *grid_y, struct cell_information *cell_info, struct back_grid *backgrid_nd, struct source_information *source, FILE *ALEX_INDEP, FILE *ALEX_DEP);

// this function is used to print data in a certain realization 
void PRINT_VELOCITY(int E_S_number, int domain_cellnumber, double *grid_y, struct cell_information *cell_info, struct back_grid *backgrid_nd, struct source_information *source, FILE *Velocity_gas, FILE *Velocity_particle);

// this function is used to renew average value (up to now, the average value is only the summation of each realization)
void RECORD_AVE(int E_S_number, int domain_cellnumber, int *avenumber, double *grid_y, struct cell_information *cell_info, struct back_grid *backgrid_nd, struct cell_average cell_ave[][ysample_number], struct source_information *source);

// this function is used to calculate the final average value for each selected point
void CALCULATE_AVE(int i, int j, int size, int *avenumber, struct cell_average cell_ave[][ysample_number]);

// this function is used to print the final average value of the selected points
void PRINT_AVE(int *avenumber, struct cell_average cell_ave[][ysample_number]);

// this function is used to calculate the important scalar on a certain odt line and to print the final average value on axis
//void PRINT_AXIS(int avenumber[], struct cell_average cell_ave[][ysample_number]);





/********************************************************************************************/
/********************* These functions are based on STANJAN software algorithm***************/
/***************************** please see details for STANJAN *******************************/
/********************************************************************************************/

void EQU_MODEL1(double *Pi, double total_moles, double *Ele_Potential, double *Gibbs_bar);
void EQU_MODEL2(double *Pi, double total_moles, double *Ele_Potential, double *Gibbs_bar);
void EQU_MODEL12(double *total_moles, double *Ele_Potential, double *Gibbs_bar);
void EQU_MODEL3(double *Pi, double *total_moles, double *Ele_Potential, double *Gibbs_bar);

/********************************************************************************************/
/*                to calculate equlibrium status based on given pressure and enthalpy       */
/********************************************************************************************/
void EQUILIBRIUM_PH(int *success, int *fail, int cell_number, struct cell_information *cell);
// to calculate equlibrium status based on given pressure and temperature
void EQUILIBRIUM_PT(int *Flag, double *Pi, struct cell_information *cell, struct property_info property_spe);

// this function is used to get the equilibrium state according to the MINIMIUM GIBBS ENERGY of the C-H-O-N system based on the given ENTHALPY and PRESSURE
void EQUILIBRIUM_ODT(int E_S_number, int domain_cellnumber, struct cell_information *cell);


// to convert in i to char string
void my_itoa(int i, char string[]);
