#include "header.h"


/*********************************************************************************
This file is used to calculate heat transfer process in ODT model
the radiation heat transfer, conduction and the heat transport due to species diffusion
process are considered in the heat transfer process
Two methods are used to solve this process: Enthalpy equations and Temperature equations
Two discrete method are used to get the solution: FE method and CN method
*********************************************************************************/	

//this function is used to calculate the final temperature according to the given enthalpy(J/mol) of a cell
double CALCULATE_CELL_TEM(struct cell_information cell)
{
    int i=0,k;
    double Tem, Tem_old, Cp,  Enthalpy, H_can, H[Species_Number];
    double deta = 1;  //the error

    Enthalpy = cell.mixture.Enthalpy*1000/cell.mixture.Molecular_Weight;       /// J/kg
    Cp = cell.mixture.Heat_Capacity*1000/cell.mixture.Molecular_Weight;        /// J/(kg.K)
    Tem = cell.mixture.Tem;                 // the candidate temperature
    while(deta>5e-4 && i<5)
    {
        // to calculate enthalpy for each species
        CALCULATE_ENTHALPY_ONECELL(Tem, H);
        H_can = 0.0;
        for(k=0; k<Species_Number; k++)
            H_can += H[k]*cell.mixture.Massfr[k];
        Tem_old = Tem;
        Tem += (Enthalpy-H_can)/Cp*0.8;  //0.8 is the relaxiation coefficient
        deta = (Tem-Tem_old)/Tem;
        i++;
    }
    if(deta>1e-3)
        printf("Tem wierd in tem renew step! Tem=%f, Tem_old=%f, cell.mixture.Tem=%f\n", Tem, Tem_old, cell.mixture.Tem);

    return(Tem);
}










// Forward-Euler explict method to solve the enthalpy equation
void HEAT_TRANSFER_FE_ENTHALPY(int E_S_number, int *Tem_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source)
{
    int i,k,i1;
    double Pre, rad_p=1.0;
    double mfr_old, mfr_new, A0, A1,r;
    double J_con_left, J_con_right, J_h_left, J_h_right, J_drag_left, J_drag_right, S_rad, Omega;
    double density_ratio;
    //    double J_con_left1, J_con_right1, J_h_left1, J_h_righ1, J_drag_left1, J_drag_right1;
    double Tem[domain_cellnumber],Tem_old[domain_cellnumber]; // cell[i].mixture.Tem!!!
    double Enthalpy_old[domain_cellnumber][Species_Number],Enthalpy[domain_cellnumber][Species_Number];

    struct cell_information cell_new[domain_cellnumber];

    //    printf("E_S_number=%d, deta_x=%f,",E_S_number, *deta_x);

    //to set boundary conditions and record the initial values of each step	
    for(i=0; i<domain_cellnumber; i++)
    {   
        cell_new[i] = cell[i];
        Tem_old[i] = cell[i].Tem_old;
    }

    // to calculate enthalpy of each species based on the original Temperature. J/kg
    CALCULATE_ENTHALPY(domain_cellnumber, Tem_old, Enthalpy_old);

    // to solve heat transfer equations where conduction, convection and radiation terms are considered
    // to calculate vector a[],b[],c[] and d[]
    for(i=0; i<domain_cellnumber; i++)
    {
        if(i==0)
        {
            //the unit of heat capacity should be convert from J/[mol K] to J/[kgK]
            A0 = *deta_x;
            mfr_old = cell[i].Density_old*cell[i].u_old*dy_co;
            A1 = mfr_old*cell[i].mixture.Enthalpy*1000/cell[i].mixture.Molecular_Weight;  //mfr*h=kg/s/m*J/kg
            // Heat flux based on old nth value!
            J_con_left = (cell[0].mixture.Conduction_Coeff+cell[domain_cellnumber-1].mixture.Conduction_Coeff)/(dy_co+dy_co)*(cell[domain_cellnumber-1].Tem_old-cell[0].Tem_old);
            J_con_right = 2*(cell[0].mixture.Conduction_Coeff*cell[1].size_old+cell[1].mixture.Conduction_Coeff*dy_co)/(cell[1].size_old+dy_co)*(cell[1].Tem_old-cell[0].Tem_old)/(cell[1].size_old+dy_co);

            J_h_left = 0.0;
            J_h_right = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                J_h_left += (cell[0].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/2.0*(Enthalpy_old[0][k]+Enthalpy_old[domain_cellnumber-1][k])/2.0*(cell[domain_cellnumber-1].Massfr_old[k]-cell[0].Massfr_old[k])/(dy_co+dy_co);
                J_h_right += (cell[0].mixture.Spe_Diff_Coeff[k]*cell[1].size_old+cell[1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[1].size_old+dy_co)*(Enthalpy_old[0][k]*cell[1].size_old+Enthalpy_old[1][k]*dy_co)/(cell[1].size_old+dy_co)*(cell[1].Massfr_old[k]-cell[0].Massfr_old[k])/(cell[1].size_old+dy_co);
            }
            J_h_left *= (cell[0].Density_old+cell[domain_cellnumber-1].Density_old);
            J_h_right *= (2*(cell[0].Density_old*cell[1].size_old+cell[1].Density_old*dy_co)/(cell[1].size_old+dy_co));

            J_drag_left = (cell[0].mixture.Kinetic_Viscosity_Coeff+cell[domain_cellnumber-1].mixture.Kinetic_Viscosity_Coeff)*(cell[domain_cellnumber-1].u_old*cell[domain_cellnumber-1].u_old-cell[0].u_old*cell[0].u_old)/(dy_co+dy_co)/2.0;

            J_drag_right = (cell[0].mixture.Kinetic_Viscosity_Coeff*cell[1].size_old+cell[1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[1].u_old*cell[1].u_old-cell[0].u_old*cell[0].u_old)/pow((dy_co+cell[1].size_old),2);

            //Radiation
            S_rad =  rad_p*SOURCE_RADIATION(cell[i],*deta_x);

            //heat source due to reaction
            Omega = 0.0;
            r = A1+A0*(J_con_left+J_con_right+J_h_left+J_h_right+J_drag_left+J_drag_right)+S_rad+Omega+source[i].Enth_h;

            cell_new[i].mixture.Enthalpy = r/(mfr_old+source[i].mass)*cell[i].mixture.Molecular_Weight/1000;  //J/mol

        }
        else if(i==domain_cellnumber-1)
        {
            //the unit of enthalpy shall be convert from J/[mol] to J/[kg]
            A0 = *deta_x;
            mfr_old = cell[i].Density_old*cell[i].u_old*dy_co;
            A1 = mfr_old*cell[i].mixture.Enthalpy*1000/cell[i].mixture.Molecular_Weight;  //mfr*h=kg/s/m*J/kg
            // Heat flux based on old nth value!
            J_con_left = 2*(cell[i].mixture.Conduction_Coeff*cell[i-1].size_old+cell[i-1].mixture.Conduction_Coeff*dy_co)/(dy_co+cell[i-1].size_old)*(cell[i-1].Tem_old-cell[i].Tem_old);
            J_con_right = (cell[0].mixture.Conduction_Coeff+cell[i].mixture.Conduction_Coeff)*(cell[0].Tem_old-cell[i].Tem_old)/(dy_co+dy_co);

            J_h_left = 0.0;
            J_h_right = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                J_h_left +=(cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[i-1].size_old+dy_co)*(Enthalpy_old[i][k]*cell[i-1].size_old+Enthalpy_old[i-1][k]*dy_co)/(cell[i-1].size_old+dy_co)*(cell[i-1].Massfr_old[k]-cell[i].Massfr_old[k])/(cell[i].size_old+dy_co);

                J_h_right += (cell[0].mixture.Spe_Diff_Coeff[k]+cell[i].mixture.Spe_Diff_Coeff[k])/2.0*(Enthalpy_old[0][k]+Enthalpy_old[i][k])/2.0*(cell[0].Massfr_old[k]-cell[i].Massfr_old[k])/(dy_co+dy_co);
            }
            J_h_left *= (2*(cell[i].Density_old*cell[i-1].size_old+cell[i-1].Density_old*dy_co)/(cell[i-1].size_old+dy_co));
            J_h_right *= (cell[i].Density_old+cell[0].Density_old);

            J_drag_left = (cell[i].mixture.Kinetic_Viscosity_Coeff*cell[i-1].size_old+cell[i-1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[i-1].u_old*cell[i-1].u_old-cell[i].u_old*cell[i].u_old)/pow((dy_co+cell[i-1].size_old),2);

            J_drag_right = (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[0].mixture.Kinetic_Viscosity_Coeff)*(cell[0].u_old*cell[0].u_old-cell[i].u_old*cell[i].u_old)/(dy_co+dy_co)/2.0;

            //Radiation
            S_rad =  rad_p*SOURCE_RADIATION(cell[i],*deta_x);
            //heat source due to reaction
            Omega = 0.0;

            r = A1+A0*(J_con_left+J_con_right+J_h_left+J_h_right+J_drag_left+J_drag_right)+S_rad+Omega+source[i].Enth_h;

            cell_new[i].mixture.Enthalpy = r/(mfr_old+source[i].mass)*cell[i].mixture.Molecular_Weight/1000;  //J/mol
        }

        else
        {   
            //the unit of heat capacity should be convert from J/[mol K] to J/[kg K]
            mfr_old = cell[i].Density_old*cell[i].u_old*dy_co;
            A0 = *deta_x;
            A1 = mfr_old*cell[i].mixture.Enthalpy*1000/cell[i].mixture.Molecular_Weight;

            // Heat flux based on old nth value!
            //            J_con_left = 2*(cell[i].mixture.Conduction_Coeff*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Conduction_Coeff*cell[i].size)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size)*(cell[i-1].Tem_old-cell[i].Tem_old)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size);
            J_con_left = -J_con_right;
            J_con_right = 2*(cell[i].mixture.Conduction_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Conduction_Coeff*cell[i].size_old)/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old)*(cell[i+1].Tem_old-cell[i].Tem_old)/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old);

            //            J_h_left = 0.0;
            J_h_left = -J_h_right;
            J_h_right = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                //                J_h_left += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old)*(Enthalpy_old[i][k]*(i==1?dy_co:cell[i-1].size_old)+Enthalpy_old[i-1][k]*cell[i].size_old)/(cell[i].size_old+(i==1?dy_co:cell[i-1].size_old))*(cell[i-1].Massfr_old[k]-cell[i].Massfr_old[k])/((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old);
                J_h_right += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old))*(Enthalpy_old[i][k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+Enthalpy_old[i+1][k]*cell[i].size_old)/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old))*(cell[i+1].Massfr_old[k]-cell[i].Massfr_old[k])/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old));
            }
            //            J_h_left *= (2*(cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/(cell[i].size_old+(i==1?dy_co:cell[i-1].size_old)));
            J_h_right *= (2*(cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)));


            J_drag_left = -J_drag_right;

            //            J_drag_left = (cell[i].mixture.Kinetic_Viscosity_Coeff*(i==2?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Kinetic_Viscosity_Coeff*cell[i].size_old)/(cell[i].size_old+(i==2?dy_co:cell[i-1].size_old))*(cell[i-1].u_old*cell[i-1].u_old-cell[i].u_old*cell[i].u_old)/(cell[i].size_old+(i==2?dy_co:cell[i-1].size_old));

            J_drag_right = (cell[i].mixture.Kinetic_Viscosity_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Kinetic_Viscosity_Coeff*cell[i].size_old)*(cell[i+1].u_old*cell[i+1].u_old-cell[i].u_old*cell[i].u_old)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old),2);

            //Radiation
            S_rad =  rad_p*SOURCE_RADIATION(cell[i],*deta_x);

            //heat source due to reaction
            Omega = 0.0;

            r = A1+A0*(J_con_left+J_con_right+J_h_left+J_h_right+J_drag_left+J_drag_right)+S_rad+Omega+source[i].Enth_h;

            cell_new[i].mixture.Enthalpy = r/(mfr_old+source[i].mass)*cell[i].mixture.Molecular_Weight/1000;  //J/mol
        }

        // to calculate the mixture temperature according to the enthalpy
        cell_new[i].mixture.Tem = CALCULATE_CELL_TEM(cell_new[i]);
        Tem[i] = cell_new[i].mixture.Tem;

        // to print wrong value in order to check the bugs
        if(!(cell_new[i].mixture.Tem>250 && cell_new[i].mixture.Tem<5000))
        {
            *Tem_Flag = 0;
            printf("%dth step: error occurs during 1st circulation in heat transfer process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,i,domain_cellnumber,*deta_x);
            printf("number  cell_size       cell_size_old   cell.u         cell.Tem       cellnew.Tem    cell.spedif    source\n");
            for(i1=0; i1<domain_cellnumber; i1++)
            {
                if(i1<10 || i1>domain_cellnumber-10)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7e\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,cell[i1].mixture.Spe_Diff_Coeff[He], source[i].Enth_h);
                if(i1>i-20 && i1<i+20)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7e\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,cell[i1].mixture.Spe_Diff_Coeff[He], source[i].Enth_h);
                if(i1%200==0)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7e\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,cell[i1].mixture.Spe_Diff_Coeff[He],source[i].Enth_h);        
            }
            printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Tem,cell[i+1].mixture.Tem,cell[i-1].mixture.Tem);
            printf("cell[0].mixture.Tem=%8.5f, cell[end].mixture.Tem=%8.5f\n",cell[0].mixture.Tem,cell[domain_cellnumber-1].mixture.Tem);
            goto abnormal_1;
        }       
    }
    
    //to restore the calculated cell[i].mixture.Massfr[k]
    for(i=0; i<domain_cellnumber; i++)
    {
       
        cell[i] = cell_new[i];
        cell[i].Density_old = cell[i].mixture.Density;
        cell[i].mixture.Density = cell[i].Density_old*cell[i].Tem_old/cell[i].mixture.Tem;
        density_ratio = cell[i].Density_old/cell[i].mixture.Density;
        // to renew the cell size
        DILATATION_MODLE(density_ratio, &(cell[i]));

        RENEW_MIXPRO_MASSY(i,cell);
    }

 abnormal_1:    ;
}










// Forward-Euler explict method to solve the enthalpy equation
void HEAT_TRANSFER_2NDFE_ENTHALPY(int E_S_number, int *Tem_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source)
{
    int i,k,i1;
    double Pre, rad_p=1.0;
    double mfr_old, mfr_new, A0, A1,r;
    double J_con_left, J_con_right, J_h_left, J_h_right, J_drag_left, J_drag_right, S_rad, Omega;
    double J_con_left_1, J_con_right_1, J_h_left_1, J_h_right_1, J_drag_left_1, J_drag_right_1;

    double density_ratio;
    //    double J_con_left1, J_con_right1, J_h_left1, J_h_righ1, J_drag_left1, J_drag_right1;
    double Tem[domain_cellnumber],Tem_old[domain_cellnumber]; // cell[i].mixture.Tem!!!
    double Enthalpy_old[domain_cellnumber][Species_Number],Enthalpy[domain_cellnumber][Species_Number];

    struct cell_information cell_new[domain_cellnumber];

    //    printf("E_S_number=%d, deta_x=%f,",E_S_number, *deta_x);

    //to set boundary conditions and record the initial values of each step	
    for(i=0; i<domain_cellnumber; i++)
    {   
        cell_new[i] = cell[i];
        Tem_old[i] = cell[i].Tem_old;
    }

    // to calculate enthalpy of each species based on the original Temperature. J/kg
    CALCULATE_ENTHALPY(domain_cellnumber, Tem_old, Enthalpy_old);

    // to solve heat transfer equations where conduction, convection and radiation terms are considered
    // to calculate vector a[],b[],c[] and d[]
    for(i=0; i<domain_cellnumber; i++)
    {
        if(i==0)
        {
            //the unit of heat capacity should be convert from J/[mol K] to J/[kgK]
            A0 = *deta_x;
            mfr_old = cell[i].Density_old*cell[i].u_old*dy_co;
            A1 = mfr_old*cell[i].mixture.Enthalpy*1000/cell[i].mixture.Molecular_Weight;  //mfr*h=kg/s/m*J/kg
            // Heat flux based on old nth value!
            J_con_left = (cell[0].mixture.Conduction_Coeff+cell[domain_cellnumber-1].mixture.Conduction_Coeff)/(dy_co+dy_co)*(cell[domain_cellnumber-1].Tem_old-cell[0].Tem_old)/2.;
            J_con_left_1 = (cell[0].mixture.Conduction_Coeff+cell[domain_cellnumber-1].mixture.Conduction_Coeff)/(dy_co+dy_co)*(cell[domain_cellnumber-1].mixture.Tem-cell[0].mixture.Tem)/2.;

            J_con_right = (cell[0].mixture.Conduction_Coeff*cell[1].size_old+cell[1].mixture.Conduction_Coeff*dy_co)/(cell[1].size_old+dy_co)*(cell[1].Tem_old-cell[0].Tem_old)/(cell[1].size_old+dy_co);
            J_con_right_1 = (cell[0].mixture.Conduction_Coeff*cell[1].size+cell[1].mixture.Conduction_Coeff*dy_co)/(cell[1].size+dy_co)*(cell[1].mixture.Tem-cell[0].mixture.Tem)/(cell[1].size+dy_co);

            J_h_left = 0.0;
            J_h_right = 0.0;
            J_h_left_1 = 0.0;
            J_h_right_1 = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                J_h_left += (cell[0].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/2.0*(Enthalpy_old[0][k]+Enthalpy_old[domain_cellnumber-1][k])/2.0*(cell[domain_cellnumber-1].Massfr_old[k]-cell[0].Massfr_old[k])/(dy_co+dy_co);
                J_h_left_1 += (cell[0].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/2.0*(Enthalpy_old[0][k]+Enthalpy_old[domain_cellnumber-1][k])/2.0*(cell[domain_cellnumber-1].mixture.Massfr[k]-cell[0].mixture.Massfr[k])/(dy_co+dy_co);

                J_h_right += (cell[0].mixture.Spe_Diff_Coeff[k]*cell[1].size_old+cell[1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[1].size_old+dy_co)*(Enthalpy_old[0][k]*cell[1].size_old+Enthalpy_old[1][k]*dy_co)/(cell[1].size_old+dy_co)*(cell[1].Massfr_old[k]-cell[0].Massfr_old[k])/(cell[1].size_old+dy_co);
                J_h_right_1 += (cell[0].mixture.Spe_Diff_Coeff[k]*cell[1].size+cell[1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[1].size+dy_co)*(Enthalpy_old[0][k]*cell[1].size+Enthalpy_old[1][k]*dy_co)/(cell[1].size+dy_co)*(cell[1].mixture.Massfr[k]-cell[0].mixture.Massfr[k])/(cell[1].size+dy_co);
            }
            J_h_left *= (cell[0].Density_old+cell[domain_cellnumber-1].Density_old)/2.0;
            J_h_left_1 *= (cell[0].mixture.Density+cell[domain_cellnumber-1].mixture.Density)/2.0;

            J_h_right *= (cell[0].Density_old*cell[1].size_old+cell[1].Density_old*dy_co)/(cell[1].size_old+dy_co);
            J_h_right_1 *= (cell[0].mixture.Density*cell[1].size+cell[1].mixture.Density*dy_co)/(cell[1].size+dy_co);

            J_drag_left = (cell[0].mixture.Kinetic_Viscosity_Coeff+cell[domain_cellnumber-1].mixture.Kinetic_Viscosity_Coeff)*(cell[domain_cellnumber-1].u_old*cell[domain_cellnumber-1].u_old-cell[0].u_old*cell[0].u_old)/(dy_co+dy_co)/4.0;
            J_drag_left_1 = (cell[0].mixture.Kinetic_Viscosity_Coeff+cell[domain_cellnumber-1].mixture.Kinetic_Viscosity_Coeff)*(cell[domain_cellnumber-1].u*cell[domain_cellnumber-1].u-cell[0].u*cell[0].u)/(dy_co+dy_co)/4.0;

            J_drag_right = (cell[0].mixture.Kinetic_Viscosity_Coeff*cell[1].size_old+cell[1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[1].u_old*cell[1].u_old-cell[0].u_old*cell[0].u_old)/pow((dy_co+cell[1].size_old),2)/2.;
            J_drag_right_1 = (cell[0].mixture.Kinetic_Viscosity_Coeff*cell[1].size+cell[1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[1].u*cell[1].u-cell[0].u*cell[0].u)/pow((dy_co+cell[1].size),2)/2.;

            //Radiation
            S_rad =  rad_p*SOURCE_RADIATION(cell[i],*deta_x);

            //heat source due to reaction
            Omega = 0.0;
            r = A1+A0*(J_con_left+J_con_left_1+J_con_right+J_con_right_1+J_h_left+J_h_left_1+J_h_right+J_h_right_1+J_drag_left+J_drag_left_1+J_drag_right+J_drag_right_1)+S_rad+Omega+source[i].Enth_h;

            cell_new[i].mixture.Enthalpy = r/(mfr_old+source[i].mass)*cell[i].mixture.Molecular_Weight/1000;  //J/mol

        }
        else if(i==domain_cellnumber-1)
        {
            //the unit of enthalpy shall be convert from J/[mol] to J/[kg]
            A0 = *deta_x;
            mfr_old = cell[i].Density_old*cell[i].u_old*dy_co;
            A1 = mfr_old*cell[i].mixture.Enthalpy*1000/cell[i].mixture.Molecular_Weight;  //mfr*h=kg/s/m*J/kg
            // Heat flux based on old nth value!
            J_con_left = (cell[i].mixture.Conduction_Coeff*cell[i-1].size_old+cell[i-1].mixture.Conduction_Coeff*dy_co)/(dy_co+cell[i-1].size_old)*(cell[i-1].Tem_old-cell[i].Tem_old);
            J_con_left_1 = (cell[i].mixture.Conduction_Coeff*cell[i-1].size+cell[i-1].mixture.Conduction_Coeff*dy_co)/(dy_co+cell[i-1].size)*(cell[i-1].mixture.Tem-cell[i].mixture.Tem);

            J_con_right = (cell[0].mixture.Conduction_Coeff+cell[i].mixture.Conduction_Coeff)*(cell[0].Tem_old-cell[i].Tem_old)/(dy_co+dy_co);
            J_con_right_1 = (cell[0].mixture.Conduction_Coeff+cell[i].mixture.Conduction_Coeff)*(cell[0].mixture.Tem-cell[i].mixture.Tem)/(dy_co+dy_co);

            J_h_left = 0.0;
            J_h_right = 0.0;
            J_h_left_1 = 0.0;
            J_h_right_1 = 0.0;

            for(k=0; k<Species_Number; k++)
            {
                J_h_left += (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[i-1].size_old+dy_co)*(Enthalpy_old[i][k]*cell[i-1].size_old+Enthalpy_old[i-1][k]*dy_co)/(cell[i-1].size_old+dy_co)*(cell[i-1].Massfr_old[k]-cell[i].Massfr_old[k])/(cell[i].size_old+dy_co);
                J_h_left_1 += (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[i-1].size+dy_co)*(Enthalpy_old[i][k]*cell[i-1].size+Enthalpy_old[i-1][k]*dy_co)/(cell[i-1].size+dy_co)*(cell[i-1].mixture.Massfr[k]-cell[i].mixture.Massfr[k])/(cell[i].size+dy_co);

                J_h_right += (cell[0].mixture.Spe_Diff_Coeff[k]+cell[i].mixture.Spe_Diff_Coeff[k])/2.0*(Enthalpy_old[0][k]+Enthalpy_old[i][k])/2.0*(cell[0].Massfr_old[k]-cell[i].Massfr_old[k])/(dy_co+dy_co);
                J_h_right_1 += (cell[0].mixture.Spe_Diff_Coeff[k]+cell[i].mixture.Spe_Diff_Coeff[k])/2.0*(Enthalpy_old[0][k]+Enthalpy_old[i][k])/2.0*(cell[0].mixture.Massfr[k]-cell[i].mixture.Massfr[k])/(dy_co+dy_co);
            }
            J_h_left *= (cell[i].Density_old*cell[i-1].size_old+cell[i-1].Density_old*dy_co)/(cell[i-1].size_old+dy_co);
            J_h_left_1 *= (cell[i].mixture.Density*cell[i-1].size+cell[i-1].mixture.Density*dy_co)/(cell[i-1].size+dy_co);
            J_h_right *= (cell[i].Density_old+cell[0].Density_old);
            J_h_right_1 *= (cell[i].mixture.Density+cell[0].mixture.Density);

            J_drag_left = (cell[i].mixture.Kinetic_Viscosity_Coeff*cell[i-1].size_old+cell[i-1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[i-1].u_old*cell[i-1].u_old-cell[i].u_old*cell[i].u_old)/pow((dy_co+cell[i-1].size_old),2)/2.;
            J_drag_left_1 = (cell[i].mixture.Kinetic_Viscosity_Coeff*cell[i-1].size+cell[i-1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[i-1].u*cell[i-1].u-cell[i].u*cell[i].u)/pow((dy_co+cell[i-1].size),2)/2.;

            J_drag_right = (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[0].mixture.Kinetic_Viscosity_Coeff)*(cell[0].u_old*cell[0].u_old-cell[i].u_old*cell[i].u_old)/(dy_co+dy_co)/4.0;
            J_drag_right_1 = (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[0].mixture.Kinetic_Viscosity_Coeff)*(cell[0].u*cell[0].u-cell[i].u*cell[i].u)/(dy_co+dy_co)/4.0;

            //Radiation
            S_rad =  rad_p*SOURCE_RADIATION(cell[i],*deta_x);
            //heat source due to reaction
            Omega = 0.0;

            r = A1+A0*(J_con_left+J_con_left_1+J_con_right+J_con_right_1+J_h_left+J_h_left_1+J_h_right+J_h_right_1+J_drag_left+J_drag_left_1+J_drag_right+J_drag_right_1)+S_rad+Omega+source[i].Enth_h;

            cell_new[i].mixture.Enthalpy = r/(mfr_old+source[i].mass)*cell[i].mixture.Molecular_Weight/1000;  //J/mol
        }

        else
        {   
            //the unit of heat capacity should be convert from J/[mol K] to J/[kg K]
            mfr_old = cell[i].Density_old*cell[i].u_old*dy_co;
            A0 = *deta_x;
            A1 = mfr_old*cell[i].mixture.Enthalpy*1000/cell[i].mixture.Molecular_Weight;

            // Heat flux based on old nth value!
            //            J_con_left = 2*(cell[i].mixture.Conduction_Coeff*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Conduction_Coeff*cell[i].size)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size)*(cell[i-1].Tem_old-cell[i].Tem_old)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size);
            J_con_left = -J_con_right;
            J_con_left_1 = -J_con_right_1;
            J_con_right = (cell[i].mixture.Conduction_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Conduction_Coeff*cell[i].size_old)/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old)*(cell[i+1].Tem_old-cell[i].Tem_old)/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old);
            J_con_right_1 = (cell[i].mixture.Conduction_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Conduction_Coeff*cell[i].size)/((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size)*(cell[i+1].mixture.Tem-cell[i].mixture.Tem)/((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size);

            //            J_h_left = 0.0;
            J_h_left = -J_h_right;
            J_h_left_1 = -J_h_right_1;
            J_h_right = 0.0;
            J_h_right_1 = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                //                J_h_left += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old)*(Enthalpy_old[i][k]*(i==1?dy_co:cell[i-1].size_old)+Enthalpy_old[i-1][k]*cell[i].size_old)/(cell[i].size_old+(i==1?dy_co:cell[i-1].size_old))*(cell[i-1].Massfr_old[k]-cell[i].Massfr_old[k])/((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old);
                J_h_right += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old))*(Enthalpy_old[i][k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+Enthalpy_old[i+1][k]*cell[i].size_old)/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old))*(cell[i+1].Massfr_old[k]-cell[i].Massfr_old[k])/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old));
                J_h_right_1 += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size)/(cell[i].size+(i==domain_cellnumber-2?dy_co:cell[i+1].size))*(Enthalpy_old[i][k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+Enthalpy_old[i+1][k]*cell[i].size)/(cell[i].size+(i==domain_cellnumber-2?dy_co:cell[i+1].size))*(cell[i+1].mixture.Massfr[k]-cell[i].mixture.Massfr[k])/(cell[i].size+(i==domain_cellnumber-2?dy_co:cell[i+1].size));
            }
            //            J_h_left *= (2*(cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/(cell[i].size_old+(i==1?dy_co:cell[i-1].size_old)));
            J_h_right *= (cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old));
            J_h_right_1 *= (cell[i].mixture.Density*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Density*cell[i].size)/(cell[i].size+(i==domain_cellnumber-2?dy_co:cell[i+1].size));

            J_drag_left = -J_drag_right;
            J_drag_left_1 = -J_drag_right_1;
            //            J_drag_left = (cell[i].mixture.Kinetic_Viscosity_Coeff*(i==2?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Kinetic_Viscosity_Coeff*cell[i].size_old)/(cell[i].size_old+(i==2?dy_co:cell[i-1].size_old))*(cell[i-1].u_old*cell[i-1].u_old-cell[i].u_old*cell[i].u_old)/(cell[i].size_old+(i==2?dy_co:cell[i-1].size_old));

            J_drag_right = (cell[i].mixture.Kinetic_Viscosity_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Kinetic_Viscosity_Coeff*cell[i].size_old)*(cell[i+1].u_old*cell[i+1].u_old-cell[i].u_old*cell[i].u_old)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old),2)/2.;
            J_drag_right_1 = (cell[i].mixture.Kinetic_Viscosity_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Kinetic_Viscosity_Coeff*cell[i].size)*(cell[i+1].u*cell[i+1].u-cell[i].u*cell[i].u)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size),2)/2.;

            //Radiation
            S_rad =  rad_p*SOURCE_RADIATION(cell[i],*deta_x);

            //heat source due to reaction
            Omega = 0.0;

            r = A1+A0*(J_con_left+J_con_left_1+J_con_right+J_con_right_1+J_h_left+J_h_left_1+J_h_right+J_h_right_1+J_drag_left+J_drag_left_1+J_drag_right+J_drag_right_1)+S_rad+Omega+source[i].Enth_h;

            cell_new[i].mixture.Enthalpy = r/(mfr_old+source[i].mass)*cell[i].mixture.Molecular_Weight/1000;  //J/mol
        }

        // to calculate the mixture temperature according to the enthalpy
        cell_new[i].mixture.Tem = CALCULATE_CELL_TEM(cell_new[i]);
        Tem[i] = cell_new[i].mixture.Tem;

        // to print wrong value in order to check the bugs
        if(!(cell_new[i].mixture.Tem>250 && cell_new[i].mixture.Tem<5000))
        {
            *Tem_Flag = 0;
            printf("%dth step: error occurs during 1st circulation in 2ndFE heat transfer process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,i,domain_cellnumber,*deta_x);
            printf("number  cell_size       cell_size_old   cell.u         cell.Tem       cellnew.Tem    cell.spedif\n");
            for(i1=0; i1<domain_cellnumber; i1++)
            {
                if(i1<10 || i1>domain_cellnumber-10)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,cell[i1].mixture.Spe_Diff_Coeff[He]);
                if(i1>i-20 && i1<i+20)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,cell[i1].mixture.Spe_Diff_Coeff[He]);
                if(i1%200==0)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,cell[i1].mixture.Spe_Diff_Coeff[He]);        
            }
            printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Tem,cell[i+1].mixture.Tem,cell[i-1].mixture.Tem);
            printf("cell[0].mixture.Tem=%8.5f, cell[end].mixture.Tem=%8.5f\n",cell[0].mixture.Tem,cell[domain_cellnumber-1].mixture.Tem);
            goto abnormal_1;
        }       
    }
    
    //to restore the calculated cell[i].mixture.Massfr[k] and to renew the density and cell size
    for(i=0; i<domain_cellnumber; i++)
    {
        cell[i].Tem_old = cell[i].mixture.Tem;  // should be equal even without this line
        cell[i].mixture.Tem = cell_new[i].mixture.Tem;  //only to get the renewed temperature, but not enthalpy!!!
        cell[i].mixture.Density *= cell[i].Tem_old/cell[i].mixture.Tem;
        density_ratio = cell[i].mixture.Tem/cell[i].Tem_old;
        // to renew the cell size
        DILATATION_MODLE(density_ratio, &(cell[i]));
    }

  // to calculate enthalpy of each species based on the candidate Temperature. J/kg
    CALCULATE_ENTHALPY(domain_cellnumber, Tem, Enthalpy);


    // the second circulation to solve heat transfer equations where conduction, convection and radiation terms are considered
    // to calculate vector a[],b[],c[] and d[]
    for(i=0; i<domain_cellnumber; i++)
    {
        if(i==0)
        {
            //the unit of heat capacity should be convert from J/[mol K] to J/[kgK]
            A0 = *deta_x;
            mfr_old = cell[i].Density_old*cell[i].u_old*dy_co;
            A1 = mfr_old*cell[i].mixture.Enthalpy*1000/cell[i].mixture.Molecular_Weight;  //mfr*h=kg/s/m*J/kg
            // Heat flux based on old nth value!
            J_con_left = (cell[0].mixture.Conduction_Coeff+cell[domain_cellnumber-1].mixture.Conduction_Coeff)/(dy_co+dy_co)*(cell[domain_cellnumber-1].Tem_old-cell[0].Tem_old)/2.;
            J_con_left_1 = (cell[0].mixture.Conduction_Coeff+cell[domain_cellnumber-1].mixture.Conduction_Coeff)/(dy_co+dy_co)*(cell[domain_cellnumber-1].mixture.Tem-cell[0].mixture.Tem)/2.;

            J_con_right = (cell[0].mixture.Conduction_Coeff*cell[1].size_old+cell[1].mixture.Conduction_Coeff*dy_co)/(cell[1].size_old+dy_co)*(cell[1].Tem_old-cell[0].Tem_old)/(cell[1].size_old+dy_co);
            J_con_right_1 = (cell[0].mixture.Conduction_Coeff*cell[1].size+cell[1].mixture.Conduction_Coeff*dy_co)/(cell[1].size+dy_co)*(cell[1].mixture.Tem-cell[0].mixture.Tem)/(cell[1].size+dy_co);

            J_h_left = 0.0;
            J_h_right = 0.0;
            J_h_left_1 = 0.0;
            J_h_right_1 = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                J_h_left += (cell[0].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/2.0*(Enthalpy_old[0][k]+Enthalpy_old[domain_cellnumber-1][k])/2.0*(cell[domain_cellnumber-1].Massfr_old[k]-cell[0].Massfr_old[k])/(dy_co+dy_co);
                J_h_left_1 += (cell[0].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/2.0*(Enthalpy[0][k]+Enthalpy[domain_cellnumber-1][k])/2.0*(cell[domain_cellnumber-1].mixture.Massfr[k]-cell[0].mixture.Massfr[k])/(dy_co+dy_co);

                J_h_right += (cell[0].mixture.Spe_Diff_Coeff[k]*cell[1].size_old+cell[1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[1].size_old+dy_co)*(Enthalpy_old[0][k]*cell[1].size_old+Enthalpy_old[1][k]*dy_co)/(cell[1].size_old+dy_co)*(cell[1].Massfr_old[k]-cell[0].Massfr_old[k])/(cell[1].size_old+dy_co);
                J_h_right_1 += (cell[0].mixture.Spe_Diff_Coeff[k]*cell[1].size+cell[1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[1].size+dy_co)*(Enthalpy[0][k]*cell[1].size+Enthalpy[1][k]*dy_co)/(cell[1].size+dy_co)*(cell[1].mixture.Massfr[k]-cell[0].mixture.Massfr[k])/(cell[1].size+dy_co);
            }
            J_h_left *= (cell[0].Density_old+cell[domain_cellnumber-1].Density_old)/2.0;
            J_h_left_1 *= (cell[0].mixture.Density+cell[domain_cellnumber-1].mixture.Density)/2.0;

            J_h_right *= (cell[0].Density_old*cell[1].size_old+cell[1].Density_old*dy_co)/(cell[1].size_old+dy_co);
            J_h_right_1 *= (cell[0].mixture.Density*cell[1].size+cell[1].mixture.Density*dy_co)/(cell[1].size+dy_co);

            J_drag_left = (cell[0].mixture.Kinetic_Viscosity_Coeff+cell[domain_cellnumber-1].mixture.Kinetic_Viscosity_Coeff)*(cell[domain_cellnumber-1].u_old*cell[domain_cellnumber-1].u_old-cell[0].u_old*cell[0].u_old)/(dy_co+dy_co)/4.0;
            J_drag_left_1 = (cell[0].mixture.Kinetic_Viscosity_Coeff+cell[domain_cellnumber-1].mixture.Kinetic_Viscosity_Coeff)*(cell[domain_cellnumber-1].u*cell[domain_cellnumber-1].u-cell[0].u*cell[0].u)/(dy_co+dy_co)/4.0;

            J_drag_right = (cell[0].mixture.Kinetic_Viscosity_Coeff*cell[1].size_old+cell[1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[1].u_old*cell[1].u_old-cell[0].u_old*cell[0].u_old)/pow((dy_co+cell[1].size_old),2)/2.;
            J_drag_right_1 = (cell[0].mixture.Kinetic_Viscosity_Coeff*cell[1].size+cell[1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[1].u*cell[1].u-cell[0].u*cell[0].u)/pow((dy_co+cell[1].size),2)/2.;

            //Radiation
            S_rad =  rad_p*SOURCE_RADIATION(cell[i],*deta_x);

            //heat source due to reaction
            Omega = 0.0;
            r = A1+A0*(J_con_left+J_con_left_1+J_con_right+J_con_right_1+J_h_left+J_h_left_1+J_h_right+J_h_right_1+J_drag_left+J_drag_left_1+J_drag_right+J_drag_right_1)+S_rad+Omega+source[i].Enth_h;

            cell_new[i].mixture.Enthalpy = r/(mfr_old+source[i].mass)*cell[i].mixture.Molecular_Weight/1000;  //J/mol

        }
        else if(i==domain_cellnumber-1)
        {
            //the unit of enthalpy shall be convert from J/[mol] to J/[kg]
            A0 = *deta_x;
            mfr_old = cell[i].Density_old*cell[i].u_old*dy_co;
            A1 = mfr_old*cell[i].mixture.Enthalpy*1000/cell[i].mixture.Molecular_Weight;  //mfr*h=kg/s/m*J/kg
            // Heat flux based on old nth value!
            J_con_left = (cell[i].mixture.Conduction_Coeff*cell[i-1].size_old+cell[i-1].mixture.Conduction_Coeff*dy_co)/(dy_co+cell[i-1].size_old)*(cell[i-1].Tem_old-cell[i].Tem_old);
            J_con_left_1 = (cell[i].mixture.Conduction_Coeff*cell[i-1].size+cell[i-1].mixture.Conduction_Coeff*dy_co)/(dy_co+cell[i-1].size)*(cell[i-1].mixture.Tem-cell[i].mixture.Tem);

            J_con_right = (cell[0].mixture.Conduction_Coeff+cell[i].mixture.Conduction_Coeff)*(cell[0].Tem_old-cell[i].Tem_old)/(dy_co+dy_co);
            J_con_right_1 = (cell[0].mixture.Conduction_Coeff+cell[i].mixture.Conduction_Coeff)*(cell[0].mixture.Tem-cell[i].mixture.Tem)/(dy_co+dy_co);

            J_h_left = 0.0;
            J_h_right = 0.0;
            J_h_left_1 = 0.0;
            J_h_right_1 = 0.0;

            for(k=0; k<Species_Number; k++)
            {
                J_h_left += (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[i-1].size_old+dy_co)*(Enthalpy_old[i][k]*cell[i-1].size_old+Enthalpy_old[i-1][k]*dy_co)/(cell[i-1].size_old+dy_co)*(cell[i-1].Massfr_old[k]-cell[i].Massfr_old[k])/(cell[i].size_old+dy_co);
                J_h_left_1 += (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[i-1].size+dy_co)*(Enthalpy[i][k]*cell[i-1].size+Enthalpy[i-1][k]*dy_co)/(cell[i-1].size+dy_co)*(cell[i-1].mixture.Massfr[k]-cell[i].mixture.Massfr[k])/(cell[i].size+dy_co);

                J_h_right += (cell[0].mixture.Spe_Diff_Coeff[k]+cell[i].mixture.Spe_Diff_Coeff[k])/2.0*(Enthalpy_old[0][k]+Enthalpy_old[i][k])/2.0*(cell[0].Massfr_old[k]-cell[i].Massfr_old[k])/(dy_co+dy_co);
                J_h_right_1 += (cell[0].mixture.Spe_Diff_Coeff[k]+cell[i].mixture.Spe_Diff_Coeff[k])/2.0*(Enthalpy[0][k]+Enthalpy[i][k])/2.0*(cell[0].mixture.Massfr[k]-cell[i].mixture.Massfr[k])/(dy_co+dy_co);
            }
            J_h_left *= (cell[i].Density_old*cell[i-1].size_old+cell[i-1].Density_old*dy_co)/(cell[i-1].size_old+dy_co);
            J_h_left_1 *= (cell[i].mixture.Density*cell[i-1].size+cell[i-1].mixture.Density*dy_co)/(cell[i-1].size+dy_co);
            J_h_right *= (cell[i].Density_old+cell[0].Density_old);
            J_h_right_1 *= (cell[i].mixture.Density+cell[0].mixture.Density);

            J_drag_left = (cell[i].mixture.Kinetic_Viscosity_Coeff*cell[i-1].size_old+cell[i-1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[i-1].u_old*cell[i-1].u_old-cell[i].u_old*cell[i].u_old)/pow((dy_co+cell[i-1].size_old),2)/2.;
            J_drag_left_1 = (cell[i].mixture.Kinetic_Viscosity_Coeff*cell[i-1].size+cell[i-1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[i-1].u*cell[i-1].u-cell[i].u*cell[i].u)/pow((dy_co+cell[i-1].size),2)/2.;

            J_drag_right = (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[0].mixture.Kinetic_Viscosity_Coeff)*(cell[0].u_old*cell[0].u_old-cell[i].u_old*cell[i].u_old)/(dy_co+dy_co)/4.0;
            J_drag_right_1 = (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[0].mixture.Kinetic_Viscosity_Coeff)*(cell[0].u*cell[0].u-cell[i].u*cell[i].u)/(dy_co+dy_co)/4.0;

            //Radiation
            S_rad =  rad_p*SOURCE_RADIATION(cell[i],*deta_x);
            //heat source due to reaction
            Omega = 0.0;

            r = A1+A0*(J_con_left+J_con_left_1+J_con_right+J_con_right_1+J_h_left+J_h_left_1+J_h_right+J_h_right_1+J_drag_left+J_drag_left_1+J_drag_right+J_drag_right_1)+S_rad+Omega+source[i].Enth_h;

            cell_new[i].mixture.Enthalpy = r/(mfr_old+source[i].mass)*cell[i].mixture.Molecular_Weight/1000;  //J/mol
        }

        else
        {   
            //the unit of heat capacity should be convert from J/[mol K] to J/[kg K]
            mfr_old = cell[i].Density_old*cell[i].u_old*dy_co;
            A0 = *deta_x;
            A1 = mfr_old*cell[i].mixture.Enthalpy*1000/cell[i].mixture.Molecular_Weight;

            // Heat flux based on old nth value!
            //            J_con_left = 2*(cell[i].mixture.Conduction_Coeff*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Conduction_Coeff*cell[i].size)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size)*(cell[i-1].Tem_old-cell[i].Tem_old)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size);
            J_con_left = -J_con_right;
            J_con_left_1 = -J_con_right_1;
            J_con_right = (cell[i].mixture.Conduction_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Conduction_Coeff*cell[i].size_old)/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old)*(cell[i+1].Tem_old-cell[i].Tem_old)/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old);
            J_con_right_1 = (cell[i].mixture.Conduction_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Conduction_Coeff*cell[i].size)/((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size)*(cell[i+1].mixture.Tem-cell[i].mixture.Tem)/((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size);

            //            J_h_left = 0.0;
            J_h_left = -J_h_right;
            J_h_left_1 = -J_h_right_1;
            J_h_right = 0.0;
            J_h_right_1 = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                //                J_h_left += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old)*(Enthalpy_old[i][k]*(i==1?dy_co:cell[i-1].size_old)+Enthalpy_old[i-1][k]*cell[i].size_old)/(cell[i].size_old+(i==1?dy_co:cell[i-1].size_old))*(cell[i-1].Massfr_old[k]-cell[i].Massfr_old[k])/((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old);
                J_h_right += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old))*(Enthalpy_old[i][k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+Enthalpy_old[i+1][k]*cell[i].size_old)/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old))*(cell[i+1].Massfr_old[k]-cell[i].Massfr_old[k])/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old));
                J_h_right_1 += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size)/(cell[i].size+(i==domain_cellnumber-2?dy_co:cell[i+1].size))*(Enthalpy[i][k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+Enthalpy[i+1][k]*cell[i].size)/(cell[i].size+(i==domain_cellnumber-2?dy_co:cell[i+1].size))*(cell[i+1].mixture.Massfr[k]-cell[i].mixture.Massfr[k])/(cell[i].size+(i==domain_cellnumber-2?dy_co:cell[i+1].size));
            }
            //            J_h_left *= (2*(cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/(cell[i].size_old+(i==1?dy_co:cell[i-1].size_old)));
            J_h_right *= (cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/(cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old));
            J_h_right_1 *= (cell[i].mixture.Density*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Density*cell[i].size)/(cell[i].size+(i==domain_cellnumber-2?dy_co:cell[i+1].size));

            J_drag_left = -J_drag_right;
            J_drag_left_1 = -J_drag_right_1;
            //            J_drag_left = (cell[i].mixture.Kinetic_Viscosity_Coeff*(i==2?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Kinetic_Viscosity_Coeff*cell[i].size_old)/(cell[i].size_old+(i==2?dy_co:cell[i-1].size_old))*(cell[i-1].u_old*cell[i-1].u_old-cell[i].u_old*cell[i].u_old)/(cell[i].size_old+(i==2?dy_co:cell[i-1].size_old));

            J_drag_right = (cell[i].mixture.Kinetic_Viscosity_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Kinetic_Viscosity_Coeff*cell[i].size_old)*(cell[i+1].u_old*cell[i+1].u_old-cell[i].u_old*cell[i].u_old)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old),2)/2.;
            J_drag_right_1 = (cell[i].mixture.Kinetic_Viscosity_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Kinetic_Viscosity_Coeff*cell[i].size)*(cell[i+1].u*cell[i+1].u-cell[i].u*cell[i].u)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size),2)/2.;

            //Radiation
            S_rad =  rad_p*SOURCE_RADIATION(cell[i],*deta_x);

            //heat source due to reaction
            Omega = 0.0;

            r = A1+A0*(J_con_left+J_con_left_1+J_con_right+J_con_right_1+J_h_left+J_h_left_1+J_h_right+J_h_right_1+J_drag_left+J_drag_left_1+J_drag_right+J_drag_right_1)+S_rad+Omega+source[i].Enth_h;

            cell_new[i].mixture.Enthalpy = r/(mfr_old+source[i].mass)*cell[i].mixture.Molecular_Weight/1000;  //J/mol
        }

        // to calculate the mixture temperature according to the enthalpy
        cell_new[i].mixture.Tem = CALCULATE_CELL_TEM(cell_new[i]);
        Tem[i] = cell_new[i].mixture.Tem;

        // to print wrong value in order to check the bugs
        if(!(cell_new[i].mixture.Tem>250 && cell_new[i].mixture.Tem<5000))
        {
            *Tem_Flag = 0;
            printf("%dth step: error occurs during 2nd circulation in 2ndFE heat transfer process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,i,domain_cellnumber,*deta_x);
            printf("number  cell_size       cell_size_old   cell.u         cell.Tem       cellnew.Tem    cell.spedif\n");
            for(i1=0; i1<domain_cellnumber; i1++)
            {
                if(i1<10 || i1>domain_cellnumber-10)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,cell[i1].mixture.Spe_Diff_Coeff[He]);
                if(i1>i-20 && i1<i+20)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,cell[i1].mixture.Spe_Diff_Coeff[He]);
                if(i1%200==0)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,cell[i1].mixture.Spe_Diff_Coeff[He]);        
            }
            printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Tem,cell[i+1].mixture.Tem,cell[i-1].mixture.Tem);
            printf("cell[0].mixture.Tem=%8.5f, cell[end].mixture.Tem=%8.5f\n",cell[0].mixture.Tem,cell[domain_cellnumber-1].mixture.Tem);
            goto abnormal_1;
        }       
    }
    
    //to restore the calculated cell[i].mixture.Massfr[k] and to renew the density and cell size
    for(i=0; i<domain_cellnumber; i++)
    {
        cell[i] = cell_new[i];  //to get the renewed Tem and Enthalpy
        cell[i].Density_old = cell[i].mixture.Density;
        cell[i].mixture.Density = cell[i].Density_old*cell[i].Tem_old/cell[i].mixture.Tem;  // to renew density
        density_ratio = cell[i].Density_old/cell[i].mixture.Density;
        // to renew the cell size
        DILATATION_MODLE(density_ratio, &(cell[i]));
        RENEW_MIXPRO_MASSY(i,cell);  //to renew the thermal property
    }

 abnormal_1:    ;
}

















// Forward-Euler explicit method to solve Temperature Equations!!!
void HEAT_TRANSFER_FE_TEM(int E_S_number, int *Tem_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source)
{
    int i,k,i1;
    double Pre;
    double rad_p = 1.0;
    double density_ratio;  //density_old/density_new=Tem_new/Tem_old
    double r, mass_fr;
    double A1,A0,B0,B1,B2,C0,C1,C2,Drag,S_rad,Omega;  //S_rad: source of radiation heat transfer, Omega: source of reaction heat release
    double Tem[domain_cellnumber],Density[domain_cellnumber]; // cell[i].mixture.Tem!!!
    double Heat_Cp_old[domain_cellnumber][Species_Number],Heat_Cp[domain_cellnumber][Species_Number];  //heat capacity of each species in the mixture

    //    printf("E_S_number=%d, deta_x=%f,",E_S_number, *deta_x);

    //to set boundary conditions and record the initial values of each step	
     for(i=0; i<domain_cellnumber; i++)
     {   
         Tem[i] = cell[i].Tem_old;
     }

    // to calculate heat capacity of each species based on the original Temperature.
    CALCULATE_Heat_Cp(domain_cellnumber, Tem, Heat_Cp_old);

    // to solve heat transfer equations where conduction, convection and radiation terms are considered
    // to calculate vector a[],b[],c[] and d[]
    for(i=0; i<domain_cellnumber; i++)
    {
        if(i==0)
        {
            //the unit of heat capacity should be convert from E_S_NUMBER/[mol K] to E_S_NUMBER/[kgK]
            mass_fr = cell[i].Density_old*cell[i].u_old*dy_co;
            A1 = mass_fr*cell[i].mixture.Heat_Capacity*1000/cell[i].mixture.Molecular_Weight;
            //            A0 = *deta_x/A1/2.0;
            A0 = *deta_x;

            // coefficients based on old nth value!
            B1 = (cell[i].mixture.Conduction_Coeff+cell[domain_cellnumber-1].mixture.Conduction_Coeff)/(dy_co+dy_co)/2.0;
            B2 = (cell[i].mixture.Conduction_Coeff*cell[i+1].size_old+cell[i+1].mixture.Conduction_Coeff*dy_co)/pow((dy_co+cell[i+1].size_old),2);
            B0 = B1+B2;

 
            C1 = 0.0;
            C2 = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                C1 += (cell[i].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/2.0*(Heat_Cp_old[i][k]+Heat_Cp_old[domain_cellnumber-1][k])/2.0*(cell[domain_cellnumber-1].Massfr_old[k]-cell[i].Massfr_old[k]);

                C2 += (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i+1].size+cell[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[i+1].size+dy_co)*(Heat_Cp_old[i][k]*cell[i+1].size+Heat_Cp_old[i+1][k]*dy_co)/(cell[i+1].size+dy_co)*(cell[i+1].Massfr_old[k]-cell[i].Massfr_old[k]);
            }

            C1 *= ((cell[i].Density_old+cell[domain_cellnumber-1].Density_old)/(dy_co+dy_co)/2.0);
            C2 *= (((cell[i].Density_old*cell[i+1].size+cell[i+1].Density_old*dy_co)/pow((dy_co+cell[i+1].size),2)));
            C0 = C1+C2;

            r = 2*A0*(B1+C1)*cell[domain_cellnumber-1].Tem_old+(A1-2*A0*(B0+C0))*cell[i].Tem_old+2*A0*(B2+C2)*cell[i+1].Tem_old;  //ghost point for cell[domain_cellnumber-1]
            r += source[i].enthalpy;  //particle source term

            // heat production by viscous
//             Drag = (cell[i].mixture.Kinetic_Viscosity_Coeff*cell[i+1].size+cell[i+1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[i+1].u*cell[i+1].u-cell[i].u*cell[i].u)/pow((cell[i+1].size+dy_co),2)/2.0;
//             Drag += (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[domain_cellnumber-1].mixture.Kinetic_Viscosity_Coeff)*(cell[domain_cellnumber-1].u*cell[domain_cellnumber-1].u-cell[i].u*cell[i].u)/(dy_co+dy_co)/4.0;
            Drag = 0.0;

            //Radiation
            S_rad = rad_p*SOURCE_RADIATION(cell[i],*deta_x);

            //heat release
            Omega = 0.0;
            r += (2*A0*Drag+(S_rad+Omega));  // to consider additional diffusion due to other facts: thermal diffusion et al.
            cell[i].Tem_old = cell[i].mixture.Tem;
            Tem[i] = r/(A1+source[i].mass*cell[i].mixture.Heat_Capacity*1000/cell[i].mixture.Molecular_Weight);
        }
        else if(i==domain_cellnumber-1)
        {
            mass_fr = cell[i].Density_old*cell[i].u_old*dy_co;
            A1=mass_fr*(cell[i].mixture.Heat_Capacity/cell[i].mixture.Molecular_Weight*1000);
            A0 = *deta_x;

            // coefficients based on old nth value!
            B1 = (cell[i].mixture.Conduction_Coeff*cell[i-1].size+cell[i-1].mixture.Conduction_Coeff*dy_co)/pow((dy_co+cell[i-1].size),2);
            B2 = (cell[i].mixture.Conduction_Coeff+cell[0].mixture.Conduction_Coeff)/(dy_co+dy_co)/2.0;
            B0 = B1+B2;

            C1 = 0.0;
            C2 = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                C1 += (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[i-1].size+dy_co)*(Heat_Cp_old[i][k]*cell[i-1].size+Heat_Cp_old[i-1][k]*dy_co)/(cell[i-1].size+dy_co)*(cell[i-1].Massfr_old[k]-cell[i].Massfr_old[k]);

                C2 += (cell[i].mixture.Spe_Diff_Coeff[k]+cell[0].mixture.Spe_Diff_Coeff[k])/2.0*(Heat_Cp_old[i][k]+Heat_Cp_old[0][k])/2.0*(cell[0].Massfr_old[k]-cell[i].Massfr_old[k]);

            }
            C1 *= (((cell[i].Density_old*cell[i-1].size+cell[i-1].Density_old*dy_co)/pow((dy_co+cell[i-1].size),2)));
            C2 *= ((cell[i].Density_old+cell[0].Density_old)/(dy_co+dy_co)/2.0);
            C0 = C1+C2;

            r = 2*A0*(B1+C1)*cell[i-1].Tem_old+(A1-2*A0*(B0+C0))*cell[i].Tem_old+2*A0*(B2+C2)*cell[0].Tem_old;  
            r += source[i].enthalpy;    //paticle temperature source term

            // heat production by viscous
            // Drag = (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[0].mixture.Kinetic_Viscosity_Coeff)*(cell[0].u*cell[0].u-cell[i].u*cell[i].u)/(dy_co+dy_co)/4.0;
            // Drag +=  (cell[i].mixture.Kinetic_Viscosity_Coeff*cell[i-1].size+cell[i-1].mixture.Kinetic_Viscosity_Coeff*dy_co)*(cell[i-1].u*cell[i-1].u-cell[i].u*cell[i].u)/pow((cell[i-1].size+dy_co),2)/2.0;
            Drag = 0.0;
            //Radiation
            S_rad = rad_p*SOURCE_RADIATION(cell[i],*deta_x);
            Omega = 0.0;
            r += (2*A0*Drag+(S_rad+Omega));  // to consider additional diffusion due to other facts: thermal diffusion et al.    
            cell[i].Tem_old = cell[i].mixture.Tem;
            Tem[i] = r/(A1+source[i].mass*cell[i].mixture.Heat_Capacity*1000/cell[i].mixture.Molecular_Weight);
        }
        else
        {   
            mass_fr = cell[i].Density_old*cell[i].u_old*cell[i].size_old;
            A1 = mass_fr*(cell[i].mixture.Heat_Capacity/cell[i].mixture.Molecular_Weight*1000);
            A0 = *deta_x;

            // coefficients based on old nth value!
            B1 = B2;
            //            B1 = 2*(cell[i].mixture.Conduction_Coeff*(i==1?dy_co:cell[i-1].size)+cell[i-1].mixture.Conduction_Coeff*cell[i].size)/pow((cell[i].size+(i==1?dy_co:cell[i-1].size)),2);
            B2 = (cell[i].mixture.Conduction_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Conduction_Coeff*cell[i].size)/pow((cell[i].size+(i==domain_cellnumber-2?dy_co:cell[i+1].size)),2);
            B0 = B1+B2;

            C1 = -C2;
            C2 = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                //                C1 += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old)*(Heat_Cp_old[i][k]*(i==1?dy_co:cell[i-1].size_old)+Heat_Cp_old[i-1][k]*cell[i].size_old)/(cell[i].size_old+(i==1?dy_co:cell[i-1].size_old))*(cell[i-1].Massfr_old[k]-cell[i].Massfr_old[k]);

                C2 += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old)*(Heat_Cp_old[i][k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+Heat_Cp_old[i+1][k]*cell[i].size_old)/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old)*(cell[i+1].Massfr_old[k]-cell[i].Massfr_old[k]);

            }

            //            C1 *= (2*((cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/pow((cell[i].size_old+(i==1?dy_co:cell[i-1].size_old)),2)));
            C2 *= (((cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/pow((cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)),2)));

            C0 = C1+C2;

            r = 2*A0*(B1+C1)*cell[i-1].Tem_old+(A1-2*A0*(B0+C0))*cell[i].Tem_old+2*A0*(B2+C2)*cell[i+1].Tem_old;  
            r += source[i].enthalpy;  //particle source term

            // heat production by viscous
            Drag = 0.0;
            //                 Drag += (cell[i].mixture.Kinetic_Viscosity_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size+cell[i+1].mixture.Kinetic_Viscosity_Coeff*cell[i].size)*(cell[i+1].u*cell[i+1].u-cell[i].u*cell[i].u)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size),2)/2.0;
            //                 Drag +=  (cell[i].mixture.Kinetic_Viscosity_Coeff*(i==1?dy_co:cell[i-1].size+cell[i-1].mixture.Kinetic_Viscosity_Coeff*cell[i].size)*(cell[i-1].u*cell[i-1].u-cell[i].u*cell[i].u)/pow(((i==1?dy_co:cell[i-1].size)+cell[i].size),2)/2.0;

            //Radiation
            S_rad =  rad_p*SOURCE_RADIATION(cell[i],*deta_x);

//             if(i==2000)
//                 printf("S_rad=%e\t A1=%f\tC_p=%f,MW=%f\n",S_rad,A1,cell[i].mixture.Heat_Capacity/cell[i].mixture.Molecular_Weight*1000,cell[i].mixture.Molecular_Weight);

            Omega = 0.0;
            //           if(i== domain_cellnumber/2)
            //                    printf("A0=%f,Drag=%f\n",A0,Drag);
            //    r[i] = r[i]+0.0*H_k+Drag+S_rad;  // to consider additional diffusion due to other facts: thermal diffusion et al.
            r += (2*A0*Drag+(S_rad+Omega));  // to consider additional diffusion due to other facts: thermal diffusion et al.    
            cell[i].Tem_old = cell[i].mixture.Tem;
            Tem[i] = r/(A1+source[i].mass*cell[i].mixture.Heat_Capacity*1000/cell[i].mixture.Molecular_Weight);
        }
    }
    //    printf("0. %d %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He]);




    //to restore the calculated cell[i].mixture.Massfr[k]
    for(i=0; i<domain_cellnumber; i++)
    {
        // to record the density before temperature change
        Density[i] = cell[i].mixture.Density*cell[i].mixture.Tem/Tem[i];


        // to print wrong value in order to check the bugs
        if(!(Tem[i]>=273 && Tem[i]<5000))
        {
            *Tem_Flag = 0;
//             printf("%dth step: error occurs during 1st circulation in heat transfer process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,i,domain_cellnumber,*deta_x);
//             printf("number  cell_size       cell_size_old   cell.u         cell.Tem       cellnew.Tem    cell.spedif\n");
//             for(i1=0; i1<domain_cellnumber; i1++)
//             {
//                 if(i1<10 || i1>domain_cellnumber-10)
//                     printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem, Tem[i1],cell[i1].mixture.Spe_Diff_Coeff[He]);
//                 if(i1>i-20 && i1<i+20)
//                     printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,Tem[i1],cell[i1].mixture.Spe_Diff_Coeff[He]);
//                 if(i1%200==0)
//                     printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,Tem[i1],cell[i1].mixture.Spe_Diff_Coeff[He]);        
//             }
     
//             //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\n",E_S_number,cell[i].mixture.Massfr[k],i==domain_cellnumber-1?cell[0].mixture.Massfr[k]:cell[i+1].mixture.Massfr[k],i==0?cell[domain_cellnumber-1].mixture.Massfr[k]:cell[i-1].mixture.Massfr[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
//             //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,cell_new[i].mixture.Massfr[k],cell_new[i+1].mixture.Massfr[k],cell_new[i-1].mixture.Massfr[k],cell_new[i].size,cell[i+1].size,cell_new[i-1].size);
//             printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Tem,cell[i+1].mixture.Tem,cell[i-1].mixture.Tem);
//             printf("cell[0].mixture.Tem=%8.5f, cell[end].mixture.Tem=%8.5f\n",cell[0].mixture.Tem,cell[domain_cellnumber-1].mixture.Tem);

            //to modify the diffusion step
            MODIFY_TEMDIFFDX(deta_x, cell[i].mixture.Tem, Tem[i]);

            goto abnormal2;
        }       
    }

    // to renew the mixture property based on the known mass fraction of each species in the mixture
    for(i=0; i<domain_cellnumber; i++)
    {
        cell[i].mixture.Tem = Tem[i];
        cell[i].Density_old = cell[i].mixture.Density;
        cell[i].mixture.Density = Density[i];
        density_ratio = cell[i].Density_old/cell[i].mixture.Density;
        DILATATION_MODLE(density_ratio, &(cell[i]));

        RENEW_MIXPRO_MASSY(i,cell);
    }


 abnormal2:    ;
}















// Crank-Nicolson implicit method followed by an explicit method to solve Temperature Equations!!!
void HEAT_TRANSFER_CN_TEM(int E_S_number, int *Tem_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source)
{
    int i,k,i1;
    double Pre;
    double rad_p = 1.0;
    double density_ratio;  //density_old/density_new=Tem_new/Tem_old
    double a[domain_cellnumber],b[domain_cellnumber],c[domain_cellnumber],r[domain_cellnumber];
    double mass_fr;
    double A1,A0,B0,B1,B2,A0_1,B0_1,B1_1,B2_1,C0,C1,C2,C0_1,C1_1,C2_1,Drag,S_rad,Omega;  //S_rad: source of radiation heat transfer, Omega: source of reaction heat release
    double Tem[domain_cellnumber],Tem_old[domain_cellnumber]; // cell[i].mixture.Tem!!!
    double Heat_Cp_old[domain_cellnumber][Species_Number],Heat_Cp[domain_cellnumber][Species_Number];  //heat capacity of each species in the mixture

    struct cell_information cell_new[domain_cellnumber];

    //    printf("E_S_number=%d, deta_x=%f,",E_S_number, *deta_x);

    //to set boundary conditions and record the initial values of each step	
    for(i=0; i<domain_cellnumber; i++)
    {   
        cell_new[i] = cell[i];
        Tem_old[i] = cell[i].Tem_old;
        if(cell[i].mixture.Tem<273 || cell[i].mixture.Tem>5000)
            printf("cell tempreature weird in heat_transfer! i=%d, Tem=%f, temold=%f\n", i, cell[i].mixture.Tem, Tem_old[i]);
    }

    // to calculate heat capacity of each species based on the original Temperature.
    CALCULATE_Heat_Cp(domain_cellnumber, Tem_old, Heat_Cp_old);

    // to solve heat transfer equations where conduction, convection and radiation terms are considered
    // to calculate vector a[],b[],c[] and d[]
    for(i=0; i<domain_cellnumber; i++)
    {
        if(i==0)
        {
            //the unit of heat capacity should be convert from E_S_NUMBER/[mol K] to E_S_NUMBER/[kgK]
            //            A1 = cell[i].Density_old*cell[i].u_old*(cell[i].mixture.Heat_Capacity*1000/cell[i].mixture.Molecular_Weight)*dy_co;
            mass_fr = cell[i].Density_old*cell[i].u_old*dy_co;
            A1 = mass_fr*cell[i].mixture.Heat_Capacity*1000/cell[i].mixture.Molecular_Weight;
            //            A0 = *deta_x/A1/2.0;
            A0 = *deta_x;
            // coefficients based on old nth value!

            B1 = (cell[i].mixture.Conduction_Coeff+cell[domain_cellnumber-1].mixture.Conduction_Coeff)/(dy_co+dy_co)/2.0;
            B2 = (cell[i].mixture.Conduction_Coeff*cell[i+1].size_old+cell[i+1].mixture.Conduction_Coeff*dy_co)/pow((dy_co+cell[i+1].size_old),2);
            B0 = B1+B2;

            // coefficients based on new nth (or n+1th) value!

            B1_1 = (cell_new[i].mixture.Conduction_Coeff+cell_new[domain_cellnumber-1].mixture.Conduction_Coeff)/(dy_co+dy_co)/2.0;
            B2_1 = (cell_new[i].mixture.Conduction_Coeff*cell_new[i+1].size+cell_new[i+1].mixture.Conduction_Coeff*dy_co)/pow((dy_co+cell[i+1].size),2);
            B0_1 = B1_1+B2_1;

            C1 = 0.0;
            C2 = 0.0;
            C1_1 = 0.0;
            C2_1 = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                C1 += (cell[i].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/2.0*(Heat_Cp_old[i][k]+Heat_Cp_old[domain_cellnumber-1][k])/2.0*(cell[domain_cellnumber-1].Massfr_old[k]-cell[i].Massfr_old[k]);

                C2 += (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i+1].size_old+cell[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[i+1].size_old+dy_co)*(Heat_Cp_old[i][k]*cell[i+1].size_old+Heat_Cp_old[i+1][k]*dy_co)/(cell[i+1].size_old+dy_co)*(cell[i+1].Massfr_old[k]-cell[i].Massfr_old[k]);

                C1_1 += (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/2.0*(Heat_Cp_old[i][k]+Heat_Cp_old[domain_cellnumber-1][k])/2.0*(cell[domain_cellnumber-1].mixture.Massfr[k]-cell[i].mixture.Massfr[k]);

                C2_1 += (cell_new[i].mixture.Spe_Diff_Coeff[k]*cell_new[i+1].size+cell_new[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell_new[i+1].size+dy_co)*(Heat_Cp_old[i][k]*cell_new[i+1].size+Heat_Cp_old[i+1][k]*dy_co)/(cell_new[i+1].size+dy_co)*(cell[i+1].mixture.Massfr[k]-cell[i].mixture.Massfr[k]);
            }
            C1 *= ((cell[i].Density_old+cell[domain_cellnumber-1].Density_old)/(dy_co+dy_co)/2.0);
            C2 *= (((cell[i].Density_old*cell[i+1].size_old+cell[i+1].Density_old*dy_co)/pow((dy_co+cell[i+1].size_old),2)));
            C1_1 *= ((cell_new[i].mixture.Density+cell_new[domain_cellnumber-1].mixture.Density)/(dy_co+dy_co)/2.0);
            C2_1 *= (((cell_new[i].mixture.Density*cell_new[i+1].size+cell_new[i+1].mixture.Density*dy_co)/pow((dy_co+cell_new[i+1].size),2)));
//             C1 *= 0.500;
//             C2 *= 0.500;
//             C1_1 *= 0.500;
//             C2_1 *= 0.500;
//             C0 = -C1-C2;
//             C0_1 = -C1_1-C2_1;
            C0 = C1+C2;
            C0_1 = C1_1+C2_1;

            a[i] = 0.0;
            // b[i] = ((cell[i].mixture.Density*cell[i].u/cell[i].Density_old/cell[i].u_old)+A0*B0_1);
            b[i] = (A1+A0*(B0_1+C0_1)); // since c_p is proportional to .mixture.Density (j/[kg.K]), cell[i].mixture.Density*cell[i].u*cell[i].size*cell[i].mixture.Heat_Capacity/(cell[i].Density_old*cell[i].u_old*cell[i].size_old*cell[i].mixture.Heat_Capacity)===1!!!!!!!!!!!!!!!
            b[i] += source[i].mass*cell[i].mixture.Heat_Capacity*1000/cell[i].mixture.Molecular_Weight;
            c[i] = -A0*(B2_1+C2_1);

            r[i] = 2*A0*(B1+C1)*cell[domain_cellnumber-1].Tem_old+(A1-A0*(B0+C0))*cell[i].Tem_old+A0*(B2+C2)*cell[i+1].Tem_old;  //ghost point for cell[domain_cellnumber-1]
            r[i] += source[i].enthalpy;

            // heat production by viscous
            //  Drag = (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[i+1].mixture.Kinetic_Viscosity_Coeff)*(cell[i+1].u*cell[i+1].u-cell[i].u*cell[i].u)/(cell[i+1].size+dy_co)/2.0+(cell[i].mixture.Kinetic_Viscosity_Coeff+cell[i+1].mixture.Kinetic_Viscosity_Coeff)*(cell[i+1].u_old*cell[i+1].u_old-cell[i].u_old*cell[i].u_old)/(cell[i+1].size_old+dy_co)/2.0;

            // Drag +=  (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[domain_cellnumber-1].mixture.Kinetic_Viscosity_Coeff)*(cell[domain_cellnumber-1].u*cell[domain_cellnumber-1].u-cell[i].u*cell[i].u)/(dy_co+dy_co)/2.0+(cell[i].mixture.Kinetic_Viscosity_Coeff+cell[domain_cellnumber-1].mixture.Kinetic_Viscosity_Coeff)*(cell[domain_cellnumber-1].u_old*cell[domain_cellnumber-1].u_old-cell[i].u_old*cell[i].u_old)/(dy_co+dy_co)/2.0;

            Drag = 0.0;

            //Radiation
            S_rad = rad_p*SOURCE_RADIATION(cell[i],*deta_x);

            //heat release
            Omega = 0.0;
            r[i] += (A0*Drag+(S_rad+Omega));  // to consider additional diffusion due to other facts: thermal diffusion et al.
        }
        else if(i==domain_cellnumber-1)
        {
            mass_fr = cell[i].Density_old*cell[i].u_old*dy_co;
            A1=mass_fr*(cell[i].mixture.Heat_Capacity/cell[i].mixture.Molecular_Weight*1000);
            A0 = *deta_x;

            // coefficients based on old nth value!
            B1 = (cell[i].mixture.Conduction_Coeff*cell[i-1].size_old+cell[i-1].mixture.Conduction_Coeff*dy_co)/pow((dy_co+cell[i-1].size_old),2);
            B2 = (cell[i].mixture.Conduction_Coeff+cell[0].mixture.Conduction_Coeff)/(dy_co+dy_co)/2.0;
            B0 = B1+B2;

            // coefficients based on new nth (or n+1th) value!
            B1_1 = (cell_new[i].mixture.Conduction_Coeff*cell[i-1].size+cell_new[i-1].mixture.Conduction_Coeff*dy_co)/pow((dy_co+cell[i-1].size),2);
            B2_1 = (cell_new[i].mixture.Conduction_Coeff+cell_new[0].mixture.Conduction_Coeff)/(dy_co+dy_co)/2.0;
            B0_1 = B1_1+B2_1;

            C1 = 0.0;
            C2 = 0.0;
            C1_1 = 0.0;
            C2_1 = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                C1 += (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell[i-1].size_old+dy_co)*(Heat_Cp_old[i][k]*cell[i-1].size_old+Heat_Cp_old[i-1][k]*dy_co)/(cell[i-1].size_old+dy_co)*(cell[i-1].Massfr_old[k]-cell[i].Massfr_old[k]);

                C2 += (cell[i].mixture.Spe_Diff_Coeff[k]+cell[0].mixture.Spe_Diff_Coeff[k])/2.0*(Heat_Cp_old[i][k]+Heat_Cp_old[0][k])/2.0*(cell[0].Massfr_old[k]-cell[i].Massfr_old[k]);

                C1_1 += (cell_new[i].mixture.Spe_Diff_Coeff[k]*cell_new[i-1].size+cell_new[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)/(cell_new[i-1].size+dy_co)*(Heat_Cp_old[i][k]*cell_new[i-1].size+Heat_Cp_old[i-1][k]*dy_co)/(cell_new[i-1].size+dy_co)*(cell[i-1].mixture.Massfr[k]-cell[i].mixture.Massfr[k]);

                C2_1 += (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[0].mixture.Spe_Diff_Coeff[k])/2.0*(Heat_Cp_old[i][k]+Heat_Cp_old[0][k])/2.0*(cell[0].mixture.Massfr[k]-cell[i].mixture.Massfr[k]);
            }
            C1 *= (((cell[i].Density_old*cell[i-1].size_old+cell[i-1].Density_old*dy_co)/pow((dy_co+cell[i-1].size_old),2)));
            C2 *= ((cell[i].Density_old+cell[0].Density_old)/(dy_co+dy_co)/2.0);
            C1_1 *= (((cell_new[i].mixture.Density*cell_new[i-1].size+cell_new[i-1].mixture.Density*dy_co)/pow((dy_co+cell_new[i-1].size),2)));
            C2_1 *= ((cell_new[i].mixture.Density+cell_new[0].mixture.Density)/(dy_co+dy_co)/2.0);
//             C1 *= 0.500;
//             C2 *= 0.500;
//             C1_1 *= 0.500;
//             C2_1 *= 0.500;
            C0 = C1+C2;
            C0_1 = C1_1+C2_1;

            a[i] = -A0*(B1_1+C1_1);
            //                b[i] = ((cell[i].mixture.Density*cell[i].u/cell[i].Density_old/cell[i].u_old)+A0*B0_1);
            b[i] = (A1+A0*(B0_1+C0_1)); // since c_p is proportional to .mixture.Density (j/[kg.K]), cell[i].mixture.Density*cell[i].u*cell[i].size*cell[i].mixture.Heat_Capacity/(cell[i].Density_old*cell[i].u_old*cell[i].size_old*cell[i].mixture.Heat_Capacity)===1!!!!!!!!!!!!!!!
            b[i] += source[i].mass*cell[i].mixture.Heat_Capacity*1000/cell[i].mixture.Molecular_Weight;
            c[i] = 0.0;

            r[i] = A0*(B1+C1)*cell[i-1].Tem_old+(A1-A0*(B0+C0))*cell[i].Tem_old+2*A0*(B2+C2)*cell[0].Tem_old;  
            r[i] += source[i].enthalpy;

            // heat production by viscous
            // Drag = (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[0].mixture.Kinetic_Viscosity_Coeff)*(cell[0].u*cell[0].u-cell[i].u*cell[i].u)/(dy_co+dy_co)/2.0+(cell[i].mixture.Kinetic_Viscosity_Coeff+cell[0].mixture.Kinetic_Viscosity_Coeff)*(cell[0].u_old*cell[0].u_old-cell[i].u_old*cell[i].u_old)/(dy_co+dy_co)/2.0;
            // Drag +=  (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[i-1].mixture.Kinetic_Viscosity_Coeff)*(cell[i-1].u*cell[i-1].u-cell[i].u*cell[i].u)/(cell[i-1].size+dy_co)/2.0+(cell[i].mixture.Kinetic_Viscosity_Coeff+cell[i-1].mixture.Kinetic_Viscosity_Coeff)*(cell[i-1].u_old*cell[i-1].u_old-cell[i].u_old*cell[i].u_old)/(cell[i-1].size_old+dy_co)/2.0;
            Drag = 0.0;
            //Radiation
            S_rad = rad_p*SOURCE_RADIATION(cell[i],*deta_x);
            Omega = 0.0;
            r[i] += (A0*Drag+(S_rad+Omega));  // to consider additional diffusion due to other facts: thermal diffusion et al.    
            //r[i] += (0.0*A0*H_k+A0*Drag);
        }
        else
        {   
            mass_fr = cell[i].Density_old*cell[i].u_old*cell[i].size_old;
            A1 = mass_fr*(cell[i].mixture.Heat_Capacity/cell[i].mixture.Molecular_Weight*1000);
            A0 = *deta_x;

            // coefficients based on old nth value!
            B1 = B2;
            //            B1 = 2*(cell[i].mixture.Conduction_Coeff*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Conduction_Coeff*cell[i].size_old)/pow((cell[i].size_old+(i==1?dy_co:cell[i-1].size_old)),2);
            B2 = (cell[i].mixture.Conduction_Coeff*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Conduction_Coeff*cell[i].size_old)/pow((cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)),2);
            B0 = B1+B2;

            // coefficients based on new nth (or n+1th) value!
            B1_1 = B2_1;
            //            B1_1 = 2*(cell_new[i].mixture.Conduction_Coeff*(i==1?dy_co:cell_new[i-1].size)+cell_new[i-1].mixture.Conduction_Coeff*cell_new[i].size)/pow((cell_new[i].size+(i==1?dy_co:cell_new[i-1].size)),2);
            B2_1 = (cell_new[i].mixture.Conduction_Coeff*(i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+cell_new[i+1].mixture.Conduction_Coeff*cell_new[i].size)/pow((cell_new[i].size+(i==domain_cellnumber-2?dy_co:cell_new[i+1].size)),2);
            B0_1 = B1_1+B2_1;


            //            C1 = 0.0;
            C1 = -C2;
            C2 = 0.0;
            //            C1_1 = 0.0;
            C1_1 = -C2_1;
            C2_1 = 0.0;
            for(k=0; k<Species_Number; k++)
            {
                //                C1 += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old)*(Heat_Cp_old[i][k]*(i==1?dy_co:cell[i-1].size_old)+Heat_Cp_old[i-1][k]*cell[i].size_old)/(cell[i].size_old+(i==1?dy_co:cell[i-1].size_old))*(cell[i-1].Massfr_old[k]-cell[i].Massfr_old[k]);

                C2 += (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old)*(Heat_Cp_old[i][k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+Heat_Cp_old[i+1][k]*cell[i].size_old)/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old)*(cell[i+1].Massfr_old[k]-cell[i].Massfr_old[k]);

                //                C1_1 += (cell_new[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell_new[i-1].size)+cell_new[i-1].mixture.Spe_Diff_Coeff[k]*cell_new[i].size)/(cell_new[i].size+(i==1?dy_co:cell_new[i-1].size))*(Heat_Cp_old[i][k]*(i==1?dy_co:cell_new[i-1].size)+Heat_Cp_old[i-1][k]*cell_new[i].size)/((i==1?dy_co:cell_new[i-1].size)+cell_new[i].size)*(cell[i-1].mixture.Massfr[k]-cell[i].mixture.Massfr[k]);

                C2_1 += (cell_new[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+cell_new[i+1].mixture.Spe_Diff_Coeff[k]*cell_new[i].size)/((i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+cell_new[i].size)*(Heat_Cp_old[i][k]*(i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+Heat_Cp_old[i+1][k]*cell_new[i].size)/((i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+cell_new[i].size)*(cell[i+1].mixture.Massfr[k]-cell[i].mixture.Massfr[k]);
            }

            //            C1 *= (2*((cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/pow((cell[i].size_old+(i==1?dy_co:cell[i-1].size_old)),2)));
            C2 *= (((cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/pow((cell[i].size_old+(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)),2)));
            //            C1_1 *= (2*((cell_new[i].mixture.Density*(i==1?dy_co:cell_new[i-1].size)+cell_new[i-1].mixture.Density*cell_new[i].size)/pow((cell_new[i].size+(i==1?dy_co:cell_new[i-1].size)),2)));
            C2_1 *= (((cell_new[i].mixture.Density*(i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+cell_new[i+1].mixture.Density*cell_new[i].size)/pow((cell_new[i].size+(i==domain_cellnumber-1?dy_co:cell_new[i+1].size)),2)));

            C0 = C1+C2;
            C0_1 = C1_1+C2_1;


            a[i] = -A0*(B1_1+C1_1);
            //  b[i] = ((cell[i].mixture.Density*cell[i].u*cell[i].size/cell[i].Density_old/cell[i].u_old/cell[i].size_old)+A0*B0_1);
            b[i] = (A1+A0*(B0_1+C0_1)); // since c_p is proportional to .mixture.Density (j/[kg.K]), cell[i].mixture.Density*cell[i].u*cell[i].size*cell[i].mixture.Heat_Capacity/(cell[i].Density_old*cell[i].u_old*cell[i].size_old*cell[i].mixture.Heat_Capacity)===1!!!!!!!!!!!!!!!
            b[i] += source[i].mass*cell[i].mixture.Heat_Capacity*1000/cell[i].mixture.Molecular_Weight;
            c[i] = -A0*(B2_1+C2_1);

            r[i] = A0*(B1+C1)*cell[i-1].Tem_old+(A1-A0*(B0+C0))*cell[i].Tem_old+A0*(B2+C2)*cell[i+1].Tem_old;  
            r[i] += source[i].enthalpy;

            // heat production by viscous
            Drag = 0.0;
            //                 Drag += (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[i+1].mixture.Kinetic_Viscosity_Coeff)*(cell[i+1].u*cell[i+1].u-cell[i].u*cell[i].u)/((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size)/2.0+(cell[i].mixture.Kinetic_Viscosity_Coeff+cell[i+1].mixture.Kinetic_Viscosity_Coeff)*(cell[i+1].u_old*cell[i+1].u_old-cell[i].u_old*cell[i].u_old)/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old)/2.0;
            //                 Drag +=  (cell[i].mixture.Kinetic_Viscosity_Coeff+cell[i-1].mixture.Kinetic_Viscosity_Coeff)*(cell[i-1].u*cell[i-1].u-cell[i].u*cell[i].u)/((i==1?dy_co:cell[i-1].size)+cell[i].size)/2.0+(cell[i].mixture.Kinetic_Viscosity_Coeff+cell[i-1].mixture.Kinetic_Viscosity_Coeff)*(cell[i-1].u_old*cell[i-1].u_old-cell[i].u_old*cell[i].u_old)/((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old)/2.0;

            //Radiation
            S_rad =  rad_p*SOURCE_RADIATION(cell[i],*deta_x);

//             if(i==2000)
//                 printf("S_rad=%e\t A1=%f\tC_p=%f,MW=%f\n",S_rad,A1,cell[i].mixture.Heat_Capacity/cell[i].mixture.Molecular_Weight*1000,cell[i].mixture.Molecular_Weight);

            Omega = 0.0;
            //           if(i== domain_cellnumber/2)
            //                    printf("A0=%f,Drag=%f\n",A0,Drag);
            //    r[i] = r[i]+0.0*H_k+Drag+S_rad;  // to consider additional diffusion due to other facts: thermal diffusion et al.
            r[i] += (A0*Drag+(S_rad+Omega));  // to consider additional diffusion due to other facts: thermal diffusion et al.    
            //r[i] += (0.0*A0*H_k+A0*Drag);

        }
    }
    //    printf("0. %d %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He]);

    //to calculate the candidate TEMPERATURE on (n+1)th odt line
    TRIDIAG(a,b,c,r,Tem,domain_cellnumber);

    //    printf("0_1. %d %8.5f  %8.5f  %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He],RMassY[int(domain_cellnumber/2)],RMassY[0]);


    //the first iteration to solve out the candidate Temperature.
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//     //the second iteration to solve out the candidate Temperature.
//     ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// the second iteration is not adopted here since it doesn't change the result that much. it can save a lot of calculation time


    //to restore the calculated cell[i].mixture.Massfr[k]
    for(i=0; i<domain_cellnumber; i++)
    {
        cell_new[i].mixture.Tem = Tem[i];   // to restore the new temperature for the mixture
        // to renew the mixture density
        // here cell_new[i].Tem_old = cell[i].mixture.Tem=cell[i].Tem_old
        // cell_new[i].mixture.density(right side)=cell[i].mixture.density!=cell[i].Density_old!!!!
        // to record the density before temperature change
        cell_new[i].mixture.Density = cell[i].mixture.Density*cell[i].mixture.Tem/Tem[i];

        density_ratio = cell[i].mixture.Density/cell_new[i].mixture.Density;  //the expansion rate
        // to renew the cell size
        // also cell_new[i].size = cell_new[i].size*Tem/cell[i].Tem_old
        //        cell_new[i].size = cell[i].size*cell[i].mixture.Density/cell_new[i].mixture.Density;

        DILATATION_MODLE(density_ratio, &(cell_new[i]));


        ///////////////////////////////////here the conduction_coefficient will be renewed in further step//////////
        ///////////////// I think this step doesn't change reasult that much//////////////////////////////////////////

        ////////// to renew \lambda here///////////////////////////

        /////////////////////////////////////////////////////////////////////////////////

        // to print wrong value in order to check the bugs
        if(!(cell_new[i].mixture.Tem>=273 && cell_new[i].mixture.Tem<5000))
        {
            *Tem_Flag = 0;
            if(*deta_x<2*dx_stir)
            {
                printf("%dth step: error occurs during 1st circulation in heat transfer process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,i,domain_cellnumber,*deta_x);
                printf("number  cell_size       cell_size_old   cell.u         cell.Tem       cellnew.Tem    source\n");
                for(i1=0; i1<domain_cellnumber; i1++)
                {
                    if(i1<10 || i1>domain_cellnumber-10)
                        printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7e\t%8.5f\t%8.5f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,source[i1].enthalpy,cell[i1].loading_rate[0],cell[i1].loading_rate[1]);
                    if(i1>i-20 && i1<i+20)
                        printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7e\t%8.5f\t%8.5f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,source[i1].enthalpy,cell[i1].loading_rate[0],cell[i1].loading_rate[1]);
                    if(i1%200==0)
                        printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7e\t%8.5f\t%8.5f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Tem,cell_new[i1].mixture.Tem,source[i1].enthalpy,cell[i1].loading_rate[0],cell[i1].loading_rate[1]);        
                }
                
                //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\n",E_S_number,cell[i].mixture.Massfr[k],i==domain_cellnumber-1?cell[0].mixture.Massfr[k]:cell[i+1].mixture.Massfr[k],i==0?cell[domain_cellnumber-1].mixture.Massfr[k]:cell[i-1].mixture.Massfr[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
                //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,cell_new[i].mixture.Massfr[k],cell_new[i+1].mixture.Massfr[k],cell_new[i-1].mixture.Massfr[k],cell_new[i].size,cell[i+1].size,cell_new[i-1].size);
                printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Tem,cell[i+1].mixture.Tem,cell[i-1].mixture.Tem);
                printf("cell[0].mixture.Tem=%8.5f, cell[end].mixture.Tem=%8.5f\n",cell[0].mixture.Tem,cell[domain_cellnumber-1].mixture.Tem);
            }

            //to modify the diffusion step
            MODIFY_TEMDIFFDX(deta_x, cell[i].mixture.Tem, cell_new[i].mixture.Tem);

            goto abnormal2;
        }       
    }

    // to renew the mixture property based on the known mass fraction of each species in the mixture
    for(i=0; i<domain_cellnumber; i++)
    {
        RENEW_MIXPRO_MASSY(i,cell_new);
        //        cell_new[i].Density_old = cell_new[i].mixture.Density;
    }

    // to restore the changed value (mass fraction and mixture density)  
    for(i=0; i<domain_cellnumber; i++)
        cell[i] = cell_new[i];

 abnormal2:    ;
}




// to change/decrease diffusion step when negative mass fraction occurs
void MODIFY_TEMDIFFDX(double *deta_x, double T_old, double T_new)
{
    int detanumber,backnumber;
    detanumber = int((*deta_x+1e-10)/dx_stir);
    backnumber = int((dx_max+1e-10)/dx_stir);
    if(T_new>5000)
    {
        detanumber *= (5000-T_old-10)/(T_new-T_old);
    }
    else if(T_new<300)
    {
        detanumber *= (T_old-300)/(T_old-T_new);

    }
    if(detanumber>int(backnumber*2/3))
        detanumber -= 5;
    else if(detanumber>int(backnumber/2))
        detanumber = int(backnumber/2);
    else if(detanumber>int(backnumber/3)+1)
        detanumber = int(backnumber/3)+1;
    else if(detanumber>int(backnumber/4))
        detanumber = int(backnumber/4);
    *deta_x = detanumber*dx_stir;
    *deta_x = *deta_x>dx_stir?(*deta_x):dx_stir;
}







// to calculate radiation heat transfer!
// to note the background radiation temperature!!!
double SOURCE_RADIATION(struct cell_information cell, double deta_x)
{
    double S_rad;
    double a_CO2, a_H2O, a_CH4, a_CO;
    double ap_total, ap_total_old;  //ap_total=sum(a_k*p_k)
    double T;  //T = cell.Tem/100
    double sigma = 5.669;

    T = cell.mixture.Tem/100;
    a_CO2 = 18.741-121.31*(10/T)+273.5*pow((10/T),2)-194.05*pow((10/T),3)+56.31*pow((10/T),4)-5.8169*pow((10/T),5);
    a_H2O = -0.23093+1.12390*(10/T)+9.4153*pow((10/T),2)-2.9988*pow((10/T),3)+0.51382*pow((10/T),4)-1.8684e-5*pow((10/T),5);
    a_CH4 = 6.6334-0.0035686*(100*T)+1.6682e-8*pow((100*T),2)+2.5611e-10*pow((100*T),3)-2.6558E-14*pow((100*T),4);
    a_CO = (T<=7.5)?4.7869+(100*T)*(-0.06953+(100*T)*(2.95775e-4+(100*T)*(-4.25732e-7+(100*T)*2.02894e-10))):10.09+(100*T)*(-0.01183+(100*T)*(4.7753e-6+(100*T)*(-5.87209e-10-2.5334e-14*(100*T))));
    ap_total = cell.mixture.Molefr[CO2]*a_CO2+cell.mixture.Molefr[H2O]*a_H2O+cell.mixture.Molefr[CH4]*a_CH4+cell.mixture.Molefr[CO]*a_CO;
    S_rad = (pow(Tem_Rad_BG/100,4)-pow(T,4))*ap_total*cell.size;

    T = cell.Tem_old/100;
    a_CO2 = 18.741-121.31*(10/T)+273.5*pow((10/T),2)-194.05*pow((10/T),3)+56.31*pow((10/T),4)-5.8169*pow((10/T),5);
    a_H2O = -0.23093+1.12390*(10/T)+9.4153*pow((10/T),2)-2.9988*pow((10/T),3)+0.51382*pow((10/T),4)-1.8684e-5*pow((10/T),5);
    a_CH4 = 6.6334-0.0035686*(100*T)+1.6682e-8*pow((100*T),2)+2.5611e-10*pow((100*T),3)-2.6558E-14*pow((100*T),4);
    a_CO = (T<=7.5)?4.7869+(100*T)*(-0.06953+(100*T)*(2.95775e-4+(100*T)*(-4.25732e-7+(100*T)*2.02894e-10))):10.09+(100*T)*(-0.01183+(100*T)*(4.7753e-6+(100*T)*(-5.87209e-10-2.5334e-14*(100*T))));
    ap_total = cell.mixture.Molefr[CO2]*a_CO2+cell.mixture.Molefr[H2O]*a_H2O+cell.mixture.Molefr[CH4]*a_CH4+cell.mixture.Molefr[CO]*a_CO;
    S_rad += (pow(Tem_Rad_BG/100,4)-pow(T,4))*ap_total*cell.size_old;

    S_rad *= 2*sigma*deta_x;
//     if(S_rad>1e-6)
//         printf("radiation wrong!T=%f, S_rad=%e\n",T,S_rad);
    return(S_rad);
}
