#include "header.h"


// to solve tri-diagonal matrix problem.
//a[]start with {0, a[1],a[2],^^^a[n-1]}
//c[]start with {c[0],c[1],^^^^c[n-1],0}
//b[]start with {b[0],b[1],^^^^^^b[n-1]}
void TRIDIAG(double *a, double *b,double *c, double *r,double *u, int domain_cellnumber)
{
    int i;
    double  b1[domain_cellnumber],r1[domain_cellnumber];

    b1[0]=b[0];
    r1[0]=r[0];

    

    //     if(b[0]==0.0) 
    //        printf("error!") ;	//nrerror("error 1 in tridag")
   
    for(i=1;i<domain_cellnumber;i++)
    {
        b1[i]=b[i]-a[i]*c[i-1]/b1[i-1];
        r1[i]=r[i]-r1[i-1]*a[i]/b1[i-1];

//         if(b1[i] == 0)
//         {
//             printf("error occrusin dia! %8.5f   %8.5f", b1[i-1], b1[i]);
//         }
    }
    for(i=domain_cellnumber-1;i>=0;i--)
    {
        if(i==domain_cellnumber-1)
            u[i]=r1[i]/b1[i];
        else
            u[i]=(r1[i]-c[i]*u[i+1])/b1[i];

       
    }
    //	free_vector(gam,1,n);
}


// matrix LU decomposition
// given a matrix a[][](Element_Number * Element_Number), this routine replaces it by the LU decompostion of a rowwise permutation of itself. 
//the algorithm of Crout's is as follows:
// 1. set a[i][i]=1; i=0...Element_Number-1
// 2. for(i=0, i<j, i++), beta[i][j]=a[i][j]-sum(al[i][k]beta[k][j])(k=0,1...i-1)
// 3. for(i=j; i<Ele_Number; i++) al[i][j]=(a[i][j]-sum(al[i][k]beta[k][j])(k=0, 1..j-1))/beta[j][j]
// then, final a[i][j]=L[i][j](i>j)+U[i][j](i<j)+I[i][j]
// a[i][j]=L[i][j]*U[j][i]
void LUDCMP(double a[][Element_Number], int n, int *indx, double *d)
{
    int i, imax=0, j, k;
    double big, dum, sum, temp;
    double vv[Element_Number];             // vv stroes the implicit scaling of each row

    *d = 1.0;				// no row interchanges yet
    // to judge if a is singular (but can't judge if it is near singular)
    for(i=0; i<n; i++)
    {
        big = 0.0;
        for(j=0; j<n; j++)
            if((temp=fabs(a[i][j])) > big) 
                big = temp;
        if(big == 0.0) 
        {
            printf("Singular matrix in routine ludcmp");
            big = TINY;
        }
        vv[i] = 1.0/big;	// save the scaling
    }

    // begin LU decomposition alfa[i][j] and beta[i][j]
    for(j=0; j<n; j++)
    {
        // to calculate beta[i][j], and store it to a[i][j], except for i==j
        for (i=0; i<j; i++)
        {
            sum = a[i][j];
            for(k=0; k<i; k++) sum -= a[i][k]*a[k][j];
            a[i][j] = sum;
        }
        big = 0.0;
        // to calculate alfa[i][j], and store it to a[i][j]
        // to calculate beta[j][j]
        for(i=j; i<n; i++)
        {
            sum = a[i][j];
            for(k=0; k<j; k++)
                sum -= a[i][k]*a[k][j];
            a[i][j] = sum;
            // if the figure of merit for the pivot better than the best so far
            if((dum=vv[i]*fabs(sum))>=big)
            {
                big = dum;
                imax = i;
            }
        }
        if (j != imax)  // to exchange two rows.
        {
            for(k=0; k<n; k++)
            {
                dum = a[imax][k];
                a[imax][k] = a[j][k];
                a[j][k] = dum;
            }
            *d = -(*d);
            vv[imax] = vv[j];
        }
        indx[j] = imax;
        if(a[j][j] == 0.0) { a[j][j] = TINY;//  printf("TINY in LU decoposition!");
        }
        if(j != n-1)
        {
            dum = 1.0/(a[j][j]);
            for (i=j+1; i<n; i++)
                a[i][j] *= dum;
        }
    }
}


// i:row, j:column
// a[i][j]*x[j]=b[i]
// to solve the problem A*X = b based on the LU matrix  A=L*U  A=>[LU]
// A=a[][Element_Number], x=final (*b)of this function. b= initial (*b)
// then, final a[i][j]=L[i][j](i>j)+U[i][j](i<j)+I[i][j]
// a[i][j]=L[i][j]*U[j][i]
// so Ly=b, Ux=y
// or: a[i][j]*Y[j] = b[i] (i>j)  and   a[i][j]*X[j]=Y[i]

void LUBKSB(double a[][Element_Number], int n, int *indx, double *b)
{
    int i, ii=0, ip, j;
    double sum;
    // Ly=b, to calculate y, and store y to *b
    // a[i][j]*b[j] = b[i](i>j), here, new b[i](Y[j]) represent old b[i] step by step
    for(i=0; i<n; i++)
    {
        ip = indx[i];
        sum = b[ip];
        b[ip] = b[i];
        if (ii)
            for(j=ii-1; j<i; j++)
                sum -= a[i][j]*b[j];
        else if (sum) ii=i+1;
        b[i] = sum;
    }
    // to calculate Ux=y, and store x to b
    // a[i][j]*b[j] = b[i](i<j), here, new b[i](x) represent old b[i](y) step by step
    for(i=n-1; i>=0; i--)
    {
        sum = b[i];
        for(j=i+1; j<n; j++)
            sum -= a[i][j]*b[j];
        b[i] = sum/a[i][i];
    }
}

// to solve inverse matrix
void INVERSMATRIX(double a[][Element_Number], double a_inv[][Element_Number],int n, int *indx, double *d)
{

    int i, j;
    double col[Element_Number];

    LUDCMP(a,n,indx,d);
    for(j=0; j<n; j++)
    {
        for(i=0; i<n; i++)
            col[i] = 0.0;
        col[j] = 1.0;
        LUBKSB(a,n,indx,col);
        for(i=0; i<n; i++)
            a_inv[i][j] = col[i];
    }
}
