#include "header.h"




// this function is used to define fp_reaction
double CALCU_FP_REACTION(struct part_infor *par_info, struct cell_information *cell, double heat_release)
{
    double Tp, Tg, fp;
    Tp = par_info->coal.Tem;
    Tg = cell[par_info->cellin].mixture.Tem;
    if(heat_release<0)
        fp = 0.1;
    else
    {
        if(Tp-Tg>400)
            fp = 0.2;
        else if(Tp-Tg<-400)
            fp = 0.8;
        else
            fp = 0.5-(Tp-Tg)/800*0.6;
    }
    return(fp);
}





// this function is used to define fp_devol
double CALCU_FP_DEVOL(struct part_infor *par_info, struct cell_information *cell)
{
    double Tp, Tg, fp;
    Tp = par_info->coal.Tem;
    Tg = cell[par_info->cellin].mixture.Tem;
    if(Tp-Tg>400)
        fp = 0.4;
    else if(Tp-Tg<-400)
        fp = 0.8;
    else
        fp = 0.6-(Tp-Tg)/2000;
    return(fp);
}





// to judge which cell each paricel is in
// and to accumulate the particle mass flux for each cell, then the loading rate is acquired
// the loading rate is very important for the convergence since as particle mass flux increases, its behavior is more different than that of a single particle, the heat transfer etal, should be weaken.
void PARTICLE_JUDGECELL(int domain_cellnumber, double *grid_y, struct cell_information *cell, struct part_infor *par_info)
{
    int i;

    // to initialize particle loading rate
    for(i=0; i<domain_cellnumber; i++)
    {
        cell[i].loading_rate[0] = 0.;
        cell[i].loading_rate[1] = 0.;
    }

    for(i=0; i<par_tracknumber; i++)
    {
        if(par_info[i].boundary_type == 0)  // the particle is in the domain
        {
            // to judge which cell this particle is in!
            while(grid_y[par_info[i].cellin]>par_info[i].y)
                par_info[i].cellin--;
            while(grid_y[par_info[i].cellin+1]<par_info[i].y)
                par_info[i].cellin++;
        }
        cell[par_info[i].cellin].loading_rate[0] += par_info[i].massflowrate;
    }

    for(i=0; i<domain_cellnumber; i++)
    {
        cell[i].loading_rate[0] /= (cell[i].size*cell[i].mixture.Density*cell[i].u);
        cell[i].loading_rate[1] = cell[i].loading_rate[0];
    }
}






/******************************************************************************************
this function is used to finish particle tracking process:
for each particle in the domain, it will firstly be judged if it will hit an eddy?
if it hit an eddy, it will be tripletted instantly according to the drag law or not
the detailed manipulation is as follows:
1. particle.eddy_on == eddynumber && particle.eddy_trip ==eddynumber: the particle is not in an eddy, it won't be tripletted, just tracking it following drag law

2. particle.eddy_on == eddynumber && particle.eddy_trip != eddynumber: the particle is just go outside an eddy. to set particle.eddy_trip=0, and just to track it following drag law

3. particle.eddy_on != eddynumber && particle.eddy_trip == particle.eddy_on: the particle is in an eddy, but this particle has been tripletted by the eddy (eddy_on), just to keep the .eddy_on and .eddy_trip, and just to track the particle following the drag law

4. particle.eddy_on != eddynumber && particle.eddy_trip == eddynumber: the particle just enters an eddy and this particle was not tripletted by this eddy before.
   do the triplet map(eddy_on th eddy)  for the particle at first, then tracking it following tracking law

5. particle.eddy_on != eddynumber && particle.eddy_trip != eddynumber && particle.eddy_on != particle.eddy_trip && particle.eddy_trip1!=particle.eddy_trip: the particle enters another eddy from an eddy, and the particle is not tripletted by the later eddy before. In this case, some subcases are listed:

  a). eddy_on>eddy_trip && edd_on.size>0.7*eddy_trip.size. the eddy enters an eddy happened later, to trip it at first and to track it following the drag law.
         let:  eddy_trip1 = eddy_trip,  eddy_trip=eddy_on
  b). eddy_on>eddy_trip && eddy_on.size<0.7*eddy_trip.size. the eddy enters an eddy happend later, but this eddy is smaller the former eddy, just to track it following the drag law
  c). eddy_on<eddy_trip && eddy_on.size>eddy_trip.size. eddy_trip1=eddy_trip, eddy_trip = eddy_on. trip it and then track it
  d). eddy_on<eddy_trip && edd_on.size<eddy_trip.size. the particle enters the next small eddy, since it has left the large eddy (the overlap region is controled by the large eddy), trip it and track it

During triplet map process, only particle.v and particle.y are changed.
During tracking process, bother u, v, w and x, y, z are changed.
During both processes, the source terms are recorded for each cells, so that a 2-way interaction method can be used in this model
******************************************************************************************/

void PARTICLE_TRACKING(int E_S_number, int domain_cellnumber, double deta_x, double *grid_y, struct cell_information *cell,struct eddy_information *eddy_info, struct part_infor *par_info,  struct source_information *source)
{
    int i,eddy_on;

    //	double term1, term2,term3;
    for(i=0; i<par_tracknumber; i++)
    {
        if(par_info[i].boundary_type == 0)  //the particle is in the domain
        {
            par_info[i].eddy_on = cell[par_info[i].cellin].eddynumber;

            if(par_info[i].eddy_on == eddy_number)  // the particle is outside any eddys case:1 & case:2
            {
                if(!(par_info[i].y>=grid_y[par_info[i].cellin] && par_info[i].y<=grid_y[par_info[i].cellin+1]))
                    printf("1_b:particle cell judgement wired! p.y=%f, grid_left=%f, grid_right=%f\n",par_info[i].y, grid_y[par_info[i].cellin], grid_y[par_info[i].cellin+1]);
                //                par_info[i].eddy_trip = eddy_number;  // to set the eddy interaction state to 0
                PARTICLE_TRACK_ONESTEP(i, E_S_number, domain_cellnumber, deta_x, grid_y, cell, par_info, source);  // to track the particle for one ODT step
      
                if(!(par_info[i].y>=grid_y[par_info[i].cellin] && par_info[i].y<=grid_y[par_info[i].cellin+1]))
                    printf("1_f:particle cell judgement wired! p.y=%f, grid_left=%f, grid_right=%f\n",par_info[i].y, grid_y[par_info[i].cellin], grid_y[par_info[i].cellin+1]);
            }
            else  // the particle is in an eddy or enters an eddy
            {
                // the particle is in an eddy and it has been tripledtted by this eddy before
                if(eddy_info[par_info[i].eddy_on].ser_number==par_info[i].eddy_trip || eddy_info[par_info[i].eddy_on].ser_number==par_info[i].eddy_trip_former)  // case:3, the particle has been triplet mapped by this eddy
                {
                    if(!(par_info[i].y>=grid_y[par_info[i].cellin] && par_info[i].y<=grid_y[par_info[i].cellin+1]))
                        printf("2_b:particle cell judgement wired! p.y=%f, grid_left=%f, grid_right=%f\n",par_info[i].y, grid_y[par_info[i].cellin], grid_y[par_info[i].cellin+1]);

                    PARTICLE_TRACK_ONESTEP(i, E_S_number, domain_cellnumber, deta_x, grid_y, cell, par_info, source);  // to track the particle for one ODT step

                    if(!(par_info[i].y>=grid_y[par_info[i].cellin] && par_info[i].y<=grid_y[par_info[i].cellin+1]))
                        printf("2_f:particle cell judgement wired! p.y=%f, grid_left=%f, grid_right=%f\n",par_info[i].y, grid_y[par_info[i].cellin], grid_y[par_info[i].cellin+1]);
                }
                else if(par_info[i].eddy_trip == 0)  //the particle is in an eddy and is not tripletted by this eddy before, case:4
                {
                    eddy_on = par_info[i].eddy_on;
                    par_info[i].eddy_trip = eddy_info[eddy_on].ser_number;

                    if(!(par_info[i].y>=grid_y[par_info[i].cellin] && par_info[i].y<=grid_y[par_info[i].cellin+1]))
                        printf("3_b:particle cell judgement wired! p.y=%f, grid_left=%f, grid_right=%f\n",par_info[i].y, grid_y[par_info[i].cellin], grid_y[par_info[i].cellin+1]);

                    PARTICLE_TRIPLET(i, grid_y, cell, eddy_info[eddy_on],par_info,source);  // to do the triplet map for the particle according to the drag law

                    if(!(par_info[i].y>=grid_y[par_info[i].cellin] && par_info[i].y<=grid_y[par_info[i].cellin+1]))
                        printf("3_m:particle cell judgement wired! p.y=%f, grid_left=%f, grid_right=%f\n",par_info[i].y, grid_y[par_info[i].cellin], grid_y[par_info[i].cellin+1]);

                    PARTICLE_TRACK_ONESTEP(i, E_S_number, domain_cellnumber, deta_x, grid_y, cell, par_info, source);  // to track the particle for one ODT step

                    if(!(par_info[i].y>=grid_y[par_info[i].cellin] && par_info[i].y<=grid_y[par_info[i].cellin+1]))
                        printf("3_f:particle cell judgement wired! p.y=%f, grid_left=%f, grid_right=%f\n",par_info[i].y, grid_y[par_info[i].cellin], grid_y[par_info[i].cellin+1]);
                }
                else  //case 5 (eddy_on.ser_number != eddy_trip && eddy_on.ser_number !=eddy_trip_former && all of them != eddynumber) according to eddy_on.ser_number judging method, eddy_on shows the larger eddies for the overlap region, eddy_on is the controling eddies!
                {
                    eddy_on = par_info[i].eddy_on;
                    par_info[i].eddy_trip_former = par_info[i].eddy_trip;
                    par_info[i].eddy_trip = eddy_info[eddy_on].ser_number;

                    if(!(par_info[i].y>=grid_y[par_info[i].cellin] && par_info[i].y<=grid_y[par_info[i].cellin+1]))
                        printf("4_b:particle cell judgement wired! p.y=%f, grid_left=%f, grid_right=%f\n",par_info[i].y, grid_y[par_info[i].cellin], grid_y[par_info[i].cellin+1]);
    
                    PARTICLE_TRIPLET(i, grid_y, cell, eddy_info[eddy_on],par_info,source);  // to do the triplet map for the particle according to the drag law
                    
                    if(!(par_info[i].y>=grid_y[par_info[i].cellin] && par_info[i].y<=grid_y[par_info[i].cellin+1]))
                        printf("4_m:particle cell judgement wired! p.y=%f, grid_left=%f, grid_right=%f\n",par_info[i].y, grid_y[par_info[i].cellin], grid_y[par_info[i].cellin+1]);
                    
                    PARTICLE_TRACK_ONESTEP(i, E_S_number, domain_cellnumber, deta_x, grid_y, cell, par_info, source);  // to track the particle for one ODT step
                    
                    if(!(par_info[i].y>=grid_y[par_info[i].cellin] && par_info[i].y<=grid_y[par_info[i].cellin+1]))
                        printf("4_f:particle cell judgement wired! p.y=%f, grid_left=%f, grid_right=%f\n",par_info[i].y, grid_y[par_info[i].cellin], grid_y[par_info[i].cellin+1]);
                }
            }
        } //correspond to if(boundary==0)
    }//correspond to for(^)
}









// This function is used to triplet a particle when it enters an eddy
void PARTICLE_TRIPLET(int i, double *grid_y, struct cell_information *cell, struct eddy_information eddy_info, struct part_infor *par_info,  struct source_information *source)
{
    int eddy_on;
    //int calculate=0;
    int random;  // a random number between 0,1 and 2;
    double eddy_xend;  //the hiehgt that the eddy end, it is a modified value according to the eddy height and its time scale. eddy_xend = E_S_old*dx_stir+eddy[].height*pow((eddy_xcellend-E_S_old)*dx_stir/(eddy[].height_judge),2)
    double f, f_shadow;  //the coefficient of drag force
    double A, Bu,Bv,Bw;  // the constant in the numerical solution
    double A1, Bu1, Bv1, Bw1;             // coefficients to calculate the shadow velocity // to decrease the double calculation of particle tracking outside the eddy
    double vel_u,vel_v,vel_w,vel_m;	//the u,v,w velocity and velocity magnitude of the gas or the eddy
    double vel_u1,vel_v1,vel_w1,vel_m1;  // the shadow u, v and w of the gas phase, which is used to avoid double calculation of particle tracking
    double punew[3], pynew[3], pushadow[3], pyshadow[3]; // u and y in last step for par_infonew and par_infoshadow
    double par_dt, par_dt_min;         //particle integration time step and the minimum particle integration time step
    double mod_par_dt,mod_par_dt1;				//modified particle integral time step
    //	double intersection_left, intersection_right,intersection_left1,intersection_right1; //intersection of particle with cell boundary
    // par_infonew is for the particle which follows the eddy, par_infoshadow is the shadow particle which follows the local gas velocity
    // par_final shows the final position and velocity at each small step (par_dt)
    // the final (real) particle position should be: par_final.y += (par_infonew.y-par_infoshadow.y)
    // the final (real) particle velocity should be: par_final.v += (par_infonew.v-par_infoshadow.v)
    struct part_infor par_infonew, par_infoshadow, par_final;

    par_dt = par_info[i].u>cell[par_info[i].cellin].u ? (100*dx_stir)/(1.2*par_info[i].u) : (100*dx_stir)/(1.2*cell[par_info[i].cellin].u);
    par_dt_min = par_dt/5e8;  // to calculate the minimum particle integration time step

    par_infonew = par_info[i];
    par_infoshadow = par_info[i];
    par_final = par_info[i];
    //	double term1, term2,term3;

    eddy_on = par_info[i].eddy_on;
    //    par_info[i].eddy_trip = par_info[i].eddy_on;           //particle was tripletted by the eddy where the particle is in
    //    par_info[i].eddy_trip_former = par_info[i].eddy_trip;        // to record the former eddy where the particle was tripletted

    random = rand()%3;	//to generate a random number among 0,1and 2, to judge the eddy velocity v
    if(random == 0)
        par_info[i].eddy_displacement = eddy_info.ystart+(par_info[i].y-eddy_info.ystart)/3.0-par_info[i].y;
    else if(random == 1)
        par_info[i].eddy_displacement = eddy_info.ystart+2.0*(eddy_info.yend-eddy_info.ystart)/3.0-(par_info[i].y-eddy_info.ystart)/3.0-par_info[i].y;
    else
        par_info[i].eddy_displacement = eddy_info.ystart+2.0*(eddy_info.yend-eddy_info.ystart)/3.0+(par_info[i].y-eddy_info.ystart)/3.0-par_info[i].y;
    
    // question: an eddy should have effects on all 3 sides of velocity although it is only a one-dimension eddy?
    // In this model, the 1-d eddy will only have effects on y direction, at first.
    vel_u = eddy_info.u;					
    //vel_v = v_latcoe*eddy_info[eddy_on].v + par_info[i].eddy_displacement/eddy_info[eddy_on].lifetime;
    vel_v = v_eddycoe*eddy_info.v + par_info[i].eddy_displacement/(eddy_info.height/eddy_info.u);
    //vel_v = fabs(vel_v)>fabs(2*vel_u) ? (vel_v/fabs(vel_v)*2*fabs(vel_u)) : vel_v;
    vel_w = eddy_info.w;
    
    //  printf("step=%d, pid=%d, vel_u=%8.5f,p_u=%8.5f,p_y=%8.5f,p_x=%8.5f,vel_v=%8.5f,vel_w=%8.5f\n",E_S_number,i,vel_u,par_info[i].u,par_info[i].y,par_info[i].x,vel_v,vel_w);
    
    // shadow velocity of the gas
    vel_u1 = eddy_info.u;					
    //vel_v = v_latcoe*eddy_info[eddy_on].v + par_info[i].eddy_displacement/eddy_info[eddy_on].lifetime;
    vel_v1 = v_eddycoe*eddy_info.v;
    //vel_v = fabs(vel_v)>fabs(2*vel_u) ? (vel_v/fabs(vel_v)*2*fabs(vel_u)) : vel_v;
    vel_w1 = eddy_info.w;

    // to initialize the u and position of the 'par_infonew' and 'par_infoshadow' 
    punew[0] = par_infonew.u;
    punew[1] = par_infonew.v;
    punew[2] = par_infonew.w;
    pushadow[0] = par_infoshadow.u;
    pushadow[1] = par_infoshadow.v;
    pushadow[2] = par_infoshadow.w;

    pynew[0] = par_infonew.x;
    pynew[1] = par_infonew.y;
    pynew[2] = par_infonew.z;
    pyshadow[0] = par_infoshadow.x;
    pyshadow[1] = par_infoshadow.y;
    pyshadow[2] = par_infoshadow.z;

    eddy_xend = par_final.x+eddy_info.height*pow((eddy_info.xcellend*dx_stir-par_final.x)/eddy_info.height_judge,2);

    while((par_final.x<eddy_xend) && (par_final.y>eddy_info.ystart) && (par_final.y<eddy_info.yend) && cell[par_final.cellin].eddynumber==eddy_on && (par_final.y>-domain_y/2.0) && par_final.y<domain_y/2.0)
    {
        vel_m = sqrt(pow(vel_u-par_infonew.u,2)+pow(vel_v-par_infonew.v,2)+pow(vel_w-par_infonew.w,2));
        par_infonew.Re_p = vel_m*par_infonew.diameter/(cell[par_infonew.cellin].mixture.Kinetic_Viscosity_Coeff/cell[par_infonew.cellin].mixture.Density); //particle Reynolds number
        f = par_infonew.Re_p > 1000 ? (0.0183*par_infonew.Re_p) : (1+0.15*pow(par_infonew.Re_p,0.687));
        // to renew particle drag force coefficient
        par_infonew.tao = par_infonew.density*pow(par_infonew.diameter,2)/(18*cell[par_infonew.cellin].mixture.Kinetic_Viscosity_Coeff);

        // shadow value
        vel_m1 = sqrt(pow(vel_u1-par_infoshadow.u,2)+pow(vel_v1-par_infoshadow.v,2)+pow(vel_w1-par_infoshadow.w,2));
        par_infoshadow.Re_p = vel_m1*par_infoshadow.diameter/(cell[par_infoshadow.cellin].mixture.Kinetic_Viscosity_Coeff/cell[par_infoshadow.cellin].mixture.Density);  // shadow particle Reynolds number
        f_shadow = par_infoshadow.Re_p > 1000 ? (0.0183*par_infoshadow.Re_p) : (1+0.15*pow(par_infoshadow.Re_p, 0.687));
        par_infoshadow.tao = par_infoshadow.density*pow(par_infoshadow.diameter,2)/(18*cell[par_infoshadow.cellin].mixture.Kinetic_Viscosity_Coeff);

        A = -f/par_infonew.tao;
        Bu = -vel_u+gravity_ac*(par_infonew.density-cell[par_infonew.cellin].mixture.Density)*par_infonew.tao/(par_infonew.density*f);
        Bv = -vel_v;
        Bw = -vel_w;

        A1 = -f_shadow/par_infoshadow.tao;
        Bu1 = -vel_u1+gravity_ac*(par_infoshadow.density-cell[par_infoshadow.cellin].mixture.Density)*par_infoshadow.tao/(par_infoshadow.density*f_shadow);
        Bv1 = -vel_v1;
        Bw1 = -vel_w1;

        // to calculate the velocity and position of par_infonew, which is only affected by the eddy triplet map effect
        // Here, only v and y in y direction are calculated. u is regarded as keeping a constant velocity of eddy_inof[i].u. so that to keep the time which the particle across the eddy is the same with the eddy life time mutiplied by a ratio (beta)
        //        par_infonew.u = (punew[0]+Bu)*exp(A*par_dt)-Bu;
        par_infonew.v = (punew[1]+Bv)*exp(A*par_dt)-Bv;
        //        par_infonew.w = (punew[2].w+Bw)*exp(A*par_dt)-Bw;
        par_infonew.x += vel_u*par_dt;
        par_infonew.y = pynew[1]-(1/A)*(punew[1]+Bv)*(1-exp(A*par_dt))-Bv*par_dt;
        //        par_infonew.z = pynew[2]-(1/A)*(punew[2]+Bw)*(1-exp(A*par_dt))-Bw*par_dt;

        // to calculate the velocity and position of par_infoshadow, which is only affected by local gas phase
        //        par_infoshadow.u = (pushadow[0]+Bu1)*exp(A1*par_dt)-Bu1;
        par_infoshadow.v = (pushadow[1]+Bv1)*exp(A1*par_dt)-Bv1;
        //        par_infoshadow.w = (pushadow[2]+Bw1)*exp(A1*par_dt)-Bw1;
        //        par_infoshadow.x = pyshadow[0]-(1/A1)*(pushadow[0]+Bu1)*(1-exp(A1*par_dt))-Bu1*par_dt;
        par_infoshadow.x += vel_u1*par_dt;
        par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*par_dt))-Bv1*par_dt;
        //        par_infoshadow.z = pyshadow[2]-(1/A1)*(pushadow[2]+Bw1)*(1-exp(A1*par_dt))-Bw1*par_dt;

        // to get the velocity and position of par_final
        //        par_final.u = par_infonew.u-par_infoshadow.u+par_info[i].u;
        //        par_final.v = par_infonew.v-par_infoshadow.v+par_info[i].v;
        //        par_final.w = par_infonew.w-par_infoshadow.w+par_info[i].w;
        //        par_final.x = par_infonew.x;
        //        par_final.y = par_infonew.y-par_infoshadow.y+par_info[i].y;
        //        par_final.z = par_infonew.z-par_infoshadow.z+par_info[i].z;

        mod_par_dt = par_dt;
        mod_par_dt1 = mod_par_dt;
        /*if after this step, particle x position exceeds the upper line, a linear assumption was made
          to recalculate the modified integral time step for the particle*/ 
        if(par_infonew.x > eddy_xend)  //this particle exceeds the upper line of the eddy
        {
            mod_par_dt = par_dt*((eddy_xend-par_final.x)/(par_infonew.x-par_final.x));  //par_infonew.x-pynew[0] = par_dt*vel_u
            if(!(mod_par_dt<=par_dt && mod_par_dt>par_dt_min))
                printf("mod_par_dt weird in triplet_map 1! par_dt=%e, mod_par_dt=%e, i=%d\n",par_dt, mod_par_dt,i);

            mod_par_dt1 = mod_par_dt;
            //            par_infonew.u = (punew[0]+Bu)*exp(A*mod_par_dt)-Bu;
            par_infonew.v = (punew[1]+Bv)*exp(A*mod_par_dt)-Bv;
            //            par_infonew.w = (punew[2]+Bw)*exp(A*mod_par_dt)-Bw;
            par_infonew.x = eddy_xend+1e-9;
            par_infonew.y = pynew[1]-(1/A)*(punew[1]+Bv)*(1-exp(A*mod_par_dt))-Bv*mod_par_dt;
            //            par_infonew.z = pynew[2]-(1/A)*(punew[2]+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;
        
            //            par_infoshadow.u = (pushadow[0]+Bu1)*exp(A1*mod_par_dt)-Bu1;
            par_infoshadow.v = (pushadow[1]+Bv1)*exp(A1*mod_par_dt)-Bv1;
            //            par_infoshadow.w = (pushadow[2]+Bw1)*exp(A1*mod_par_dt)-Bw1;
            par_infoshadow.x = eddy_xend+1e-9;
            par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*mod_par_dt))-Bv1*mod_par_dt;
            //            par_infoshadow.z = pyshadow[2]-(1/A1)*(pushadow[2]+Bw1)*(1-exp(A1*mod_par_dt))-Bw1*mod_par_dt;
        }
        if(!(par_infonew.u<=180 && par_infonew.u>0))
            printf("neg p_u in tripletmap! %d\t%f\t%f\t%f\t%f\n",i,par_infonew.u,par_infonew.y,par_info[i].u,par_info[i].y);

        // exceeds the right boundary of the eddy 
        if(par_infonew.y-par_infoshadow.y+par_info[i].y > eddy_info.yend)
        {
            mod_par_dt = par_dt*(eddy_info.yend-par_final.y)/((par_infonew.y-par_infoshadow.y+par_info[i].y)-par_final.y);
            if(!(mod_par_dt<=par_dt && mod_par_dt>par_dt_min))
                printf("mod_par_dt weird 2 in triplet map! mod_par_dt=%e, i=%d\n",mod_par_dt,i);

            //            par_infonew.u = (punew[0]+Bu)*exp(A*mod_par_dt)-Bu;
            par_infonew.v = (punew[1]+Bv)*exp(A*mod_par_dt)-Bv;
            //            par_infonew.w = (punew[2]+Bw)*exp(A*mod_par_dt)-Bw;
            //            par_infonew.x = pynew[0]-(1/A)*(punew[0]+Bu)*(1-exp(A*mod_par_dt))-Bu*mod_par_dt;
            par_infonew.x += (mod_par_dt-mod_par_dt1)*vel_u;
            par_infonew.y = pynew[1]-(1/A)*(punew[1]+Bv)*(1-exp(A*mod_par_dt))-Bv*mod_par_dt;
            //            par_infonew.z = pynew[2]-(1/A)*(punew[2]+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;
        
            //            par_infoshadow.u = (pushadow[0]+Bu1)*exp(A1*mod_par_dt)-Bu1;
            par_infoshadow.v = (pushadow[1]+Bv1)*exp(A1*mod_par_dt)-Bv1;
            //            par_infoshadow.w = (pushadow[2]+Bw1)*exp(A1*mod_par_dt)-Bw1;
            par_infoshadow.x += (mod_par_dt-mod_par_dt1)*vel_u;
            // par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*mod_par_dt))-Bv1*mod_par_dt;
            par_infoshadow.y = par_info[i].y+par_infonew.y-(eddy_info.yend+1e-10);   // actually, this value is almost equal to "par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*mod_par_dt))-Bv1*mod_par_dt", however, inorder to terminate the circulation while(), the par_final.y should be equal to (eddy_info.yend+1e-10)
            //            par_infoshadow.z = pyshadow[2]-(1/A1)*(pushadow[2]+Bw1)*(1-exp(A1*mod_par_dt))-Bw1*mod_par_dt;

            //   par_final.y = eddy_info.yend+1e-10;
            mod_par_dt1 = mod_par_dt;
        }

        if(par_infonew.y>domain_y/2.0 || par_infoshadow.y>domain_y/2.0)
        {
            par_info[i].boundary_type = 1;
            printf("particle exceeds the right boundary in tripletmap!");
            break;
        }

        // it the particle exceeds the left boundary of the eddy
        if(par_infonew.y-par_infoshadow.y+par_info[i].y < eddy_info.ystart)
        {
            mod_par_dt = par_dt*(eddy_info.ystart-par_final.y)/((par_infonew.y-par_infoshadow.y+par_info[i].y)-par_final.y);
            if(!(mod_par_dt<=par_dt && mod_par_dt>par_dt_min))
                printf("mod_par_dt weird 3 in tripletmap!mod_par_dt=%e, i=%d\n", mod_par_dt,i);

            //            par_infonew.u = (punew[0]+Bu)*exp(A*mod_par_dt)-Bu;
            par_infonew.v = (punew[1]+Bv)*exp(A*mod_par_dt)-Bv;
            //            par_infonew.w = (punew[2]+Bw)*exp(A*mod_par_dt)-Bw;
            //            par_infonew.x = pynew[0]-(1/A)*(punew[0]+Bu)*(1-exp(A*mod_par_dt))-Bu*mod_par_dt;
            par_infonew.x += (mod_par_dt-mod_par_dt1)*vel_u;
            par_infonew.y = pynew[1]-(1/A)*(punew[1]+Bv)*(1-exp(A*mod_par_dt))-Bv*mod_par_dt;
            //            par_infonew.z = pynew[2]-(1/A)*(punew[2]+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;
        
            //            par_infoshadow.u = (pushadow[0]+Bu1)*exp(A1*mod_par_dt)-Bu1;
            par_infoshadow.v = (pushadow[1]+Bv1)*exp(A1*mod_par_dt)-Bv1;
            //            par_infoshadow.w = (pushadow[2]+Bw1)*exp(A1*mod_par_dt)-Bw1;
            par_infoshadow.x += (mod_par_dt-mod_par_dt1)*vel_u;
            par_infoshadow.y = par_info[i].y+par_infonew.y-(eddy_info.ystart-1e-10);   // actually, this value is almost equal to "par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*mod_par_dt))-Bv1*mod_par_dt", however, inorder to terminate the circulation while(), the par_final.y should be equal to (eddy_info.ystart-1e-10)
            // par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*mod_par_dt))-Bv1*mod_par_dt;
            //            par_infoshadow.z = pyshadow[2]-(1/A1)*(pushadow[2]+Bw1)*(1-exp(A1*mod_par_dt))-Bw1*mod_par_dt;

            //   par_final.y = eddy_info.ystart-1e-10;
            mod_par_dt1 = mod_par_dt;
        }

        if(par_infonew.y<-domain_y/2.0 || par_infoshadow.y<-domain_y/2.0)
        {
            par_info[i].boundary_type = 1;
            printf("particle exceeds left boundary in tripletmap!\n");
            break;
        }

    recal:			;

        ////////////////////////////////////////////////////////////////////////////////////////////
        /* particle exceeds the right edge of this cell */
        if((par_infonew.y-par_infoshadow.y+par_info[i].y) > grid_y[par_final.cellin+1])  
        {
            //the particle does not exceed the right edge of next cell on its right side
            if((par_infonew.y-par_infoshadow.y+par_info[i].y) < grid_y[par_final.cellin+2])
            {
                par_final.cellin++; 
                cell[par_final.cellin-1].loading_rate[1] -= par_final.massflowrate/(cell[par_final.cellin-1].mixture.Density*cell[par_final.cellin-1].u*cell[par_final.cellin-1].size);
                cell[par_final.cellin].loading_rate[1] += par_final.massflowrate/(cell[par_final.cellin].mixture.Density*cell[par_final.cellin].u*cell[par_final.cellin].size);
                //the particle momentum source term was distributed to two cells according to linear assumption	if this particle enters another cell
                
                source[par_final.cellin].mom_v += ((par_infonew.y-par_infoshadow.y+par_info[i].y)-grid_y[par_final.cellin])/((par_infonew.y-par_infoshadow.y+par_info[i].y)-par_final.y)*par_final.mass*par_final.particle_number*(par_final.v-(par_infonew.v+par_info[i].v-par_infoshadow.v));
                
                source[par_final.cellin-1].mom_v += (grid_y[par_final.cellin]-par_final.y)/((par_infonew.y-par_infoshadow.y+par_info[i].y)-par_final.y)*par_final.mass*par_final.particle_number*(par_final.v-(par_infonew.v-par_infoshadow.v+par_info[i].v));
            }

            //if the particle exceeds the right boundary of the next right cell, to judge the integral time
            // so that the new particle tracking process was conducted to keep the particle not exceed the 
            // right boundary of the next right cell!
            else
            {
                //	printf("%d R! %f  %f  %f  %f  %f\n", i,par_infonew.y,par_info[i].y, grid_y[par_info[i].cellin],grid_y[par_info[i].cellin+1],grid_y[par_info[i].cellin+2]);
                mod_par_dt *= 0.5;
                if(!(mod_par_dt<=par_dt && mod_par_dt>par_dt_min))
                    printf("mod_par_dt weird 4 in tripletmap! mod_par_dt=%f, i=%d\n",mod_par_dt,i);

                //            par_infonew.u = (punew[0]+Bu)*exp(A*mod_par_dt)-Bu;
                par_infonew.v = (punew[1]+Bv)*exp(A*mod_par_dt)-Bv;
                //            par_infonew.w = (punew[2]+Bw)*exp(A*mod_par_dt)-Bw;
                //            par_infonew.x = pynew[0]-(1/A)*(punew[0]+Bu)*(1-exp(A*mod_par_dt))-Bu*mod_par_dt;
                par_infonew.x += (mod_par_dt-mod_par_dt1)*vel_u;
                par_infonew.y = pynew[1]-(1/A)*(punew[1]+Bv)*(1-exp(A*mod_par_dt))-Bv*mod_par_dt;
                //            par_infonew.z = pynew[2]-(1/A)*(punew[2]+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;
                
                //            par_infoshadow.u = (pushadow[0]+Bu1)*exp(A1*mod_par_dt)-Bu1;
                par_infoshadow.v = (pushadow[1]+Bv1)*exp(A1*mod_par_dt)-Bv1;
                //            par_infoshadow.w = (pushadow[2]+Bw1)*exp(A1*mod_par_dt)-Bw1;
                par_infoshadow.x += (mod_par_dt-mod_par_dt1)*vel_u;
                par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*mod_par_dt))-Bv1*mod_par_dt;   // actually, this value is almost equal to "par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*mod_par_dt))-Bv1*mod_par_dt", however, inorder to terminate the circulation while(), the par_final.y should be equal to (eddy_info.ystart-1e-10)
                // par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*mod_par_dt))-Bv1*mod_par_dt;
                //            par_infoshadow.z = pyshadow[2]-(1/A1)*(pushadow[2]+Bw1)*(1-exp(A1*mod_par_dt))-Bw1*mod_par_dt;
                mod_par_dt1 = mod_par_dt;
                goto recal;
            }

            if(!(fabs(source[par_final.cellin].mom_u)<100000 && fabs(source[par_final.cellin-1].mom_u)<100000))
            {
                printf("right source error in tripletmap!%d\t%e\t%e\t%e\n",par_final.cellin,source[par_final.cellin].mom_u,source[par_final.cellin-1].mom_u,par_final.mass*par_final.particle_number*(par_final.u-par_infonew.u));
                // printf("%f\t%f\t%f\t%f\n", par_infonew.y, par_info[i].y, intersection_right, intersection_right1);
            }
        }

        // the particle is still in this cell, only source terms were refeshed
        else if((par_infonew.y-par_infoshadow.y+par_info[i].y)<=grid_y[par_final.cellin+1] && (par_infonew.y-par_infoshadow.y+par_info[i].y)>=grid_y[par_final.cellin])
        {
            source[par_final.cellin].mom_v += par_final.mass*par_final.particle_number*(par_final.v-(par_infonew.v-par_infoshadow.v+par_info[i].v));
            if(!(fabs(source[par_final.cellin].mom_u)<100000))
                printf("center source error in tripletmap! %d\t%e\n",par_final.cellin,source[par_final.cellin].mom_u);
        }

        /* particle exceeds the left edge of this cell */
        else if((par_infonew.y-par_infoshadow.y+par_info[i].y) < grid_y[par_final.cellin])  
        {
            //the particle doesn't exceed the left boundary of the next cell on its left side
            if((par_infonew.y-par_infoshadow.y+par_info[i].y) > grid_y[par_final.cellin-1])
            {
                par_final.cellin--;

                cell[par_final.cellin+1].loading_rate[1] -= par_final.massflowrate/(cell[par_final.cellin+1].mixture.Density*cell[par_final.cellin+1].u*cell[par_final.cellin+1].size);
                cell[par_final.cellin].loading_rate[1] += par_final.massflowrate/(cell[par_final.cellin].mixture.Density*cell[par_final.cellin].u*cell[par_final.cellin].size);

                //the particle momentum source term was distributed to two cells according to linear assumption	if this particle enters another cell

                source[par_final.cellin].mom_v += ((par_infonew.y-par_infoshadow.y+par_info[i].y)-grid_y[par_final.cellin+1])/((par_infonew.y-par_infoshadow.y+par_info[i].y)-par_final.y)*par_final.mass*par_final.particle_number*(par_final.v-(par_infonew.v-par_infoshadow.v+par_info[i].v));
                source[par_final.cellin+1].mom_v += (grid_y[par_final.cellin+1]-par_final.y)/((par_infonew.y-par_infoshadow.y+par_info[i].y)-par_final.y)*par_final.mass*par_final.particle_number*(par_final.v-(par_infonew.v-par_infoshadow.v+par_info[i].v));
						
            }

            //if the particle exceeds the left boundary of the next left cell, to judge the integral time
            // so that the new particle tracking process was conducted to keep the particle not exceed the 
            // left boundary of the next left cell!
            else
            {
                //			printf("%d L! %f  %f  %f  %f  %f\n", i,par_infonew.y,par_info[i].y,grid_y[par_info[i].cellin],grid_y[par_info[i].cellin+1],grid_y[par_info[i].cellin-1]);

                mod_par_dt *= 0.5;  
                if(!(mod_par_dt<par_dt && mod_par_dt>par_dt_min))
                    printf("mod_par_dt weird 5 in tripletmap! mod_par_dt=%e,i=%d, final.y=%f, g.y=%f, p.y=%f\n",mod_par_dt,i,par_infonew.y-par_infoshadow.y+par_info[i].y, grid_y[par_final.cellin-1], par_info[i].y);
      
                //            par_infonew.u = (punew[0]+Bu)*exp(A*mod_par_dt)-Bu;
                par_infonew.v = (punew[1]+Bv)*exp(A*mod_par_dt)-Bv;
                //            par_infonew.w = (punew[2]+Bw)*exp(A*mod_par_dt)-Bw;
                //            par_infonew.x = pynew[0]-(1/A)*(punew[0]+Bu)*(1-exp(A*mod_par_dt))-Bu*mod_par_dt;
                par_infonew.x += (mod_par_dt-mod_par_dt1)*vel_u;
                par_infonew.y = pynew[1]-(1/A)*(punew[1]+Bv)*(1-exp(A*mod_par_dt))-Bv*mod_par_dt;
                //            par_infonew.z = pynew[2]-(1/A)*(punew[2]+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;
                
                //            par_infoshadow.u = (pushadow[0]+Bu1)*exp(A1*mod_par_dt)-Bu1;
                par_infoshadow.v = (pushadow[1]+Bv1)*exp(A1*mod_par_dt)-Bv1;
                //            par_infoshadow.w = (pushadow[2]+Bw1)*exp(A1*mod_par_dt)-Bw1;
                par_infoshadow.x += (mod_par_dt-par_dt)*vel_u;
                par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*mod_par_dt))-Bv1*mod_par_dt;    // actually, this value is almost equal to "par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*mod_par_dt))-Bv1*mod_par_dt", however, inorder to terminate the circulation while(), the par_final.y should be equal to (eddy_info.ystart-1e-10)
                // par_infoshadow.y = pyshadow[1]-(1/A1)*(pushadow[1]+Bv1)*(1-exp(A1*mod_par_dt))-Bv1*mod_par_dt;
                //            par_infoshadow.z = pyshadow[2]-(1/A1)*(pushadow[2]+Bw1)*(1-exp(A1*mod_par_dt))-Bw1*mod_par_dt;
                mod_par_dt1 = mod_par_dt;
                goto recal;
            }

            if(!(fabs(source[par_final.cellin].mom_u)<100000 && fabs(source[par_final.cellin+1].mom_u)<100000))
            {
                printf("left source error in tripletmap!%d\t%e\t%e\t%e\n",par_final.cellin,source[par_final.cellin].mom_u,source[par_final.cellin+1].mom_u,par_final.mass*par_final.particle_number*(par_final.u-(par_infonew.u-par_infoshadow.u+par_info[i].u)));
                //    printf("%f\t%f\t%f\t%f\n", par_infonew.y, par_info[i].y, intersection_left, intersection_left1);
            }
        }
        
        // to identify which cells par_infonew and par_infoshadow is in
        if(par_infonew.y < pynew[1] && par_infonew.y>-domain_y/2.0)  // particle moves left
        {
            while(par_infonew.y<grid_y[par_infonew.cellin] && par_infonew.cellin>0)
            {
                par_infonew.cellin --;
                //par_infonew.cellin == 0 ? printf("par_infonew.cellin=0!\n") : ;
            }
        }
        else
        {
            while(par_infonew.y > grid_y[par_infonew.cellin+1] && par_infoshadow.y<domain_y/2.0)
            {
                par_infonew.cellin ++;
                //par_infonew.y > domain_y/2.0 ? par_infonew.cellin-- :;
            }
        }

          
        // to identify which cells par_infonew and par_infoshadow is in
        if(par_infoshadow.y < pyshadow[1] && par_infoshadow.y>-domain_y/2.0)  // particle moves left
        {
            while(par_infoshadow.y < grid_y[par_infoshadow.cellin] && par_infoshadow.cellin>0)
            {
                par_infoshadow.cellin --;
                //par_infoshadow.cellin == 0 ? printf("par_infonew.cellin=0!\n") : ;
            }
        }
        else
        {
            while(par_infoshadow.y>grid_y[par_infoshadow.cellin+1] && par_infoshadow.y<domain_y/2.0)
            {
                par_infoshadow.cellin ++;
                //par_infoshadow.y>domain_y/2.0 ? par_infonew.cellin-- : ;
            }
        }

        // to get the velocity and position of par_final
        //        par_final.u = par_infonew.u-par_infoshadow.u+par_info[i].u;
        par_final.v = par_infonew.v-par_infoshadow.v+par_info[i].v;
        //par_final.w = par_infonew.w-par_infoshadow.w+par_info[i].w;
        par_final.x = par_infonew.x;
        par_final.y = par_infonew.y-par_infoshadow.y+par_info[i].y;
        //par_final.z = par_infonew.z-par_infoshadow.z+par_info[i].z;

        // to store the renewd value for the 'par_infonew' and 'par_infoshadow' 
        punew[0] = par_infonew.u;
        punew[1] = par_infonew.v;
        punew[2] = par_infonew.w;
        pushadow[0] = par_infoshadow.u;
        pushadow[1] = par_infoshadow.v;
        pushadow[2] = par_infoshadow.w;
        
        pynew[0] = par_infonew.x;
        pynew[1] = par_infonew.y;
        pynew[2] = par_infonew.z;
        pyshadow[0] = par_infoshadow.x;
        pyshadow[1] = par_infoshadow.y;
        pyshadow[2] = par_infoshadow.z;

    }  //correspond to while()
    //to get the final particle position and velocity after this eddy
    //    par_info[i].u = par_infonew.u;
    par_info[i].v = par_final.v;
    //    par_info[i].w = par_infonew.w;
    //    par_info[i].x = par_infonew.x;
    par_info[i].y = par_final.y;
    //    par_info[i].z = par_infonew.z;
    par_info[i].cellin = par_final.cellin;   // it is very importment to renew this value, since the particle.cellin will be changed during particl triplet map tracking process
}








// this function is used to track a particle motion through a ODT diffusion step
// during the tracking procedure, the particle reaction process should be considered.....
// the reason why *par_info (the whole string of particle) was kept in this function is in the future potential use for considering particle-particle interactions
void PARTICLE_TRACK_ONESTEP(int i, int E_S_number, int domain_cellnumber, double deta_x, double *grid_y, struct cell_information *cell, struct part_infor *par_info,  struct source_information *source)
{
    //int calculate=0;
    double f;  //the coefficient of drag force
    double A, Bu,Bv,Bw;  // the constant in the numerical solution
    double vel_u,vel_v,vel_w,vel_m;	//the u,v,w velocity and velocity magnitude of the gas or the eddy
    double par_dt, par_dt_min;          // particle integration time step and minimum particle integration time step
    double h_old, h_new;   /// h_old: the height all the particle staying at, h_new:the new height all the particle will come to. h_new=E_S_number*dx_stir, h_old=h_new-deta_x
    double mod_par_dt,mod_par_dt1;				//modified particle integral time step
    //	double intersection_left, intersection_right,intersection_left1,intersection_right1; //intersection of particle with cell boundary
    struct part_infor par_infonew;

    h_new = E_S_number*dx_stir;
    h_old = h_new-deta_x;
    if(fabs(par_info[200].x-h_old)>dx_stir && fabs(par_info[200].x-h_new)>dx_stir)
        printf("par_info[200].x weird! par.x=%f, h_old=%f, deta_x=%f\n", par_info[200].x, h_old, deta_x);  // for debug

    par_infonew = par_info[i];
    par_dt = par_info[i].u>cell[par_info[i].cellin].u ? deta_x/(2.0*par_info[i].u) : deta_x/(2.0*cell[par_info[i].cellin].u);

    par_dt_min = par_dt/5000;

    if(par_info[i].boundary_type == 0)  //the particle is in the domain
    {
        while(par_info[i].x < E_S_number*dx_stir-1e-15)
        {
            vel_u = cell[par_info[i].cellin].u;	//to choose u velocity of the gas phase
            vel_v = v_latcoe*cell[par_info[i].cellin].v_lateral+v_tripcoe*cell[par_info[i].cellin].v_trip;  //including lateral velocity according to triplet map process
            vel_w = cell[par_info[i].cellin].w;
 
            //  printf("step=%d, pid=%d, vel_u=%8.5f,p_u=%8.5f,p_y=%8.5f,p_x=%8.5f,vel_v=%8.5f,vel_w=%8.5f\n",E_S_number,i,vel_u,par_info[i].u,par_info[i].y,par_info[i].x,vel_v,vel_w);

            vel_m = sqrt(pow(vel_u-par_info[i].u,2)+pow(vel_v-par_info[i].v,2)+pow(vel_w-par_info[i].w,2));  //velocity magnitude of the difference between gas phase and the particle
            par_info[i].Re_p = fabs(vel_m)*par_info[i].diameter/(cell[par_info[i].cellin].mixture.Kinetic_Viscosity_Coeff/cell[par_info[i].cellin].mixture.Density); //particle Reynolds number
            f = par_info[i].Re_p > 1000 ? (0.0183*par_info[i].Re_p) : (1+0.15*pow(par_info[i].Re_p,0.687));
            // to renew particle drag force coefficient
            par_info[i].tao = par_info[i].density*pow(par_info[i].diameter,2)/(18*cell[par_info[i].cellin].mixture.Kinetic_Viscosity_Coeff);
            A = -f/par_info[i].tao;
            if(!(fabs(A)>0))
                printf("A weird in onestep! tao=%f,Den=%e, Vis_Coe=%e,  f=%f, Re_p=%f, cellin=%d, cellnumber=%d\n", par_info[i].tao, par_info[i].density, cell[par_info[i].cellin].mixture.Kinetic_Viscosity_Coeff, f, par_info[i].Re_p, par_info[i].cellin, domain_cellnumber);
            Bu = -vel_u+gravity_ac*(par_info[i].density-cell[par_info[i].cellin].mixture.Density)*par_info[i].tao/(par_info[i].density*f);
            Bv = -vel_v;
            Bw = -vel_w;
            par_infonew.u = (par_info[i].u+Bu)*exp(A*par_dt)-Bu;
            par_infonew.v = (par_info[i].v+Bv)*exp(A*par_dt)-Bv;
            par_infonew.w = (par_info[i].w+Bw)*exp(A*par_dt)-Bw;
            par_infonew.x = par_info[i].x-(1/A)*(par_info[i].u+Bu)*(1-exp(A*par_dt))-Bu*par_dt;
            par_infonew.y = par_info[i].y-(1/A)*(par_info[i].v+Bv)*(1-exp(A*par_dt))-Bv*par_dt;
            par_infonew.z = par_info[i].z-(1/A)*(par_info[i].w+Bw)*(1-exp(A*par_dt))-Bw*par_dt;

            mod_par_dt = par_dt;
            mod_par_dt1 = mod_par_dt;
            /*if after this step, particle x position exceeds the upper line, a linear assumption was made
              to recalculate the modified integral time step for the particle*/ 
            if(par_infonew.x > (E_S_number)*dx_stir)  //this particle exceeds the upper line
            {
                mod_par_dt = par_dt*((E_S_number)*dx_stir-par_info[i].x)/(par_infonew.x-par_info[i].x);
                par_infonew.u = (par_info[i].u+Bu)*exp(A*mod_par_dt)-Bu;
                par_infonew.v = (par_info[i].v+Bv)*exp(A*mod_par_dt)-Bv;
                par_infonew.w = (par_info[i].w+Bw)*exp(A*mod_par_dt)-Bw;
                par_infonew.x = (E_S_number)*dx_stir;  // to set .x to the final position 
                par_infonew.y = par_info[i].y-(1/A)*(par_info[i].v+Bv)*(1-exp(A*mod_par_dt))-Bv*mod_par_dt;
                par_infonew.z = par_info[i].z-(1/A)*(par_info[i].w+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;
                mod_par_dt1 = mod_par_dt;
            }
            if(!(par_infonew.u<180 && par_infonew.u>0))
                printf("neg p_u in onestep! %d\tu_new=%e, y_new=%e, u_old=%e, y_old=%e, dt=%e, A=%e, Bv=%e, cell.u=%f, cell.v=%f\n",i,par_infonew.u,par_infonew.y,par_info[i].u,par_info[i].y, mod_par_dt, A, Bv, cell[par_infonew.cellin].u,cell[par_infonew.cellin].v_lateral);

            //particle exceeds the right calculation domain
            if(!(par_infonew.y<=domain_y/2))
            {
                if(par_infonew.y > domain_y/2)
                {
                    mod_par_dt = mod_par_dt1*(domain_y/2-par_info[i].y)/(par_infonew.y-par_info[i].y);
                    par_infonew.u = (par_info[i].u+Bu)*exp(A*mod_par_dt)-Bu;
                    par_infonew.v = (par_info[i].v+Bv)*exp(A*mod_par_dt)-Bv;
                    par_infonew.w = (par_info[i].w+Bw)*exp(A*mod_par_dt)-Bw;
                    par_infonew.x = (E_S_number)*dx_stir;   // to terminate the circulation in next step
                    par_infonew.y = domain_y/2+1e-10;
                    par_infonew.z = par_info[i].z-(1/A)*(par_info[i].w+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;
                    
                    // to judge which law should be considered, source term should be renewed in this function
                    if(mod_par_dt<=mod_par_dt1)
                        COAL_LAW_SWITCH(i, mod_par_dt, cell, par_info, &par_infonew, source);
                }
                else
                    printf("particle right boundary weird in onestep! .y_old=%e, .y_new=%e, A=%e, dt=%f, cellin=%d, number=%d\n", par_info[i].y, par_infonew.y, A, mod_par_dt, par_info[i].cellin, domain_cellnumber);

                par_info[i].boundary_type = 1;
                par_infonew.boundary_type = 1;
                printf("particle exceeds the right boundary in onestep!\n");
                mod_par_dt1 = mod_par_dt;
                break;  //break while()
            }
            // particle exceed the right boundary of the cell which the particle is in
            else if(par_infonew.y>grid_y[par_info[i].cellin+1])
            {
                // if the distance of the particle to the right boundary is larger than 1e-8, to modify the particle time step in order to make sure that particle JUST across the boundary.
                if(grid_y[par_info[i].cellin+1]-par_info[i].y>0.01*cell[par_info[i].cellin].size)
                {
                    mod_par_dt = mod_par_dt1*(grid_y[par_info[i].cellin+1]-par_info[i].y)/(par_infonew.y-par_info[i].y);
                    if(!(fabs(par_infonew.y-par_info[i].y)>0.0))
                        printf("p_displacement weird in onestep! R1, y_new=%e, y_old=%e\n",  par_infonew.y, par_info[i].y);

                    par_infonew.u = (par_info[i].u+Bu)*exp(A*mod_par_dt)-Bu;
                    par_infonew.v = (par_info[i].v+Bv)*exp(A*mod_par_dt)-Bv;
                    par_infonew.w = (par_info[i].w+Bw)*exp(A*mod_par_dt)-Bw;
                    par_infonew.x = par_info[i].x-(1/A)*(par_info[i].u+Bu)*(1-exp(A*mod_par_dt))-Bu*mod_par_dt;
                    // par_infonew.y = par_info[i].y-(1/A)*(par_info[i].v+Bv)*(1-exp(A*mod_par_dt))-Bv*mod_par_dt;
                    par_infonew.y = grid_y[par_info[i].cellin+1]+1e-15;
                    par_infonew.z = par_info[i].z-(1/A)*(par_info[i].w+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;
                    
                    // to judge which law should be considered, source terms are renewed in this function
                    COAL_LAW_SWITCH(i, mod_par_dt, cell, par_info, &par_infonew, source);
                    
                    par_info[i].cellin++;
                    par_infonew.cellin++;

                    cell[par_info[i].cellin-1].loading_rate[1] -= par_info[i].massflowrate/(cell[par_info[i].cellin-1].mixture.Density*cell[par_info[i].cellin-1].u*cell[par_info[i].cellin-1].size);
                    cell[par_info[i].cellin].loading_rate[1] += par_info[i].massflowrate/(cell[par_info[i].cellin].mixture.Density*cell[par_info[i].cellin].u*cell[par_info[i].cellin].size);

                    mod_par_dt1 = mod_par_dt;
                }
                // if this distance is small than 0.01cell.size, the modification of time step will usually cause a very small modified time step, which leads to a slow interation of particle tracking process, this case should be avoided by not modifying the time step.
                else if(par_infonew.y<grid_y[par_info[i].cellin+2])  // the new particle position is in the right neighbour cell
                {
                    par_info[i].cellin++;
                    par_infonew.cellin++;

                    cell[par_info[i].cellin-1].loading_rate[1] -= par_info[i].massflowrate/(cell[par_info[i].cellin-1].mixture.Density*cell[par_info[i].cellin-1].u*cell[par_info[i].cellin-1].size);
                    cell[par_info[i].cellin].loading_rate[1] += par_info[i].massflowrate/(cell[par_info[i].cellin].mixture.Density*cell[par_info[i].cellin].u*cell[par_info[i].cellin].size);

                    // to calculate particle physical/chemical change based on the properties of the right next cell, and to renew the source term for the right neighbour cell
                    COAL_LAW_SWITCH(i, mod_par_dt, cell, par_info, &par_infonew, source);
                }
                else  //the particle exceeds the right boundary of the right neighbour cell, to adjust the time step
                {
                    mod_par_dt = mod_par_dt1*(grid_y[par_info[i].cellin+2]-par_info[i].y)/(par_infonew.y-par_info[i].y);
                    if(!(fabs(par_infonew.y-par_info[i].y)>0))
                        printf("p_displace weird in onestep! right, y_new=%e, y_old=%e\n", par_infonew.y, par_info[i].y);
                    par_infonew.u = (par_info[i].u+Bu)*exp(A*mod_par_dt)-Bu;
                    par_infonew.v = (par_info[i].v+Bv)*exp(A*mod_par_dt)-Bv;
                    par_infonew.w = (par_info[i].w+Bw)*exp(A*mod_par_dt)-Bw;
                    par_infonew.x = par_info[i].x-(1/A)*(par_info[i].u+Bu)*(1-exp(A*mod_par_dt))-Bu*mod_par_dt;
                    // par_infonew.y = par_info[i].y-(1/A)*(par_info[i].v+Bv)*(1-exp(A*mod_par_dt))-Bv*mod_par_dt;
                    par_infonew.y = grid_y[par_info[i].cellin+2]+1e-15;
                    par_infonew.z = par_info[i].z-(1/A)*(par_info[i].w+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;
                    
                    par_info[i].cellin++;
                    par_infonew.cellin++;

                    cell[par_info[i].cellin-1].loading_rate[1] -= par_info[i].massflowrate/(cell[par_info[i].cellin-1].mixture.Density*cell[par_info[i].cellin-1].u*cell[par_info[i].cellin-1].size);
                    cell[par_info[i].cellin].loading_rate[1] += par_info[i].massflowrate/(cell[par_info[i].cellin].mixture.Density*cell[par_info[i].cellin].u*cell[par_info[i].cellin].size);
                     // to calculate particle physical/chemical change based on the properties of the right next cell, and to renew the source term for the right neighbour cell
                    COAL_LAW_SWITCH(i, mod_par_dt, cell, par_info, &par_infonew, source);
                    // to set the final value of par.cellin
                    par_info[i].cellin++;
                    par_infonew.cellin++;

                    cell[par_info[i].cellin-1].loading_rate[1] -= par_info[i].massflowrate/(cell[par_info[i].cellin-1].mixture.Density*cell[par_info[i].cellin-1].u*cell[par_info[i].cellin-1].size);
                    cell[par_info[i].cellin].loading_rate[1] += par_info[i].massflowrate/(cell[par_info[i].cellin].mixture.Density*cell[par_info[i].cellin].u*cell[par_info[i].cellin].size);
                    mod_par_dt1 = mod_par_dt;
                }
            }
            else if(!(par_infonew.y>=(-domain_y/2)))       // if the particle exceeds the left boundary
            {
                if(par_infonew.y<-domain_y/2)
                {
                    mod_par_dt = mod_par_dt1*(-domain_y/2-par_info[i].y)/(par_infonew.y-par_info[i].y);
                    par_infonew.u = (par_info[i].u+Bu)*exp(A*mod_par_dt)-Bu;
                    par_infonew.v = (par_info[i].v+Bv)*exp(A*mod_par_dt)-Bv;
                    par_infonew.w = (par_info[i].w+Bw)*exp(A*mod_par_dt)-Bw;
                    par_infonew.x = (E_S_number)*dx_stir;   //to terminate the circulation
                    par_infonew.y = -domain_y/2-1e-8;
                    par_infonew.z = par_info[i].z-(1/A)*(par_info[i].w+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;
                    // to judge which law should be considered, source.enthalpy and source.massflux_species should be renewed in this function
                    if(mod_par_dt<=mod_par_dt1)
                        COAL_LAW_SWITCH(i, mod_par_dt, cell, par_info, &par_infonew,source);
                }
                else
                    printf("particle left boundary weird! .y_old=%e, .y_new=%e, A=%e, dt=%f, cellin=%d, number=%d\n", par_info[i].y, par_infonew.y, A, mod_par_dt, par_info[i].cellin, domain_cellnumber);

                par_info[i].boundary_type = 1;
                par_infonew.boundary_type = 1;
                printf("particle exceeds left boundary in one step!\n");
                mod_par_dt1 = mod_par_dt;
                break;
            }
            // particle exceeds the left boundary of the cell which the particle is in 
            else if(par_infonew.y < grid_y[par_info[i].cellin])  
            {
                // if the distance of the particle to the left boundary is larger than 0.01cell.size, to modify the partice integration time step
                if((par_info[i].y-grid_y[par_info[i].cellin])>0.01*cell[par_info[i].cellin].size)
                {
                    mod_par_dt = mod_par_dt1*(grid_y[par_info[i].cellin]-par_info[i].y)/(par_infonew.y-par_info[i].y);
                    if(!(fabs(par_infonew.y-par_info[i].y)>0))
                        printf("p_displacement weird in onestep, L1, y.new=%e, y.old=%e\n", par_infonew.y, par_info[i].y);

                    par_infonew.u = (par_info[i].u+Bu)*exp(A*mod_par_dt)-Bu;
                    par_infonew.v = (par_info[i].v+Bv)*exp(A*mod_par_dt)-Bv;
                    par_infonew.w = (par_info[i].w+Bw)*exp(A*mod_par_dt)-Bw;
                    //             par_infonew.x = (E_S_number)*dx_stir;
                    par_infonew.x = par_info[i].x-(1/A)*(par_info[i].u+Bu)*(1-exp(A*mod_par_dt))-Bu*mod_par_dt;
                    par_infonew.y = grid_y[par_info[i].cellin]-1e-15;
                    par_infonew.z = par_info[i].z-(1/A)*(par_info[i].w+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;

                    // to judge which law should be considered, source.enthalpy and source.massflux_species should be renewed in this function
                    COAL_LAW_SWITCH(i, mod_par_dt, cell, par_info, &par_infonew, source);
                    
                    par_info[i].cellin--;
                    par_infonew.cellin--;

                    cell[par_info[i].cellin+1].loading_rate[1] -= par_info[i].massflowrate/(cell[par_info[i].cellin+1].mixture.Density*cell[par_info[i].cellin+1].u*cell[par_info[i].cellin+1].size);
                    cell[par_info[i].cellin].loading_rate[1] += par_info[i].massflowrate/(cell[par_info[i].cellin].mixture.Density*cell[par_info[i].cellin].u*cell[par_info[i].cellin].size);
                    mod_par_dt1 = mod_par_dt;
                }
                // if this distance is small than 0.01cell.size, the modification of time step will usually cause a very small modified time step, which leads to a slow interation of particle tracking process, this case should be avoided by not modifying the time step.
                else if(par_infonew.y > grid_y[par_info[i].cellin-1])  //particle do not exceed the left boundary of the left neighbour cell
                {
                    par_info[i].cellin--;
                    par_infonew.cellin--;
                    cell[par_info[i].cellin+1].loading_rate[1] -= par_info[i].massflowrate/(cell[par_info[i].cellin+1].mixture.Density*cell[par_info[i].cellin+1].u*cell[par_info[i].cellin+1].size);
                    cell[par_info[i].cellin].loading_rate[1] += par_info[i].massflowrate/(cell[par_info[i].cellin].mixture.Density*cell[par_info[i].cellin].u*cell[par_info[i].cellin].size);
                    // to calculate particle physical/chemical change based on the properties of the left neigthbour cell, and to renew the source term for the left neighbour cell
                    COAL_LAW_SWITCH(i, mod_par_dt, cell, par_info, &par_infonew, source);
                }
                else  // particle exceeds the left boundary of the left neighbour cell
                {
                    mod_par_dt = mod_par_dt1*(grid_y[par_info[i].cellin-1]-par_info[i].y)/(par_infonew.y-par_info[i].y);
                    if(!(fabs(par_infonew.y-par_info[i].y)>0))
                        printf("p_displacement weird in onestep, Left, y.new=%e, y.old=%e\n", par_infonew.y, par_info[i].y);
                    par_infonew.u = (par_info[i].u+Bu)*exp(A*mod_par_dt)-Bu;
                    par_infonew.v = (par_info[i].v+Bv)*exp(A*mod_par_dt)-Bv;
                    par_infonew.w = (par_info[i].w+Bw)*exp(A*mod_par_dt)-Bw;
                    //             par_infonew.x = (E_S_number)*dx_stir;
                    par_infonew.x = par_info[i].x-(1/A)*(par_info[i].u+Bu)*(1-exp(A*mod_par_dt))-Bu*mod_par_dt;
                    par_infonew.y = grid_y[par_info[i].cellin-1]-1e-15;
                    par_infonew.z = par_info[i].z-(1/A)*(par_info[i].w+Bw)*(1-exp(A*mod_par_dt))-Bw*mod_par_dt;

                    par_info[i].cellin--;
                    par_infonew.cellin--;
                    cell[par_info[i].cellin+1].loading_rate[1] -= par_info[i].massflowrate/(cell[par_info[i].cellin+1].mixture.Density*cell[par_info[i].cellin+1].u*cell[par_info[i].cellin+1].size);
                    cell[par_info[i].cellin].loading_rate[1] += par_info[i].massflowrate/(cell[par_info[i].cellin].mixture.Density*cell[par_info[i].cellin].u*cell[par_info[i].cellin].size);
                    // to calculate particle physical/chemical change based on the properties of the left neighbour cell, and to renew the source term for the right neighbour cell
                    COAL_LAW_SWITCH(i, mod_par_dt, cell, par_info, &par_infonew, source);
                    // to set the final value of par.cellin
                    par_info[i].cellin--;
                    par_infonew.cellin--;
                                                                                          cell[par_info[i].cellin+1].loading_rate[1] -= par_info[i].massflowrate/(cell[par_info[i].cellin+1].mixture.Density*cell[par_info[i].cellin+1].u*cell[par_info[i].cellin+1].size);
                                                                                          cell[par_info[i].cellin].loading_rate[1] += par_info[i].massflowrate/(cell[par_info[i].cellin].mixture.Density*cell[par_info[i].cellin].u*cell[par_info[i].cellin].size);
                    mod_par_dt1 = mod_par_dt;
                }
            }
            else    // .y in the same cell
            {
                // to judge which law should be considered, source.enthalpy and source.massflux_species should be renewed in this function
                COAL_LAW_SWITCH(i, mod_par_dt, cell, par_info, &par_infonew, source);
                mod_par_dt1 = mod_par_dt;
                if(!(fabs(mod_par_dt>0)))
                    printf("mod_par_dt weird in onestep in the same cell! dt=%e\n", mod_par_dt);
            }

            //to record the final particle values in this integral step
            par_info[i] = par_infonew;

            if(!(par_info[i].cellin<domain_cellnumber-1 && par_info[i].cellin>0))  //if particle escapes the domain
            {
                printf("0! particle escapes domain! particle number: %d\t%d\t%d\t%f\n",i,domain_cellnumber, par_info[i].cellin, par_info[i].y);
                par_info[i].boundary_type=1;
                break;
            }
            if(!(par_info[i].y<=domain_y/2 && par_info[i].y>=-domain_y/2))  //if particle escapes the domain
            {
                printf("1!particle escapes domain! particle number: %d\t%f\n",i,par_info[i].y);
                par_info[i].boundary_type=1;
                break;
            }
        }  //correspond to while()
    } //correspond to if(boundary==0)
}









// to judge which law should be considered, source.enthalpy and source.massflux_species should be renewed in this function
// par_info[i]:the old particle information, par_info_new: the new particle information
void COAL_LAW_SWITCH(int i, double mod_par_dt, struct cell_information *cell, struct part_infor *par_info, struct part_infor *par_info_new, struct source_information *source)
{
    int j;
    double Tem;

    if(par_info[i].coal.Tem>=Tem_vap)
    {
        if(par_info[i].coal.Mass_P[Pry_M] > 0)
        {
            if(par_info[i].coal.Tem<Tem_boiling)
            {
//                 if(i==0)
//                     printf("vaporization law!  %f\t%f\t%e\t%e\t%e\t%e\t%e\t%e\n",par_info[i].coal.Tem, cell[par_info[i].cellin].mixture.Tem, par_info[i].coal.density[4],par_info[i].coal.Mass[1],par_info[i].coal.Mass_P[Pry_M],par_info[i].coal.Mass_P[Pry_V],par_info[i].coal.Mass_P[Pry_C],par_info[i].coal.heat_capacity[4]);

                LAW_VAPORIZATION(mod_par_dt, par_info_new, cell, source);
                //                printf("vap for %dth particle!\n", i);
            }
            else if(par_info[i].coal.Tem == Tem_boiling)
            {
                if(par_info[i].coal.Tem<=cell[par_info[i].cellin].mixture.Tem)
                    LAW_BOILING(mod_par_dt, par_info_new, cell, source);
                else
                    LAW_VAPORIZATION(mod_par_dt, par_info_new, cell, source);   
            }
            else
            {
                if(par_info[i].coal.Mass_P[Pry_M]>1e-20)
                {
                    printf("par_info[%d].T weird when there is still Water! T=%f,%f, water=%e\n", i, par_info[i].coal.Tem,cell[par_info[i].cellin].mixture.Tem, par_info[i].coal.Mass_P[Pry_M]);
                    par_info[i].coal.Tem = Tem_boiling;  //reset particle temperature
                    LAW_BOILING(mod_par_dt, par_info_new, cell, source);
                }
            }
        }
        else if(par_info[i].coal.Tem < Tem_devol)
        {
//             if(i==0)
//                 printf("heating law!before devol, %f\t%f\t%e\t%e\t%e\t%e\t%e\t%e\n",par_info[i].coal.Tem, cell[par_info[i].cellin].mixture.Tem, par_info[i].coal.density[4],par_info[i].coal.Mass[1],par_info[i].coal.Mass_P[Pry_M],par_info[i].coal.Mass_P[Pry_V],par_info[i].coal.Mass_P[Pry_C],par_info[i].coal.heat_capacity[4]);

            LAW_INERT_HEATING(mod_par_dt, par_info_new, cell, source);  
        }
        else if(par_info[i].coal.Tem >= Tem_devol)
        {
            //            if(par_info[i].coal.Mass[1] > par_info[i].coal.Mass[0]*(1-coal_primary_mfr[Pry_M]-coal_primary_mfr[Pry_V]))
            if(par_info[i].coal.Mass_P[Pry_V]>0)
            {
                if(par_info[i].coal.Tem>700 && par_info[i].coal.Mass_P[Pry_C]>0)
                {
//                     if(i==0)
//                         printf("Devol&Reaction law!  %f\t%f\t%e\t%e\t%e\t%e\t%e\t%e\n",par_info[i].coal.Tem, cell[par_info[i].cellin].mixture.Tem, par_info[i].coal.density[4],par_info[i].coal.Mass[1],cell[par_info[i].cellin].mixture.Molefr[O2],par_info[i].coal.Mass_P[Pry_V],par_info[i].coal.Mass_P[Pry_C],par_info[i].coal.heat_capacity[4]);


                    LAW_DEVOL_CHAR_REACTION(mod_par_dt, par_info_new, cell, source);
                }
                else
                {
//                     if(i==0)
//                         printf("only Devolatilization law!  %f\t%f\t%e\t%e\t%e\t%e\t%e\t%e\n",par_info[i].coal.Tem, cell[par_info[i].cellin].mixture.Tem, par_info[i].coal.density[4],par_info[i].coal.Mass[1],par_info[i].coal.Mass_P[Pry_M],par_info[i].coal.Mass_P[Pry_V],par_info[i].coal.Mass_P[Pry_C],par_info[i].coal.heat_capacity[4]);

                    LAW_DEVOLATILIZATION(mod_par_dt, par_info_new, cell, source);
                }
            }
            //            else if(par_info[i].coal.Mass[1] > par_info[i].coal.Mass[0]*coal_primary_mfr[Pry_A])
            else if(par_info[i].coal.Mass_P[Pry_C] > 0)
            {
//                 if(i==800)
//                     printf("Reaction law!  %f\t%f\t%e\t%e\t%e\t%e\t%e\t%e\n",par_info[i].coal.Tem, cell[par_info[i].cellin].mixture.Tem, par_info[i].coal.density[4],par_info[i].coal.Mass[1],cell[par_info[i].cellin].mixture.Molefr[O2],par_info[i].coal.Mass_P[Pry_V],par_info[i].coal.Mass_P[Pry_C],par_info[i].coal.heat_capacity[4]);

                LAW_CHAR_REACTION(mod_par_dt, par_info_new, cell, source);
            }
            else 
            {
//                 if(i==800)
//                     printf("heating law! after reaction,  %f\t%f\t%e\t%e\t%e\t%e\t%e\t%e\n",par_info[i].coal.Tem, cell[par_info[i].cellin].mixture.Tem, par_info[i].coal.density[4],par_info[i].coal.Mass[1],par_info[i].coal.Mass_P[Pry_M],par_info[i].coal.Mass_P[Pry_V],par_info[i].coal.Mass_P[Pry_C],par_info[i].coal.heat_capacity[4]);

                LAW_INERT_HEATING(mod_par_dt, par_info_new, cell, source);
            }
        }
    }
    else
    {
//         if(i==800)
//             printf("heating law begin!  %f\t%f\t%e\t%e\t%e\t%e\t%e\t%e\n",par_info[i].coal.Tem, cell[par_info[i].cellin].mixture.Tem, par_info[i].coal.density[4],par_info[i].coal.Mass[1],par_info[i].coal.Mass_P[Pry_M],par_info[i].coal.Mass_P[Pry_V],par_info[i].coal.Mass_P[Pry_C],par_info[i].coal.heat_capacity[4]);

        LAW_INERT_HEATING(mod_par_dt, par_info_new, cell, source);
    }

    // to renew the  momentum source terms
    source[par_info[i].cellin].mom_u += par_info[i].particle_number*(par_info[i].coal.Mass[1]*par_info[i].u-par_info_new->coal.Mass[1]*par_info_new->u);
    source[par_info[i].cellin].mom_v += par_info[i].particle_number*(par_info[i].coal.Mass[1]*par_info[i].v-par_info_new->coal.Mass[1]*par_info_new->v);
    source[par_info[i].cellin].mom_w += par_info[i].particle_number*(par_info[i].coal.Mass[1]*par_info[i].w-par_info_new->coal.Mass[1]*par_info_new->w);
    source[par_info[i].cellin].mom_u += par_info[i].particle_number*(-gravity_ac*mod_par_dt*(par_info[i].coal.Mass[1]+par_info_new->coal.Mass[1])/2.0);  //to subtract the momentum increase/change according to the gravity force!
}









// the first law of the coal particles: inert particle heating
// in this function, the partice will be heated according to convection, conduction and radiation,
// the momentum/mass source term won't be calculated in this function, instead, it will be renewed in PARTICLE_TRACK_ONESTEP, since the particle scalar(state) has been calculated out. but the enthalpy source term would be renewed
void LAW_INERT_HEATING(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source)
{

    double A, B, Nu, h;
    double Tp, Tg;
    double Pr;
    double B_1, Tp_1;

    Tp = par_info->coal.Tem;
    Tg = cell[par_info->cellin].mixture.Tem;
    Pr = cell[par_info->cellin].mixture.Kinetic_Viscosity_Coeff*cell[par_info->cellin].mixture.Heat_Capacity/cell[par_info->cellin].mixture.Conduction_Coeff;                   //Pr = v/alfa = miu.cp/lambda
    Nu = 2.0 + 0.6*sqrt(par_info->Re_p)*pow(Pr,1/3);
    h = Nu*cell[par_info->cellin].mixture.Conduction_Coeff/par_info->diameter;
    
    // to modify the heat transfer coefficient according to the loading rate
    //    h *= 1/(1+cell[par_info->cellin].loading_rate[1]);

    A = -par_info->surface*(h+epsilon_p*Sigma*pow(Tp,3))/((par_info->coal.Mass[1])*(par_info->coal.heat_capacity[4]));
          
    B = -(h*Tg+epsilon_p*Sigma*pow(Tem_Rad_BG,4))/(h+epsilon_p*Sigma*pow(Tp,3));
    //    B_1 = -(h*Tg+epsilon_p*Sigma*pow(Tg,4))/(h+epsilon_p*Sigma*pow(Tp,3));
    //    printf("A=%f, B=%f, h=%f\n", A, B, h);
    //    printf("convection=%f, rad=%f, Tp=%f, Tg=%f\n", h*par_info->surface*(Tg-Tp)/par_info->coal.Mass[1]/par_info->coal.heat_capacity[4], epsilon_p*Sigma*par_info->surface*(pow(Tg,4)-pow(Tp,4))/par_info->coal.Mass[1]/par_info->coal.heat_capacity[4], Tp, Tg);

    par_info->coal.Tem = (par_info->coal.Tem+B)*exp(A*mod_par_dt)-B;
    //    Tp_1 = (Tp+B_1)*exp(A*mod_par_dt)-B_1;

    if(par_info->coal.Tem > 4800)
        printf("too high particle temperature in inert heating law! Tp=%f, Tp_new=%f, Tg=%f\n", Tp, par_info->coal.Tem, Tg);

    //    source[par_info->cellin].enthalpy += par_info->particle_number*par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-par_info->coal.Tem);

    //    source[par_info->cellin].enthalpy += par_info->particle_number*par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1);
    //    source[par_info->cellin].Enth_h += par_info->particle_number*par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1);
    source[par_info->cellin].enthalpy += par_info->particle_number*(Tp-Tg)*par_info->surface*h*mod_par_dt;  //the heat source term is only considering convective heat transfer;
    source[par_info->cellin].Enth_h += par_info->particle_number*(Tp-Tg)*par_info->surface*h*mod_par_dt;

    RENEW_COAL_PROPERTY(par_info);
}







// the second law of the coal particles: vaporization process
// in this function, the partice will be heated according to convection, conduction and radiation, the steam would be release according to vaporization process
// the momentum/mass source term won't be calculated in this function, instead, it will be renewed in PARTICLE_TRACK_ONESTEP, since the particle scalar(state) has been calculated out. but the enthalpy source term and the H2O source term would be renewed
void LAW_VAPORIZATION(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source)
{
    int k;
    double D_pv, P_sat, Nu, Sc, Pr, h, h_fg[2],H_steam;
    double Evap_1, Evap_2;
    double Pmass_new, Pmass_old, Tp_new, Tp, Tg;
    double A, B;
    double Tem[2], Pre;
    double mod_par_dt1;  //the modified time step
    double vapo_drive;
    double A_1,B_1, Tp_1;
    //    double mfr_pry[6];
    extern double Molecular_Weight[Species_Number+2];

    if(mod_par_dt==0)
        printf("p_dt weird in vaporization process!\n");  // for debug

    Pmass_old = par_info->coal.Mass[1];  // to record the original coal weight
    Tp = par_info->coal.Tem;
    Tg = cell[par_info->cellin].mixture.Tem;
    if(Tp>5000 || Tg>5000)
        printf("Temperature too high at law_vapo! Tp=%f, Tg=%f\n",Tp, Tg);
    else if(Tp<273 || Tg<273)
        printf("Temperature too low at law_vapo! Tp=%f, Tg=%f\n", Tp, Tg);
    Tem[0] = Tp;
    Tem[1] = Tg;
    Pre = cell[par_info->cellin].pressure;
    h_fg[0] = CAL_LATENT_HEAT(Tem, Pre);   // to calculate the latent heat when water become steam(from water(Tp) to steam(Tg)
    Tem[1] = Tp;
    CAL_LATENT_HEAT_H(Tem, Pre, &(h_fg[1]), &H_steam);   // to calculate the latent heat, from water(Tp) to Steam(Tp) and the enthalpy of steam (for calculation of enthalpy source term);
    Sc = cell[par_info->cellin].mixture.Kinetic_Viscosity_Coeff/(cell[par_info->cellin].mixture.Density*cell[par_info->cellin].mixture.Spe_Diff_Coeff_mix);  //Sc=v/D=miu/(rou*D)
    P_sat = exp(-0.000073375*Tp*Tp+0.09048861*Tp-25.8925454)*1e6;   // saturation pressure of steam at Tp, Pa
    Nu = 2.0+0.6*sqrt(par_info->Re_p)*pow(Sc, 1/3);
    D_pv = Nu*cell[par_info->cellin].mixture.Spe_Diff_Coeff_mix/par_info->diameter;
    Pr = cell[par_info->cellin].mixture.Kinetic_Viscosity_Coeff*cell[par_info->cellin].mixture.Heat_Capacity/cell[par_info->cellin].mixture.Conduction_Coeff;                   //Pr = v/alfa = miu.cp/lambda
    Nu = 2.0 + 0.6*sqrt(par_info->Re_p)*pow(Pr,1/3);
    h = Nu*cell[par_info->cellin].mixture.Conduction_Coeff/par_info->diameter;

    // to modify the heat transfer coefficient according to particle loading rate
    //    h *= 1/(1+cell[par_info->cellin].loading_rate[1]);

    vapo_drive = P_sat/Tp-Pre*cell[par_info->cellin].mixture.Molefr[H2O]/Tg;
    vapo_drive = vapo_drive<0?0.0:vapo_drive;  //there is no condense process 

    Evap_1 = D_pv*vapo_drive*par_info->surface*Molecular_Weight[H2O]*mod_par_dt/(1000*R);
    Evap_2 = (h*par_info->surface*(Tg-Tp)+epsilon_p*Sigma*par_info->surface*(pow(Tem_Rad_BG,4)-pow(Tp,4)))*mod_par_dt/h_fg[1];
              
    if(Evap_2 >0)
        Evap_1 = Evap_1>Evap_2?Evap_2:Evap_1;

    Evap_1 = Evap_1>0.?Evap_1:0.;

    //    vapo_drive *= 1/(1+cell[par_info->cellin].loading_rate[1]);
    //    vapo_drive *= 0.001;
    // strictly speaking, there should be another judgement: Pmass_new>=.Mass[0]*coal_primary_mfr[0]. but it doesn't matter to ommit this judgement if we can tolerate a small variation of water content. it is actually small enough to omit it.
    Pmass_new = par_info->coal.Mass[1]-Evap_1;  // to calculate the final mass weight;

    A = -par_info->surface*(h+epsilon_p*Sigma*pow(Tp,3))/(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]);
    A_1 = -par_info->surface*h/(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]);

    B = -(h*Tg+epsilon_p*Sigma*pow(Tem_Rad_BG,4)-(par_info->coal.Mass[1]-Pmass_new)*h_fg[1]/par_info->surface/mod_par_dt)/(h+epsilon_p*Sigma*pow(Tp,3));
    B_1 = -(h*Tg-(par_info->coal.Mass[1]-Pmass_new)*h_fg[1]/par_info->surface/mod_par_dt)/(h);

    par_info->coal.Tem = (par_info->coal.Tem+B)*exp(A*mod_par_dt)-B;  // to get the new particle temperature
    Tp_1 = (Tp+B_1)*exp(A_1*mod_par_dt)-B_1;

    // in case more steam yield out than steam contant
    //    if(Pmass_new < par_info->coal.Mass[0]*(1-coal_primary_mfr[Pry_M]))
    if(Pmass_old-Pmass_new > par_info->coal.Mass_P[Pry_M])
    {
        mod_par_dt1 = mod_par_dt*par_info->coal.Mass_P[Pry_M]/(par_info->coal.Mass[1]-Pmass_new);  // adjust time step
        Pmass_new = par_info->coal.Mass[1]-par_info->coal.Mass_P[Pry_M];
        par_info->coal.Tem = (par_info->coal.Tem+B)*exp(A*mod_par_dt1)-B;
        Tp_1 = (Tp+B_1)*exp(A_1*mod_par_dt1)-B_1;

        if(par_info->coal.Tem > Tem_boiling)   // final tem>Tem_boiling
        {
            mod_par_dt1 = mod_par_dt1*(Tem_boiling-Tp)/(par_info->coal.Tem-Tp);  //adjust time step a second time
            Pmass_new = par_info->coal.Mass[1]-D_pv*vapo_drive*par_info->surface*Molecular_Weight[H2O]*mod_par_dt1/(1000*R);  // to calculate the final mass weight;
            par_info->coal.Tem = Tem_boiling;
            Tp_1 = (Tp+B_1)*exp(A_1*mod_par_dt1)-B_1;
            // to renew source term
            source[par_info->cellin].mass += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            source[par_info->cellin].mixture_fr[0] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            //            source[par_info->cellin].massflux_species[H2O] += source[par_info->cellin].mass;
            source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            source[par_info->cellin].enthalpy +=  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)-(par_info->coal.Mass[1]-Pmass_new)*h_fg[0]);  //or -(par_info->coall.Mass[1]-Pmass_new)*(h_fg[1]+(h_fg[0]-h_fg[1]))

            source[par_info->cellin].Enth_h += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)-(par_info->coal.Mass[1]-Pmass_new)*h_fg[1]+(par_info->coal.Mass[1]-Pmass_new)*H_steam);  //the enthalpy source term for the enthalpy equation.

            mod_par_dt1 = mod_par_dt-mod_par_dt1;  //the rest time step
            par_info->coal.Mass[1] = Pmass_new;    // to renew particle mass

            // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
            par_info->coal.Mass_P[Pry_M] -= (Pmass_old-Pmass_new);  // to get new mass of Moisture
            for(k=0; k<4; k++)
                par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;

            RENEW_COAL_PROPERTY(par_info);
            LAW_BOILING(mod_par_dt1, par_info, cell, source);              // boiling law for rest time step
        }
        else   //final tem<=Tem_boiling
        {
            Pmass_new = par_info->coal.Mass[1]-par_info->coal.Mass_P[Pry_M];
            source[par_info->cellin].mass += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            source[par_info->cellin].mixture_fr[0] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            //            source[par_info->cellin].massflux_species[H2O] += source[par_info->cellin].mass;
            source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            //            source[par_info->cellin].enthalpy += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-par_info->coal.Tem)-(par_info->coal.Mass[1]-Pmass_new)*h_fg[0]);  //or -(par_info->coall.Mass[1]-Pmass_new)*(h_fg[1]+(h_fg[0]-h_fg[1]))
            source[par_info->cellin].enthalpy += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)-(par_info->coal.Mass[1]-Pmass_new)*h_fg[0]);  //or -(par_info->coall.Mass[1]-Pmass_new)*(h_fg[1]+(h_fg[0]-h_fg[1]))

            source[par_info->cellin].Enth_h += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)-(par_info->coal.Mass[1]-Pmass_new)*h_fg[1]+(par_info->coal.Mass[1]-Pmass_new)*H_steam);  //the enthalpy source term for the enthalpy equation.

            mod_par_dt1 = mod_par_dt-mod_par_dt1;  // the rest time step
            par_info->coal.Mass[1] = Pmass_new;

            // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
            //            par_info->coal.Mass_P[Pry_M] -= (Pmass_old-Pmass_new);  // to get new mass of Moisture
            par_info->coal.Mass_P[Pry_M] = 0.0;
            for(k=0; k<4; k++)
                par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;
            // renew coal property
            RENEW_COAL_PROPERTY(par_info);
            LAW_INERT_HEATING(mod_par_dt1, par_info, cell, source);  //inert heating for the rest step
        }
    }
    else //there is liquid water remains in coal particle after mod_par_dt
    {
        if(par_info->coal.Tem>Tem_boiling)
        {
            Pmass_new -= par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(par_info->coal.Tem-Tem_boiling)/h_fg[1];  // according to the boiling rules.
            par_info->coal.Tem = Tem_boiling;

            source[par_info->cellin].mass += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            source[par_info->cellin].mixture_fr[0] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            //            source[par_info->cellin].massflux_species[H2O] += source[par_info->cellin].mass;
            source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            source[par_info->cellin].enthalpy += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)-(par_info->coal.Mass[1]-Pmass_new)*h_fg[0]);  //or -(par_info->coall.Mass[1]-Pmass_new)*(h_fg[1]+(h_fg[0]-h_fg[1]))

            source[par_info->cellin].Enth_h += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)-(par_info->coal.Mass[1]-Pmass_new)*h_fg[1]+(par_info->coal.Mass[1]-Pmass_new)*H_steam);  //the enthalpy source term for the enthalpy equation.

            par_info->coal.Mass[1] = Pmass_new;    // to renew particle mass

            // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
            par_info->coal.Mass_P[Pry_M] -= (Pmass_old-Pmass_new);  // to get new mass of Moisture
            for(k=0; k<4; k++)
                par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;
            RENEW_COAL_PROPERTY(par_info);
        }
        else
        {
            source[par_info->cellin].mass += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            source[par_info->cellin].mixture_fr[0] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
            //            source[par_info->cellin].massflux_species[H2O] += source[par_info->cellin].mass;
            //            source[par_info->cellin].enthalpy += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-par_info->coal.Tem)-(par_info->coal.Mass[1]-Pmass_new)*h_fg[0]);  //or -(par_info->coall.Mass[1]-Pmass_new)*(h_fg[1]+(h_fg[0]-h_fg[1]))
            source[par_info->cellin].enthalpy += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)-(par_info->coal.Mass[1]-Pmass_new)*h_fg[0]);  //or -(par_info->coall.Mass[1]-Pmass_new)*(h_fg[1]+(h_fg[0]-h_fg[1]))

            source[par_info->cellin].Enth_h += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)-(par_info->coal.Mass[1]-Pmass_new)*h_fg[1]+(par_info->coal.Mass[1]-Pmass_new)*H_steam);  //the enthalpy source term for the enthalpy equation.

            par_info->coal.Mass[1] = Pmass_new;   // to record the final particle mass

            // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
            par_info->coal.Mass_P[Pry_M] -= (Pmass_old-Pmass_new);  // to get new mass of Moisture
            for(k=0; k<4; k++)
                par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;
            RENEW_COAL_PROPERTY(par_info);
        }
    }
    if((par_info->coal.Tem<293 || par_info->coal.Tem>Tem_boiling) && par_info->coal.Mass_P[Pry_M]>0.)
        printf("particle temperature weird in evaporation law! Tp=%f, Moisture=%e\n", par_info->coal.Tem, par_info->coal.Mass_P[Pry_M]);
}








// the third law of the coal particles: boiling process
// in this function, the water boiling process makes the water become steam due to convection, conduction and radiation heat transfer
// the momentum/mass source term won't be calculated in this function, instead, it will be renewed in PARTICLE_TRACK_ONESTEP, since the particle scalar(state) has been calculated out. but the enthalpy source term and the H2O source term would be renewed
void LAW_BOILING(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source)
{
    int k;
    double Pr, Nu, h, h_fg[2], H_steam;
    double Pmass_new, Pmass_old, Tp, Tg, Tem[2], Pre;
    double A, B;
    double mod_par_dt1;
    double mfr_pry[6];
    extern double Molecular_Weight[Species_Number+2];

    Tp = par_info->coal.Tem;
    Tg = cell[par_info->cellin].mixture.Tem;
    if(Tp>5000 || Tg>5000)
        printf("Temperature too high at law_vapo! Tp=%f, Tg=%f\n",Tp, Tg);
    else if(Tp<273 || Tg<273)
        printf("Temperature too low at law_vapo! Tp=%f, Tg=%f\n", Tp, Tg);
    Pmass_old = par_info->coal.Mass[1];  // to record the oritinal coal weight
    Tem[0] = Tp;
    Tem[1] = Tg;
    Pre = cell[par_info->cellin].pressure;
    h_fg[0] = CAL_LATENT_HEAT(Tem, Pre); // to calculate the higher latent heat
    Tem[1] = Tp;
    CAL_LATENT_HEAT_H(Tem, Pre, &(h_fg[1]), &H_steam); // to calculate real latent heat

    Pr = cell[par_info->cellin].mixture.Kinetic_Viscosity_Coeff*cell[par_info->cellin].mixture.Heat_Capacity/cell[par_info->cellin].mixture.Conduction_Coeff;                   //Pr = v/alfa = miu.cp/lambda
    Nu = 2.0 + 0.6*sqrt(par_info->Re_p)*pow(Pr,1/3);
    h = Nu*cell[par_info->cellin].mixture.Conduction_Coeff/par_info->diameter;
    //    h *= 1./(1.+cell[par_info->cellin].loading_rate[1]);

    Pmass_new = par_info->coal.Mass[1]-(h*par_info->surface*(Tg-Tp)+epsilon_p*Sigma*par_info->surface*(pow(Tem_Rad_BG,4)-pow(Tp,4)))*mod_par_dt/h_fg[1];

    // in case Pmass_new<.Mass[0]*(1-coal_primary_mfr[0])
    //    if(Pmass_new < par_info->coal.Mass[0]*(1-coal_primary_mfr[Pry_M]))
    if(Pmass_old-Pmass_new > par_info->coal.Mass_P[Pry_M])
    {
        mod_par_dt1 = mod_par_dt*par_info->coal.Mass_P[Pry_M]/(par_info->coal.Mass[1]-Pmass_new);
        mod_par_dt1 = mod_par_dt-mod_par_dt1;  //the rest time step
        Pmass_new = par_info->coal.Mass[1]-par_info->coal.Mass_P[Pry_M];

        source[par_info->cellin].mass += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
        source[par_info->cellin].mixture_fr[0] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
        source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
        source[par_info->cellin].enthalpy += par_info->particle_number*(Pmass_new-par_info->coal.Mass[1])*h_fg[0];
        source[par_info->cellin].Enth_h += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new)*(H_steam-h_fg[1]);
        par_info->coal.Mass[1] = Pmass_new;

        LAW_INERT_HEATING(mod_par_dt1, par_info, cell, source);
    }
    else  // in normal case
    {
        source[par_info->cellin].mass += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
        source[par_info->cellin].mixture_fr[0] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
        source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new);
        source[par_info->cellin].enthalpy += par_info->particle_number*(Pmass_new-par_info->coal.Mass[1])*h_fg[0];
        source[par_info->cellin].Enth_h += par_info->particle_number*(par_info->coal.Mass[1]-Pmass_new)*(H_steam-h_fg[1]);
        par_info->coal.Mass[1] = Pmass_new;
    }

    // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
    par_info->coal.Mass_P[Pry_M] -= (Pmass_old-Pmass_new);  // to get new mass of Moisture
    for(k=0; k<4; k++)
        par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;

    RENEW_COAL_PROPERTY(par_info);
}







// the fourth law of the coal particles: devolatilization process
// in this function, the devolatilizaiton process was predicted by the 1st order rules.
// the momentum/mass source term won't be calculated in this function, instead, it will be renewed in PARTICLE_TRACK_ONESTEP, since the particle scalar(state) has been calculated out. but the enthalpy source term and the H2O source term would be renewed
void LAW_DEVOLATILIZATION(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source)
{
    int k;
    double Pr, Nu, h, h_fv;
    double Vol_yield, Pmass_new, Pmass_old, Tp, Tg, Pre;
    double A, B;
    double mod_par_dt1;
    double Enthalpy[Species_Number], Enthalpy1[Species_Number],H_Cp[Species_Number];
    double Enthalpy_Vol, H_Vol_p, H_Vol_g; //enthalpy of volatile J/kg, but they are calculated by different ways. Enthalpy_Vol is according to h=cp*dT, H_Vol_p is the enthalpy at tp, H_Vol_g is the enthalpy at tg, Enthalpy_Vol~H_Vol_g-H_Vol_p
    double total_vol;   //vol mass fraction
    double h_vol;      // enthalpy of volatile yielding out
    double A_1,B_1, Tp_1;
    double fp_devol; // fraction of the heat release of devolatilization absorbed by particle

    extern double Molecular_Weight[Species_Number+2];

    mod_par_dt1 = mod_par_dt;
    fp_devol = CALCU_FP_DEVOL(par_info, cell);  //to calculate fp_devol

    if(fp_devol>1)
        printf("fp_devol error! fp_devol=%f\n", fp_devol);

    if(mod_par_dt == 0)
        printf("p_dt weird in devolatilization process!%f\n", mod_par_dt);  // for debug

    Tp = par_info->coal.Tem;
    Tg = cell[par_info->cellin].mixture.Tem;
    if(Tp>5000 || Tg>5000)
        printf("Temperature too high at law_devola! Tp=%f, Tg=%f\n",Tp, Tg);
    else if(Tp<273 || Tg<273)
        printf("Temperature too low at law_devola! Tp=%f, Tg=%f\n", Tp, Tg);
 
    Pre = cell[par_info->cellin].pressure;
    Pmass_old = par_info->coal.Mass[1];

    // to calculate the average heat capacity for each species. J/kg
    CALCULATE_Heat_Cp_ONECELL((Tp+Tg)/2.0,H_Cp);

    // these two should be better, which will be tested later.
    CALCULATE_ENTHALPY_ONECELL(Tp, Enthalpy);   // enthalpy at Tp
    CALCULATE_ENTHALPY_ONECELL(Tg, Enthalpy1);  // enthalpy at Tg
    Enthalpy_Vol = 0.0;   // the enthalpy used to heat volatile material from Tp to Tg
    H_Vol_p = 0.0;
    H_Vol_g = 0.0;
    // it is the average heat capacity of volatile J/(kg.K)
    for(k=0; k<Species_Number; k++)
    {
        Enthalpy_Vol += H_Cp[k]*par_info->coal.mfr_species[k];
        H_Vol_p += Enthalpy[k]*par_info->coal.mfr_species[k];
        H_Vol_g += Enthalpy1[k]*par_info->coal.mfr_species[k];
    }
    //    Enthalpy_Vol /= coal_primary_mfr[Pry_V];  // this is the real enthalpy of volatile!

    // to calculate convection heat transfer coefficient
    Pr = cell[par_info->cellin].mixture.Kinetic_Viscosity_Coeff*cell[par_info->cellin].mixture.Heat_Capacity/cell[par_info->cellin].mixture.Conduction_Coeff;                   //Pr = v/alfa = miu.cp/lambda
    Nu = 2.0 + 0.6*sqrt(par_info->Re_p)*pow(Pr,1/3);
    h = Nu*cell[par_info->cellin].mixture.Conduction_Coeff/par_info->diameter;
    // to modify heat transfer coefficient according to loading rate
    //    h *= 1./(1.+cell[par_info->cellin].loading_rate[1]);

    // to calculate the weight of volatile yielded out
    Vol_yield = DEVOL_FIRST_ORDER(mod_par_dt,Tp,Pre,par_info);
    //    Vol_yield *= 1./(1.+sqrt(cell[par_info->cellin].loading_rate[1]));
    // to get the new particle weight
    Pmass_new = Pmass_old-Vol_yield;  // to get the particle weight after mod_par_dt, the mass loss (volatile yield) is then equal to .Mass[1]-Pmass_new
    if(Pmass_new > Pmass_old)
        printf("Pmass_new wrong! dt=%f\n", mod_par_dt);  //for debug

    A = -par_info->surface*(h+epsilon_p*Sigma*pow(Tp,3))/par_info->coal.Mass[1]/par_info->coal.heat_capacity[4];
    A_1 = -par_info->surface*h/par_info->coal.Mass[1]/par_info->coal.heat_capacity[4];
    B = -(h*Tg+epsilon_p*Sigma*pow(Tem_Rad_BG,4)+fp_devol*Vol_yield*par_info->coal.heat_release[1]/par_info->surface/mod_par_dt)/(h+epsilon_p*Sigma*pow(Tp,3));
    B_1 = -(h*Tg+fp_devol*Vol_yield*par_info->coal.heat_release[1]/par_info->surface/mod_par_dt)/(h);

    total_vol = par_info->coal.mfr_species[N2]+par_info->coal.mfr_species[H2O]+par_info->coal.mfr_species[CO]+par_info->coal.mfr_species[CO2]+par_info->coal.mfr_species[C2H2]+par_info->coal.mfr_species[CH4]+par_info->coal.mfr_species[H2];  // for debug

    // when volatile that yield out is larger than volatile that remains, particle integration time step mod_par_dt is modified, the rest time step is for char reaction process
    //    if(Pmass_new < par_info->coal.Mass[0]*(1-coal_primary_mfr[Pry_M]-coal_primary_mfr[Pry_V]))
    if(Vol_yield > par_info->coal.Mass_P[Pry_V])
    {
        // to calculate the modified integration time step
        mod_par_dt1 = mod_par_dt*par_info->coal.Mass_P[Pry_V]/Vol_yield;
        if(mod_par_dt1<0)
            printf("dt weird in devol process! dt1=%f, dt=%f, Vol1=%e, Vol2=%e\n", mod_par_dt1, mod_par_dt, par_info->coal.Mass_P[Pry_V], Vol_yield);

        Vol_yield = par_info->coal.Mass_P[Pry_V];
        Pmass_new = Pmass_old-Vol_yield;
        B = -(h*Tg+epsilon_p*Sigma*pow(Tem_Rad_BG,4)+fp_devol*Vol_yield*par_info->coal.heat_release[1]/par_info->surface/mod_par_dt1)/(h+epsilon_p*Sigma*pow(Tp,3));
        B_1 = -(h*Tg+fp_devol*Vol_yield*par_info->coal.heat_release[1]/par_info->surface/mod_par_dt1)/(h);
        par_info->coal.Tem = (Tp+B)*exp(A*mod_par_dt1)-B;
        Tp_1 = (Tp+B_1)*exp(A_1*mod_par_dt1)-B_1;
//        // to renew species source term (universal form)
//         for(k=0; k<Species_Number; k++)
//             source[par_info->cellin].massflux_species[k] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[k]/(coal_primary_mfr[Pry_V];

        // to renew species source term (just for this work)
        source[par_info->cellin].mass += par_info->particle_number*Vol_yield;
        source[par_info->cellin].mixture_fr[1] += par_info->particle_number*Vol_yield;
        source[par_info->cellin].massflux_species[N2] += par_info->particle_number*(Vol_yield)*par_info->coal.mfr_species[N2]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(Vol_yield)*par_info->coal.mfr_species[H2O]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[CO] += par_info->particle_number*(Vol_yield)*par_info->coal.mfr_species[CO]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[CO2] += par_info->particle_number*(Vol_yield)*par_info->coal.mfr_species[CO2]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[C2H2] += par_info->particle_number*(Vol_yield)*par_info->coal.mfr_species[C2H2]/coal_primary_mfr[Pry_V];
//         if(source[par_info->cellin].massflux_species[C2H2]<0)
//            printf("S.C2H2 weird! %f, %f, %f\n", source[par_info->cellin].massflux_species[C2H2],par_info->particle_number*(Vol_yield)*par_info->coal.mfr_species[C2H2]/coal_primary_mfr[Pry_V],(Vol_yield));  //for debug
        source[par_info->cellin].massflux_species[CH4] += par_info->particle_number*(Vol_yield)*par_info->coal.mfr_species[CH4]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[H2] += par_info->particle_number*(Vol_yield)*par_info->coal.mfr_species[H2]/coal_primary_mfr[Pry_V];


        if(fabs(total_vol-coal_primary_mfr[Pry_V])>1e-6)
            printf("coal composition weird! in devolatilization process!%f\n", fabs(total_vol-coal_primary_mfr[Pry_V])*1e6);  //for debug!

        Enthalpy_Vol *= Vol_yield*(Tp-Tg)/coal_primary_mfr[Pry_V];
        H_Vol_p *= Vol_yield/coal_primary_mfr[Pry_V];
        H_Vol_g *= Vol_yield/coal_primary_mfr[Pry_V];

        if(((Enthalpy_Vol+H_Vol_g-H_Vol_p)/Enthalpy_Vol) > 1e-1)
            printf("large difference for H_Vol calculation: Cpdt=%e, Hp=%e, Hg=%e, err=%f\n", Enthalpy_Vol, H_Vol_p, H_Vol_g, (Enthalpy_Vol+H_Vol_g-H_Vol_p)/Enthalpy_Vol);

        // it is a negative value, represent that Enthalpy_Vol J is needed to heat (par_info->coal.Mass[1]-Pmass_new)kg volatil from Tp to Tg

        //        h_vol =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-par_info->coal.Tem)+Vol_yield*par_info->coal.heat_release[1]+Enthalpy_Vol);
        h_vol =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+Vol_yield*par_info->coal.heat_release[1]+(H_Vol_p-H_Vol_g));
        if(fabs(h_vol)>6e4)
        {
            printf("hvol weird! h_vol=%f, par.Tem=%f, %f, %f, Tem=%f, Enthalpy_vol=%e\n", h_vol, par_info->coal.Tem, Tp, Tp_1, cell[par_info->cellin].mixture.Tem, Enthalpy_Vol);
            printf("dt=%e, dt1=%e, Vol_yield=%e, V=%e\n", mod_par_dt, mod_par_dt1, Vol_yield, par_info->coal.Mass_P[Pry_V]);
            printf("A=%e, B=%e\n",A, B);
        }

        //to renew the enthalpy source term
        source[par_info->cellin].enthalpy += h_vol;
        source[par_info->cellin].Enth_h += (h_vol+par_info->particle_number*H_Vol_g);

        if(fabs(source[par_info->cellin].enthalpy)>4e5)
            printf("s.h weird in devol process!1, s.h=%f, %f\n", source[par_info->cellin].enthalpy, h_vol);  //for debug

        par_info->coal.Mass[1] = Pmass_new;    // to renew the particle mass, now the coal is just composed of char and ash

        // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
        par_info->coal.Mass_P[Pry_V] -= Vol_yield;  // to get new mass of Moisture
        for(k=0; k<4; k++)
            par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;

        RENEW_COAL_PROPERTY(par_info);

        mod_par_dt1 = mod_par_dt-mod_par_dt1;  // the rest time step;
        //for the rest time step, to consider char reaction laws.
        if(par_info->coal.Mass_P[Pry_C]>0)
            LAW_CHAR_REACTION(mod_par_dt1, par_info, cell, source);
        else
            LAW_INERT_HEATING(mod_par_dt1, par_info, cell, source);
    }
    else
    {
        par_info->coal.Tem = (Tp+B)*exp(A*mod_par_dt)-B;
        Tp_1 = (Tp+B_1)*exp(A_1*mod_par_dt)-B_1;

        source[par_info->cellin].mass += par_info->particle_number*Vol_yield;
        source[par_info->cellin].mixture_fr[1] += par_info->particle_number*Vol_yield;

//        // to renew species source term (universal form)
//         for(k=0; k<Species_Number; k++)
//             source[par_info->cellin].massflux_species[k] += Vol_yield*par_info->coal.mfr_species[k]/(coal_primary_mfr[Pry_V];

        // to renew species source term (just for this work)
        source[par_info->cellin].massflux_species[N2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[N2]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[H2O]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[CO] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CO]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[CO2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CO2]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[C2H2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[C2H2]/coal_primary_mfr[Pry_V];
//         if(source[par_info->cellin].massflux_species[C2H2]<0)
//             printf("S.C2H2 weird! %f, %f, %f\n", source[par_info->cellin].massflux_species[C2H2],par_info->particle_number*Vol_yield*par_info->coal.mfr_species[C2H2]/coal_primary_mfr[Pry_V], par_info->coal.Mass[1]-Pmass_new);  //for debug
        source[par_info->cellin].massflux_species[CH4] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CH4]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[H2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[H2]/coal_primary_mfr[Pry_V];

        if(fabs(total_vol-coal_primary_mfr[Pry_V])>1e-6)
            printf("coal composition weird! in devolatilization process!2 %f \n", fabs(total_vol-coal_primary_mfr[Pry_V])*1e6);  //for debug!

        Enthalpy_Vol *= (Vol_yield*(Tp-Tg)/coal_primary_mfr[Pry_V]);
        H_Vol_p *= Vol_yield/coal_primary_mfr[Pry_V];
        H_Vol_g *= Vol_yield/coal_primary_mfr[Pry_V];
        if(((Enthalpy_Vol+H_Vol_g-H_Vol_p)/Enthalpy_Vol) > 1e-1)
            printf("large difference for H_Vol calculation: Cpdt=%e, Hp=%e, Hg=%e, err=%f\n", Enthalpy_Vol, H_Vol_p, H_Vol_g, (Enthalpy_Vol+H_Vol_g-H_Vol_p)/Enthalpy_Vol);

        // it is a negative value, represent that Enthalpy_Vol J is needed to heat Vol_yieldkg volatil from Tp to Tg

        //        h_vol =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-par_info->coal.Tem)+Vol_yield*par_info->coal.heat_release[1]+Enthalpy_Vol);
        h_vol =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+Vol_yield*par_info->coal.heat_release[1]+(H_Vol_p-H_Vol_g));

        if(fabs(h_vol)>6e4)
        {
            printf("hvol weird!2 h_vol=%f, par.Tem=%f, %f, %f, Tem=%f, Enthalpy_vol=%e\n", h_vol, par_info->coal.Tem, Tp, Tp_1, cell[par_info->cellin].mixture.Tem, Enthalpy_Vol);
            printf("dt=%e, dt1=%e, Vol_yield=%e, V=%e\n", mod_par_dt, mod_par_dt1, Vol_yield, par_info->coal.Mass_P[Pry_V]);
            printf("A=%e, B=%e, fp_devol=%e\t", A, B, fp_devol);
            printf("heat_relaease=%e, h=%e, Enthalpy=%e\n", par_info->coal.heat_release[1], h, Enthalpy_Vol/Vol_yield/(Tp-Tg));
        }


        //to renew the enthalpy source term
        source[par_info->cellin].enthalpy += h_vol;
        source[par_info->cellin].Enth_h += (h_vol+par_info->particle_number*H_Vol_g);

        if(fabs(source[par_info->cellin].enthalpy)>4e5)
            printf("s.h weird in devol process!2. s.h=%f, %f\n", source[par_info->cellin].enthalpy, h_vol);  //for debug

        par_info->coal.Mass[1] = Pmass_new;    // to renew the particle mass, now the coal is just composed of char and ash

        // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
        par_info->coal.Mass_P[Pry_V] -= Vol_yield;  // to get new mass of Moisture
        for(k=0; k<4; k++)
            par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;
        RENEW_COAL_PROPERTY(par_info);
    }
}







// to calculate devolatilization rate according to the first order rules, the returned value is the weight of volatile that yield out
// in order to consider the pressurized environment, Pre was also introduced although in this code it has no effect.
double DEVOL_FIRST_ORDER(double mod_par_dt, double Tp, double Pre, struct part_infor *par_info)
{
    double A_v, E_v, A, B;
    double Pmass_new;
    double Vol_yield;
    double tiny = 1e-8;

    A_v = 52000;   //pre-exponential factor, 1/s
    E_v = 2.58e4;   //activation energy of devolatilization process, J/mol
    A = -A_v*exp(-E_v/(R*(Tp>1500?1500:Tp)));
    //    B = -(par_info->coal.Mass_P[2]+par_info->coal.Mass_P[3]);
    //    Pmass_new = (par_info->coal.Mass[1]+B)*exp(A*mod_par_dt)-B-tiny*par_info->coal.Mass[0];
    Pmass_new = par_info->coal.Mass_P[Pry_V]*exp(A*mod_par_dt)-tiny*par_info->coal.Mass[0];  // new volatile after mod_par_dt
    Vol_yield = par_info->coal.Mass_P[Pry_V]-Pmass_new;
    if(Pmass_new > par_info->coal.Mass_P[Pry_V])
        printf("devol weird!\n");
    // to subject a very small value so that Pmass_new>0 any time
    return(Vol_yield);  //the final particle weight after mod_par_dt s
}










// the fifth law of the coal particles: heterogeneous char reaction process
// in this function, the heterogeneous char reaction model is based on the empirical nth law to consider the pressure effect.
// the momentum/mass source term won't be calculated in this function, instead, it will be renewed in PARTICLE_TRACK_ONESTEP, since the particle scalar(state) has been calculated out. but the enthalpy source term and the H2O source term would be renewed
void LAW_CHAR_REACTION(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source)
{
    int k;
    double Tp, Tg, Tem, Pre;
    double mod_par_dt1;  // the modified time step
    double Char_consume[4],Heat_release[4];  //carbon weight loss and heat release due to each reaction
    //    double f_p=0.5;  //what part of heat release was absorbed by char particle
    double Tem_reaction[4];  //used to calculate Heat_release[4]
    double Pmass_new,Pmass_old, C_consume_total=0.0;  //total carbon consumption during this time step
    double h, Nu, Pr, A, B;        // for heat transfer
    double h_reaction;             // for source term
    double Enthalpy_reaction=0;  //Enthalpy_reaction = f_p*sum(Ri*Hri)
    double H_Cs;                // enthalpy of solid carbon at Tp, J/kg
    double A_1, Tp_1, B_1;
    double fp_reaction;    // fraction of the heat release of char reactions absorbed by particle
    extern double Molecular_Weight[Species_Number+2];


    if(!(mod_par_dt > 0))
        printf("p_dt weird in reaction process!%f\n", mod_par_dt);  // for debug

    //initialize
    Pmass_old = par_info->coal.Mass[1];
    mod_par_dt1 = mod_par_dt;
    Tp = par_info->coal.Tem;
    Tg = cell[par_info->cellin].mixture.Tem;
    if(Tp>5000 || Tg>5000)
        printf("Temperature too high at law_char_reaction! Tp=%f, Tg=%f\n",Tp, Tg);
    else if(Tp<273 || Tg<273)
        printf("Temperature too low at law_char_reaction! Tp=%f, Tg=%f\n", Tp, Tg);

    Pre = cell[par_info->cellin].pressure;
    Tem = (Tp+Tg)/2.0;  //reaction temperature
    //    for(k=0; k<4; k++)
    //  Char_consume[k] = 0.0;   // initial char consumption is 0.0

    // to calculate carbon consumption according to each reaction
    CHAR_REACTION_EMPIRICAL(mod_par_dt, Tem, Pre, Char_consume, par_info, cell);
    // to modify the reaction rate according to the mass loading_rate of particle
//     for(k=0; k<4; k++)
//         Char_consume[k] *= 1/(1+cell[par_info->cellin].loading_rate[1]);
    // to calculate total carbon weight loss in mod_par_dt
    C_consume_total = 0.;
    for(k=0; k<4; k++)
        C_consume_total += Char_consume[k];
    Pmass_new = Pmass_old-C_consume_total;  // the final particle weight

    if(!(C_consume_total>0))
    {
        printf("Char_reaction weird in law_reaction_1!Tem=%f, Pre=%f, dt=%e, Mass_old=%e, C_0=%e, C_1=%e, C_2=%e, C_3=%e\n", Tem, Pre, mod_par_dt, Pmass_old, Char_consume[0], Char_consume[1], Char_consume[2],Char_consume[3]);
        printf("cell info: cellin=%d, M_O2=%e, M_CO2=%e, M_H2O=%e, M_H=%e, dp=%e\n", par_info->cellin, cell[par_info->cellin].mixture.Molefr[O2],cell[par_info->cellin].mixture.Molefr[CO2],cell[par_info->cellin].mixture.Molefr[H2O],cell[par_info->cellin].mixture.Molefr[H2],par_info->coal.diameter);
    }

    // for convective heat transfer
    Pr = cell[par_info->cellin].mixture.Kinetic_Viscosity_Coeff*cell[par_info->cellin].mixture.Heat_Capacity/cell[par_info->cellin].mixture.Conduction_Coeff;                   //Pr = v/alfa = miu.cp/lambda
    Nu = 2.0 + 0.6*sqrt(par_info->Re_p)*pow(Pr,1/3);
    h = Nu*cell[par_info->cellin].mixture.Conduction_Coeff/par_info->diameter;
    // to modify heat transfer coefficient according to the mass loading rate
    //    h *= 1/(1+cell[par_info->cellin].loading_rate[1]);

    A = -par_info->surface*(h+epsilon_p*Sigma*pow(Tp,3))/par_info->coal.Mass[1]/par_info->coal.heat_capacity[4];
    A_1 = -par_info->surface*h/par_info->coal.Mass[1]/par_info->coal.heat_capacity[4];

    // to calculate enthalpy of solid carbon at Tp J/kg
    H_Cs = CAL_ENTHALPY_CS(Tp, Pre);

    // to set surface temperature for each reaction
    Tem_reaction[0] = Tp;
    Tem_reaction[1] = Tg;
    Tem_reaction[2] = Tg;
    Tem_reaction[3] = Tg;

    // to calculate heat release for each reaction, J/kg
    Heat_release[0] = CAL_HEAT_CARBON_OXYGEN(Tem_reaction, Pre);
    Heat_release[1] = CAL_HEAT_CARBON_CO2(Tem_reaction, Pre);
    Heat_release[2] = CAL_HEAT_CARBON_STEAM(Tem_reaction, Pre);
    Heat_release[3] = CAL_HEAT_CARBON_CH4(Tem_reaction, Pre);

    // to calculate the total heat release of char in mod_par_dt J/kg
    Enthalpy_reaction = 0.0;
    for(k=0; k<4; k++)
        Enthalpy_reaction += Char_consume[k]*Heat_release[k];

    fp_reaction = CALCU_FP_REACTION(par_info, cell, Enthalpy_reaction);

    // for heat reaction term in B equation
    Enthalpy_reaction = Enthalpy_reaction*fp_reaction/mod_par_dt/par_info->surface;

    B = -(h*Tg+epsilon_p*Sigma*pow(Tem_Rad_BG,4)+Enthalpy_reaction)/(h+epsilon_p*Sigma*pow(Tp,3));
    B_1 = -(h*Tg+Enthalpy_reaction)/(h);

    // when carbon loss is larger than the carbon remains, time step is adjusted, the rest time step= char_remains/C_consume_total.
    //    if(Pmass_new < par_info->coal.Mass[0]*coal_primary_mfr[Pry_A])
    if(C_consume_total > par_info->coal.Mass_P[Pry_C])
    {
        mod_par_dt1 = mod_par_dt*par_info->coal.Mass_P[Pry_C]/C_consume_total;
        if(!(mod_par_dt1>0))
            printf("time weird in law_char_reaction!_dt1_adjust=%e, C=%e,C_consumption=%e, dt=%e\n",mod_par_dt1,par_info->coal.Mass_P[Pry_C], C_consume_total,mod_par_dt);
        if(mod_par_dt1 == 0 )
            mod_par_dt1 = 1e-20;

        Pmass_new = Pmass_old-par_info->coal.Mass_P[Pry_C];  // the final particle weight

        par_info->coal.Tem = (Tp+B)*exp(A*mod_par_dt1)-B;   // to calculate final temperature
        Tp_1 = (Tp+B_1)*exp(A_1*mod_par_dt1)-B_1; 

        source[par_info->cellin].mass += par_info->particle_number*par_info->coal.Mass_P[Pry_C];
        source[par_info->cellin].mixture_fr[2] += par_info->particle_number*par_info->coal.Mass_P[Pry_C];

        source[par_info->cellin].massflux_species[O2] -= par_info->particle_number*Char_consume[0]*Molecular_Weight[O2]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;
        source[par_info->cellin].massflux_species[CO2] += par_info->particle_number*(Char_consume[0]-Char_consume[1])*Molecular_Weight[CO2]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;
        source[par_info->cellin].massflux_species[CO] += par_info->particle_number*(2*Char_consume[1]+Char_consume[2])*Molecular_Weight[CO]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;
        source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(-Char_consume[2])*Molecular_Weight[H2O]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;
        source[par_info->cellin].massflux_species[H2] += par_info->particle_number*(Char_consume[2]-2*Char_consume[3])*Molecular_Weight[H2]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;
        source[par_info->cellin].massflux_species[CH4] += par_info->particle_number*Char_consume[3]*Molecular_Weight[CH4]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;

        //        h_reaction =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-par_info->coal.Tem)+(fp_reaction==0?0.0:Enthalpy_reaction*mod_par_dt1*par_info->surface/fp_reaction));
        h_reaction =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+(fp_reaction==0?0.0:Enthalpy_reaction*mod_par_dt1*par_info->surface/fp_reaction));
        source[par_info->cellin].enthalpy += h_reaction;
        source[par_info->cellin].Enth_h += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+H_Cs*par_info->coal.Mass_P[Pry_C]);

        if(fabs(source[par_info->cellin].enthalpy)>4e5)
            printf("s.h weird in reaction process!1. s.h=%f, %f\n", source[par_info->cellin].enthalpy, h_reaction);  //for debug

        par_info->coal.Mass[1] = Pmass_new;

        // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
        par_info->coal.Mass_P[Pry_C] = 0.0;  // to get new mass of Char
        for(k=0; k<4; k++)
            par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;
        RENEW_COAL_PROPERTY(par_info);

        mod_par_dt1 = mod_par_dt-mod_par_dt1;   // the rest time step
        LAW_INERT_HEATING(mod_par_dt1, par_info, cell, source);
    }
    else
    {
        par_info->coal.Tem = (Tp+B)*exp(A*mod_par_dt)-B;   // to calculate final temperature
        Tp_1 = (Tp+B_1)*exp(A_1*mod_par_dt)-B_1;

        source[par_info->cellin].mass += par_info->particle_number*C_consume_total;
        source[par_info->cellin].mixture_fr[2] += par_info->particle_number*C_consume_total;
        source[par_info->cellin].massflux_species[O2] -= par_info->particle_number*Char_consume[0]*Molecular_Weight[O2]/Molecular_Weight[C_S];
        source[par_info->cellin].massflux_species[CO2] += par_info->particle_number*(Char_consume[0]-Char_consume[1])*Molecular_Weight[CO2]/Molecular_Weight[C_S];
        source[par_info->cellin].massflux_species[CO] += par_info->particle_number*(2*Char_consume[1]+Char_consume[2])*Molecular_Weight[CO]/Molecular_Weight[C_S];
        source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(-Char_consume[2])*Molecular_Weight[H2O]/Molecular_Weight[C_S];
        source[par_info->cellin].massflux_species[H2] += par_info->particle_number*(Char_consume[2]-2*Char_consume[3])*Molecular_Weight[H2]/Molecular_Weight[C_S];
        source[par_info->cellin].massflux_species[CH4] += par_info->particle_number*Char_consume[3]*Molecular_Weight[CH4]/Molecular_Weight[C_S];

        //        h_reaction =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-par_info->coal.Tem)+fp_reaction==0.0?0.0:Enthalpy_reaction*mod_par_dt*par_info->surface/fp_reaction);
        h_reaction =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+fp_reaction==0.0?0.0:Enthalpy_reaction*mod_par_dt*par_info->surface/fp_reaction);

        source[par_info->cellin].enthalpy += h_reaction;
        source[par_info->cellin].Enth_h += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+H_Cs*C_consume_total);

        if(fabs(source[par_info->cellin].enthalpy)>4e5)
            printf("s.h weird in reaction process!2. s.h=%f, %f\n", source[par_info->cellin].enthalpy, h_reaction);  //for debug

        par_info->coal.Mass[1] = Pmass_new;

        // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
        par_info->coal.Mass_P[Pry_C] -= C_consume_total;  // to get new mass of Moisture

        if(!(par_info->coal.Mass_P[Pry_C]>0))
        printf("Char_reaction weird in law_reaction_2!Tem=%f, dt=%e, P_C=%e\n", Tem, mod_par_dt, par_info->coal.Mass_P[Pry_C]);

        for(k=0; k<4; k++)
            par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;
        RENEW_COAL_PROPERTY(par_info);
    }
}







// this function is used to calculate carbon consumption during time step mod_par_dt according to the empirical nth law.
// the reaction rates are calculated in this model. the carbon consumption in each reaction after mod_par_dt are identified and stored in Char_consume[], which is the information provided by this function
void CHAR_REACTION_EMPIRICAL(double mod_par_dt, double Tem, double Pre, double *Char_consume, struct part_infor *par_info, struct cell_information *cell)
{
    int i;
    double Tem_gas, Tem_par;
    double R_t[4],R_diff[4], R_che[4];  //reaction rate, diffusion reaction rate and chemical reaction rate for each heterogeneous reaction
    double C_diff[4];  //gas diffusion coefficient for each reaction
    double A_reaction[4] = {7000, 14240, 82.5, 3.62};        //pre-exponential coefficient, kg/(m^2.s.atm^n)
    double E_reaction[4] = {1.15e5, 2.1e5, 1.42e5, 1.5e5};  // activation energy, J/mol
    double n[4] = {0.65, 0.6, 0.4, 1.0};                   // nth order for each reaction


    if(!(mod_par_dt>0) || !(Tem>0))
        printf("dt weird in law_char_rec! dt=%e, Tem=%e\n", mod_par_dt, Tem);
    for(i=0; i<4; i++)
        C_diff[i] = 5e-12;        //s/K^0.75

    Tem_gas = Tem>3000?3000:Tem;
    Tem_par = par_info->coal.Tem>3000?3000:par_info->coal.Tem;

    // reaction rate due to diffusion/chemical kinetics process for each reaction
    R_diff[0] = C_diff[0]*pow(Tem_gas,0.75)*Pre*cell[par_info->cellin].mixture.Molefr[O2]/par_info->coal.diameter;
    R_che[0] = A_reaction[0]*exp(-E_reaction[0]/R/Tem_par)*pow((cell[par_info->cellin].mixture.Molefr[O2]*Pre/1e5),Pre>1.1e5?n[0]:1.)*pow((1-cell[par_info->cellin].mixture.Molefr[CO2]),1/2);  //to consider the negative effect of CO2

    R_diff[1] = C_diff[1]*pow(Tem_gas,0.75)*Pre*cell[par_info->cellin].mixture.Molefr[CO2]/par_info->coal.diameter;
    R_che[1] = A_reaction[1]*exp(-E_reaction[1]/R/Tem_par)*pow((cell[par_info->cellin].mixture.Molefr[CO2]*Pre/1e5),Pre>1.1e5?n[1]:1.)*pow((1-cell[par_info->cellin].mixture.Molefr[O2]),1.5);  //to consider the negative effect of O2

    R_diff[2] = C_diff[2]*pow(Tem_gas,0.75)*Pre*cell[par_info->cellin].mixture.Molefr[H2O]/par_info->coal.diameter;
    R_che[2] = A_reaction[2]*exp(-E_reaction[2]/R/Tem_par)*pow((cell[par_info->cellin].mixture.Molefr[H2O]*Pre/1e5),Pre>1.1e5?n[2]:1.)*pow((1-cell[par_info->cellin].mixture.Molefr[O2]),1.5);

    R_diff[3] = C_diff[3]*pow(Tem_gas,0.75)*Pre*cell[par_info->cellin].mixture.Molefr[H2]/par_info->coal.diameter;
    R_che[3] = A_reaction[3]*exp(-E_reaction[3]/R/Tem_par)*pow((cell[par_info->cellin].mixture.Molefr[H2]*Pre/1e5),Pre>1.1e5?n[3]:1.)*pow((1-cell[par_info->cellin].mixture.Molefr[O2]),2.);

    // to calculate the total reaction rate and the char loss weight due to each reaction in mod_par_dt
    for(i=0; i<4; i++)
    {
        R_t[i] = (R_diff[i]+R_che[i])>0.?R_diff[i]*R_che[i]/(R_diff[i]+R_che[i]):0.0;
        //R_t[i] = R_diff[i]*R_che[i]/(R_diff[i]+R_che[i]);

        Char_consume[i] = par_info->surface*R_t[i]*mod_par_dt;
    }
}








// the sixth law of the coal particles: devolatilization with heterogeneous char reaction process
// in this function, the heterogeneous char reaction model is based on the empirical nth law to consider the pressure effect. the devolatilization law is based on 1st order model
// the momentum/mass source term won't be calculated in this function, instead, it will be renewed in PARTICLE_TRACK_ONESTEP, since the particle scalar(state) has been calculated out. but the enthalpy source term and the H2O source term would be renewed
void LAW_DEVOL_CHAR_REACTION(double mod_par_dt, struct part_infor *par_info, struct cell_information *cell, struct source_information *source)
{
    int k;
    double Tp, Tg, Tem, Pre;
    double mod_par_dt1;  // the modified time step
    double Vol_yield, total_vol;     // the weight of volatile yield out in mod_par_dt  and vol massfr
    double Char_consume[4],Heat_release[4];  //carbon weight loss and heat release due to each reaction
    //    double f_p=0.5;  //what part of heat release was absorbed by char particle
    double Tem_reaction[4];
    double Pmass_new,Pmass_old, C_consume_total=0.0;  //total carbon weight loss in mod_par_dt
    double h, Nu, Pr, A, B;  // for convection heat transfer
    double Enthalpy[Species_Number], Enthalpy1[Species_Number],H_Cp[Species_Number];  // for vol properties
    double h_reaction, h_fv;  //heat source of reaction and devolatilization
    double Enthalpy_reaction=0, Enthalpy_Vol,H_Vol_g, H_Vol_p, H_Cs,  h_vol; //Enthalpy_reaction = f_p*sum(Ri*Hri)
    double t1, t2;   //the modified time step 
    double A_1, B_1, Tp_1;
    double fp_reaction;    // fraction of the heat release of char reactions absorbed by particle
    double fp_devol; // fraction of the heat release of devolatilization absorbed by particle
    extern double Molecular_Weight[Species_Number+2];

    fp_devol = CALCU_FP_DEVOL(par_info, cell);

    if(!(mod_par_dt>0))
        printf("p_dt weird in Vol&Rea step! dt=%e\n",mod_par_dt);  //for debug

    //initialize
    Pmass_old = par_info->coal.Mass[1];
    mod_par_dt1 = mod_par_dt;
    Tp = par_info->coal.Tem;
    Tg = cell[par_info->cellin].mixture.Tem;
    if(Tp>5000 || Tg>5000)
        printf("Temperature too high at law_devola_char_reac! Tp=%f, Tg=%f\n",Tp, Tg);
    else if(Tp<273 || Tg<273)
        printf("Temperature too low at law_devola_char_reac! Tp=%f, Tg=%f\n", Tp, Tg);

    Pre = cell[par_info->cellin].pressure;
    Tem = (Tp+Tg)/2.0;  //reaction temperature

    if(Tp>5000 || Tg>5000)
        printf("Too high temperature in char reaction, Tp=%f, Tg=%f\n", Tp, Tg);
    if(Tp<273 || Tg<273)
        printf("Too low tempreature in char reaction, Tp=%f, Tg=%f\n", Tp, Tg);

    //    for(k=0; k<4; k++)
    //  Char_consume[k] = 0.0;   // initial char consumption is 0.0

    // to calculate the average heat capacity for each species in volatile. J/kg
    CALCULATE_Heat_Cp_ONECELL((Tp+Tg)/2.0,H_Cp);

    // these two should be better, which will be tested later.
    CALCULATE_ENTHALPY_ONECELL(Tp, Enthalpy);   // enthalpy at Tp
    CALCULATE_ENTHALPY_ONECELL(Tg, Enthalpy1);  // enthalpy at Tg
    Enthalpy_Vol = 0.0;   // the enthalpy used to heat volatile material from Tp to Tg
    H_Vol_g = 0.0;
    H_Vol_p = 0.0;
    // it is the average heat capacity of volatile J/(kg.K)
    for(k=0; k<Species_Number; k++)
    {
        Enthalpy_Vol += H_Cp[k]*par_info->coal.mfr_species[k];
        H_Vol_g += Enthalpy1[k]*par_info->coal.mfr_species[k];
        H_Vol_p += Enthalpy[k]*par_info->coal.mfr_species[k];
    }
    //    Enthalpy_Vol /= coal_primary_mfr[Pry_V];  // this is the real enthalpy of volatile!
    H_Cs = CAL_ENTHALPY_CS(Tp, Pre);  // to calculate enthalpy of solid carbon at Tp, J/kg

    Vol_yield = DEVOL_FIRST_ORDER(mod_par_dt,Tp,Pre,par_info);  // to get the weight of volatile yield out after mod_par_dt
    // to modify voltile yield rate according to the mass loading rate of the particles
    //    Vol_yield *= 1/(1+sqrt(cell[par_info->cellin].loading_rate[1]));
    // to calculate carbon consumption according to each reaction
    CHAR_REACTION_EMPIRICAL(mod_par_dt, Tem, Pre, Char_consume, par_info, cell);
    // to modify char reaction rate according to the mass loading rate of the particles
//     for(k=0; k<4; k++)
//         Char_consume[k] *= 1./(1.+cell[par_info->cellin].loading_rate[1]);

    // to calculate total carbon weight loss
    C_consume_total = 0.0;
    for(k=0; k<4; k++)
        C_consume_total += Char_consume[k];

    if(!(C_consume_total>0))
        printf("Char_reaction weird in law_dev&char_1!Tem=%f, dt=%e\n", Tem, mod_par_dt);

    //for convection heat transfer
    Pr = cell[par_info->cellin].mixture.Kinetic_Viscosity_Coeff*cell[par_info->cellin].mixture.Heat_Capacity/cell[par_info->cellin].mixture.Conduction_Coeff;                   //Pr = v/alfa = miu.cp/lambda
    Nu = 2.0 + 0.6*sqrt(par_info->Re_p)*pow(Pr,1/3);
    h = Nu*cell[par_info->cellin].mixture.Conduction_Coeff/par_info->diameter;
    // to modify heat transfer coefficient according to the mass loading rate
    //    h *= 1/(1+cell[par_info->cellin].loading_rate[1]);

    A = -par_info->surface*(h+epsilon_p*Sigma*pow(Tp,3))/par_info->coal.Mass[1]/par_info->coal.heat_capacity[4];
    A_1 = -par_info->surface*(h)/par_info->coal.Mass[1]/par_info->coal.heat_capacity[4];
    // total volatile massfr
    total_vol = par_info->coal.mfr_species[N2]+par_info->coal.mfr_species[H2O]+par_info->coal.mfr_species[CO]+par_info->coal.mfr_species[CO2]+par_info->coal.mfr_species[C2H2]+par_info->coal.mfr_species[CH4]+par_info->coal.mfr_species[H2];  // for debug

    // to set surface temperature for each reaction
    Tem_reaction[0] = Tp;
    Tem_reaction[1] = Tg;
    Tem_reaction[2] = Tg;
    Tem_reaction[3] = Tg;

    // to calculate heat release for each reaction, J/kg
    Heat_release[0] = CAL_HEAT_CARBON_OXYGEN(Tem_reaction, Pre);
    Heat_release[1] = CAL_HEAT_CARBON_CO2(Tem_reaction, Pre);
    Heat_release[2] = CAL_HEAT_CARBON_STEAM(Tem_reaction, Pre);
    Heat_release[3] = CAL_HEAT_CARBON_CH4(Tem_reaction, Pre);

    Enthalpy_reaction = 0.0;
    for(k=0; k<4; k++)
        Enthalpy_reaction += Char_consume[k]*Heat_release[k];

    fp_reaction = CALCU_FP_REACTION(par_info, cell, Enthalpy_reaction);    
    Enthalpy_reaction = Enthalpy_reaction*fp_reaction/mod_par_dt/par_info->surface;

    B =  -(h*Tg+epsilon_p*Sigma*pow(Tem_Rad_BG,4)+fp_devol*Vol_yield*par_info->coal.heat_release[1]/par_info->surface/mod_par_dt+Enthalpy_reaction)/(h+epsilon_p*Sigma*pow(Tp,3));
    B_1 =  -(h*Tg+fp_devol*Vol_yield*par_info->coal.heat_release[1]/par_info->surface/mod_par_dt+Enthalpy_reaction)/(h);

    // when carbon weight loss is larger than carbon remains, or volatile that yields out is larger than volatile remains, time step is adjusted, the rest time step is set as oterh laws.
    //    if(Pmass_new < par_info->coal.Mass[0]*coal_primary_mfr[Pry_A])
    if(Vol_yield>par_info->coal.Mass_P[Pry_V] || C_consume_total>par_info->coal.Mass_P[Pry_C])
    {
        t1 = mod_par_dt*par_info->coal.Mass_P[Pry_V]/Vol_yield;
        t2 = mod_par_dt*par_info->coal.Mass_P[Pry_C]/C_consume_total;
        if((!(t1>0)) || (!(t2>0)))
           printf("adjust t weird in car_vol law! t1=%e, V=%e, t2=%e, C=%e, t_old=%e\n",t1,Vol_yield, t2, C_consume_total, mod_par_dt);
        // to choose modified devolatilization time step
        if(t1<t2 && t1>0)  
        {
            mod_par_dt1 = t1;
            Vol_yield = par_info->coal.Mass_P[Pry_V];  // to get the weight of volatile yield out
            // to modify volatile yield out rate according to mass loading rate
            //            Vol_yield *= 1/(1+sqrt(cell[par_info->cellin].loading_rate[1]));
            // to recalculate carbon consumption of each reaction
            CHAR_REACTION_EMPIRICAL(mod_par_dt1, Tem, Pre, Char_consume, par_info, cell);
            // to modify char reaction rate according to mass loading rate
//             for(k=0; k<4; k++)
//                 Char_consume[k] *= 1/(1+cell[par_info->cellin].loading_rate[1]);
            // to calculate total carbon weight loss
            C_consume_total = 0.0;
            for(k=0; k<4; k++)
                C_consume_total += Char_consume[k];

            Enthalpy_reaction = 0.0;
            for(k=0; k<4; k++)
                Enthalpy_reaction += Char_consume[k]*Heat_release[k];

            fp_reaction = CALCU_FP_REACTION(par_info, cell, Enthalpy_reaction);
            Enthalpy_reaction = Enthalpy_reaction*fp_reaction/mod_par_dt1/par_info->surface;
            B =  -(h*Tg+epsilon_p*Sigma*pow(Tem_Rad_BG,4)+fp_devol*Vol_yield*par_info->coal.heat_release[1]/par_info->surface/mod_par_dt1+Enthalpy_reaction)/(h+epsilon_p*Sigma*pow(Tp,3));
            B_1 =  -(h*Tg+fp_devol*Vol_yield*par_info->coal.heat_release[1]/par_info->surface/mod_par_dt1+Enthalpy_reaction)/(h);
            
            Pmass_new = Pmass_old-Vol_yield-C_consume_total;  // the final particle weight
            par_info->coal.Tem = (Tp+B)*exp(A*mod_par_dt1)-B;   // to calculate final temperature
            Tp_1 = (Tp+B_1)*exp(A_1*mod_par_dt1)-B_1;   // to calculate final temperature

            source[par_info->cellin].mass += par_info->particle_number*(Pmass_old-Pmass_new);  // mass source
            source[par_info->cellin].mixture_fr[1] += par_info->particle_number*Vol_yield;
            source[par_info->cellin].mixture_fr[2] += par_info->particle_number*C_consume_total;

            // to renew species source term according to the reaction
            source[par_info->cellin].massflux_species[O2] -= par_info->particle_number*Char_consume[0]*Molecular_Weight[O2]/Molecular_Weight[C_S];
            source[par_info->cellin].massflux_species[CO2] += par_info->particle_number*(Char_consume[0]-Char_consume[1])*Molecular_Weight[CO2]/Molecular_Weight[C_S];
            source[par_info->cellin].massflux_species[CO] += par_info->particle_number*(2*Char_consume[1]+Char_consume[2])*Molecular_Weight[CO]/Molecular_Weight[C_S];
            source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(-Char_consume[2])*Molecular_Weight[H2O]/Molecular_Weight[C_S];
            source[par_info->cellin].massflux_species[H2] += par_info->particle_number*(Char_consume[2]-2*Char_consume[3])*Molecular_Weight[H2]/Molecular_Weight[C_S];
            source[par_info->cellin].massflux_species[CH4] += par_info->particle_number*Char_consume[3]*Molecular_Weight[CH4]/Molecular_Weight[C_S];

            if(fabs(total_vol-coal_primary_mfr[Pry_V])>1e-6)
                printf("coal composition weird! in devolatilization process!%f\n", fabs(total_vol-coal_primary_mfr[Pry_V])*1e6);  //for debug!

            // to renew species source term according to the devolatilization(just for this work)
            source[par_info->cellin].massflux_species[N2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[N2]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[H2O]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[CO] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CO]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[CO2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CO2]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[C2H2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[C2H2]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[CH4] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CH4]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[H2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[H2]/coal_primary_mfr[Pry_V];

            Enthalpy_Vol *= Vol_yield*(Tp-Tg)/coal_primary_mfr[Pry_V];
            H_Vol_g *= Vol_yield/coal_primary_mfr[Pry_V];
            H_Vol_p *= Vol_yield/coal_primary_mfr[Pry_V];
            if((Enthalpy_Vol+H_Vol_g-H_Vol_p)/Enthalpy_Vol >1e-1)
                printf("too large difference in Devol_Reac! Hvol_cp=%e, Hvol_tp=%e, Hvol_tg=%e, err=%f\n", Enthalpy_Vol, H_Vol_p, H_Vol_g, (Enthalpy_Vol+H_Vol_g-H_Vol_p)/Enthalpy_Vol);

            //            h_reaction =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-par_info->coal.Tem)+fp_reaction==0.0?0.0:Enthalpy_reaction*mod_par_dt1*par_info->surface/fp_reaction+Vol_yield*par_info->coal.heat_release[1]+Enthalpy_Vol);
            h_reaction =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+fp_reaction==0.0?0.0:Enthalpy_reaction*mod_par_dt1*par_info->surface/fp_reaction+Vol_yield*par_info->coal.heat_release[1]+(H_Vol_p-H_Vol_g));

            source[par_info->cellin].enthalpy += h_reaction;
            source[par_info->cellin].Enth_h += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+H_Cs*C_consume_total+Vol_yield*par_info->coal.heat_release[1]+H_Vol_p);

            if(fabs(source[par_info->cellin].enthalpy)>4e5)
                printf("s.h weird in reaction process!1. s.h=%f, %f\n", source[par_info->cellin].enthalpy, h_reaction);

            par_info->coal.Mass[1] = Pmass_new;

            // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
            par_info->coal.Mass_P[Pry_V] -= Vol_yield;  // to get new mass of Moisture
            par_info->coal.Mass_P[Pry_C] -= C_consume_total;
            for(k=0; k<4; k++)
                par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;
            RENEW_COAL_PROPERTY(par_info);

            mod_par_dt1 = mod_par_dt-mod_par_dt1;   // the rest time step
            //for the rest time step, to consider char reaction laws.
            LAW_CHAR_REACTION(mod_par_dt1, par_info, cell, source);
        }
        else //t1>t2, to choose modified reaction time
        {
            mod_par_dt1 = t2;
            Vol_yield = DEVOL_FIRST_ORDER(mod_par_dt1,Tp, Pre, par_info);  // to get the weight of volatile yield out
            // to modify volatile yield out rate according to mass loading rate
            //            Vol_yield *= 1/(1+sqrt(cell[par_info->cellin].loading_rate[1]));
            C_consume_total = par_info->coal.Mass_P[Pry_C];  // to adjust the carbon consumption

            B =  -(h*Tg+epsilon_p*Sigma*pow(Tem_Rad_BG,4)+fp_devol*Vol_yield*par_info->coal.heat_release[1]/par_info->surface/mod_par_dt1+Enthalpy_reaction)/(h+epsilon_p*Sigma*pow(Tp,3));
            B_1 =  -(h*Tg+fp_devol*Vol_yield*par_info->coal.heat_release[1]/par_info->surface/mod_par_dt1+Enthalpy_reaction)/(h);
            
            Pmass_new = Pmass_old-Vol_yield-par_info->coal.Mass_P[Pry_C];  // the final particle weight
            par_info->coal.Tem = (Tp+B)*exp(A*mod_par_dt1)-B;   // to calculate final temperature
            Tp_1 = (Tp+B_1)*exp(A_1*mod_par_dt1)-B_1;
            source[par_info->cellin].mass += par_info->particle_number*(Pmass_old-Pmass_new);  // mass source
            source[par_info->cellin].mixture_fr[1] += par_info->particle_number*Vol_yield;
            source[par_info->cellin].mixture_fr[2] += par_info->particle_number*C_consume_total;

            // to renew species source term according to the reaction
            source[par_info->cellin].massflux_species[O2] -= par_info->particle_number*Char_consume[0]*Molecular_Weight[O2]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;
            source[par_info->cellin].massflux_species[CO2] += par_info->particle_number*(Char_consume[0]-Char_consume[1])*Molecular_Weight[CO2]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;
            source[par_info->cellin].massflux_species[CO] += par_info->particle_number*(2*Char_consume[1]+Char_consume[2])*Molecular_Weight[CO]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;
            source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(-Char_consume[2])*Molecular_Weight[H2O]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;
            source[par_info->cellin].massflux_species[H2] += par_info->particle_number*(Char_consume[2]-2*Char_consume[3])*Molecular_Weight[H2]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;
            source[par_info->cellin].massflux_species[CH4] += par_info->particle_number*Char_consume[3]*Molecular_Weight[CH4]/Molecular_Weight[C_S]*mod_par_dt1/mod_par_dt;

            if(fabs(total_vol-coal_primary_mfr[Pry_V])>1e-6)
                printf("coal composition weird! in devolatilization process!%f\n", fabs(total_vol-coal_primary_mfr[Pry_V])*1e6);  //for debug!

            // to renew species source term according to the devolatilization(just for this work)
            source[par_info->cellin].massflux_species[N2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[N2]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[H2O]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[CO] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CO]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[CO2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CO2]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[C2H2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[C2H2]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[CH4] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CH4]/coal_primary_mfr[Pry_V];
            source[par_info->cellin].massflux_species[H2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[H2]/coal_primary_mfr[Pry_V];

            Enthalpy_Vol *= Vol_yield*(Tp-Tg)/coal_primary_mfr[Pry_V];
            H_Vol_g *= Vol_yield/coal_primary_mfr[Pry_V];
            H_Vol_p *= Vol_yield/coal_primary_mfr[Pry_V];
            if((Enthalpy_Vol+H_Vol_g-H_Vol_p)/Enthalpy_Vol >1e-1)
                printf("too large difference in Devol_Reac!!! Hvol_cp=%e, Hvol_tp=%e, Hvol_tg=%e, err=%f\n", Enthalpy_Vol, H_Vol_p, H_Vol_g, (Enthalpy_Vol+H_Vol_g-H_Vol_p)/Enthalpy_Vol);

            //            h_reaction =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-par_info->coal.Tem)+fp_reaction==0.0?0.0:Enthalpy_reaction*mod_par_dt1*par_info->surface/fp_reaction+Vol_yield*par_info->coal.heat_release[1]+Enthalpy_Vol);
            h_reaction =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+fp_reaction==0.0?0.0:Enthalpy_reaction*mod_par_dt1*par_info->surface/fp_reaction+Vol_yield*par_info->coal.heat_release[1]+(H_Vol_p-H_Vol_g));

            source[par_info->cellin].enthalpy += h_reaction;
            source[par_info->cellin].Enth_h += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+H_Cs*C_consume_total+Vol_yield*par_info->coal.heat_release[1]+H_Vol_p);

            if(fabs(source[par_info->cellin].enthalpy)>4e5)
                printf("s.h weird in reaction process!1. s.h=%f, %f\n", source[par_info->cellin].enthalpy, h_reaction);

            par_info->coal.Mass[1] = Pmass_new;

            // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
            par_info->coal.Mass_P[Pry_V] -= Vol_yield;  // to get new mass of Moisture
            par_info->coal.Mass_P[Pry_C] = 0.0;
            for(k=0; k<4; k++)
                par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;
            RENEW_COAL_PROPERTY(par_info);

            mod_par_dt1 = mod_par_dt-mod_par_dt1;   // the rest time step

            //for the rest time step, to consider devolatilization laws.
           LAW_DEVOLATILIZATION(mod_par_dt1, par_info, cell, source);
        }
    }
    else  //nomal case
    {
        Pmass_new = Pmass_old-Vol_yield-C_consume_total;  // the final particle weight
        par_info->coal.Tem = (Tp+B)*exp(A*mod_par_dt)-B;   // to calculate final temperature
        Tp_1 = (Tp+B_1)*exp(A_1*mod_par_dt)-B_1;
        source[par_info->cellin].mass += par_info->particle_number*(Pmass_old-Pmass_new);  // mass source
        source[par_info->cellin].mixture_fr[1] += par_info->particle_number*Vol_yield;
        source[par_info->cellin].mixture_fr[2] += par_info->particle_number*C_consume_total;

        // to renew species source term according to the reaction
        source[par_info->cellin].massflux_species[O2] -= par_info->particle_number*Char_consume[0]*Molecular_Weight[O2]/Molecular_Weight[C_S];
        source[par_info->cellin].massflux_species[CO2] += par_info->particle_number*(Char_consume[0]-Char_consume[1])*Molecular_Weight[CO2]/Molecular_Weight[C_S];
        source[par_info->cellin].massflux_species[CO] += par_info->particle_number*(2*Char_consume[1]+Char_consume[2])*Molecular_Weight[CO]/Molecular_Weight[C_S];
        source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*(-Char_consume[2])*Molecular_Weight[H2O]/Molecular_Weight[C_S];
        source[par_info->cellin].massflux_species[H2] += par_info->particle_number*(Char_consume[2]-2*Char_consume[3])*Molecular_Weight[H2]/Molecular_Weight[C_S];
        source[par_info->cellin].massflux_species[CH4] += par_info->particle_number*Char_consume[3]*Molecular_Weight[CH4]/Molecular_Weight[C_S];
        
        if(fabs(total_vol-coal_primary_mfr[Pry_V])>1e-6)
            printf("coal composition weird! in devolatilization process!%f\n", fabs(total_vol-coal_primary_mfr[Pry_V])*1e6);  //for debug!
        
        // to renew species source term according to the devolatilization(just for this work)
        source[par_info->cellin].massflux_species[N2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[N2]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[H2O] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[H2O]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[CO] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CO]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[CO2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CO2]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[C2H2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[C2H2]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[CH4] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[CH4]/coal_primary_mfr[Pry_V];
        source[par_info->cellin].massflux_species[H2] += par_info->particle_number*Vol_yield*par_info->coal.mfr_species[H2]/coal_primary_mfr[Pry_V];

        Enthalpy_Vol *= Vol_yield*(Tp-Tg)/coal_primary_mfr[Pry_V];
        H_Vol_g *= Vol_yield/coal_primary_mfr[Pry_V];
        H_Vol_p *= Vol_yield/coal_primary_mfr[Pry_V];
        if((Enthalpy_Vol+H_Vol_g-H_Vol_p)/Enthalpy_Vol >1e-1)
            printf("too large difference in Devol_Reac!!!!! Hvol_cp=%e, Hvol_tp=%e, Hvol_tg=%e, err=%f\n", Enthalpy_Vol, H_Vol_p, H_Vol_g, (Enthalpy_Vol-H_Vol_p+H_Vol_g)/Enthalpy_Vol);

        //        h_reaction =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-par_info->coal.Tem)+fp_reaction==0.0?0.0:Enthalpy_reaction*mod_par_dt*par_info->surface/fp_reaction+Vol_yield*par_info->coal.heat_release[1]+Enthalpy_Vol);
        h_reaction =  par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+fp_reaction==0.0?0.0:Enthalpy_reaction*mod_par_dt*par_info->surface/fp_reaction+Vol_yield*par_info->coal.heat_release[1]+(H_Vol_p-H_Vol_g));
        source[par_info->cellin].Enth_h += par_info->particle_number*(par_info->coal.Mass[1]*par_info->coal.heat_capacity[4]*(Tp-Tp_1)+H_Cs*C_consume_total+Vol_yield*par_info->coal.heat_release[1]+H_Vol_p);
        source[par_info->cellin].enthalpy += h_reaction;

        if(fabs(source[par_info->cellin].enthalpy)>4e5)
            printf("s.h weird in reaction process!1. s.h=%f, %f\n", source[par_info->cellin].enthalpy, h_reaction);

        par_info->coal.Mass[1] = Pmass_new;
        
        // to change coal.mfr_primary[4] and coal.mfr_ultimate[], mfr_species[]
        par_info->coal.Mass_P[Pry_V] -= Vol_yield;  // to get new mass of Moisture
        par_info->coal.Mass_P[Pry_C] -= C_consume_total;
        for(k=0; k<4; k++)
            par_info->coal.mfr_primary[k] = par_info->coal.Mass_P[k]/Pmass_new;
        RENEW_COAL_PROPERTY(par_info);
    }
}
