#include "header.h"


// this function is used to renew grid (when a diffusion process occurs) so that the correct eddy selection can be implemented
void RENEW_Y(int domain_cellnumber, double *grid_y, struct cell_information *cell)
{
    int i;
    double new_domainlength=0.0;  // new domain length after triplet map process!!!


    for(i=0; i<domain_cellnumber; i++)
        new_domainlength += cell[i].size;  // to calculate new domain length

    //printf("new_length=%8.5f\n",new_domainlength);

    //the new boundary of the domain is set as follows: to imagine that the domain will shrink or expand the same value on both side of the origional domain
    grid_y[0] = -(new_domainlength/2.0);

    for(i=1; i<domain_cellnumber+1; i++)
        grid_y[i] = grid_y[i-1]+cell[i-1].size;   

    for(i=0; i<domain_cellnumber; i++)
        cell[i].y = (grid_y[i]+grid_y[i+1])/2;
}






// to add or decrease cells in order to keep a constant domain width
void REGRID_KEEPCONSTANT(int *core_bdcell, int *domain_cellnumber, int eddy_occur, double *grid_y, struct cell_information *cell, struct part_infor *par_info, struct eddy_information *eddy_info)
{
    int i, k, n;
    int olddomain_cellnumber, newdomain_cellnumber;
    int leftcellnumber, rightcellnumber;
    double domain_width=0.0, domain_expansion, half_expansion;
    double ranum;
    //    double old_bd_l, old_bd_r, new_bd_l, new_bd_r; // the old coordination and new coordination of the left/right boundary.
	
    struct cell_information cell_old[*domain_cellnumber];

    olddomain_cellnumber = *domain_cellnumber;

    for(i=0; i<olddomain_cellnumber; i++)
        cell_old[i] = cell[i];
    //to calculate new domain width and new left domain width	

    for(i=0; i<olddomain_cellnumber; i++)
    {
        domain_width += cell_old[i].size;  //to calculate the new domain width;
//         if(cell_old[i].size<0 || cell_old[i].size>0.1)
//             printf("error_regrid! %d\t%f\t%f\t%f\n", i, cell_old[i].size,cell_old[i].u,cell_old[i].u);
    }

    //because the domain will shrink or expand. to guarantee a constant domain width and thus a unchangable
    //boundary conditions, a number of cells should be added or subtracted;

    //since usually, new domain width will shrinks with a small fraction deta_y, deta_y=domain_y/2-domain_half_width<<dy=0.0001m.
    //if we add an small cell whose width is deta_y every time, there should be a lot of small cells at last
    //so if there r small cells, they should be merged! 
    //the new cell number added to left boundary is calculated by: addedleft_cellnumber = (int)(cell_old[0].size+(domain_y-domain_width)/2)/dy
    //the new cell number added to right boundary is calculated by: addedright_cellnumber = (int)(cell_old[old_domaincellnumber-1].size+(domain_y-domain_width).2)/dy
    if(domain_width < domain_y)  // old domain shrinks
    {
        //to calculate newly added cell number
        leftcellnumber = (int)(((domain_y-domain_width)/2.0+cell_old[0].size)/dy_co);
        rightcellnumber = (int)(((domain_y-domain_width)/2.0+cell_old[olddomain_cellnumber-1].size)/dy_co);

        if(leftcellnumber >100)
            printf("large shrink! d_y=%e, cell_old[0].size=%e\n", domain_width,cell_old[0].size);

        core_bdcell[0] += leftcellnumber;
        core_bdcell[1] += leftcellnumber;

        for(i=0; i<eddy_occur; i++)
        {
            eddy_info[i].ycellstart += leftcellnumber;
            eddy_info[i].ycellend += leftcellnumber;
            if(eddy_info[i].ycellend>olddomain_cellnumber)
            {
                printf("in REGRID! eddy[%d].cellend=%d, %d, leftcellnumber=%d\n",i,eddy_info[i].ycellend-leftcellnumber, eddy_info[i].ycellend,leftcellnumber);
                eddy_info[i].ycellend = olddomain_cellnumber;
            }
        }

        //      printf(" !!  core_bdcell[0]=%d, core_bdcell[1]=%d, cor_l=%f, cor_r=%f\n",core_bdcell[0],core_bdcell[1],cell[core_bdcell[0]].y, cell[core_bdcell[1]].y);

        if(leftcellnumber!=rightcellnumber)
            printf("shrinks wierd! left=%d, right=%d, shrink=%8.5f,cell[0].size=%8.5f,u=%8.5f,cell[end].size=%8.5f,u=%8.5f\n",leftcellnumber, rightcellnumber,(domain_y-domain_width)/2.0*1e3,cell_old[0].size*1e3,cell_old[0].u,cell[olddomain_cellnumber-1].size*1e3,cell[olddomain_cellnumber-1].u);

        //to calculate new domain cell number
        newdomain_cellnumber = olddomain_cellnumber+leftcellnumber+rightcellnumber;  

        for(i=1; i<olddomain_cellnumber-1; i++)
            cell[i+leftcellnumber] = cell_old[i];

        if(leftcellnumber == 0)  // small shrinks at left side, merge small cells
        {
            cell[0] = cell_old[0];
            cell[0].size = (domain_y-domain_width)/2.0+cell_old[0].size;
            cell[0].size_old = cell[0].size;  //size_old is used in species diffusion process
            i = 0;
            //to set the mole fraction of each species, u, temperature and pressure information for the added cells
            INI_BOUNDARY_CELL(i, cell_old[0].u, cell);
            //to acquire the massfraction information and other information of the added cell based on given T, P and X[i]
            //This step is important and necessary
            RENEW_MIXPRO_MOLEX(i,cell);

        }
        else  //large shrinks at left side
        {
            cell[leftcellnumber] = cell_old[0];
            cell[leftcellnumber].size = dy_co;
            cell[leftcellnumber].size_old = dy_co;  //to give the original cell.size, which should be used in species diffusion process, but at the boundary, this .size_old is better to be set to dy_co;

            for(i=leftcellnumber-1; i>=0; i--)
            {
                cell[i].size = dy_co;
                cell[i].size_old = cell[i].size;
                ranum=(double) rand()/RAND_MAX;
                ranum=ranum*2-1;
                // cell[i].u = covelocity*(1+ranum*0.5*turb_intensity_coflow); 
                cell[i].v_lateral = cell_old[0].v_lateral;
                cell[i].v_trip = cell_old[0].v_trip;
                ranum=(double) rand()/RAND_MAX;
                ranum=ranum*2-1;
                cell[i].w = covelocity*ranum*0.5*turb_intensity_coflow; 

                //to set the mole fraction of each species, u, temperature and pressure information for the added cells
                INI_BOUNDARY_CELL(i, cell_old[0].u, cell);
                //to acquire the massfraction information and other information of the added cell based on given T, P and X[i]
                //This step is important and necessary
                RENEW_MIXPRO_MOLEX(i,cell);

            }
            cell[0].size = (domain_y-domain_width)/2.0+cell_old[0].size-leftcellnumber*dy_co;
            cell[0].size_old = cell[0].size;
        }

        //for right domain shrink        
        k = leftcellnumber+olddomain_cellnumber-1;
        if(rightcellnumber == 0)  // small shrinks at right side, merge small cells
        {
            cell[k] = cell_old[olddomain_cellnumber-1];
            cell[k].size = (domain_y-domain_width)/2.0+cell_old[olddomain_cellnumber-1].size;
            cell[k].size_old = cell_old[k].size;  //size_old is used in species diffusion process
            i = k;
            //to set the mole fraction of each species, u, temperature and pressure information for the added cells
            INI_BOUNDARY_CELL(i, cell_old[olddomain_cellnumber-1].u, cell);
            //to acquire the massfraction information and other information of the added cell based on given T, P and X[i]
            //This step is important and necessary
            RENEW_MIXPRO_MOLEX(i,cell);
        }
        else  //large shrinks at right side
        {
            cell[k] = cell_old[olddomain_cellnumber-1];
            cell[k].size = dy_co;
            cell[k].size_old = dy_co;  //to give the original cell.size, which should be used in species diffusion process, but at the boundary, this .size_old is better to be set to dy_co;

            for(i=k+1; i<newdomain_cellnumber; i++)
            {
                cell[i].size = dy_co;
                cell[i].size_old = cell[i].size;
                ranum=(double) rand()/RAND_MAX;
                ranum=ranum*2-1;
                // cell[i].u = covelocity*(1+ranum*0.5*turb_intensity_coflow); 
                cell[i].v_lateral = cell_old[olddomain_cellnumber-1].v_lateral;
                cell[i].v_trip = cell_old[olddomain_cellnumber-1].v_trip;
                ranum=(double) rand()/RAND_MAX;
                ranum=ranum*2-1;
                cell[i].w = covelocity*ranum*0.5*turb_intensity_coflow; 

                //to set the mole fraction of each species, u, temperature and pressure information for the added cells
                INI_BOUNDARY_CELL(i, cell_old[olddomain_cellnumber-1].u, cell);
                //to acquire the massfraction information and other information of the added cell based on given T, P and X[i]
                //This step is important and necessary
                RENEW_MIXPRO_MOLEX(i,cell);
            }
            cell[newdomain_cellnumber-1].size = (domain_y-domain_width)/2.0+cell_old[olddomain_cellnumber-1].size-rightcellnumber*dy_co;
            cell[newdomain_cellnumber-1].size_old = cell[newdomain_cellnumber-1].size;  // this term is not strickly correct. but it doesn't matter since in species_diffusion.cpp, the boundary condition is based on ghost point
        }
    }

    if(domain_width >= domain_y) //domain expands
    {
        //to calculate domain expansion
        domain_expansion = (domain_width-domain_y);
        half_expansion = 0.0;

        for(i=0; i<olddomain_cellnumber; i++)
        {
            half_expansion += cell_old[i].size;
            if(half_expansion > domain_expansion/2.0) // the standard expansion for each side of the domain is half of domain_expansion
            {
                leftcellnumber=i;          // the serial number of the cell which should be cut	
                i=olddomain_cellnumber+1;  // interrupt the circulation
            }
        }
        cell[0] = cell_old[leftcellnumber];
        cell[0].size = half_expansion-domain_expansion/2.0;
        cell[0].size_old = cell[0].size;

        half_expansion = 0.0;
        for(i=olddomain_cellnumber-1; i>0; i--)
        {
            half_expansion += cell_old[i].size;
            if(half_expansion > domain_expansion/2.0)  // the standard expansion for right side of the domain is half of domain_expansion
            {
                rightcellnumber=i;        // the serial number of the cell which should be cut
                i=0;  //to interrupt the circulation
            }
        }

        newdomain_cellnumber = rightcellnumber-leftcellnumber+1;  //to calculate the new cell number
        cell[newdomain_cellnumber-1] = cell_old[rightcellnumber];
        cell[newdomain_cellnumber-1].size = half_expansion-domain_expansion/2.0;
        cell[newdomain_cellnumber-1].size_old = cell[newdomain_cellnumber-1].size;


        core_bdcell[0] -= leftcellnumber;
        core_bdcell[0] = core_bdcell[0]>100?core_bdcell[0]:100;
        core_bdcell[1] -= leftcellnumber;
        core_bdcell[1] = core_bdcell[1]<(newdomain_cellnumber-101)?core_bdcell[1]:(newdomain_cellnumber-101);

        for(i=0; i<eddy_occur; i++)
        {
            eddy_info[i].ycellstart -= leftcellnumber;
            eddy_info[i].ycellstart = eddy_info[i].ycellstart>50?eddy_info[i].ycellstart:50;
            eddy_info[i].ycellend -= leftcellnumber;
            eddy_info[i].ycellend = eddy_info[i].ycellend>60?eddy_info[i].ycellend:60;
            if(eddy_info[i].ycellstart<0)
            {
                printf("in REGRID! eddy[%d].ycellstart=%d,%d, leftcellnumber=%d\n",i,eddy_info[i].ycellstart+leftcellnumber, eddy_info[i].ycellstart,leftcellnumber);
                eddy_info[i].ycellstart = 0;
            }
        }

        // to reset the value for other cells
        for(i=1; i<newdomain_cellnumber-1; i++)
            cell[i] = cell_old[i+leftcellnumber];
    }

    // to calculate the new cell number of the boundary of the core region
    *domain_cellnumber = newdomain_cellnumber;

    //to calculate new grid_y and cell_y
    grid_y[0]=-domain_y/2;
    for(i=1; i<*domain_cellnumber+1; i++)
    {
        grid_y[i] = cell[i-1].size+grid_y[i-1];
        if(cell[i-1].size<0|| cell[i-1].size>0.1)
            printf("error_regrid! %d\t%f\n", i, cell[i-1].size);
    }
    for(i=0; i<*domain_cellnumber; i++)
        cell[i].y = (grid_y[i]+grid_y[i+1])/2;

    for(i=0; i<eddy_occur; i++)
    {
        eddy_info[i].ystart = grid_y[eddy_info[i].ycellstart];
        eddy_info[i].yend = grid_y[eddy_info[i].ycellend];
        
    }

    //    printf(" %d\t%8.5f\t%8.5f\n",*domain_cellnumber, grid_y[0],grid_y[*domain_cellnumber]);
}







// this function is used to acquire the velocity and particle information of the uniform grid based on non-uniform grid
void RENEW_BACKGRID(int domain_cellnumber, double deta_x, double *grid_y, struct cell_information *cell, struct part_infor *par_info, struct back_grid *backgrid_nd)
{
    int i, k, startpoint, endpoint, unit, cells;	
    startpoint=0;
    unit=0;
    for(i=1;i<domain_cellnumber+1;i++)
    {
        //when new grid point is larger than new uniform grid point(background grid)
        if(grid_y[i]>=(unit+1-int((domain_y/2+1e-10)/dy_bg))*dy_bg-1e-8)
        {	
            endpoint=i;	
            // to record mass of last ODT step
            backgrid_nd[unit].mass0 = backgrid_nd[unit].mass1;  
            // to record u velocity of last ODT step
            backgrid_nd[unit].ugas0 = backgrid_nd[unit].ugas1;
            // to record v diffusion velocity of last ODT step
            backgrid_nd[unit].vgas0 = backgrid_nd[unit].vgas1;
            // to record numberdensity information of last step
            
            //if it is left boundary
            if(unit==0)
            {
                //to initialize boundary numberdensity information of this step
                backgrid_nd[unit].mass1 = cell[0].size*cell[0].mixture.Density;
                backgrid_nd[unit].ugas1 = cell[0].u*cell[0].size*cell[0].mixture.Density;
            }

            //not left boundary
            else
            {
                backgrid_nd[unit].mass1 = (grid_y[startpoint]-(unit-int((domain_y+1e-10)/dy_bg/2))*dy_bg)*cell[startpoint-1].mixture.Density;
                backgrid_nd[unit].ugas1 = (grid_y[startpoint]-(unit-int((domain_y+1e-10)/dy_bg/2))*dy_bg)*cell[startpoint-1].u*cell[startpoint-1].mixture.Density;
            }
            for(k=startpoint+1;k<endpoint;k++)
            {
                backgrid_nd[unit].mass1 += cell[k-1].size*cell[k-1].mixture.Density;
                backgrid_nd[unit].ugas1 += cell[k-1].u*cell[k-1].size*cell[k-1].mixture.Density;
            }

            backgrid_nd[unit].mass1 += ((unit+1-int((domain_y/2+1e-10)/dy_bg))*dy_bg-grid_y[endpoint-1])*cell[endpoint-1].mixture.Density;
            backgrid_nd[unit].ugas1 += ((unit+1-int((domain_y/2+1e-10)/dy_bg))*dy_bg-grid_y[endpoint-1])*cell[endpoint-1].u*cell[endpoint-1].mixture.Density;
            backgrid_nd[unit].ugas1 /= backgrid_nd[unit].mass1;

            unit++;
			
            //if a cell size is larger than dy
            if(grid_y[endpoint]>=(unit+1-int((domain_y/2+1e-10)/dy_bg))*dy_bg-1e-8)
            {

                cells=int((grid_y[endpoint]-(unit-int((domain_y+1e-10)/dy_bg/2))*dy_bg)/dy_bg);
                for(k=0; k<cells; k++)
                {
                    backgrid_nd[unit].mass1 = cell[endpoint-1].mixture.Density*dy_bg;
                    backgrid_nd[unit].ugas1 = cell[endpoint-1].u;
                    unit++;
                }
            }
            startpoint=endpoint;

    //         if(backgrid_nd[unit].ugas1>30)
//                 printf("here! bg[%d]ugas1=%8.5f, mass=%8.5f\n",unit, backgrid_nd[unit].ugas1, backgrid_nd[unit].mass1);
        }
    }

    // to initialize particle information based on uniform grid
    for(i=0; i<(int)((domain_y+1e-10)/dy_bg); i++)
    {
        backgrid_nd[i].par_size = 0.0;
        backgrid_nd[i].par_surface = 0.0;
        backgrid_nd[i].par_volume = 0.0;
        backgrid_nd[i].par_d10 = 0.0;
        backgrid_nd[i].par_d32 = 0.0;

        for(k=0; k<Par_SN; k++)
        {
            backgrid_nd[i].par_numberdensity[k] = 0.0;
            backgrid_nd[i].par_number[k] = 0.0;
            backgrid_nd[i].par_massflux[k] = 0.0;
            backgrid_nd[i].par_u[k] = 0.0;
            backgrid_nd[i].par_v[k] = 0.0;
            backgrid_nd[i].par_Re[k] = 0.0;
            backgrid_nd[i].par_nd0th_old[k] = backgrid_nd[i].par_nd0th_new[k];
            backgrid_nd[i].par_nd0th_new[k] = 0.0;
            backgrid_nd[i].par_0thdiffcoe[k] = 0.0;
            backgrid_nd[i].Tem[k] = 0.0;
            backgrid_nd[i].Mass_P[1][k] = 0.0;
            backgrid_nd[i].Mass_P[2][k] = 0.0;
            backgrid_nd[i].Mass_P[3][k] = 0.0;
            backgrid_nd[i].Mass_P[4][k] = 0.0;
            backgrid_nd[i].Mass_P[5][k] = 0.0;
            backgrid_nd[i].density[k] = 0.0;
        }

        backgrid_nd[i].par_nd1st_old = backgrid_nd[i].par_nd1st_new;
        backgrid_nd[i].par_nd2nd_old = backgrid_nd[i].par_nd2nd_new;
        backgrid_nd[i].par_nd3rd_old = backgrid_nd[i].par_nd3rd_new;
    
        backgrid_nd[i].par_nd1st_new = 0.0;
        backgrid_nd[i].par_nd2nd_new = 0.0;
        backgrid_nd[i].par_nd3rd_new = 0.0;
        
        backgrid_nd[i].par_1stdiffcoe = 0.0;
        backgrid_nd[i].par_2nddiffcoe = 0.0;
        backgrid_nd[i].par_3rddiffcoe = 0.0;
}


    for(i=0; i<par_tracknumber; i++)
    {
        // to calculate total particle mass flow rate per area in this cell
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_number[Par_SN-1]++;
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_massflux[Par_SN-1] += par_info[i].massflowrate;

        // to calculate total Re/u/v/numberdensity in this cell, This kind of averaging Rep/u/v method is based on mass weight average!
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_Re[Par_SN-1] += par_info[i].Re_p*par_info[i].massflowrate;
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_u[Par_SN-1] += par_info[i].u*par_info[i].massflowrate;
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_v[Par_SN-1] += par_info[i].v*par_info[i].massflowrate;
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_numberdensity[Par_SN-1] += par_info[i].particle_number/par_info[i].u/dy_bg;

            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Tem[Par_SN-1] += par_info[i].coal.Tem*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[0][Par_SN-1] += par_info[i].coal.Mass_P[Pry_M]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[1][Par_SN-1] += par_info[i].coal.Mass_P[Pry_V]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[2][Par_SN-1] += par_info[i].coal.Mass_P[Pry_C]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[3][Par_SN-1] += par_info[i].coal.Mass_P[Pry_A]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[4][Par_SN-1] += par_info[i].coal.Mass[1]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].density[Par_SN-1] += par_info[i].coal.density[4]*par_info[i].massflowrate;

        // to calculate the 0th/1st/2nd/3rd moments of particle phase in this cell
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_size += par_info[i].diameter*par_info[i].particle_number/par_info[i].u/dy_bg;
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_surface += PI*pow(par_info[i].diameter,2)*par_info[i].particle_number/par_info[i].u/dy_bg;
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_volume += PI*pow(par_info[i].diameter,3)/6*par_info[i].particle_number/par_info[i].u/dy_bg;

        //particle d10 was the mass weighted particle average diameter
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_d10 += par_info[i].diameter*par_info[i].massflowrate;

        //to calculate the 0th/1st/2nd/3rd moment flux of particle phase in this cell
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_nd0th_new[Par_SN-1] += par_info[i].particle_number/dy_bg;
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_nd1st_new += par_info[i].particle_number*par_info[i].diameter/dy_bg;
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_nd2nd_new += PI*pow(par_info[i].diameter, 2)*par_info[i].particle_number/dy_bg;
        backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_nd3rd_new += PI*pow(par_info[i].diameter, 3)/6*par_info[i].particle_number/dy_bg;
 
        if(par_info[i].diameter == par_size_dist[0])
        {
            // to calculate particle mass flow rate of the particles with a size of par_size_dist[0] per area in this cell
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_number[0]++;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_massflux[0] += par_info[i].massflowrate;

            // to calculate  Re/u/v/numberdensity of the particles with a size of par_size_dist[0] in this cell, The averaging Rep/u/v method is based on mass weight average!
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_Re[0] += par_info[i].Re_p*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_u[0] += par_info[i].u*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_v[0] += par_info[i].v*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_numberdensity[0] += par_info[i].particle_number/par_info[i].u/dy_bg;

            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Tem[0] += par_info[i].coal.Tem*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[0][0] += par_info[i].coal.Mass_P[Pry_M]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[1][0] += par_info[i].coal.Mass_P[Pry_V]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[2][0] += par_info[i].coal.Mass_P[Pry_C]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[3][0] += par_info[i].coal.Mass_P[Pry_A]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[4][0] += par_info[i].coal.Mass[1]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].density[0] += par_info[i].coal.density[4]*par_info[i].massflowrate;

            // to calculate 0th/1st/2nd/3rd moment flux of particle with a size of par_size_dist[0] in this cell
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_nd0th_new[0] += par_info[i].particle_number/dy_bg;
        }
        else if(par_info[i].diameter == par_size_dist[1])
        {
            // to calculate particle mass flow rate of the particles with a size of par_size_dist[0] per area in this cell
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_number[1]++;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_massflux[1] += par_info[i].massflowrate;

            // to calculate  Re/u/v/numberdensity of the particles with a size of par_size_dist[0] in this cell, The averaging Rep/u/v method is based on mass weight average!
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_Re[1] += par_info[i].Re_p*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_u[1] += par_info[i].u*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_v[1] += par_info[i].v*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_numberdensity[1] += par_info[i].particle_number/par_info[i].u/dy_bg;

            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Tem[1] += par_info[i].coal.Tem*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[0][1] += par_info[i].coal.Mass_P[Pry_M]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[1][1] += par_info[i].coal.Mass_P[Pry_V]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[2][1] += par_info[i].coal.Mass_P[Pry_C]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[3][1] += par_info[i].coal.Mass_P[Pry_A]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[4][1] += par_info[i].coal.Mass[1]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].density[1] += par_info[i].coal.density[4]*par_info[i].massflowrate;

            // to calculate 0th/1st/2nd/3rd moment flux of particle with a size of par_size_dist[0] in this cell
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_nd0th_new[1] += par_info[i].particle_number/dy_bg;

        }
        else
        {
            // to calculate particle mass flow rate of the particles with a size of par_size_dist[0] per area in this cell
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_number[2]++;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_massflux[2] += par_info[i].massflowrate;

            // to calculate  Re/u/v/numberdensity of the particles with a size of par_size_dist[0] in this cell, The averaging Rep/u/v method is based on mass weight average!
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_Re[2] += par_info[i].Re_p*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_u[2] += par_info[i].u*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_v[2] += par_info[i].v*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_numberdensity[2] += par_info[i].particle_number/par_info[i].u/dy_bg;

            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Tem[2] += par_info[i].coal.Tem*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[0][2] += par_info[i].coal.Mass_P[Pry_M]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[1][2] += par_info[i].coal.Mass_P[Pry_V]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[2][2] += par_info[i].coal.Mass_P[Pry_C]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[3][2] += par_info[i].coal.Mass_P[Pry_A]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].Mass_P[4][2] += par_info[i].coal.Mass[1]*par_info[i].massflowrate;
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].density[2] += par_info[i].coal.density[4]*par_info[i].massflowrate;

            // to calculate 0th/1st/2nd/3rd moment flux of particle with a size of par_size_dist[0] in this cell
            backgrid_nd[(int)((par_info[i].y+domain_y/2)/dy_bg)].par_nd0th_new[2] += par_info[i].particle_number/dy_bg;
        }
    }
    // to calculate the final particle Re/u/v, Tem, Mass_P, density, and size, surface, volume, d10, d32
    for(i=0; i<(int)((domain_y+1e-10)/dy_bg); i++)
    {
        for(k=0; k<Par_SN; k++)
        {
            if(backgrid_nd[i].par_number[k]!=0)
            {
                backgrid_nd[i].par_Re[k] /= backgrid_nd[i].par_massflux[k];
                backgrid_nd[i].par_u[k] /= backgrid_nd[i].par_massflux[k];
                backgrid_nd[i].par_v[k] /= backgrid_nd[i].par_massflux[k];
                backgrid_nd[i].Tem[k] /= backgrid_nd[i].par_massflux[k];
                backgrid_nd[i].Mass_P[0][k] /= backgrid_nd[i].par_massflux[k];
                backgrid_nd[i].Mass_P[1][k] /= backgrid_nd[i].par_massflux[k];
                backgrid_nd[i].Mass_P[2][k] /= backgrid_nd[i].par_massflux[k];
                backgrid_nd[i].Mass_P[3][k] /= backgrid_nd[i].par_massflux[k];
                backgrid_nd[i].Mass_P[4][k] /= backgrid_nd[i].par_massflux[k];
                backgrid_nd[i].density[k] /= backgrid_nd[i].par_massflux[k];
            }
        }
        if(backgrid_nd[i].par_numberdensity[Par_SN-1]!=0)
        {
            //            backgrid_nd[i].par_size /= backgrid_nd[i].par_numberdensity[Par_SN-1];          //numberdensity averaged particle size   m/#
            //            backgrid_nd[i].par_surface /= backgrid_nd[i].par_numberdensity[Par_SN-1];        //number density averaged particle surface m2/#
            //            backgrid_nd[i].par_volume /= backgrid_nd[i].par_numberdensity[Par_SN-1];        // number density averaged particle volume m3/#
            backgrid_nd[i].par_d10 /= backgrid_nd[i].par_massflux[Par_SN-1];
            backgrid_nd[i].par_d32 = backgrid_nd[i].par_volume*6/backgrid_nd[i].par_surface;
        }
    }

/*     printf("unit=%d\n",unit); */
/*     printf("backgrid: %8.5f  %8.5f  %8.5f  %8.5f  %8.5f  %8.5f  %8.5f  %8.5f\n", backgrid_nd[unit/2-1].ugas, backgrid_nd[unit/2-1].ugas1, backgrid_nd[unit/2-1].vgas, backgrid_nd[unit/2-1].vgas1, backgrid_nd[1].ugas, backgrid_nd[1].ugas1, backgrid_nd[1].vgas, backgrid_nd[1].vgas1); */


    //for boundary cell number, 1st and 2nd derivative of particle number density is set to 0
    for(k=0; k<Par_SN; k++)
    {
        backgrid_nd[0].par_0thdiffcoe[k] = 0.0;
        backgrid_nd[(int)((domain_y+1e-10)/dy_bg-1)].par_0thdiffcoe[k] = 0.0;
    }
    backgrid_nd[0].par_1stdiffcoe = 0.0;
    backgrid_nd[0].par_2nddiffcoe = 0.0;
    backgrid_nd[0].par_3rddiffcoe = 0.0;
    backgrid_nd[(int)((domain_y+1e-10)/dy_bg-1)].par_1stdiffcoe = 0.0;
    backgrid_nd[(int)((domain_y+1e-10)/dy_bg-1)].par_2nddiffcoe = 0.0;
    backgrid_nd[(int)((domain_y+1e-10)/dy_bg-1)].par_3rddiffcoe = 0.0;

    // to calculate the 2nd derivative of particle number density
    for(i=1; i<(int)((domain_y+1e-10)/dy_bg)-1; i++)
    {
        for(k=0; k<Par_SN; k++)
        {
            backgrid_nd[i].par_ddnd0th[k] = (backgrid_nd[i+1].par_numberdensity[k]-2*backgrid_nd[i].par_numberdensity[k]+backgrid_nd[i-1].par_numberdensity[k])/dy_bg/dy_bg;
            //to calculate the diffusion coefficent of 0th moment of each size of particle
            if(backgrid_nd[i].par_ddnd0th[k] != 0.0)
                backgrid_nd[i].par_0thdiffcoe[k] = (backgrid_nd[i].par_nd0th_new[k]-backgrid_nd[i].par_nd0th_old[k])/deta_x/backgrid_nd[i].par_ddnd0th[k];
            else
                backgrid_nd[i].par_0thdiffcoe[k] = 0.0;
        }

        backgrid_nd[i].par_ddnd1st = (backgrid_nd[i+1].par_size-2*backgrid_nd[i].par_size+backgrid_nd[i-1].par_size)/dy_bg/dy_bg;
        backgrid_nd[i].par_ddnd2nd = (backgrid_nd[i+1].par_surface-2*backgrid_nd[i].par_surface+backgrid_nd[i-1].par_surface)/dy_bg/dy_bg;
        backgrid_nd[i].par_ddnd3rd = (backgrid_nd[i+1].par_volume-2*backgrid_nd[i].par_volume+backgrid_nd[i-1].par_volume)/dy_bg/dy_bg;

        //to calculate the diffusion coefficient of the 0th/1st/2nd/3rd moments of the total particle
        if(backgrid_nd[i].par_ddnd1st != 0.0)
            backgrid_nd[i].par_1stdiffcoe = (backgrid_nd[i].par_nd1st_new-backgrid_nd[i].par_nd1st_old)/deta_x/backgrid_nd[i].par_ddnd1st;
        else
            backgrid_nd[i].par_1stdiffcoe = 0.0;

        if(backgrid_nd[i].par_ddnd2nd != 0.0)
            backgrid_nd[i].par_2nddiffcoe = (backgrid_nd[i].par_nd2nd_new-backgrid_nd[i].par_nd2nd_old)/deta_x/backgrid_nd[i].par_ddnd2nd;
        else
            backgrid_nd[i].par_2nddiffcoe = 0.0;

        if(backgrid_nd[i].par_ddnd3rd != 0.0)
            backgrid_nd[i].par_3rddiffcoe = (backgrid_nd[i].par_nd3rd_new-backgrid_nd[i].par_nd3rd_old)/deta_x/backgrid_nd[i].par_ddnd3rd;
        else
            backgrid_nd[i].par_3rddiffcoe = 0.0;
    }
}








//to merge cell_1 and cell_2 into cell according to the mixing law
void  MERGECELL(struct cell_information *cell, struct cell_information cell_1, struct cell_information cell_2)
{
    int k;
    double  mfr, momflux,spe_flux;
    double size;

    mfr = cell_1.size*cell_1.u*cell_1.mixture.Density+cell_2.size*cell_2.u*cell_2.mixture.Density;
    momflux = cell_1.size*cell_1.u*cell_1.u*cell_1.mixture.Density+cell_2.size*cell_2.u*cell_2.u*cell_2.mixture.Density;
    cell->u = momflux/mfr;
    for(k=0; k<Species_Number; k++)
    {
        spe_flux = cell_1.size*cell_1.u*cell_1.mixture.Massfr[k]*cell_1.mixture.Density+cell_2.size*cell_2.mixture.Massfr[k]*cell_2.u*cell_2.mixture.Density;
        cell->mixture.Massfr[k] = spe_flux/mfr;
    }
    RENEW_MIXPRO_MASSY(0, cell);
    size = cell->size;
    cell->size = mfr/cell->u/cell->mixture.Density;
    cell->size_old = cell->size/size*cell->size_old;
    cell->y = cell->y-size/2.+cell->size/2.;
}








//to split or merge cells whose size is too large or too small
void MESH_MANAGEMENT(int E_S_number, int *core_bdcell, int *domain_cellnumber, double *grid_y,struct cell_information *cell)
{
    int i,k,k1,old_cellnumber,new_cellnumber,index;
    int core_bdcell_new[2];
    double domain_width=0.0;
    double size_var,u_var,mfr_var;
    double size_ratio;
    double mfr_left, mfr_center, mfr_right;
    struct cell_information cell_old[domain_cellnumber[1]],cell_neighbour;

    old_cellnumber = domain_cellnumber[1];
    new_cellnumber = domain_cellnumber[1];

    core_bdcell_new[0] = core_bdcell[0];
    core_bdcell_new[1] = core_bdcell[1];

    //to restore the orginal cell information
    for(i=0; i<old_cellnumber; i++)
        cell_old[i] = cell[i];

    // k1 means how many cells r merged during this odt step, k means how many cells r splitted during this odt step
    k1 = 0;
    k = 0;

    // first to merge small cells!!!!,but not for the boundary cell!!!
    for(i=0; i<old_cellnumber; i++)
    {
        cell[i+k-k1] = cell_old[i];
        //too small cell.size, need for further steps
        // to merge cells should be satisfy such conditions as:
        // Case 1.
        // 1. cell.size < 0.5*dy_cen, usually, the small cells are caused by the increment of velocity for the coflow cells & 2 & 3
        // 2. fabs(cell.u-cell_next.u)/cell.u <0.01
        // 3. fabs((cell.mixture.MixtureFr_Coal[1]+cell.mixture.MixtureFr_Coal[2])-(cell_nex.mixture.MixtureFr_Coal[1]+cell_nex.mixture.MixtureFr_Coal[2]))/(max(cell.mfr, cell_next.mfr)))<0.01 
        // Case 2.
        //4. cell.size < 0.2*dy_cen (finer than the set-up resolution), to merge it!
        if(i>10 && i<old_cellnumber-10)  //not for the boundary cell!!!
        {
            if(cell_old[i].size<0.2*dy_cen)  //finer than the resolution! Case2
            {
                mfr_left = cell_old[i-1].size*cell_old[i-1].mixture.Density*cell_old[i-1].u;
                mfr_center = cell_old[i].size*cell_old[i].mixture.Density*cell_old[i].u;
                mfr_right = cell_old[i+1].size*cell_old[i+1].mixture.Density*cell_old[i+1].u;               

                // the principle is to merge small cells and leave large cells alone
                if(mfr_left<=mfr_right) //merge left
                {
                    u_var = fabs(cell_old[i].u-cell_old[i-1].u)/cell_old[i].u;
                    mfr_var = ((cell_old[i].mixture.MixtureFr_Coal[1]+cell_old[i].mixture.MixtureFr_Coal[2])-(cell[i-1].mixture.MixtureFr_Coal[1]+cell[i-1].mixture.MixtureFr_Coal[2]))/(cell_old[i].mixture.MixtureFr_Coal[1]+cell_old[i].mixture.MixtureFr_Coal[2]);

                    if(i<=core_bdcell[0])
                    {
                        core_bdcell_new[0]--;
                        core_bdcell_new[1]--;
                    }
                    else if(i>core_bdcell[0] && i<=core_bdcell[1])
                    {
                        core_bdcell_new[1]--;
                    }
                    else
                        ;
                    
                    if(u_var<0.01 && mfr_var<0.01)  //small difference
                    {
                        if(mfr_left<mfr_center) // to merge the cell from left based on ith cell
                        {
                            cell_neighbour = cell_old[i];
                            cell_neighbour.size = cell[i+k-k1-1].size+cell_old[i].size;
                            cell_neighbour.size_old = cell[i+k-k1-1].size_old+cell_old[i].size_old;
                            cell_neighbour.y = cell[i+k-k1-1].y+cell_old[i].size/2.0;

                            //    printf("merging cells from i, small_diff: cell[%d].size=%8.5f, cell[%d].u=%8.5f,cell_neighbour.size=%8.5f,cell_neighbour.u=%8.5f, cell_final.size=%8.5f, cell_final.u=%8.5f\n",i,cell[i+k-k1-1].size*1000,i,cell[i+k-k1-1].u,cell_old[i].size*1000,cell_old[i].u,cell_neighbour.size*1000, cell_neighbour.u);
                            cell[i+k-k1-1] = cell_neighbour;
                        }
                        else  //to merge the cell from left based on the left cell
                        {
                            cell_neighbour = cell[i+k-k1-1];
                            cell[i+k-k1-1].size += cell_old[i].size;
                            cell[i+k-k1-1].size_old += cell_old[i].size_old;
                            cell[i+k-k1-1].y += cell_old[i].size/2.0;

                            //     printf("merging cells from left, small_diff: cell[%d].size=%8.5f, cell[%d].u=%8.5f,cell_neighbour.size=%8.5f,cell_neighbour.u=%8.5f, cell_final.size=%8.5f, cell_final.u=%8.5f\n",i,cell[i+k-k1-1].size*1000,i,cell[i+k-k1-1].u,cell_old[i].size*1000,cell_old[i].u,cell_neighbour.size*1000, cell_neighbour.u);
                        }
                    }
                    else  //large difference, mixing the cell according to the conservation and mixing law
                    {
                        cell_neighbour = mfr_left>mfr_center?cell_old[i-1]:cell_old[i];
                        MERGECELL(&cell_neighbour, cell[i+k-k1-1], cell_old[i]);

                        //    printf("merging cells from left_largediff: cell[%d].size=%8.5f, cell[%d].u=%8.5f,cell_neighbour.size=%8.5f,cell_neighbour.u=%8.5f, cell_final.size=%8.5f, cell_final.u=%8.5f\n",i,cell[i+k-k1-1].size*1000,i,cell[i+k-k1-1].u,cell_old[i].size*1000,cell_old[i].u,cell_neighbour.size*1000, cell_neighbour.u);

                        cell[i+k-k1-1] = cell_neighbour;
                    }
                    k1++;
                }
                else  //merge right
                {
                    u_var = fabs(cell_old[i].u-cell_old[i+1].u)/cell_old[i].u;
                    mfr_var = ((cell_old[i].mixture.MixtureFr_Coal[1]+cell_old[i].mixture.MixtureFr_Coal[2])-(cell[i+1].mixture.MixtureFr_Coal[1]+cell[i+1].mixture.MixtureFr_Coal[2]))/(cell_old[i].mixture.MixtureFr_Coal[1]+cell_old[i].mixture.MixtureFr_Coal[2]);

                    if(i<core_bdcell[0])
                    {
                        core_bdcell_new[0]--;
                        core_bdcell_new[1]--;
                    }
                    else if(i>=core_bdcell[0] && i<core_bdcell[1])
                    {
                        core_bdcell_new[1]--;
                    }
                    else
                        ;
                    
                    if(u_var<0.01 && mfr_var<0.01)  //small difference
                    {
                        if(mfr_right<mfr_center) // to merge the cell from right based on the ith cell
                        {
                            cell_neighbour = cell_old[i];
                            cell_neighbour.size = cell[i+k-k1].size+cell_old[i].size;
                            cell_neighbour.size_old = cell[i+k-k1].size_old+cell_old[i].size_old;
                            cell_neighbour.y = cell[i+k-k1].y+cell_old[i].size/2.0;
                            cell[i+k-k1] = cell_neighbour;
                        }
                        else  //to merge the cell from left based on the right cell
                        {
                            cell[i+k-k1].size += cell_old[i+1].size;
                            cell[i+k-k1].size_old += cell_old[i+1].size_old;
                            cell[i+k-k1].y -= cell_old[i+1].size/2.0;
                        }
                    }
                    else  //large difference, mixing the cell according to the conservation and mixing law
                    {
                        cell_neighbour = mfr_right>mfr_center?cell_old[i+1]:cell_old[i];
                        MERGECELL(&cell_neighbour, cell[i+k-k1], cell_old[i+1]);
                        //    printf("merging cells from right_largediff: cell[%d].size=%8.5f, cell[%d].u=%8.5f,cell_neighbour.size=%8.5f,cell_neighbour.u=%8.5f, cell_final.size=%8.5f, cell_final.u=%8.5f\n",i,cell[i+k-k1].size*1000,i,cell[i+k-k1].u,cell_old[i+1].size*1000,cell_old[i+1].u,cell_neighbour.size*1000, cell_neighbour.u);
                        cell[i+k-k1] = cell_neighbour;
                    }
                    cell_old[i+1] = cell[i+k-k1];  //to restore the cell information of the merged cell to replace the old cell information
                    k1++;
                    i++;  //to skip the next cell(very important!!!)
                }
            }
            else if(cell_old[i].size < 0.4*dy_cen)  //Case 1-1 ?
            {
                u_var = fabs(cell_old[i].u-cell_old[i-1].u)>fabs(cell_old[i].u-cell_old[i+1].u)? fabs(cell_old[i].u-cell_old[i+1].u):fabs(cell_old[i].u-cell_old[i-1].u);

                // the velocity between those two cells are smooth enough, Case1-2 || Case2
                if(u_var/cell_old[i].u<0.005)
                {
                    // to identify which cell should merge the small cell!
                    cell_neighbour = fabs(cell_old[i].u-cell_old[i-1].u)>fabs(cell_old[i].u-cell_old[i+1].u)?cell_old[i+1]:cell_old[i-1];
                    // to judge merge left or right?
                    index = fabs(cell_old[i].u-cell_old[i-1].u)>fabs(cell_old[i].u-cell_old[i+1].u)?1:0;  // 1 means i+1/right, 0 means i-1/left 
                    mfr_var = fabs((cell_old[i].mixture.MixtureFr_Coal[1]+cell_old[i].mixture.MixtureFr_Coal[2])-(cell_neighbour.mixture.MixtureFr_Coal[1]+cell_neighbour.mixture.MixtureFr_Coal[2]));
                    //the species between those two cells r smooth enough, Case1-3 || Case2
                    if(mfr_var/(cell_old[i].mixture.MixtureFr_Coal[1]+cell_old[i].mixture.MixtureFr_Coal[2]+0.0001)<0.005 && cell_old[i].size/cell_neighbour.size<1.5 && cell_old[i].size/cell_neighbour.size>0.5)
                    {
                        //merge left, and to keep the core boundary not changed by merging
                        if(index==0 && ((i!=core_bdcell[0])||(i!=core_bdcell[1]+1)))
                        {
                            if(cell_old[i-1].size == cell[i+k-k1-1].size)  //no left merge before
                            {
                                //      printf("merging cells from left: cell[%d].size=%8.5f, cell[%d].u=%8.5f,cell_neighbour.size=%8.5f,cell_neighbour.u=%8.5f\n",i,cell[i+k-k1-1].size*1000,i+k-k1-1,cell[i+k-k1-1].u,cell_neighbour.size*1000,cell_neighbour.u);
                                if(i<core_bdcell[0])
                                {
                                    core_bdcell_new[0]--;
                                    core_bdcell_new[1]--;
                                }
                                else if(i>core_bdcell[0] && i<=core_bdcell[1])
                                {
                                    core_bdcell_new[1]--;
                                }
                                else
                                    ;
                                
                                // to merge the cell from left
                                cell[i+k-k1-1].size += cell_old[i].size;
                                cell[i+k-k1-1].size_old += cell_old[i].size_old;
                                cell[i+k-k1-1].y += cell_old[i].size/2.0;
                                k1++;
                            }
                            else  //the previous cell has been merged to the left or right, this cell is then merged to the right.
                                index=1;  //merge right
                        }
                        //merge right
                        if(index==1 && ((i!=core_bdcell[0]-1)||(i!=core_bdcell[1])))
                        {
                            //     printf("merging cells from right: cell[%d].size=%8.5f, cell[%d].u=%8.5f,cell_neighbour.size=%8.5f,cell_neighbour.u=%8.5f\n",i,cell[i].size*1000,i,cell[i].u,cell_neighbour.size*1000,cell_neighbour.u);
                            if(i<core_bdcell[0])
                            {
                                core_bdcell_new[0]--;
                                core_bdcell_new[1]--;
                            }
                            else if(i>=core_bdcell[0] && i<core_bdcell[1])
                            {
                                core_bdcell_new[1] --;
                            }
                            else
                                ;
                            
                            cell[i+k-k1] = cell_old[i+1];
                            cell[i+k-k1].size += cell_old[i].size;
                            cell[i+k-k1].size_old += cell_old[i].size_old;
                            cell[i+k-k1].y -= cell_old[i].size/2.0;
                            i++;  //since the right cell has been merged, this cell should be skipped in the following step
                            k1++;
                        }
                    }
                }
            }
        }
    }

    // to calculate new cell numbers after merging step
    old_cellnumber -= k1;

    // to record the merged cell array
    for(i=0; i<old_cellnumber; i++)
        cell_old[i] = cell[i];

    core_bdcell[0] = core_bdcell_new[0];
    core_bdcell[1] = core_bdcell_new[1];


    //to split large cells
    //Case1. cell[i].size > 3*dy_co
    //Case2.
    //  cell[i].size>1.5*dy_co && cell[i].u>0.7*cellnext.u  (cellnext=cell[i+1])
    //Case3.
    // 1. cell[i].size > 0.5*dy_cen
    // 2. cell[i].u>0.7*cellneighbour.u
    // 3. ((cell[i].mixture.MixtureFr_Coal[1]+cell[i].mixture.MixtureFr_Coal[2])-(cellnext.mixture.MixtureFr_Coal[1]+cellnext.mixture.MixtureFr_Coal[2]))/(cell[i].mixture.MixtureFr_Coal[1]+cell[i].mixture.MixtureFr_Coal[2])>0.2 && (cell[i].mixture.Tem-cellnext.mixture.Tem)>100 && (cell[i].u-cellnext.u)/cell[i].u>0.3
    // 4. cell[i].size/cellnext.size >2.5, and mfr[i]/mfr_next>1.25 to split in ratio!

    for(i=0; i<old_cellnumber; i++)
    {
        cell[i+k] = cell_old[i];
        // too large cell.size, split it into 2 pieces
        if(i>10 && i<old_cellnumber-10)
        {
            //Case4
            if(cell_old[i].size/cell_old[i+1].size>2.5)  //this cell>cellnext, to split this cell, but not skip the next cell judgement(no i++)
            {
                if(cell_old[i].size*cell_old[i].u*cell_old[i].mixture.Density>1.25*cell_old[i+1].size*cell_old[i+1].u*cell_old[i+1].mixture.Density)
                {
                    if(i<core_bdcell[0])
                    {
                        core_bdcell_new[0]++;
                        core_bdcell_new[1]++;
                    }
                    else if(i>=core_bdcell[0] && i<=core_bdcell[1])
                    {
                        core_bdcell_new[1]++;
                    }
                    else
                        ;
                    size_ratio = cell_old[i].size/cell_old[i+1].size;
                    
                    cell[i+k] = cell_old[i];
                    cell[i+k+1] = cell_old[i];
                    // to make cell_new[i+k].size/cell_new[i+k+1].size = cell_new[i+k+1].size/cell_old[i+1].size
                    cell[i+k].size = cell_old[i+1].size*(2.0*size_ratio+1-sqrt(1+4*size_ratio))/2.0;
                    cell[i+k+1].size = cell_old[i+1].size*(sqrt(1+4*size_ratio)-1)/2.0;
                    
                    size_ratio = (sqrt(1+4*size_ratio)-1)/2.0;
                    cell[i+k].size_old = cell_old[i].size_old*size_ratio/(size_ratio+1);
                    cell[i+k+1].size_old = cell_old[i].size_old/(size_ratio+1);
                    
                    cell[i+k].y = cell_old[i].y-cell_old[i].size/2.0+cell[i+k].size/2.0;
                    cell[i+k+1].y = cell_old[i].y-cell_old[i].size/2.0+cell[i+k].size+cell[i+k+1].size/2.0;
                    
                    //    printf("split C4_left!: cell[%d].size=%8.5f,cell[%d].size=%8.5f, cell_new[%d].size=%8.5f, cell_new[%d].size=%8.5f\n",i,cell_old[i].size*1000,i+1,cell_old[i+1].size*1000,i+k,cell[i+k].size*1000,i+k+1,cell[i+k+1].size*1000);
                    k++;
                }

            }
            else if(cell_old[i+1].size/cell_old[i].size>2.5)  //cellnext>cell[i], to split next cell, and to skip the next cell(i++)
            {
                if(cell_old[i+1].size*cell_old[i+1].u*cell_old[i+1].mixture.Density>1.25*cell_old[i].size*cell_old[i].u*cell_old[i].mixture.Density)
                {
                    i++;  //to split the right cell and to skip the next cell
                    
                    if(i<core_bdcell[0])
                    {
                        core_bdcell_new[0]++;
                        core_bdcell_new[1]++;
                    }
                    else if(i>=core_bdcell[0] && i<=core_bdcell[1])
                    {
                        core_bdcell_new[1]++;
                    }
                    else
                        ;
                    size_ratio = cell_old[i].size/cell_old[i-1].size;
                    
                    cell[i+k] = cell_old[i];
                    cell[i+k+1] = cell_old[i];
                    // to make cell_new[i+k].size/cell_new[i+k+1].size = cell_new[i+k+1].size/cell_old[i+1].size
                    cell[i+k].size = cell_old[i-1].size*(sqrt(1+4*size_ratio)-1)/2.0;
                    cell[i+k+1].size = cell_old[i-1].size*(2.0*size_ratio+1-sqrt(1+4*size_ratio))/2.0;
                    
                    size_ratio = (sqrt(1+4*size_ratio)-1)/2.0;
                    cell[i+k].size_old = cell_old[i].size_old/(size_ratio+1);
                    cell[i+k+1].size_old = cell_old[i].size_old*size_ratio/(size_ratio+1);
                    
                    cell[i+k].y = cell_old[i].y-cell_old[i].size/2.0+cell[i+k].size/2.0;
                    cell[i+k+1].y = cell_old[i].y-cell_old[i].size/2.0+cell[i+k].size+cell[i+k+1].size/2.0;
                    
                    //    printf("split C4_right!: cell[%d].size=%8.5f, cell[%d].size=%8.5f, cell_new[%d].size=%8.5f, cell_new[%d].size=%8.5f\n",i-1, cell_old[i-1].size*1000, i,cell_old[i].size*1000,i+k,cell[i+k].size*1000,i+k+1,cell[i+k+1].size*1000);
                    
                    k++;
                }
            }
            else if(cell_old[i].size > 1.4*dy_co) //Case 1, Case2
            {
                if(cell_old[i].u>0.7*cell_old[i+1].u || cell_old[i].size>3.0*dy_co)
                {
                    if(i<core_bdcell[0])
                    {
                        core_bdcell_new[0]++;
                        core_bdcell_new[1]++;
                    }
                    else if(i>=core_bdcell[0] && i<=core_bdcell[1])
                    {
                        core_bdcell_new[1]++;
                    }
                    else
                        ;

                    cell[i+k] = cell_old[i];
                    cell[i+k+1] = cell_old[i];
                    cell[i+k].size = cell_old[i].size/2.0;
                    cell[i+k].size_old = cell_old[i].size_old/2.0;
                    cell[i+k].y = cell_old[i].y-cell_old[i].size/4.0;
                    cell[i+k+1].size = cell_old[i].size/2.0;
                    cell[i+k+1].size_old = cell_old[i].size_old/2.0;
                    cell[i+k+1].y = cell_old[i].y+cell_old[i].size/4.0;
                  
                    //    printf("split!Case1&2: cell[%d].size=%8.5f, cell_new[%d].size=%8.5f, cell_new[%d].size=%8.5f\n",i,cell_old[i].size*1000,i+k,cell[i+k].size*1000,i+k+1,cell[i+k+1].size*1000);
                    k++;
                }
            }
            else if(cell_old[i].size > 0.8*dy_cen)   //Case3, big variance between the neighbour cell, split both of them
            {
                if(cell_old[i].u>0.7*cell_old[i+1].u)
                {
                    if(fabs(cell_old[i].mixture.Tem-cell_old[i-1].mixture.Tem)>50)
                    {
                        mfr_var = ((cell_old[i].mixture.MixtureFr_Coal[1]+cell_old[i].mixture.MixtureFr_Coal[2])-(cell_old[i+1].mixture.MixtureFr_Coal[1]+cell_old[i+1].mixture.MixtureFr_Coal[2]))/(cell_old[i].mixture.MixtureFr_Coal[1]+cell_old[i].mixture.MixtureFr_Coal[2]);
                    
                        u_var = (cell_old[i].u-cell_old[i+1].u)/cell_old[i].u;
                        if(mfr_var>0.2 || u_var>0.5)
                        {
                            // to split the cell with the larger  mass flow rate
                            if(cell_old[i].u*cell_old[i].size*cell_old[i].mixture.Density>1.6*cell_old[i+1].u*cell_old[i+1].size*cell_old[i+1].mixture.Density)
                            {
                                if(i<core_bdcell[0])
                                {
                                    core_bdcell_new[0]++;
                                    core_bdcell_new[1]++;
                                }
                                else if(i>=core_bdcell[0] && i<=core_bdcell[1])
                                {
                                    core_bdcell_new[1]++;
                                }
                                else
                                    ;
                                
                                cell[i+k] = cell_old[i];
                                cell[i+k+1] = cell_old[i];
                                cell[i+k].size = cell_old[i].size/2.0;
                                cell[i+k].size_old = cell_old[i].size_old/2.0;
                                cell[i+k].y = cell_old[i].y-cell_old[i].size/4.0;
                                cell[i+k+1].size = cell_old[i].size/2.0;
                                cell[i+k+1].size_old = cell_old[i].size_old/2.0;
                                cell[i+k+1].y = cell_old[i].y+cell_old[i].size/4.0;
                                
                                //         printf("split!Case3_left!: cell[%d].size=%8.5f, cell_new[%d].size=%8.5f, cell_new[%d].size=%8.5f\n",i,cell_old[i].size*1000,i+k,cell[i+k].size*1000,i+k+1,cell[i+k+1].size*1000);
                                k++;
                            }
                            else if(1.6*cell_old[i].u*cell_old[i].size*cell_old[i].mixture.Density<cell_old[i+1].u*cell_old[i+1].size*cell_old[i+1].mixture.Density) // split right 
                            {
                                i++;  //to split the right cell and to skip this cell for next step
 if(i<core_bdcell[0])
                                {
                                    core_bdcell_new[0]++;
                                    core_bdcell_new[1]++;
                                }
                                else if(i>=core_bdcell[0] && i<=core_bdcell[1])
                                {
                                    core_bdcell_new[1]++;
                                }
                                else
                                    ;
                                
                                cell[i+k] = cell_old[i];
                                cell[i+k+1] = cell_old[i];
                                cell[i+k].size = cell_old[i].size/2.0;
                                cell[i+k].size_old = cell_old[i].size_old/2.0;
                                cell[i+k].y = cell_old[i].y-cell_old[i].size/4.0;
                                cell[i+k+1].size = cell_old[i].size/2.0;
                                cell[i+k+1].size_old = cell_old[i].size_old/2.0;
                                cell[i+k+1].y = cell_old[i].y+cell_old[i].size/4.0;
                                
                                //      printf("split!Case3_right!: cell[%d].size=%8.5f, cell_new[%d].size=%8.5f, cell_new[%d].size=%8.5f\n",i,cell_old[i].size*1000,i+k,cell[i+k].size*1000,i+k+1,cell[i+k+1].size*1000);
                                k++;
                            }
                        }
                    }
                }
            }
        }
    }

    new_cellnumber = old_cellnumber+k;
    domain_cellnumber[1] = new_cellnumber;
    domain_cellnumber[2] = new_cellnumber;

    core_bdcell[0] = core_bdcell_new[0];
    core_bdcell[1] = core_bdcell_new[1];

    for(i=0; i<new_cellnumber; i++)
        domain_width += cell[i].size;

    grid_y[0] = -domain_width/2.0;
    for(i=1; i<=new_cellnumber; i++)
        grid_y[i] = grid_y[i-1]+cell[i-1].size;

    for(i=0; i<new_cellnumber; i++)
        cell[i].y = (grid_y[i]+grid_y[i+1])/2.0;

//      if(k>0 || k1>0)
//      {
//          printf("%dth step, %d cells are splited, %d cells are merged, old_cellnumber=%d, new_number= %d\n",E_S_number, k,k1,old_cellnumber+k1, new_cellnumber);
//          printf("d_l=%8.5f, core_l=%8.5f, core_r=%8.5f, d_r=%8.5f\n", grid_y[0], grid_y[core_bdcell[0]],grid_y[core_bdcell[1]+1],grid_y[new_cellnumber]);
//      }
    
}






//to split or merge cells whose size is too large or too small
void MESH_MANAGEMENT_OLD(int E_S_number, int *core_bdcell, int *domain_cellnumber, double *grid_y,struct cell_information *cell)
{
    int i,k,k1,old_cellnumber,new_cellnumber,index;
    int core_bdcell_new[2];
    double domain_width=0.0;
    double size_var,u_var,mfr_var;
    struct cell_information cell_old[domain_cellnumber[1]],cell_neighbour;

    old_cellnumber = domain_cellnumber[1];
    new_cellnumber = domain_cellnumber[1];

    core_bdcell_new[0] = core_bdcell[0];
    core_bdcell_new[1] = core_bdcell[1];

    //to restore the orginal cell information
    for(i=0; i<old_cellnumber; i++)
        cell_old[i] = cell[i];

    // k1 means how many cells r merged during this odt step, k+k1 means how many cells r splitted during this odt step
    k1 = 0;
    k = 0;

    // first to merge small cells!!!!
    for(i=0; i<old_cellnumber; i++)
    {
        cell[i+k-k1] = cell_old[i];
        //too small cell.size, need for further steps
        // to merge cells should be satisfy such conditions as:
        // 1. cell.size < 0.8*dy_cen, usually, too small cells is caused by the increment of velocity for the coflow cells
        // 2. fabs(cell.u-cell_next.u)/cell.u <0.01
        // 3. fabs((cell.mixture_fraction-cell_nex.mixture_fraction)/(max(cell.mfr, cell_next.mfr)))<0.01 
        if(cell_old[i].size < 0.7*dy_cen && i>5 && i<old_cellnumber-5)  //no merge for boundary cells!
        {
            u_var = fabs(cell_old[i].u-cell_old[i-1].u)>fabs(cell_old[i].u-cell_old[i+1].u)? fabs(cell_old[i].u-cell_old[i+1].u):fabs(cell_old[i].u-cell_old[i-1].u);

            // the velocity between those two cells are smooth enough
            if(u_var/cell_old[i].u<0.005 || cell_old[i].size < 0.3*dy_cen)
            {
                // to identify which cell should merge the small cell!
                cell_neighbour = fabs(cell_old[i].u-cell_old[i-1].u)>fabs(cell_old[i].u-cell_old[i+1].u)?cell_old[i+1]:cell_old[i-1];
                index = fabs(cell_old[i].u-cell_old[i-1].u)>fabs(cell_old[i].u-cell_old[i+1].u)?1:0;  // 1 means i+1/right, 0 means i-1/left 
                mfr_var = fabs(cell_old[i].mixture.Mixture_Fraction[4]-cell_neighbour.mixture.Mixture_Fraction[4]);
                //the species between those two cells r smooth enough
                if(mfr_var/((cell_old[i].mixture.Mixture_Fraction[4]+0.0001)>cell_neighbour.mixture.Mixture_Fraction[4]?(cell_old[i].mixture.Mixture_Fraction[4]+0.0001):cell_neighbour.mixture.Mixture_Fraction[4])<0.005 || cell_old[i].size<0.3*dy_cen)
                {
                    //merge left, but to keep the core boundary not changed by merging
                    if(index==0 && ((i!=core_bdcell[0])||(i!=core_bdcell[1]+1)))
                    {
                        //                        printf("merging cells from left: cell[%d].size=%8.5f, cell[%d].u=%8.5f,cell_neighbour.size=%8.5f,cell_neighbour.u=%8.5f\n",i,cell[i].size*1000,i,cell[i].u,cell_neighbour.size*1000,cell_neighbour.u);
                        if(i<core_bdcell[0])
                        {
                            core_bdcell_new[0]--;
                            core_bdcell_new[1]--;
                        }
                        else if(i>core_bdcell[0] && i<=core_bdcell[1])
                        {
                            core_bdcell_new[1]--;
                        }
                        else
                            ;

                        cell[i+k-k1-1].size += cell_old[i].size;
                        cell[i+k-k1-1].size_old += cell_old[i].size_old;
                        cell[i+k-k1-1].y += cell_old[i].size/2.0;
                        k1++;
                    }
                    //merge right
                    if(index==1 && ((i!=core_bdcell[0]-1)||(i!=core_bdcell[1])))
                    {
                        //                        printf("merging cells from right: cell[%d].size=%8.5f, cell[%d].u=%8.5f,cell_neighbour.size=%8.5f,cell_neighbour.u=%8.5f\n",i,cell[i].size*1000,i,cell[i].u,cell_neighbour.size*1000,cell_neighbour.u);
                        if(i<core_bdcell[0])
                        {
                            core_bdcell_new[0]--;
                            core_bdcell_new[1]--;
                        }
                        else if(i>=core_bdcell[0] && i<core_bdcell[1])
                        {
                            core_bdcell_new[1] --;
                        }
                        else
                            ;

                        cell[i+k-k1] = cell_old[i+1];
                        cell[i+k-k1].size += cell_old[i].size;
                        cell[i+k-k1].size_old += cell_old[i].size_old;
                        cell[i+k-k1].y += cell_old[i].size/2.0;
                        i++;  //since the right cell has been merged, this cell should be skipped in the following step
                        k1++;
                    }
                    //                    k1++;
                }
            }
        }
    }

    // to calculate new cell numbers after merging step
    old_cellnumber -= k1;

    // to record the merged cell array
    for(i=0; i<old_cellnumber; i++)
        cell_old[i] = cell[i];

    core_bdcell[0] = core_bdcell_new[0];
    core_bdcell[1] = core_bdcell_new[1];
    //to split large cells
    for(i=0; i<old_cellnumber; i++)
    {
        cell[i+k] = cell_old[i];
        // too large cell.size, split it into 2 pieces
        if(cell_old[i].size > 1.3*dy_co)
        {
            if(cell_old[i].u>0.8*covelocity || cell_old[i].size>3.0*dy_co)
            {
                if(i<core_bdcell[0])
                {
                    core_bdcell_new[0]++;
                    core_bdcell_new[1]++;
                }
                else if(i>=core_bdcell[0] && i<=core_bdcell[1])
                {
                    core_bdcell_new[1]++;
                }
                else
                    ;

                cell[i+k] = cell_old[i];
                cell[i+k+1] = cell_old[i];
                cell[i+k].size = cell_old[i].size/2.0;
                cell[i+k].size_old = cell_old[i].size_old/2.0;
                cell[i+k].y = cell_old[i].y-cell_old[i].size/4.0;
                cell[i+k+1].size = cell_old[i].size/2.0;
                cell[i+k+1].size_old = cell_old[i].size_old/2.0;
                cell[i+k+1].y = cell_old[i].y+cell_old[i].size/4.0;
                //                printf("split!: cell[%d].size=%8.5f, cell_new[%d].size=%8.5f, cell_new[%d].u=%8.5f\n",i,cell_old[i].size*1000,i+k,cell[i+k].size*1000,i+k+1,cell[i+k+1].u);
                k++;
            }
        }
    }

    new_cellnumber = old_cellnumber+k;
    domain_cellnumber[1] = new_cellnumber;
    domain_cellnumber[2] = new_cellnumber;

    core_bdcell[0] = core_bdcell_new[0];
    core_bdcell[1] = core_bdcell_new[1];

    for(i=0; i<new_cellnumber; i++)
        domain_width += cell[i].size;

    grid_y[0] = -domain_width/2.0;
    for(i=1; i<=new_cellnumber; i++)
        grid_y[i] = grid_y[i-1]+cell[i-1].size;

    for(i=0; i<new_cellnumber; i++)
        cell[i].y = (grid_y[i]+grid_y[i+1])/2.0;

//      if(k>0 || k1>0)
//      {
//          printf("%dth step, %d cells are splited, %d cells are merged, old_cellnumber=%d, new_number= %d\n",E_S_number, k,k1,old_cellnumber+k1, new_cellnumber);
//          printf("d_l=%8.5f, core_l=%8.5f, core_r=%8.5f, d_r=%8.5f\n", grid_y[0], grid_y[core_bdcell[0]],grid_y[core_bdcell[1]+1],grid_y[new_cellnumber]);
//      }
    
}


