#include "header.h"

/**********************************************************************
this file is used to counter the 2-way coupling interaction between
the gas phase and coal particles. The source term is give in particle-
tracking process, here, the source term will be added to the gas phase.
The flow field and scalar field of the gas phase would be changed.
**********************************************************************/


// this function is used to consider 2-way coupling between gas phase and particles. the source term had been given, the gas phase flow field would be changed.
void TWO_WAY_COUPLING_SOURCE(int domain_cellnumber, struct cell_information *cell, struct source_information *source)
{
    int i,k;
    double massfr_total,new_mflr, old_mflr;
    double mom;
    double H_cp_ave; //H_cp_ave = (cell[i].mixture.Heat_Capacity+cell_new[i].mixture.Heat_Capacity)/2.0;
    double density_ratio; //density_ratio = cell_new[i].mixture.Tem/cell[i].mixutre.Tem
    struct cell_information cell_new[domain_cellnumber];

    for(i=0; i<domain_cellnumber; i++)
        cell_new[i] = cell[i];

    // to renew the scalar field according to the given source term
    for(i=0; i<domain_cellnumber; i++)
    {
        old_mflr = cell[i].size*cell[i].mixture.Density*cell[i].u;
        new_mflr = old_mflr+source[i].mass;  // the new mass flux
        if(!(new_mflr>0 && new_mflr<1))
           printf("new_mflr[%d] wierd! old_mflr=%f, source.mass=%f", i, old_mflr, source[i].mass);
        massfr_total = 0.0;
        for(k=0; k<Species_Number; k++)  // to renew mass fraction for each species
        {
            cell_new[i].mixture.Massfr[k] = (source[i].massflux_species[k]+old_mflr*cell[i].mixture.Massfr[k])/new_mflr;
            if(cell_new[i].mixture.Massfr[k]<-1e-6)
            {
                printf("too large source for species:%s, Mfr_old=%f, Mfr_new=%f, source=%f\n",Species_Name[k],cell[i].mixture.Massfr[k],cell_new[i].mixture.Massfr[k],source[i].massflux_species[k]);
                cell_new[i].mixture.Massfr[k] = 0.0;
            }
            else if(cell_new[i].mixture.Massfr[k]>=-1e-6 && cell_new[i].mixture.Massfr[k]<0)
                cell_new[i].mixture.Massfr[k] = 0.0;
            massfr_total += cell_new[i].mixture.Massfr[k];
        }

        if(!(fabs(massfr_total-1)<=1e-6))
            printf("massfr_total wierd! total=%13.10f, %f, %f\n",(massfr_total-1)*1e6,source[i].mass*1e6, source[i].massflux_species[H2O]*1e6);

        for(k=0; k<Species_Number; k++)  // uniform mass fraction (should not be used in future)
        {
            cell_new[i].mixture.Massfr[k] /= massfr_total;
            if(cell_new[i].mixture.Massfr[k] <0.0)
                printf("wierd for %dth species in %dth cell, %f\n", k, i,cell_new[i].mixture.Massfr[k]);
        }

        // actually, only cell_new.density and cell_new.mixture.Hcp is needed, this function can be rewrite to save time
        
        RENEW_MIXPRO_MASSY(i, cell_new);   //to renew mixture properties based on given mass fraction of each species
        
        if(!(cell_new[i].mixture.Tem>273 && cell_new[i].mixture.Tem<5000))
            printf("Tem wierd!!!!0 i=%d, source.h=%f, Tem_old=%f, Tem_new=%f\n", i, source[i].enthalpy, cell[i].mixture.Tem, cell_new[i].mixture.Tem);

        // to renew velocities for the gas phase
        mom = source[i].mom_u+old_mflr*cell[i].u;
        if(!(mom > 0.0))
            printf("too large source for mom_u=%f, mom=%f\n",source[i].mom_u, mom);
        else
            cell_new[i].u = mom/new_mflr;  // to calculate new axial velocity

//         mom = source[i].mom_v+old_mflr*(cell[i].v_lateral);
//         if(mom < 0.0)
//             printf("too large source for mom_v=%f, mom=%f\n",source[i].mom_v, mom);
//         else
//             cell_new[i].v_lateral = mom/new_mflr;

//         mom = source[i].mom_w+old_mflr*cell[i].w;
//         if(mom < 0.0)
//             printf("too large source for mom_w=%f, mom=%f\n",source[i].mom_w, mom);
//         else
//             cell_new[i].w = mom/new_mflr;

        cell_new[i].size = new_mflr/(cell_new[i].mixture.Density*cell_new[i].u);  //to get the final cell size under a constant temperature

        if(!(cell_new[i].size/cell[i].size<10 && cell_new[i].size/cell[i].size>0.1))
            printf("size wierd! old=%f, new=%f, old_u=%f, new_u=%f", cell[i].size, cell_new[i].size, cell[i].u, cell_new[i].u);

        H_cp_ave = cell_new[i].mixture.Heat_Capacity*1000/cell_new[i].mixture.Molecular_Weight;  // to notice the unit! should be J/(kg.K)!!!

        cell_new[i].mixture.Tem = (source[i].enthalpy+old_mflr*cell[i].mixture.Heat_Capacity*1000/cell[i].mixture.Molecular_Weight*cell[i].mixture.Tem)/(new_mflr*H_cp_ave);

        if(!(cell_new[i].mixture.Tem>273 && cell_new[i].mixture.Tem<5000))
            printf("Tem wierd!!!!1 i=%d, source.h=%f, Tem_old=%f, Tem_new=%f\n", i, source[i].enthalpy, cell[i].mixture.Tem, cell_new[i].mixture.Tem);

        // only cp is needed, this function can be rewrite to save cpu time
        RENEW_MIXPRO_MASSY(i, cell_new);  //to get the cell properties based on new temperature

        H_cp_ave = (H_cp_ave+cell_new[i].mixture.Heat_Capacity*1000/cell_new[i].mixture.Molecular_Weight)/2.0;  // to get the final heat capacity

        cell_new[i].mixture.Tem = (source[i].enthalpy+old_mflr*cell[i].mixture.Heat_Capacity*1000/cell[i].mixture.Molecular_Weight*cell[i].mixture.Tem)/(new_mflr*H_cp_ave);  // to get the final temperature

        if(!(cell_new[i].mixture.Tem>273 && cell_new[i].mixture.Tem<5000))
            printf("Tem wierd!!!!2 i=%d, source.h=%f, Tem_old=%f, Tem_new=%f\n", i, source[i].enthalpy, cell[i].mixture.Tem, cell_new[i].mixture.Tem);

        RENEW_MIXPRO_MASSY(i, cell_new); // to get the final properties

        density_ratio = cell_new[i].mixture.Tem/cell[i].mixture.Tem;  // also = cell_new[i].mixture.Density'/cell_new[i].mixture.Density

        DILATATION_MODLE(density_ratio, &(cell_new[i]));  // to change cell size and velocity
        cell[i] = cell_new[i];   // to get the final value
    }
}






// the dilatation model, cell.size and cell.u should be changed in this model when cell.Tem changes
// density_ratio = density_old/density_new = Tem_new/Tem_old
void DILATATION_MODLE(double density_ratio, struct cell_information *cell)
{
    //    cell->size *= density_ratio;
    cell->size *= pow(density_ratio, (2./3.));
    cell->u *= pow(density_ratio, (1./3.));
}


