#include "header.h"




//*************************************************************************************//



//*************************************************************************************//
// Forward_Euler explicit method to solve species diffusion process
void SPECIES_DIFFUSION_FE(int E_S_number, int *SpeDiff_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source)
{
    int i,k,i1;
    double Tem, Pre;
    double mass_fr;   // old mass flow rate for ith cell
    double sum_massfr, mix_density;  // summation of mole fraction from 1st species to (Species_Number-1)th species, but not for 0th species!!!
    //  a[i]u[i-1]+b[i]u[i]+c[i]u[i+1]=r, vector of tridiaginal matrix
    //    double a[domain_cellnumber],b[domain_cellnumber],c[domain_cellnumber],r[domain_cellnumber];
    double A0,B0,B1,B2;
    double RMassY[domain_cellnumber], r; // cell[i].mixture.Massfr[k]!!!
    //    double Density[domain_cellnumber][Species_Number];   // density of each species;
    extern double Density_Standard[Species_Number];

    struct cell_information cell_new[domain_cellnumber];


    //to set boundary conditions and record the initial values of each step	
    for(i=0; i<domain_cellnumber; i++)
    {   
        cell_new[i] = cell[i];
        for(k=0; k<Species_Number; k++)
        { 
            if(cell[i].mixture.Molefr[k]<0-TINY||cell[i].mixture.Molefr[k]>1+1e-6 )
            {
                printf("wierd! %d,%s,%20.18f\n",i,Species_Name[k],cell[i].mixture.Molefr[k]);
                goto abnormal2;
            }
        }
    }

    //to solve all species diffusion process, include 0th species, then to normalize the species mass fraction
    for(k=0; k<Species_Number; k++)
    {
        // to calculate vector a[],b[],c[] and d[]
        for(i=0; i<domain_cellnumber; i++)
        {
            if(i==0)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old; // cell.u has changed, shouldn't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[domain_cellnumber-1].mixture.Density)/2.0;
                B2 = 2*(cell[i].mixture.Spe_Diff_Coeff[k]*cell[i+1].size_old+cell[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i+1].size_old+cell[i+1].mixture.Density*dy_co)/pow((dy_co+cell[i+1].size_old),3);
                B0 = B1+B2;

                r = A0*B1*cell[domain_cellnumber-1].mixture.Massfr[k]+(mass_fr-A0*B0)*cell[i].mixture.Massfr[k]+A0*B2*cell[i+1].mixture.Massfr[k];  //ghost point for cell[domain_cellnumber-1]
                r += 0.0;  // to consider additional diffusion due to other facts: thermal diffusion et al.
                r += source[i].massflux_species[k];  // to consider reaction source term. Here, the reaction will be considered separately, but particle source term should be considered in this part.
                RMassY[i] = r/(mass_fr+source[i].mass);
            }
            else if(i==domain_cellnumber-1)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old;  // cell.u has changed, can't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = 2*(cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i-1].size_old+cell[i-1].mixture.Density*dy_co)/pow((cell[i-1].size_old+dy_co),3);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[0].mixture.Density)/2.0;
                B0 = B1+B2;
            
                r = A0*B1*cell[i-1].mixture.Massfr[k]+(mass_fr-A0*B0)*cell[i].mixture.Massfr[k]+A0*B2*cell[0].mixture.Massfr[k];  //ghost point for cell[0]
                r += 0.0;  // to consider addition diffusion 
                r += source[i].massflux_species[k];  // to consider reaction source term, including particle reaction source
                RMassY[i] = r/(mass_fr+source[i].mass);
            }
            else
            {
                mass_fr = cell[i].u_old*cell[i].size_old*cell[i].Density_old; // the mass flow rate is the old mass flow rate! so .u, .size can't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = B2;
                //B1 = 2*(cell[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/pow(((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old),3);
                B2 = 2*(cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)*(cell[i].mixture.Density*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Density*cell[i].size_old)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old),3);
                B0 = B1+B2;

                r = A0*B1*cell[i-1].mixture.Massfr[k]+(mass_fr-A0*B0)*cell[i].mixture.Massfr[k]+A0*B2*cell[i+1].mixture.Massfr[k];

                r += 0.0;  // to consider additional diffusion
                r += source[i].massflux_species[k];
                RMassY[i] = r/(mass_fr+source[i].mass);
            }
        }

        for(i=0; i<domain_cellnumber; i++)
        {
            // to adjust RMassY if ignorable error occurs.
            RMassY[i] = (RMassY[i]<0. && RMassY[i]>-1e-7) ? 0.0 : RMassY[i];
            RMassY[i] = (RMassY[i]>1. && RMassY[i]<1+1e-7) ? 1.0 : RMassY[i];

            if(!(RMassY[i]>=0 && RMassY[i]<=1))
            {
                *SpeDiff_Flag = 0;
              //   printf("%dth step: error occurs during %dth species(%s) in 1st circulation in species diffusion process!cells=%d, cell number=%d, (*deta_x)=%f\n",E_S_number,k,Species_Name[k],i,domain_cellnumber,(*deta_x));
//                 printf("number  cell_size       cell_size_old   cell.u         cell.Massfr       cellnew.Massfr    cell.spedif     cell.density    cell.Tem\n");
//                 for(i1=0; i1<domain_cellnumber; i1++)
//                 {
//                     if(i1<10 || i1>domain_cellnumber-10)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1>i-20 && i1<i+20)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1%200==0)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);        
//                 }
     
//                 printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Density,cell[i+1].mixture.Density,cell[i-1].mixture.Density);
//                 printf("cell[0].mixture.Massfr[k]=%8.5f, cell[end].mixture.Massfr[k]=%8.5f\n",cell[0].mixture.Molefr[k],cell[domain_cellnumber-1].mixture.Massfr[k]);
                // to modify the diffusion step
                MODIFY_SPEDIFFDX(deta_x, cell[i].mixture.Massfr[k], RMassY[i]);

                goto abnormal2;
            } 
            else  //to store the new massfraction
                cell_new[i].mixture.Massfr[k] = RMassY[i];      
        } 
    }  //for(k=..


// uniform the mass fraction
    for(i=0; i<domain_cellnumber; i++)
    {
        sum_massfr = 0.0;
        for(k=0; k<Species_Number; k++)
            sum_massfr += cell_new[i].mixture.Massfr[k];
        for(k=0; k<Species_Number; k++)
            cell_new[i].mixture.Massfr[k] /= sum_massfr;
    }

    // to renew the mixture property based on the known mass fraction of each species in the mixture
    for(i=0; i<domain_cellnumber; i++)
        RENEW_MIXPRO_MASSY(i,cell_new);

    // to restore the changed value (mass fraction and mixture density)  
    for(i=0; i<domain_cellnumber; i++)
    {
//         cell_new[i].size_old = cell[i].size_old;
//         cell_new[i].u_old = cell[i].u_old;
//         cell_new[i].Density_old = cell[i].Density_old;
//         for(k=0; k<Species_Number; k++)
//             cell_new[i].Massfr_old[k] = cell[i].Massfr_old[k];
        cell[i] = cell_new[i];
        cell[i].size = cell_new[i].size*cell[i].Density_old/cell[i].mixture.Density;
    }

 abnormal2:    ;
}










//*************************************************************************************//
// Forward_Euler explicit method to solve species diffusion process
void SPECIES_DIFFUSION_2NDFE(int E_S_number, int *SpeDiff_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source)
{
    int i,k,i1;
    double Tem, Pre;
    double mass_fr;   // old mass flow rate for ith cell
    double sum_massfr, mix_density;  // summation of mole fraction from 1st species to (Species_Number-1)th species, but not for 0th species!!!
    double A0,B0,B1,B2;
    double B0_1, B1_1, B2_1;
    double RMassY[domain_cellnumber], r; // cell[i].mixture.Massfr[k]!!!
    //    double Density[domain_cellnumber][Species_Number];   // density of each species;
    extern double Density_Standard[Species_Number];

    struct cell_information cell_new[domain_cellnumber];


    //to set boundary conditions and record the initial values of each step	
    for(i=0; i<domain_cellnumber; i++)
    {   
        cell_new[i] = cell[i];
        for(k=0; k<Species_Number; k++)
        { 
            if(cell[i].mixture.Molefr[k]<0-TINY||cell[i].mixture.Molefr[k]>1+1e-6 )
            {
                printf("wierd! %d,%s,%20.18f\n",i,Species_Name[k],cell[i].mixture.Molefr[k]);
                goto abnormal2;
            }
        }
    }

    //to solve all species diffusion process, include 0th species, then to normalize the species mass fraction
    for(k=0; k<Species_Number; k++)
    {
        // to calculate vector a[],b[],c[] and d[]
        for(i=0; i<domain_cellnumber; i++)
        {
            if(i==0)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old; // cell.u has changed, shouldn't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].Density_old+cell[domain_cellnumber-1].Density_old)/4.0;
                B1_1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[domain_cellnumber-1].mixture.Density)/4.0;
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i+1].size_old+cell[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].Density_old*cell[i+1].size_old+cell[i+1].Density_old*dy_co)/pow((dy_co+cell[i+1].size_old),3);
                B2_1 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i+1].size+cell[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i+1].size+cell[i+1].mixture.Density*dy_co)/pow((dy_co+cell[i+1].size),3);
                B0 = B1+B2;
                B0_1 = B1_1+B2_1;

                r = A0*(B1+B1_1)*cell[domain_cellnumber-1].mixture.Massfr[k]+(mass_fr-A0*(B0+B0_1))*cell[i].mixture.Massfr[k]+A0*(B2+B2_1)*cell[i+1].mixture.Massfr[k];  //ghost point for cell[domain_cellnumber-1]
                r += 0.0;  // to consider additional diffusion due to other facts: thermal diffusion et al.
                r += source[i].massflux_species[k];  // to consider reaction source term. Here, the reaction will be considered separately, but particle source term should be considered in this part.
                RMassY[i] = r/(mass_fr+source[i].mass);
            }
            else if(i==domain_cellnumber-1)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old;  // cell.u has changed, can't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i-1].size_old+cell[i-1].mixture.Density*dy_co)/pow((cell[i-1].size_old+dy_co),3);
                B1_1 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i-1].size+cell[i-1].mixture.Density*dy_co)/pow((cell[i-1].size+dy_co),3);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[0].mixture.Density)/4.0;
                B2_1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[0].mixture.Density)/4.0;
                B0 = B1+B2;
                B0_1 = B1_1+B2_1;
            
                r = A0*(B1+B1_1)*cell[i-1].mixture.Massfr[k]+(mass_fr-A0*(B0+B0_1))*cell[i].mixture.Massfr[k]+A0*(B2+B2_1)*cell[0].mixture.Massfr[k];  //ghost point for cell[0]
                r += 0.0;  // to consider addition diffusion 
                r += source[i].massflux_species[k];  // to consider reaction source term, including particle reaction source
                RMassY[i] = r/(mass_fr+source[i].mass);
            }
            else
            {
                mass_fr = cell[i].u_old*cell[i].size_old*cell[i].Density_old; // the mass flow rate is the old mass flow rate! so .u, .size can't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = B2;
                B1_1 = B2_1;
                //B1 = 2*(cell[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/pow(((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old),3);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old),3);
                B2_1 = (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size)*(cell[i].mixture.Density*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Density*cell[i].size)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size),3);
                B0 = B1+B2;
                B0_1 = B1_1+B2_1;

                r = A0*(B1+B1_1)*cell[i-1].mixture.Massfr[k]+(mass_fr-A0*(B0+B0_1))*cell[i].mixture.Massfr[k]+A0*(B2+B2_1)*cell[i+1].mixture.Massfr[k];

                r += 0.0;  // to consider additional diffusion
                r += source[i].massflux_species[k];
                RMassY[i] = r/(mass_fr+source[i].mass);
            }
        }

        for(i=0; i<domain_cellnumber; i++)
        {
            // to adjust RMassY if ignorable error occurs.
            RMassY[i] = (RMassY[i]<0 && RMassY[i]>-1e-7) ? 0.0 : RMassY[i];
            RMassY[i] = (RMassY[i]>1 && RMassY[i]<1+1e-7) ? 1.0 : RMassY[i];

            if(!(RMassY[i]>=0 && RMassY[i]<=1))
            {
                *SpeDiff_Flag = 0;
               //  printf("%dth step: error occurs during %dth species(%s) in 1st circulation in species diffusion process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,k,Species_Name[k],i,domain_cellnumber,(*deta_x));
//                 printf("number  cell_size       cell_size_old   cell.u         cell.Massfr       cellnew.Massfr    cell.spedif     cell.density    cell.Tem\n");
//                 for(i1=0; i1<domain_cellnumber; i1++)
//                 {
//                     if(i1<10 || i1>domain_cellnumber-10)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1>i-20 && i1<i+20)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1%200==0)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);        
//                 }
     
// //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\n",E_S_number,cell[i].mixture.Massfr[k],i==domain_cellnumber-1?cell[0].mixture.Massfr[k]:cell[i+1].mixture.Massfr[k],i==0?cell[domain_cellnumber-1].mixture.Massfr[k]:cell[i-1].mixture.Massfr[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
// //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,RMassY[i],cell_new[i+1].mixture.Massfr[k],cell_new[i-1].mixture.Massfr[k],cell_new[i].size,cell[i+1].size,cell_new[i-1].size);
//                 printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Density,cell[i+1].mixture.Density,cell[i-1].mixture.Density);
//                 printf("cell[0].mixture.Massfr[k]=%8.5f, cell[end].mixture.Massfr[k]=%8.5f\n",cell[0].mixture.Molefr[k],cell[domain_cellnumber-1].mixture.Massfr[k]);

                // to modify the diffusion step
                MODIFY_SPEDIFFDX(deta_x, cell[i].mixture.Massfr[k], RMassY[i]);

                goto abnormal2;
            } 
            else  //to store the new massfraction
                cell_new[i].mixture.Massfr[k] = RMassY[i];      
        } 
    }  //for(k=..


// uniform the mass fraction
    for(i=0; i<domain_cellnumber; i++)
    {
        sum_massfr = 0.0;
        for(k=0; k<Species_Number; k++)
            sum_massfr += cell_new[i].mixture.Massfr[k];
        for(k=0; k<Species_Number; k++)
            cell_new[i].mixture.Massfr[k] /= sum_massfr;
    }

    // to renew the mixture density for each cell
    for(i=0; i<domain_cellnumber; i++)
        CALCU_MIXTURE_DENSITY(i, cell_new);



    //to solve all species diffusion process, include 0th species, then to normalize the species mass fraction
    for(k=0; k<Species_Number; k++)
    {
        // to calculate vector a[],b[],c[] and d[]
        for(i=0; i<domain_cellnumber; i++)
        {
            if(i==0)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old; // cell.u has changed, shouldn't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].Density_old+cell[domain_cellnumber-1].Density_old)/4.0;
                B1_1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell_new[i].mixture.Density+cell_new[domain_cellnumber-1].mixture.Density)/4.0;
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i+1].size_old+cell[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].Density_old*cell[i+1].size_old+cell[i+1].Density_old*dy_co)/pow((dy_co+cell[i+1].size_old),3);
                B2_1 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i+1].size+cell[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell_new[i].mixture.Density*cell[i+1].size+cell_new[i+1].mixture.Density*dy_co)/pow((dy_co+cell[i+1].size),3);
                B0 = B1+B2;
                B0_1 = B1_1+B2_1;

                r = A0*(B1*cell[domain_cellnumber-1].mixture.Massfr[k]+B1_1*cell_new[domain_cellnumber-1].mixture.Massfr[k])+(mass_fr-A0*(B0*cell[i].mixture.Massfr[k]+B0_1*cell_new[i].mixture.Massfr[k]))+A0*(B2*cell[i+1].mixture.Massfr[k]+B2_1*cell_new[i+1].mixture.Massfr[k]);  //ghost point for cell[domain_cellnumber-1]
                r += 0.0;  // to consider additional diffusion due to other facts: thermal diffusion et al.
                r += source[i].massflux_species[k];  // to consider reaction source term. Here, the reaction will be considered separately, but particle source term should be considered in this part.
                RMassY[i] = r/(mass_fr+source[i].mass);
            }
            else if(i==domain_cellnumber-1)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old;  // cell.u has changed, can't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].Density_old*cell[i-1].size_old+cell[i-1].Density_old*dy_co)/pow((cell[i-1].size_old+dy_co),3);
                B1_1 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell_new[i].mixture.Density*cell[i-1].size+cell_new[i-1].mixture.Density*dy_co)/pow((cell[i-1].size+dy_co),3);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].Density_old+cell[0].Density_old)/4.0;
                B2_1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell_new[i].mixture.Density+cell_new[0].mixture.Density)/4.0;
                B0 = B1+B2;
                B0_1 = B1_1+B2_1;
            
                r = A0*(B1*cell[i-1].mixture.Massfr[k]+B1_1*cell_new[i-1].mixture.Massfr[k])+(mass_fr-A0*(B0*cell[i].mixture.Massfr[k]+B0_1*cell_new[i].mixture.Massfr[k]))+A0*(B2*cell[0].mixture.Massfr[k]+B2_1*cell_new[0].mixture.Massfr[k]);  //ghost point for cell[0]
                r += 0.0;  // to consider addition diffusion 
                r += source[i].massflux_species[k];  // to consider reaction source term, including particle reaction source
                RMassY[i] = r/(mass_fr+source[i].mass);
            }
            else
            {
                mass_fr = cell[i].u_old*cell[i].size_old*cell[i].Density_old; // the mass flow rate is the old mass flow rate! so .u, .size can't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = B2;
                B1_1 = B2_1;
                //B1 = 2*(cell[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/pow(((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old),3);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old),3);
                B2_1 = (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size)*(cell_new[i].mixture.Density*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell_new[i+1].mixture.Density*cell[i].size)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size),3);
                B0 = B1+B2;
                B0_1 = B1_1+B2_1;

                r = A0*(B1*cell[i-1].mixture.Massfr[k]+B1_1*cell_new[i-1].mixture.Massfr[k])+(mass_fr-A0*(B0*cell[i].mixture.Massfr[k]+B0_1*cell_new[i].mixture.Massfr[k]))+A0*(B2*cell[i+1].mixture.Massfr[k]+B2_1*cell_new[i+1].mixture.Massfr[k]);

                r += 0.0;  // to consider additional diffusion
                r += source[i].massflux_species[k];
                RMassY[i] = r/(mass_fr+source[i].mass);
            }
        }

        for(i=0; i<domain_cellnumber; i++)
        {
            // to adjust RMassY if ignorable error occurs.
            RMassY[i] = (RMassY[i]<0 && RMassY[i]>-1e-7) ? 0.0 : RMassY[i];
            RMassY[i] = (RMassY[i]>1 && RMassY[i]<1+1e-7) ? 1.0 : RMassY[i];

            if(!(RMassY[i]>=0 && RMassY[i]<=1))
            {
                *SpeDiff_Flag = 0;
              //   printf("%dth step: error occurs during %dth species(%s) in 1st circulation in species diffusion process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,k,Species_Name[k],i,domain_cellnumber,(*deta_x));
//                 printf("number  cell_size       cell_size_old   cell.u         cell.Massfr       cellnew.Massfr    cell.spedif     cell.density    cell.Tem\n");
//                 for(i1=0; i1<domain_cellnumber; i1++)
//                 {
//                     if(i1<10 || i1>domain_cellnumber-10)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1>i-20 && i1<i+20)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1%200==0)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);        
//                 }
     
// //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\n",E_S_number,cell[i].mixture.Massfr[k],i==domain_cellnumber-1?cell[0].mixture.Massfr[k]:cell[i+1].mixture.Massfr[k],i==0?cell[domain_cellnumber-1].mixture.Massfr[k]:cell[i-1].mixture.Massfr[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
// //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,RMassY[i],cell_new[i+1].mixture.Massfr[k],cell_new[i-1].mixture.Massfr[k],cell_new[i].size,cell[i+1].size,cell_new[i-1].size);
//                 printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Density,cell[i+1].mixture.Density,cell[i-1].mixture.Density);
//                 printf("cell[0].mixture.Massfr[k]=%8.5f, cell[end].mixture.Massfr[k]=%8.5f\n",cell[0].mixture.Molefr[k],cell[domain_cellnumber-1].mixture.Massfr[k]);
                // to modify the diffusion step
                MODIFY_SPEDIFFDX(deta_x, cell[i].mixture.Massfr[k], RMassY[i]);

                goto abnormal2;
            } 
            else  //to store the new massfraction
                cell_new[i].mixture.Massfr[k] = RMassY[i];      
        } 
    }  //for(k=..


// uniform the mass fraction
    for(i=0; i<domain_cellnumber; i++)
    {
        sum_massfr = 0.0;
        for(k=0; k<Species_Number; k++)
            sum_massfr += cell_new[i].mixture.Massfr[k];
        for(k=0; k<Species_Number; k++)
            cell_new[i].mixture.Massfr[k] /= sum_massfr;
    }

    // to renew the mixture property based on the known mass fraction of each species in the mixture
    for(i=0; i<domain_cellnumber; i++)
        RENEW_MIXPRO_MASSY(i,cell_new);

    // to restore the changed value (mass fraction and mixture density)  
    for(i=0; i<domain_cellnumber; i++)
    {
//         cell_new[i].size_old = cell[i].size_old;
//         cell_new[i].u_old = cell[i].u_old;
//         cell_new[i].Density_old = cell[i].Density_old;
//         for(k=0; k<Species_Number; k++)
//             cell_new[i].Massfr_old[k] = cell[i].Massfr_old[k];
        cell[i] = cell_new[i];
    }

 abnormal2:    ;
}












/*********************************************************************************
This file is used to simulatie the species diffusion process in ODT model
one of the simplest model of the diffusion process: the Fick's laws is used
in this method.
Crank-Nicolson method was used to discrete the PDEs
*********************************************************************************/	

// we should note that the species diffusion process is based on the constant cell size assumption!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


// Crank-Nicolson implicit method followed by an explicit method to solve species diffusion process
void SPECIES_DIFFUSION_CNONLY(int E_S_number, int *SpeDiff_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell)
{
    int i,k,i1;
    double Tem, Pre;
    double sum_massfr, mix_density;  // summation of mole fraction from 1st species to (Species_Number-1)th species, but not for 0th species!!!
    //  a[i]u[i-1]+b[i]u[i]+c[i]u[i+1]=r[i], vector of tridiaginal matrix
    double a[domain_cellnumber],b[domain_cellnumber],c[domain_cellnumber],r[domain_cellnumber];
    double A0,B0,B1,B2,A0_1,B0_1,B1_1,B2_1;
    double RMassY[domain_cellnumber]; // cell[i].mixture.Massfr[k]!!!

    struct property_info species_property;
    struct cell_information cell_new[domain_cellnumber];

    //to set boundary conditions and record the initial values of each step	
    for(i=0; i<domain_cellnumber; i++)
    {   
        cell_new[i] = cell[i];
        for(k=0; k<Species_Number; k++)
        { 
            if(cell[i].mixture.Molefr[k]<0-TINY||cell[i].mixture.Molefr[k]>1+1e-6 )
            {
                printf("wierd! %d,%s,%20.18f\n",i,Species_Name[k],cell[i].mixture.Molefr[k]);
                goto abnormal2;
            }
        }
    }

    //to solve all species diffusion process, but not for 0th species, because sum(Y[i])=1.0. 0th species should be the one whose Y[0th] is the maximum value!
    for(k=1; k<Species_Number; k++)
    {
        // to calculate vector a[],b[],c[] and d[]
        for(i=0; i<domain_cellnumber; i++)
        {
            if(i==0)
            {
                A0 = (*deta_x)/(cell[i].u_old*dy_co)/2;
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[i+1].mixture.Spe_Diff_Coeff[k])/(dy_co+cell[i+1].size_old);
                B0 = B1+B2;

                // coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co);
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[i+1].mixture.Spe_Diff_Coeff[k])/(dy_co+cell[i+1].size);
                B0_1 = B1_1+B2_1;

                a[i] = 0.0;
                b[i] = (1+A0*B0_1)*cell_new[i].mixture.Density;
                c[i] = -A0*B2_1*cell_new[i+1].mixture.Density;
                r[i] = 2*A0*B1*cell[domain_cellnumber-1].Density_old*cell[domain_cellnumber-1].Massfr_old[k]+(1-A0*B0)*cell[i].Density_old*cell[i].Massfr_old[k]+A0*B2*cell[i+1].Density_old*cell[i+1].Massfr_old[k];  //ghost point for cell[domain_cellnumber-1]
                r[i] += 0.0;  // to consider additional diffusion due to other facts: thermal diffusion et al.
            }
            else if(i==domain_cellnumber-1)
            {
                A0 = (*deta_x)/(cell[i].u_old*dy_co)/2;
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[i-1].mixture.Spe_Diff_Coeff[k])/(cell[i-1].size_old+dy_co);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co);
                B0 = B1+B2;
            
                // coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[i-1].mixture.Spe_Diff_Coeff[k])/(cell[i-1].size+dy_co);
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co);
                B0_1 = B1_1+B2_1;

                // coefficients of the algebratic equations
                a[i] = -A0*B1_1*cell_new[i-1].mixture.Density;
                b[i] = (1+A0*B0_1)*cell_new[i].mixture.Density;
                c[i] = 0.0;
                r[i] = A0*B1*cell[i-1].Density_old*cell[i-1].Massfr_old[k]+(1-A0*B0)*cell[i].Density_old*cell[i].Massfr_old[k]+2*A0*B2*cell[0].Density_old*cell[0].Massfr_old[k];  //ghost point for cell[0]
                r[i] += 0.0;  // to consider addition diffusion 
            }
            else
            {
                A0 = (*deta_x)/(cell[i].u_old*cell[i].size_old)/2;
             //    if((cell[i].u_old*cell[i].size_old-cell[i].u*cell[i].size)/(cell[i].u*cell[i].size)*100>0.001)
//                     printf("u*y wierd!%d,  %10.7f =  %10.7f\n",i, cell[i].u_old*cell[i].size_old*1e5,cell[i].u*cell[i].size*1e5);
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[i-1].mixture.Spe_Diff_Coeff[k])/((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[i+1].mixture.Spe_Diff_Coeff[k])/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old);
                B0 = B1+B2;

                //coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[i-1].mixture.Spe_Diff_Coeff[k])/((i==1?dy_co:cell[i-1].size)+cell[i].size);
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[i+1].mixture.Spe_Diff_Coeff[k])/((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size);
                B0_1 = B1_1+B2_1;


                a[i] = -A0*B1_1*cell_new[i-1].mixture.Density;
                b[i] = (1+A0*B0_1)*cell_new[i].mixture.Density;
                c[i] = -A0*B2_1*cell_new[i+1].mixture.Density;
                r[i] = A0*B1*cell[i-1].Density_old*cell[i-1].Massfr_old[k]+(1-A0*B0)*cell[i].Density_old*cell[i].Massfr_old[k]+A0*B2*cell[i+1].Density_old*cell[i+1].Massfr_old[k];
             //    if((cell[i].Density_old-cell[i].mixture.Density)/cell[i].Density_old*100>0.001)
//                     printf("Density wierd!%d,  %10.7f === %10.7f\n",i, cell[i].Density_old, cell[i].mixture.Density);

                r[i] += 0.0;  // to consider additional diffusion
            }
        }
        //    printf("0. %d %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He]);

        //to calculate the candidate mole fraction of kth species velocity on (n+1)th odt line
        TRIDIAG(a,b,c,r,RMassY,domain_cellnumber);

        //    printf("0_1. %d %8.5f  %8.5f  %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He],RMassY[int(domain_cellnumber/2)],RMassY[0]);

        //to restore the calculated cell[i].mixture.Massfr[k]
        for(i=0; i<domain_cellnumber; i++)
        {
            cell_new[i].mixture.Massfr[k] = RMassY[i];   // to restore the new mass fraction for kth species in the mixture
            // if there are some disturbations according to calculation but not physical model, to modify them at first
         //    if(cell_new[i].mixture.Massfr[k]<0 && cell_new[i].mixture.Massfr[k]>-1e-6)
//                 cell_new[i].mixture.Massfr[k] = 0.0;
//             if(cell_new[i].mixture.Massfr[k]>1 && cell_new[i].mixture.Massfr[k]<1+1e-6)
//                 cell_new[i].mixture.Massfr[k] = 1.0;


            cell_new[i].mixture.Massfr[k] = (cell_new[i].mixture.Massfr[k]<0 && cell_new[i].mixture.Massfr[k]>-1e-6) ? 0.0 : cell_new[i].mixture.Massfr[k];
            cell_new[i].mixture.Massfr[k] = (cell_new[i].mixture.Massfr[k]>1 && cell_new[i].mixture.Massfr[k]<1+1e-6) ? 1.0 : cell_new[i].mixture.Massfr[k];

            if(cell_new[i].mixture.Massfr[k]<0 || cell_new[i].mixture.Massfr[k]>1)
            {
                *SpeDiff_Flag = 0;
               //  printf("%dth step: error occurs during %dth species(%s) in 1st circulation in species diffusion process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,k,Species_Name[k],i,domain_cellnumber,(*deta_x));
//                 printf("number  cell_size       cell_size_old   cell.u         cell.Massfr       cellnew.Massfr    cell.spedif     cell.density    cell.Tem\n");
//                 for(i1=0; i1<domain_cellnumber; i1++)
//                 {
//                     if(i1<10 || i1>domain_cellnumber-10)
//                         printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1>i-20 && i1<i+20)
//                         printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1%200==0)
//                         printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);        
//                 }
     
// //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\n",E_S_number,cell[i].mixture.Massfr[k],i==domain_cellnumber-1?cell[0].mixture.Massfr[k]:cell[i+1].mixture.Massfr[k],i==0?cell[domain_cellnumber-1].mixture.Massfr[k]:cell[i-1].mixture.Massfr[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
// //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,cell_new[i].mixture.Massfr[k],cell_new[i+1].mixture.Massfr[k],cell_new[i-1].mixture.Massfr[k],cell_new[i].size,cell[i+1].size,cell_new[i-1].size);
//                 printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Density,cell[i+1].mixture.Density,cell[i-1].mixture.Density);
//                 printf("cell[0].mixture.Massfr[k]=%8.5f, cell[end].mixture.Massfr[k]=%8.5f\n",cell[0].mixture.Massfr[k],cell[domain_cellnumber-1].mixture.Massfr[k]);

                // to modify the diffusion step
                MODIFY_SPEDIFFDX(deta_x, cell[i].mixture.Massfr[k], cell_new[i].mixture.Massfr[k]);

                goto abnormal2;
            }       

        }
    }

//     // to calculate new mass fraction of 0th species in the mixture for each cell
//     for(i=0; i<domain_cellnumber; i++)
//     {
//         sum_massfr = 0.0;
//         for(k=1; k<Species_Number; k++)
//             sum_massfr += cell_new[i].mixture.Massfr[k];
//         cell_new[i].mixture.Massfr[0] = 1.0-sum_massfr;

//         cell_new[i].mixture.Massfr[0] = (cell_new[i].mixture.Massfr[0]<1+1e-6&&cell_new[i].mixture.Massfr[0]>1) ? 1.0 : cell_new[i].mixture.Massfr[0];
//         cell_new[i].mixture.Massfr[0] = (cell_new[i].mixture.Massfr[0]<0&&cell_new[i].mixture.Massfr[0]>-1e-6) ? 0.0 : cell_new[i].mixture.Massfr[0];

//         if(cell_new[i].mixture.Massfr[k]<0 || cell_new[i].mixture.Massfr[k]>1)
//         {
//             *SpeDiff_Flag = 0;
//             printf("%dth step: 0th species error occurs during %dth species in 1st circulation in species diffusion process!cells=%d, cell number=%d, deta_x=%f",E_S_number,k,i,domain_cellnumber,(*deta_x));
//             printf("number  cell_size       cell_size_old   cell.u         cell.Massfr       cellnew.Massfr    cell.spedif\n");
//             for(i1=0; i1<domain_cellnumber; i1++)
//             {
//                 if(i1<10 || i1>domain_cellnumber-10)
//                     printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k]);
//                 if(i1>i-20 && i1<i+20)
//                     printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k]);
//                 if(i1%200==0)
//                     printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k]);        
//             }
     
//             //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\n",E_S_number,cell[i].mixture.Massfr[k],i==domain_cellnumber-1?cell[0].mixture.Massfr[k]:cell[i+1].mixture.Massfr[k],i==0?cell[domain_cellnumber-1].mixture.Massfr[k]:cell[i-1].mixture.Massfr[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
//             //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,cell_new[i].mixture.Massfr[k],cell_new[i+1].mixture.Massfr[k],cell_new[i-1].mixture.Massfr[k],cell_new[i].size,cell[i+1].size,cell_new[i-1].size);
//             printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Density,cell[i+1].mixture.Density,cell[i-1].mixture.Density);
//             printf("cell[0].mixture.Massfr[k]=%8.5f, cell[end].mixture.Massfr[k]=%8.5f\n",cell[0].mixture.Massfr[k],cell[domain_cellnumber-1].mixture.Massfr[k]);
//             goto abnormal2;
//         }       
//     }

    // to make sure sum_massfr === 1.0
    for(i=0; i<domain_cellnumber; i++)
    {
        sum_massfr = 0.0;
        for(k=0; k<Species_Number; k++)
            sum_massfr += cell_new[i].mixture.Massfr[k];
        for(k=0; k<Species_Number; k++)
            cell_new[i].mixture.Massfr[k] /= sum_massfr;
    }


    //    printf("1. %d %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He]);

    // to calculate the new density based on the new mass fraction
    for(i=0; i<domain_cellnumber; i++)
    {
        mix_density = 0.0;
        Tem = cell_new[i].mixture.Tem;
        Pre = cell_new[i].pressure;
        CALCU_PROPERTY_DENSITY(Tem, Pre, &species_property);  // to calculate thermal properties for each species in the mixture
        for(k=0; k<Species_Number; k++)
            mix_density += cell_new[i].mixture.Massfr[k]/species_property.Density[k];
        mix_density = 1.0/mix_density;
        cell_new[i].mixture.Density = mix_density;
    }




    //    printf("2. %d %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He]);


    //to solve all species diffusion process based on the candidate species massfraction and density, but not for 0th species, because sum(Y[i])=1.0. 0th species should be the one whose Y[0th] is the maximum value!
    for(k=1; k<Species_Number; k++)
    {
        // to calculate vector a[],b[],c[] and d[]
        for(i=0; i<domain_cellnumber; i++)
        {
            if(i==0)
            {
                A0 = (*deta_x)/(cell[i].u_old*dy_co)/2;
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[i+1].mixture.Spe_Diff_Coeff[k])/(dy_co+cell[i+1].size_old);
                B0 = B1+B2;

                // coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co);
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[i+1].mixture.Spe_Diff_Coeff[k])/(dy_co+cell[i+1].size);
                B0_1 = B1_1+B2_1;

                a[i] = 0.0;
                b[i] = (1+A0*B0_1)*cell_new[i].mixture.Density;
                c[i] = -A0*B2_1*cell_new[i+1].mixture.Density;
                r[i] = 2*A0*B1*cell[domain_cellnumber-1].Density_old*cell[domain_cellnumber-1].Massfr_old[k]+(1-A0*B0)*cell[i].Density_old*cell[i].Massfr_old[k]+A0*B2*cell[i+1].Density_old*cell[i+1].Massfr_old[k];  //ghost point for cell[domain_cellnumber-1]
                r[i] += 0.0;  // to consider additional diffusion due to other facts: thermal diffusion et al.
            }
            else if(i==domain_cellnumber-1)
            {
                A0 = (*deta_x)/(cell[i].u_old*dy_co)/2;
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[i-1].mixture.Spe_Diff_Coeff[k])/(cell[i-1].size_old+dy_co);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co);
                B0 = B1+B2;
            
                // coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[i-1].mixture.Spe_Diff_Coeff[k])/(cell[i-1].size+dy_co);
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co);
                B0_1 = B1_1+B2_1;

                // coefficients of the algebratic equations
                a[i] = -A0*B1_1*cell_new[i-1].mixture.Density;
                b[i] = (1+A0*B0_1)*cell_new[i].mixture.Density;
                c[i] = 0.0;
                r[i] = A0*B1*cell[i-1].Density_old*cell[i-1].Massfr_old[k]+(1-A0*B0)*cell[i].Density_old*cell[i].Massfr_old[k]+2*A0*B2*cell[0].Density_old*cell[0].Massfr_old[k];  //ghost point for cell[0]
                r[i] += 0.0;  // to consider addition diffusion 
            }
            else
            {
                A0 = (*deta_x)/(cell[i].u_old*cell[i].size_old)/2;
               //  if((cell[i].u_old*cell[i].size_old-cell[i].u*cell[i].size)/(cell[i].u*cell[i].size)*100>0.001)
//                     printf("2! u*size wierd! %d,  %10.7f == %10.7f\n",i, cell[i].u_old*cell[i].size_old*1e5, cell[i].u*cell[i].size*1e5);
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[i-1].mixture.Spe_Diff_Coeff[k])/((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[i+1].mixture.Spe_Diff_Coeff[k])/((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old);
                B0 = B1+B2;

                //coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[i-1].mixture.Spe_Diff_Coeff[k])/((i==1?dy_co:cell[i-1].size)+cell[i].size);
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[i+1].mixture.Spe_Diff_Coeff[k])/((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size);
                B0_1 = B1_1+B2_1;


                a[i] = -A0*B1_1*cell_new[i-1].mixture.Density;
                b[i] = (1+A0*B0_1)*cell_new[i].mixture.Density;
                c[i] = -A0*B2_1*cell_new[i+1].mixture.Density;
                r[i] = A0*B1*cell[i-1].Density_old*cell[i-1].Massfr_old[k]+(1-A0*B0)*cell[i].Density_old*cell[i].Massfr_old[k]+A0*B2*cell[i+1].Density_old*cell[i+1].Massfr_old[k];

//                 if((cell[i].Density_old-cell[i].mixture.Density)/cell[i].Density_old*100>0.001)
//                     printf("2! density wierd! %d, %10.7f =====%10.7f\n", i,cell[i].Density_old, cell[i].mixture.Density);
                r[i] += 0.0;  // to consider additional diffusion
            }
        }

        //to calculate the candidate cell velocity on (n+1)th odt line
        TRIDIAG(a,b,c,r,RMassY,domain_cellnumber);


        //to restore the calculated cell[i].mixture.Massfr[k]
        for(i=0; i<domain_cellnumber; i++)
        {
            cell_new[i].mixture.Massfr[k] = RMassY[i];   // to restore the new mass fraction for kth species in the mixture

            // if there are some disturbations according to calculation but not physical model, to modify them at first
//             if(cell_new[i].mixture.Massfr[k]<0 && cell_new[i].mixture.Massfr[k]>-1e-6)
//                 cell_new[i].mixture.Massfr[k] = 0.0;
//             if(cell_new[i].mixture.Massfr[k]>1 && cell_new[i].mixture.Massfr[k]<1+1e-6)
//                 cell_new[i].mixture.Massfr[k] = 1.0;

            cell_new[i].mixture.Massfr[k] = (cell_new[i].mixture.Massfr[k]<0 && cell_new[i].mixture.Massfr[k]>-1e-6) ? 0.0 : cell_new[i].mixture.Massfr[k];
            cell_new[i].mixture.Massfr[k] = (cell_new[i].mixture.Massfr[k]>1 && cell_new[i].mixture.Massfr[k]<1+1e-6) ? 1.0 : cell_new[i].mixture.Massfr[k];

            if(cell_new[i].mixture.Massfr[k]<0 || cell_new[i].mixture.Massfr[k]>1)
            {
                *SpeDiff_Flag = 0;
              //   printf("%dth step: error occurs during %dth species(%s) in 2nd circulation in species diffusion process!cell number: %d\t,velo:%8.5f,dx:%8.5f\n",E_S_number,k,Species_Name[k],i,cell_new[i].mixture.Massfr[k],(*deta_x));
//                 for(i1=(i-20<0?0:i-20); i1<(i+20>domain_cellnumber-1?domain_cellnumber-1:i+20); i1++)
//                      printf("%d\t%8.5f\t%8.5f\t%8.5f,%8.5f\n",i1,cell[i1].size,cell[i1].mixture.Density,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k]);

//                 printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,cell[i].mixture.Massfr[k],i==domain_cellnumber-1?cell[0].mixture.Massfr[k]:cell[i+1].mixture.Massfr[k],i==0?cell[domain_cellnumber-1].mixture.Massfr[k]:cell[i-1].mixture.Massfr[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
//                 printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,cell_new[i].mixture.Massfr[k],cell_new[i+1].mixture.Massfr[k],cell[i-1].mixture.Massfr[k],cell_new[i].size,cell_new[i+1].size,cell_new[i-1].size);
//                 printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Density,cell[i+1].mixture.Density,cell[i-1].mixture.Density);
//                 printf("cell[0].mixture.Massfr[k]=%8.5f, cell[end].mixture.Massfr[k]=%8.5f\n",cell[0].mixture.Massfr[k],cell[domain_cellnumber-1].mixture.Massfr[k]);

                // to modify the diffusion step
                MODIFY_SPEDIFFDX(deta_x, cell[i].mixture.Massfr[k], cell_new[i].mixture.Massfr[k]);
                goto abnormal2;
            }       

        }
    }

    //    printf("3. %d %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He]);
    
    

    // to calculate new mass fraction of 0th species in the mixture for each cell
    for(i=0; i<domain_cellnumber; i++)
    {
        sum_massfr = 0.0;
        for(k=1; k<Species_Number; k++)
            sum_massfr += cell_new[i].mixture.Massfr[k];
        cell_new[i].mixture.Massfr[0] = 1.0-sum_massfr;

        cell_new[i].mixture.Massfr[0] = (cell_new[i].mixture.Massfr[0]<1+1e-6&&cell_new[i].mixture.Massfr[0]>1) ? 1.0 : cell_new[i].mixture.Massfr[0];
        cell_new[i].mixture.Massfr[0] = (cell_new[i].mixture.Massfr[0]<0&&cell_new[i].mixture.Massfr[0]>-1e-6) ? 0.0 : cell_new[i].mixture.Massfr[0];

        if(cell_new[i].mixture.Massfr[k]<0 || cell_new[i].mixture.Massfr[k]>1)
        {
            *SpeDiff_Flag = 0;
            printf("%dth step: 0th species error occurs during %dth species in 1st circulation in species diffusion process!cells=%d, cell number=%d, deta_x=%f",E_S_number,k,i,domain_cellnumber,(*deta_x));
            printf("number  cell_size       cell_size_old   cell.u         cell.Massfr       cellnew.Massfr    cell.spedif\n");
            for(i1=0; i1<domain_cellnumber; i1++)
            {
                if(i1<10 || i1>domain_cellnumber-10)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[He]);
                if(i1>i-20 && i1<i+20)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[He]);
                if(i1%200==0)
                    printf("%6d  %8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[He]);        
            }
     
            //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\n",E_S_number,cell[i].mixture.Massfr[k],i==domain_cellnumber-1?cell[0].mixture.Massfr[k]:cell[i+1].mixture.Massfr[k],i==0?cell[domain_cellnumber-1].mixture.Massfr[k]:cell[i-1].mixture.Massfr[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
            //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,cell_new[i].mixture.Massfr[k],cell_new[i+1].mixture.Massfr[k],cell_new[i-1].mixture.Massfr[k],cell_new[i].size,cell[i+1].size,cell_new[i-1].size);
            printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Density,cell[i+1].mixture.Density,cell[i-1].mixture.Density);
            printf("cell[0].mixture.Massfr[k]=%8.5f, cell[end].mixture.Massfr[k]=%8.5f\n",cell[0].mixture.Massfr[k],cell[domain_cellnumber-1].mixture.Massfr[k]);
            goto abnormal2;
        }       
    }


    // to make sure sum_massfr === 1.0
    for(i=0; i<domain_cellnumber; i++)
    {
        sum_massfr = 0.0;
        for(k=0; k<Species_Number; k++)
            sum_massfr += cell_new[i].mixture.Massfr[k];
        for(k=0; k<Species_Number; k++)
            cell_new[i].mixture.Massfr[k] /= sum_massfr;
    }

    // to renew the mixture property based on the known mass fraction of each species in the mixture
    for(i=0; i<domain_cellnumber; i++)
        RENEW_MIXPRO_MASSY(i,cell_new);


    // to restore the changed value (mass fraction and mixture density)  
    for(i=0; i<domain_cellnumber; i++)
    {
//         cell_new[i].size_old = cell[i].size_old;
//         cell_new[i].u_old = cell[i].u_old;
//         cell_new[i].Density_old = cell[i].Density_old;
//         for(k=0; k<Species_Number; k++)
//             cell_new[i].Massfr_old[k] = cell[i].Massfr_old[k];
        cell[i] = cell_new[i];
    }

 abnormal2:    ;
}


	








//*************************************************************************************//



//*************************************************************************************//




// Crank-Nicolson implicit method followed by an explicit method to solve species diffusion process
void SPECIES_DIFFUSION_CN(int E_S_number, int *SpeDiff_Flag, int domain_cellnumber, double *deta_x, struct cell_information *cell, struct source_information *source)
{
    int i,k,i1;
    double Tem, Pre;
    double mass_fr;   // old mass flow rate for ith cell
    double sum_massfr, mix_density;  // summation of mole fraction from 1st species to (Species_Number-1)th species, but not for 0th species!!!
    //  a[i]u[i-1]+b[i]u[i]+c[i]u[i+1]=r[i], vector of tridiaginal matrix
    double a[domain_cellnumber],b[domain_cellnumber],c[domain_cellnumber],r[domain_cellnumber];
    double A0,B0,B1,B2,A0_1,B0_1,B1_1,B2_1;
    double RMassY[domain_cellnumber]; // cell[i].mixture.Massfr[k]!!!
    //    double Density[domain_cellnumber][Species_Number];   // density of each species;
    extern double Density_Standard[Species_Number];

    // struct property_info species_property;
    struct cell_information cell_new[domain_cellnumber];

    //to set boundary conditions and record the initial values of each step	
    for(i=0; i<domain_cellnumber; i++)
    {   
        cell_new[i] = cell[i];
        for(k=0; k<Species_Number; k++)
        { 
            if(cell[i].mixture.Molefr[k]<0-TINY||cell[i].mixture.Molefr[k]>1+1e-6 )
            {
                printf("wierd! %d,%s,%20.18f\n",i,Species_Name[k],cell[i].mixture.Molefr[k]);
                goto abnormal2;
            }
        }
    }

    //to solve all species diffusion process, include 0th species, then to normalize the species mass fraction
    for(k=0; k<Species_Number; k++)
    {
        // to calculate vector a[],b[],c[] and d[]
        for(i=0; i<domain_cellnumber; i++)
        {
            if(i==0)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old; // cell.u has changed, shouldn't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[domain_cellnumber-1].mixture.Density)/4.0;
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i+1].size_old+cell[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i+1].size_old+cell[i+1].mixture.Density*dy_co)/pow((dy_co+cell[i+1].size_old),3);
                B0 = B1+B2;

                // coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[domain_cellnumber-1].mixture.Density)/4.0;
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]*cell_new[i+1].size+cell_new[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell_new[i+1].size+cell[i+1].mixture.Density*dy_co)/pow((cell_new[i+1].size+dy_co),3);
                B0_1 = B1_1+B2_1;

                a[i] = 0.0;
                b[i] = (mass_fr+A0*B0_1);
                b[i] += source[i].mass;
                c[i] = -A0*B2_1;
                r[i] = 2*A0*B1*cell[domain_cellnumber-1].Massfr_old[k]+(mass_fr-A0*B0)*cell[i].Massfr_old[k]+A0*B2*cell[i+1].Massfr_old[k];  //ghost point for cell[domain_cellnumber-1]
                r[i] += 0.0;  // to consider additional diffusion due to other facts: thermal diffusion et al.
                r[i] += source[i].massflux_species[k];  // to consider reaction source term. Here, the reaction will be considered separately, but particle source term should be considered in this part.
            }
            else if(i==domain_cellnumber-1)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old;  // cell.u has changed, can't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i-1].size_old+cell[i-1].mixture.Density*dy_co)/pow((cell[i-1].size_old+dy_co),3);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[0].mixture.Density)/4.0;
                B0 = B1+B2;
            
                // coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]*cell_new[i-1].size+cell_new[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell_new[i-1].size+cell[i-1].mixture.Density*dy_co)/pow((cell_new[i-1].size+dy_co),3);
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[0].mixture.Density)/4.0;
                B0_1 = B1_1+B2_1;

                // coefficients of the algebratic equations
                a[i] = -A0*B1_1;
                b[i] = (mass_fr+A0*B0_1);
                b[i] += source[i].mass;
                c[i] = 0.0;
                r[i] = A0*B1*cell[i-1].Massfr_old[k]+(mass_fr-A0*B0)*cell[i].Massfr_old[k]+2*A0*B2*cell[0].Massfr_old[k];  //ghost point for cell[0]
                r[i] += 0.0;  // to consider addition diffusion 
                r[i] += source[i].massflux_species[k];  // to consider reaction source term, including particle reaction source
            }
            else
            {
                mass_fr = cell[i].u_old*cell[i].size_old*cell[i].Density_old; // the mass flow rate is the old mass flow rate! so .u, .size can't be used here!
                A0 = (*deta_x);
             //    if((cell[i].u_old*cell[i].size_old-cell[i].u*cell[i].size)/(cell[i].u*cell[i].size)*100>0.001)
//                     printf("u*y wierd!%d,  %10.7f =  %10.7f\n",i, cell[i].u_old*cell[i].size_old*1e5,cell[i].u*cell[i].size*1e5);
                // coefficients based on old nth value!
                B1 = B2;
                //B1 = 2*(cell[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/pow(((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old),3);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old),3);
                B0 = B1+B2;

                //coefficients based on new nth (or n+1th) value!
                B1_1 = B2_1;
                //B1_1 = 2*(cell_new[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell_new[i-1].size)+cell_new[i-1].mixture.Spe_Diff_Coeff[k]*cell_new[i].size)*(cell_new[i].mixture.Density*(i==1?dy_co:cell_new[i-1].size)+cell_new[i-1].mixture.Density*cell_new[i].size)/pow(((i==1?dy_co:cell_new[i-1].size)+cell_new[i].size),3);
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+cell_new[i+1].mixture.Spe_Diff_Coeff[k]*cell_new[i].size)*(cell_new[i].mixture.Density*(i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+cell_new[i+1].mixture.Density*cell_new[i].size)/pow(((i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+cell_new[i].size),3);
                B0_1 = B1_1+B2_1;


                a[i] = -A0*B1_1;
                b[i] = (mass_fr+A0*B0_1);
                b[i] += source[i].mass;
                c[i] = -A0*B2_1;
                r[i] = A0*B1*cell[i-1].Massfr_old[k]+(mass_fr-A0*B0)*cell[i].Massfr_old[k]+A0*B2*cell[i+1].Massfr_old[k];
             //    if((cell[i].Density_old-cell[i].mixture.Density)/cell[i].Density_old*100>0.001)
//                     printf("Density wierd!%d,  %10.7f === %10.7f\n",i, cell[i].Density_old, cell[i].mixture.Density);

                r[i] += 0.0;  // to consider additional diffusion
                r[i] += source[i].massflux_species[k];
            }
        }
        //    printf("0. %d %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He]);

        //to calculate the candidate massfraction of kth species velocity on (n+1)th odt line
        TRIDIAG(a,b,c,r,RMassY,domain_cellnumber);

        //    printf("0_1. %d %8.5f  %8.5f  %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He],RMassY[int(domain_cellnumber/2)],RMassY[0]);

        // RMassY = rho_mix*Yi, rho_mix and Yi should be calculated out separately here
        for(i=0; i<domain_cellnumber; i++)
        {
            //to restore the calculated cell[i].mixture.Massfr[k]*rho_mix
            cell_new[i].mixture.Massfr[k] = RMassY[i];  
            // to calculate the new mole fraction according to Xi = Yi*rho_mix/rho_i


            cell_new[i].mixture.Massfr[k] = (cell_new[i].mixture.Massfr[k]<0.0 && cell_new[i].mixture.Massfr[k]>-1e-7) ? 0.0 : cell_new[i].mixture.Massfr[k];
            cell_new[i].mixture.Massfr[k] = (cell_new[i].mixture.Massfr[k]>1.0 && cell_new[i].mixture.Massfr[k]<1+1e-7) ? 1.0 : cell_new[i].mixture.Massfr[k];

        

            if(!(cell_new[i].mixture.Massfr[k]>=0.0 && cell_new[i].mixture.Massfr[k]<=1.0))
            {
                *SpeDiff_Flag = 0;
              //   printf("%dth step: error occurs during %dth species(%s) in 1st circulation in species diffusion process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,k,Species_Name[k],i,domain_cellnumber,(*deta_x));
//                 printf("number  cell_size       cell_size_old   cell.u         cell.Massfr       cellnew.Massfr    cell.spedif     cell.density    cell.Tem\n");
//                 for(i1=0; i1<domain_cellnumber; i1++)
//                 {
//                     if(i1<10 || i1>domain_cellnumber-10)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1>i-20 && i1<i+20)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1%200==0)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);        
//                 }
     
// //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\n",E_S_number,cell[i].mixture.Massfr[k],i==domain_cellnumber-1?cell[0].mixture.Massfr[k]:cell[i+1].mixture.Massfr[k],i==0?cell[domain_cellnumber-1].mixture.Massfr[k]:cell[i-1].mixture.Massfr[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
// //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,cell_new[i].mixture.Massfr[k],cell_new[i+1].mixture.Massfr[k],cell_new[i-1].mixture.Massfr[k],cell_new[i].size,cell[i+1].size,cell_new[i-1].size);
//                 printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Density,cell[i+1].mixture.Density,cell[i-1].mixture.Density);
//                 printf("cell[0].mixture.Massfr[k]=%8.5f, cell[end].mixture.Massfr[k]=%8.5f\n",cell[0].mixture.Molefr[k],cell[domain_cellnumber-1].mixture.Massfr[k]);
                // to modify the diffusion step
                MODIFY_SPEDIFFDX(deta_x, cell[i].mixture.Massfr[k], cell_new[i].mixture.Massfr[k]);
                goto abnormal2;
            }
        }
    }

//     // to calculate new mole fraction of 0th species in the mixture for each cell
//     for(i=0; i<domain_cellnumber; i++)
//     {
//         sum_massfr = 0.0;
//         for(k=1; k<Species_Number; k++)
//             sum_massfr += cell_new[i].mixture.Massfr[k];
//         cell_new[i].mixture.Massfr[0] = 1.0-sum_massfr;

//         if(cell_new[i].mixture.Massfr[0] > 1.0)
//         {
//             printf("0th species molefr in %dth cell too large! modification should be done\n",i);
            
//         }
//         else if(cell_new[i].mixture.Massfr[0]<0.0)
//         {
//             printf("0th species molefr in %dth cell too small(%f, %f, %f)! modification should be done\n",i,sum_massfr, cell_new[i].mixture.Massfr[0],cell[i].mixture.Massfr[0]);
//             sum_massfr += cell[i].mixture.Massfr[0];
//             cell_new[i].mixture.Massfr[0] = cell[i].mixture.Massfr[0]/sum_massfr;
//             for(k=1; k<Species_Number; k++)
//                 cell_new[i].mixture.Massfr[k] /= sum_massfr;
//         }
//     }

// uniform the mass fraction
    for(i=0; i<domain_cellnumber; i++)
    {
        sum_massfr = 0.0;
        for(k=0; k<Species_Number; k++)
            sum_massfr += cell_new[i].mixture.Massfr[k];
        for(k=0; k<Species_Number; k++)
            cell_new[i].mixture.Massfr[k] /= sum_massfr;
    }

    for(i=0; i<domain_cellnumber; i++)
        CALCU_MIXTURE_DENSITY(i,cell_new);   // to calculate mixture density 

    // the second iteration
    //to solve all species diffusion process, but not for 0th species, because sum(Y[i])=1.0. 0th species should be the one whose Y[0th] is the maximum value!
    for(k=0; k<Species_Number; k++)
    {
        // to calculate vector a[],b[],c[] and d[]
        for(i=0; i<domain_cellnumber; i++)
        {
            if(i==0)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old;  // cell.u has changed, can't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[domain_cellnumber-1].mixture.Density)/4.0;
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i+1].size_old+cell[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i+1].size_old+cell[i+1].mixture.Density*dy_co)/pow((dy_co+cell[i+1].size_old),3);
                B0 = B1+B2;

                // coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[domain_cellnumber-1].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell_new[i].mixture.Density+cell_new[domain_cellnumber-1].mixture.Density)/4.0;
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]*cell_new[i+1].size+cell_new[i+1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell_new[i].mixture.Density*cell_new[i+1].size+cell_new[i+1].mixture.Density*dy_co)/pow((cell_new[i+1].size+dy_co),3);
                B0_1 = B1_1+B2_1;

                a[i] = 0.0;
                b[i] = (cell[i].u_old*dy_co*cell_new[i].mixture.Density+A0*B0_1);
                b[i] += source[i].mass;
                c[i] = -A0*B2_1;
                r[i] = 2*A0*B1*cell[domain_cellnumber-1].Massfr_old[k]+(mass_fr-A0*B0)*cell[i].Massfr_old[k]+A0*B2*cell[i+1].Massfr_old[k];  //ghost point for cell[domain_cellnumber-1]
                r[i] += 0.0;  // to consider additional diffusion due to other facts: thermal diffusion et al.
                r[i] += source[i].massflux_species[k];  // to consider reaction source term. Here, the reaction will be considered separately.
            }
            else if(i==domain_cellnumber-1)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old;  // cell.u has changed, can't be used here!
                A0 = (*deta_x);
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.Spe_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i-1].size_old+cell[i-1].mixture.Density*dy_co)/pow((cell[i-1].size_old+dy_co),3);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]+cell[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[0].mixture.Density)/4.0;
                B0 = B1+B2;
            
                // coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]*cell_new[i-1].size+cell_new[i-1].mixture.Spe_Diff_Coeff[k]*dy_co)*(cell_new[i].mixture.Density*cell_new[i-1].size+cell_new[i-1].mixture.Density*dy_co)/pow((cell_new[i-1].size+dy_co),3);
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]+cell_new[0].mixture.Spe_Diff_Coeff[k])/(dy_co+dy_co)*(cell_new[i].mixture.Density+cell_new[0].mixture.Density)/4.0;
                B0_1 = B1_1+B2_1;

                // coefficients of the algebratic equations
                a[i] = -A0*B1_1;
                b[i] = (mass_fr*cell_new[i].mixture.Density/cell[i].mixture.Density+A0*B0_1);
                b[i] += source[i].mass;
                c[i] = 0.0;
                r[i] = A0*B1*cell[i-1].Massfr_old[k]+(mass_fr-A0*B0)*cell[i].Massfr_old[k]+2*A0*B2*cell[0].Massfr_old[k];  //ghost point for cell[0]
                r[i] += 0.0;  // to consider addition diffusion 
                r[i] += source[i].massflux_species[k];  // to consider reaction source term
            }
            else
            {
                mass_fr = cell[i].u_old*cell[i].size_old*cell[i].Density_old;  // cell.u has changed, can't be used here!
                A0 = (*deta_x);
             //    if((cell[i].u_old*cell[i].size_old-cell[i].u*cell[i].size)/(cell[i].u*cell[i].size)*100>0.001)
//                     printf("u*y wierd!%d,  %10.7f =  %10.7f\n",i, cell[i].u_old*cell[i].size_old*1e5,cell[i].u*cell[i].size*1e5);
                // coefficients based on old nth value!
                B1 = B2;
                //B1 = 2*(cell[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/pow(((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old),3);
                B2 = (cell[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.Spe_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old),3);
                B0 = B1+B2;

                //coefficients based on new nth (or n+1th) value!
                B1_1 = B2_1;
                //B1_1 = 2*(cell_new[i].mixture.Spe_Diff_Coeff[k]*(i==1?dy_co:cell_new[i-1].size)+cell_new[i-1].mixture.Spe_Diff_Coeff[k]*cell_new[i].size)*(cell_new[i].mixture.Density*(i==1?dy_co:cell_new[i-1].size)+cell_new[i-1].mixture.Density*cell_new[i].size)/pow(((i==1?dy_co:cell_new[i-1].size)+cell_new[i].size),3);
                B2_1 = (cell_new[i].mixture.Spe_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+cell_new[i+1].mixture.Spe_Diff_Coeff[k]*cell_new[i].size)*(cell_new[i].mixture.Density*(i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+cell_new[i+1].mixture.Density*cell_new[i].size)/pow(((i==domain_cellnumber-2?dy_co:cell_new[i+1].size)+cell_new[i].size),3);
                B0_1 = B1_1+B2_1;

                a[i] = -A0*B1_1;
                //                b[i] = (cell_new[i].mixture.Density*mass_fr/cell[i].Density_old+A0*B0_1);
                b[i] = (mass_fr+A0*B0_1);
                b[i] += source[i].mass;
                c[i] = -A0*B2_1;
                r[i] = A0*B1*cell[i-1].Massfr_old[k]+(mass_fr-A0*B0)*cell[i].Massfr_old[k]+A0*B2*cell[i+1].Massfr_old[k];
             //    if((cell[i].Density_old-cell[i].mixture.Density)/cell[i].Density_old*100>0.001)
//                     printf("Density wierd!%d,  %10.7f === %10.7f\n",i, cell[i].Density_old, cell[i].mixture.Density);
                r[i] += source[i].massflux_species[k];
                r[i] += 0.0;  // to consider additional diffusion
            }
        }
        //    printf("0. %d %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He]);

        //to calculate the candidate massfraction of kth species velocity on (n+1)th odt line
        TRIDIAG(a,b,c,r,RMassY,domain_cellnumber);

        //    printf("0_1. %d %8.5f  %8.5f  %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.Massfr[He],cell_new[0].mixture.Massfr[He],RMassY[int(domain_cellnumber/2)],RMassY[0]);

        // RMassY = rho_mix*Yi, rho_mix and Yi should be calculated out separately here
        for(i=0; i<domain_cellnumber; i++)
        {
            //to restore the calculated cell[i].mixture.Massfr[k]*rho_mix
            cell_new[i].mixture.Massfr[k] = RMassY[i];  
            // to calculate the new mole fraction according to Xi = Yi*rho_mix/rho_i

            cell_new[i].mixture.Massfr[k] = (cell_new[i].mixture.Massfr[k]<0.0 && cell_new[i].mixture.Massfr[k]>-1e-7) ? 0.0 : cell_new[i].mixture.Massfr[k];
            cell_new[i].mixture.Massfr[k] = (cell_new[i].mixture.Massfr[k]>1.0 && cell_new[i].mixture.Massfr[k]<1.+1e-7) ? 1.0 : cell_new[i].mixture.Massfr[k];

            if(!(cell_new[i].mixture.Massfr[k]>=0. && cell_new[i].mixture.Massfr[k]<=1.))
            {
                *SpeDiff_Flag = 0;
              //   printf("%dth step: error occurs during %dth species(%s) in 1st circulation in species diffusion process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,k,Species_Name[k],i,domain_cellnumber,(*deta_x));
//                 printf("number  cell_size       cell_size_old   cell.u         cell.Massfr       cellnew.Massfr    cell.spedif     cell.density    cell.Tem\n");
//                 for(i1=0; i1<domain_cellnumber; i1++)
//                 {
//                     if(i1<10 || i1>domain_cellnumber-10)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1>i-20 && i1<i+20)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
//                     if(i1%200==0)
//                         printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.Massfr[k],cell_new[i1].mixture.Massfr[k],cell[i1].mixture.Spe_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);        
//                 }
     
// //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\n",E_S_number,cell[i].mixture.Massfr[k],i==domain_cellnumber-1?cell[0].mixture.Massfr[k]:cell[i+1].mixture.Massfr[k],i==0?cell[domain_cellnumber-1].mixture.Massfr[k]:cell[i-1].mixture.Massfr[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
// //                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,cell_new[i].mixture.Massfr[k],cell_new[i+1].mixture.Massfr[k],cell_new[i-1].mixture.Massfr[k],cell_new[i].size,cell[i+1].size,cell_new[i-1].size);
//                 printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Density,cell[i+1].mixture.Density,cell[i-1].mixture.Density);
//                 printf("cell[0].mixture.Massfr[k]=%8.5f, cell[end].mixture.Massfr[k]=%8.5f\n",cell[0].mixture.Molefr[k],cell[domain_cellnumber-1].mixture.Massfr[k]);
                // to modify the diffusion step
                MODIFY_SPEDIFFDX(deta_x, cell[i].mixture.Massfr[k], cell_new[i].mixture.Massfr[k]);
                goto abnormal2;
            }       
        }
    }

//     // to calculate new mole fraction of 0th species in the mixture for each cell
//     for(i=0; i<domain_cellnumber; i++)
//     {
//         sum_massfr = 0.0;
//         for(k=1; k<Species_Number; k++)
//             sum_massfr += cell_new[i].mixture.Massfr[k];
//         cell_new[i].mixture.Massfr[0] = 1.0-sum_massfr;

//         if(cell_new[i].mixture.Massfr[0] > 1.0)
//         {
//             printf("0th species molefr in %dth cell too large! modification should be done\n",i);
            
//         }
//         else if(cell_new[i].mixture.Massfr[0]<0.0)
//         {
//             printf("0th species molefr in %dth cell too small(%f, %f, %f)! modification should be done\n",i,sum_massfr, cell_new[i].mixture.Massfr[0],cell[i].mixture.Massfr[0]);
//             sum_massfr += cell[i].mixture.Massfr[0];
//             cell_new[i].mixture.Massfr[0] = cell[i].mixture.Massfr[0]/sum_massfr;
//             for(k=1; k<Species_Number; k++)
//                 cell_new[i].mixture.Massfr[k] /= sum_massfr;
//         }
//     }

    for(i=0; i<domain_cellnumber; i++)
    {
        sum_massfr = 0.0;
        for(k=0; k<Species_Number; k++)
            sum_massfr += cell_new[i].mixture.Massfr[k];
        for(k=0; k<Species_Number; k++)
            cell_new[i].mixture.Massfr[k] /= sum_massfr;
    }

    // to renew the mixture property based on the known mass fraction of each species in the mixture
    for(i=0; i<domain_cellnumber; i++)
        RENEW_MIXPRO_MASSY(i,cell_new);

    // to restore the changed value (mass fraction and mixture density)  
    for(i=0; i<domain_cellnumber; i++)
    {
//         cell_new[i].size_old = cell[i].size_old;
//         cell_new[i].u_old = cell[i].u_old;
//         cell_new[i].Density_old = cell[i].Density_old;
//         for(k=0; k<Species_Number; k++)
//             cell_new[i].Massfr_old[k] = cell[i].Massfr_old[k];
        cell[i] = cell_new[i];
        cell[i].size = cell_new[i].size*cell[i].Density_old/cell[i].mixture.Density;
    }

 abnormal2:    ;
}






// to change/decrease diffusion step when negative mass fraction occurs
void MODIFY_SPEDIFFDX(double *deta_x, double Y_old, double Y_new)
{
    int stepnumber;
    stepnumber = int((*deta_x+1e-10)/dx_stir);
    if(Y_new<0.)
    {
        stepnumber *= (Y_old-1e-2*Y_old)/(Y_old-Y_new);
        if(stepnumber>20)
            stepnumber -= 5;
        else if(stepnumber>10)
            stepnumber = 10;
        else if(stepnumber>7)
            stepnumber =7;
        else if(stepnumber>5)
            stepnumber = 5;
        *deta_x = stepnumber*dx_stir;
        *deta_x = *deta_x>dx_min?(*deta_x):dx_min;
    }
    else if(Y_new>1.)
    {
        stepnumber *= (1.-1e-2*Y_old-Y_old)/(Y_new-Y_old);
        if(stepnumber>20)
            stepnumber -= 5;
        else if(stepnumber>10)
            stepnumber = 10;
        else if(stepnumber>7)
            stepnumber = 7;
        else if(stepnumber>5)
            stepnumber = 5;
        *deta_x = stepnumber*dx_stir;
        *deta_x = *deta_x>dx_min?(*deta_x):dx_min;
    }
    printf("in species diffusion! deta_x = %f\n", *deta_x);
}



//*************************************************************************************//


//*************************************************************************************//
// Forward_Euler explicit method to solve species diffusion process
void MFR_DIFFUSION_FE(int E_S_number, int *MFRDiff_Flag, int domain_cellnumber, double deta_x, struct cell_information *cell, struct source_information *source)
{
    int i,k,i1;
    double Tem, Pre;
    double mass_fr;   // old mass flow rate for ith cell
    double sum_massfr, mix_density;  // summation of mole fraction from 1st species to (Species_Number-1)th species, but not for 0th species!!!
    //  a[i]u[i-1]+b[i]u[i]+c[i]u[i+1]=r, vector of tridiaginal matrix
    //    double a[domain_cellnumber],b[domain_cellnumber],c[domain_cellnumber],r[domain_cellnumber];
    double A0,B0,B1,B2;
    double Mixture_Fr[domain_cellnumber], r; // cell[i].mixture.Massfr[k]!!!
    //    double Density[domain_cellnumber][Species_Number];   // density of each species;
    struct cell_information cell_new[domain_cellnumber];


    //to set boundary conditions and record the initial values of each step	
    for(i=0; i<domain_cellnumber; i++)
    {   
        cell_new[i] = cell[i];
        for(k=0; k<5; k++)
        { 
            if(cell[i].mixture.MixtureFr_Coal[k]<0-TINY||cell[i].mixture.MixtureFr_Coal[k]>1+1e-6 )
            {
                printf("wierd! %d,%s,%20.18f\n",i,Species_Name[k],cell[i].mixture.MixtureFr_Coal[k]);
                goto abnormal2;
            }
        }
    }

    //to solve all species diffusion process, include 0th species, then to normalize the species mass fraction
    for(k=0; k<5; k++)
    {
        // to calculate vector a[],b[],c[] and d[]
        for(i=0; i<domain_cellnumber; i++)
        {
            if(i==0)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old; // cell.u has changed, shouldn't be used here!
                A0 = deta_x;
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.MFR_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.MFR_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].Density_old+cell[domain_cellnumber-1].Density_old)/2.0;
                B2 = 2*(cell[i].mixture.MFR_Diff_Coeff[k]*cell[i+1].size_old+cell[i+1].mixture.MFR_Diff_Coeff[k]*dy_co)*(cell[i].Density_old*cell[i+1].size_old+cell[i+1].Density_old*dy_co)/pow((dy_co+cell[i+1].size_old),3);
                B0 = B1+B2;

                r = A0*B1*cell[domain_cellnumber-1].mixture.MixtureFr_Coal[k]+(mass_fr-A0*B0)*cell[i].mixture.MixtureFr_Coal[k]+A0*B2*cell[i+1].mixture.MixtureFr_Coal[k];  //ghost point for cell[domain_cellnumber-1]
                r += 0.0;  // to consider additional diffusion due to other facts: thermal diffusion et al.
                if(k<3)
                    r += source[i].mixture_fr[k];  // to consider reaction source term. Here, the reaction will be considered separately, but particle source term should be considered in this part.
                Mixture_Fr[i] = r/(mass_fr+source[i].mass);
            }
            else if(i==domain_cellnumber-1)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old;  // cell.u has changed, can't be used here!
                A0 = deta_x;
                // coefficients based on old nth value!
                B1 = 2*(cell[i].mixture.MFR_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.MFR_Diff_Coeff[k]*dy_co)*(cell[i].Density_old*cell[i-1].size_old+cell[i-1].Density_old*dy_co)/pow((cell[i-1].size_old+dy_co),3);
                B2 = (cell[i].mixture.MFR_Diff_Coeff[k]+cell[0].mixture.MFR_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].Density_old+cell[0].Density_old)/2.0;
                B0 = B1+B2;
            
                r = A0*B1*cell[i-1].mixture.MixtureFr_Coal[k]+(mass_fr-A0*B0)*cell[i].mixture.MixtureFr_Coal[k]+A0*B2*cell[0].mixture.MixtureFr_Coal[k];  //ghost point for cell[0]
                r += 0.0;  // to consider addition diffusion 
                if(k<3)
                    r += source[i].mixture_fr[k];  // to consider reaction source term, including particle reaction source
                Mixture_Fr[i] = r/(mass_fr+source[i].mass);
            }
            else
            {
                mass_fr = cell[i].u_old*cell[i].size_old*cell[i].Density_old; // the mass flow rate is the old mass flow rate! so .u, .size can't be used here!
                A0 = deta_x;
                // coefficients based on old nth value!
                B1 = B2;
                //B1 = 2*(cell[i].mixture.MFR_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.MFR_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/pow(((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old),3);
                B2 = 2*(cell[i].mixture.MFR_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.MFR_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old),3);
                B0 = B1+B2;

                r = A0*B1*cell[i-1].mixture.MixtureFr_Coal[k]+(mass_fr-A0*B0)*cell[i].mixture.MixtureFr_Coal[k]+A0*B2*cell[i+1].mixture.MixtureFr_Coal[k];

                r += 0.0;  // to consider additional diffusion
                if(k<3)
                    r += source[i].mixture_fr[k];
                Mixture_Fr[i] = r/(mass_fr+source[i].mass);
            }
        }

        for(i=0; i<domain_cellnumber; i++)
        {
            // to adjust Mixture_Fr if ignorable error occurs.
            Mixture_Fr[i] = (Mixture_Fr[i]<0 && Mixture_Fr[i]>-1e-7) ? 0.0 : Mixture_Fr[i];
            Mixture_Fr[i] = (Mixture_Fr[i]>1 && Mixture_Fr[i]<1+1e-7) ? 1.0 : Mixture_Fr[i];

            if(!(Mixture_Fr[i]>=0 && Mixture_Fr[i]<=1))
            {
                *MFRDiff_Flag = 0;
                printf("%dth step: error occurs during %dth species(%s) in 1st circulation in species diffusion process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,k,Species_Name[k],i,domain_cellnumber,deta_x);
                printf("number  cell_size       cell_size_old   cell.u         cell.MFr_Coal       cellnew.MixtureFr_Coal    cell.spedif     cell.density    cell.Tem\n");
                for(i1=0; i1<domain_cellnumber; i1++)
                {
                    if(i1<10 || i1>domain_cellnumber-10)
                        printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.MixtureFr_Coal[k],cell_new[i1].mixture.MixtureFr_Coal[k],cell[i1].mixture.MFR_Diff_Coeff[k],cell[i1].Density_old,cell[i1].mixture.Tem);
                    if(i1>i-20 && i1<i+20)
                        printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.MixtureFr_Coal[k],cell_new[i1].mixture.MixtureFr_Coal[k],cell[i1].mixture.MFR_Diff_Coeff[k],cell[i1].Density_old,cell[i1].mixture.Tem);
                    if(i1%200==0)
                        printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.MixtureFr_Coal[k],cell_new[i1].mixture.MixtureFr_Coal[k],cell[i1].mixture.MFR_Diff_Coeff[k],cell[i1].Density_old,cell[i1].mixture.Tem);        
                }
     
                printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].Density_old,cell[i+1].Density_old,cell[i-1].Density_old);
                printf("cell[0].mixture.MixtureFr_Coal[k]=%8.5f, cell[end].mixture.MixtureFr_Coal[k]=%8.5f\n",cell[0].mixture.MixtureFr_Coal[k],cell[domain_cellnumber-1].mixture.MixtureFr_Coal[k]);
                goto abnormal2;
            } 
            else  //to store the new massfraction
                cell_new[i].mixture.MixtureFr_Coal[k] = Mixture_Fr[i];      
        } 
    }  //for(k=..


// uniform the mass fraction
    for(i=0; i<domain_cellnumber; i++)
    {
        sum_massfr = 0.0;
        for(k=0; k<5; k++)
            sum_massfr += cell_new[i].mixture.MixtureFr_Coal[k];
        for(k=0; k<5; k++)
            cell_new[i].mixture.MixtureFr_Coal[k] /= sum_massfr;
    }

    // to restore the changed value (mass fraction and mixture density)  
    for(i=0; i<domain_cellnumber; i++)
    {
//         cell_new[i].size_old = cell[i].size_old;
//         cell_new[i].u_old = cell[i].u_old;
//         cell_new[i].Density_old = cell[i].Density_old;
//         for(k=0; k<5; k++)
//             cell_new[i].MixtureFr_Coal_old[k] = cell[i].MixtureFr_Coal_old[k];
        cell[i] = cell_new[i];
    }
 abnormal2:    ;
}










//*************************************************************************************//
// Forward_Euler explicit method to solve species diffusion process
void MFR_DIFFUSION_2NDFE(int E_S_number, int *MFRDiff_Flag, int domain_cellnumber, double deta_x, struct cell_information *cell, struct source_information *source)
{
    int i,k,i1;
    double Tem, Pre;
    double mass_fr;   // old mass flow rate for ith cell
    double sum_massfr, mix_density;  // summation of mole fraction from 1st species to (5-1)th species, but not for 0th species!!!
    double A0,B0,B1,B2;
    double B0_1, B1_1, B2_1;
    double Mixture_Fr[domain_cellnumber], r; // cell[i].mixture.MixtureFr_Coal[k]!!!
    //    double Density[domain_cellnumber][5];   // density of each species;
    extern double Density_Standard[Species_Number];

    struct cell_information cell_new[domain_cellnumber];


    //to set boundary conditions and record the initial values of each step	
    for(i=0; i<domain_cellnumber; i++)
    {   
        cell_new[i] = cell[i];
        for(k=0; k<5; k++)
        { 
            if(cell[i].mixture.MixtureFr_Coal[k]<0-TINY||cell[i].mixture.MixtureFr_Coal[k]>1+1e-6 )
            {
                printf("wierd! %d,%s,%20.18f\n",i,Species_Name[k],cell[i].mixture.MixtureFr_Coal[k]);
                goto abnormal2;
            }
        }
    }

    //to solve all species diffusion process, include 0th species, then to normalize the species mass fraction
    for(k=0; k<5; k++)
    {
        // to calculate vector a[],b[],c[] and d[]
        for(i=0; i<domain_cellnumber; i++)
        {
            if(i==0)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old; // cell.u has changed, shouldn't be used here!
                A0 = deta_x;
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.MFR_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.MFR_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].Density_old+cell[domain_cellnumber-1].Density_old)/4.0;
                B1_1 = (cell[i].mixture.MFR_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.MFR_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[domain_cellnumber-1].mixture.Density)/4.0;
                B2 = (cell[i].mixture.MFR_Diff_Coeff[k]*cell[i+1].size_old+cell[i+1].mixture.MFR_Diff_Coeff[k]*dy_co)*(cell[i].Density_old*cell[i+1].size_old+cell[i+1].Density_old*dy_co)/pow((dy_co+cell[i+1].size_old),3);
                B2_1 = (cell[i].mixture.MFR_Diff_Coeff[k]*cell[i+1].size+cell[i+1].mixture.MFR_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i+1].size+cell[i+1].mixture.Density*dy_co)/pow((dy_co+cell[i+1].size),3);
                B0 = B1+B2;
                B0_1 = B1_1+B2_1;

                r = A0*(B1+B1_1)*cell[domain_cellnumber-1].mixture.MixtureFr_Coal[k]+(mass_fr-A0*(B0+B0_1))*cell[i].mixture.MixtureFr_Coal[k]+A0*(B2+B2_1)*cell[i+1].mixture.MixtureFr_Coal[k];  //ghost point for cell[domain_cellnumber-1]
                r += 0.0;  // to consider additional diffusion due to other facts: thermal diffusion et al.
                if(k<3)   
                    r += source[i].mixture_fr[k];  // to consider reaction source term. Here, the reaction will be considered separately, but particle source term should be considered in this part.
                Mixture_Fr[i] = r/(mass_fr+source[i].mass);
            }
            else if(i==domain_cellnumber-1)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old;  // cell.u has changed, can't be used here!
                A0 = deta_x;
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.MFR_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.MFR_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i-1].size_old+cell[i-1].mixture.Density*dy_co)/pow((cell[i-1].size_old+dy_co),3);
                B1_1 = (cell[i].mixture.MFR_Diff_Coeff[k]*cell[i-1].size+cell[i-1].mixture.MFR_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i-1].size+cell[i-1].mixture.Density*dy_co)/pow((cell[i-1].size+dy_co),3);
                B2 = (cell[i].mixture.MFR_Diff_Coeff[k]+cell[0].mixture.MFR_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].Density_old+cell[0].Density_old)/4.0;
                B2_1 = (cell[i].mixture.MFR_Diff_Coeff[k]+cell[0].mixture.MFR_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[0].mixture.Density)/4.0;
                B0 = B1+B2;
                B0_1 = B1_1+B2_1;
            
                r = A0*(B1+B1_1)*cell[i-1].mixture.MixtureFr_Coal[k]+(mass_fr-A0*(B0+B0_1))*cell[i].mixture.MixtureFr_Coal[k]+A0*(B2+B2_1)*cell[0].mixture.MixtureFr_Coal[k];  //ghost point for cell[0]
                r += 0.0;  // to consider addition diffusion 
                if(k<3)
                    r += source[i].mixture_fr[k];  // to consider reaction source term, including particle reaction source
                Mixture_Fr[i] = r/(mass_fr+source[i].mass);
            }
            else
            {
                mass_fr = cell[i].u_old*cell[i].size_old*cell[i].Density_old; // the mass flow rate is the old mass flow rate! so .u, .size can't be used here!
                A0 = deta_x;
                // coefficients based on old nth value!
                B1 = B2;
                B1_1 = B2_1;
                //B1 = 2*(cell[i].mixture.MFR_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.MFR_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/pow(((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old),3);
                B2 = (cell[i].mixture.MFR_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.MFR_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old),3);
                B2_1 = (cell[i].mixture.MFR_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.MFR_Diff_Coeff[k]*cell[i].size)*(cell[i].mixture.Density*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Density*cell[i].size)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size),3);
                B0 = B1+B2;
                B0_1 = B1_1+B2_1;

                r = A0*(B1+B1_1)*cell[i-1].mixture.MixtureFr_Coal[k]+(mass_fr-A0*(B0+B0_1))*cell[i].mixture.MixtureFr_Coal[k]+A0*(B2+B2_1)*cell[i+1].mixture.MixtureFr_Coal[k];

                r += 0.0;  // to consider additional diffusion
                if(k<3)
                    r += source[i].mixture_fr[k];
                Mixture_Fr[i] = r/(mass_fr+source[i].mass);
            }
        }

        for(i=0; i<domain_cellnumber; i++)
        {
            // to adjust Mixture_Fr if ignorable error occurs.
            Mixture_Fr[i] = (Mixture_Fr[i]<0 && Mixture_Fr[i]>-1e-7) ? 0.0 : Mixture_Fr[i];
            Mixture_Fr[i] = (Mixture_Fr[i]>1 && Mixture_Fr[i]<1+1e-7) ? 1.0 : Mixture_Fr[i];

            if(!(Mixture_Fr[i]>=0 && Mixture_Fr[i]<=1))
            {
                *MFRDiff_Flag = 0;
                printf("%dth step: error occurs during %dth species(%s) in 1st circulation in species diffusion process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,k,Species_Name[k],i,domain_cellnumber,deta_x);
                printf("number  cell_size       cell_size_old   cell.u         cell.MixtureFr_Coal       cellnew.MixtureFr_Coal    cell.spedif     cell.density    cell.Tem\n");
                for(i1=0; i1<domain_cellnumber; i1++)
                {
                    if(i1<10 || i1>domain_cellnumber-10)
                        printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.MixtureFr_Coal[k],cell_new[i1].mixture.MixtureFr_Coal[k],cell[i1].mixture.MFR_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
                    if(i1>i-20 && i1<i+20)
                        printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.MixtureFr_Coal[k],cell_new[i1].mixture.MixtureFr_Coal[k],cell[i1].mixture.MFR_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
                    if(i1%200==0)
                        printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.MixtureFr_Coal[k],cell_new[i1].mixture.MixtureFr_Coal[k],cell[i1].mixture.MFR_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);        
                }
     
//                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\n",E_S_number,cell[i].mixture.MixtureFr_Coal[k],i==domain_cellnumber-1?cell[0].mixture.MixtureFr_Coal[k]:cell[i+1].mixture.MixtureFr_Coal[k],i==0?cell[domain_cellnumber-1].mixture.MixtureFr_Coal[k]:cell[i-1].mixture.MixtureFr_Coal[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
//                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,Mixture_Fr[i],cell_new[i+1].mixture.MixtureFr_Coal[k],cell_new[i-1].mixture.MixtureFr_Coal[k],cell_new[i].size,cell[i+1].size,cell_new[i-1].size);
                printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Density,cell[i+1].mixture.Density,cell[i-1].mixture.Density);
                printf("cell[0].mixture.MixtureFr_Coal[k]=%8.5f, cell[end].mixture.MixtureFr_Coal[k]=%8.5f\n",cell[0].mixture.MixtureFr_Coal[k],cell[domain_cellnumber-1].mixture.MixtureFr_Coal[k]);
                goto abnormal2;
            } 
            else  //to store the new massfraction
                cell_new[i].mixture.MixtureFr_Coal[k] = Mixture_Fr[i];      
        } 
    }  //for(k=..


// uniform the mass fraction
    for(i=0; i<domain_cellnumber; i++)
    {
        sum_massfr = 0.0;
        for(k=0; k<5; k++)
            sum_massfr += cell_new[i].mixture.MixtureFr_Coal[k];
        for(k=0; k<5; k++)
            cell_new[i].mixture.MixtureFr_Coal[k] /= sum_massfr;
    }

    // to restore the changed value (mass fraction and mixture density)  
    for(i=0; i<domain_cellnumber; i++)
    {
//         cell_new[i].size_old = cell[i].size_old;
//         cell_new[i].u_old = cell[i].u_old;
//         cell_new[i].Density_old = cell[i].Density_old;
//         for(k=0; k<5; k++)
//             cell_new[i].MixtureFr_Coal_old[k] = cell[i].MixtureFr_Coal_old[k];
        cell[i] = cell_new[i];
    }

 abnormal2:    ;
}















//*************************************************************************************//



//*************************************************************************************//
// Crank-Nicolson implicit method followed by an explicit method to solve species diffusion process
void MFR_DIFFUSION_CN(int E_S_number, int *MFRDiff_Flag, int domain_cellnumber, double deta_x, struct cell_information *cell, struct source_information *source)
{
    int i,k,i1;
    double Tem, Pre;
    double mass_fr;   // old mass flow rate for ith cell
    double sum_massfr, mix_density;  // summation of mole fraction from 1st species to (5-1)th species, but not for 0th species!!!

    //  a[i]u[i-1]+b[i]u[i]+c[i]u[i+1]=r[i], vector of tridiaginal matrix
    double a[domain_cellnumber],b[domain_cellnumber],c[domain_cellnumber],r[domain_cellnumber];
    double A0,B0,B1,B2,A0_1,B0_1,B1_1,B2_1;
    double Mixture_Fr[domain_cellnumber]; // cell[i].mixture.MixtureFr_Coal[k]!!!

    // struct property_info species_property;
    struct cell_information cell_new[domain_cellnumber];

    //to set boundary conditions and record the initial values of each step	
    for(i=0; i<domain_cellnumber; i++)
    {   
        cell_new[i] = cell[i];
        for(k=0; k<5; k++)
        { 
            if(cell[i].mixture.MixtureFr_Coal[k]<0-TINY||cell[i].mixture.MixtureFr_Coal[k]>1+1e-6 )
            {
                printf("wierd!in MFR_Diff process %d,%s,%20.18f\n",i,Species_Name[k],cell[i].mixture.MixtureFr_Coal[k]);
                goto abnormal2;
            }
        }
    }

    //to solve all species diffusion process, include 0th species, then to normalize the species mass fraction
    for(k=0; k<5; k++)
    {
        // to calculate vector a[],b[],c[] and d[]
        for(i=0; i<domain_cellnumber; i++)
        {
            if(i==0)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old; // cell.u has changed, shouldn't be used here!
                A0 = deta_x;
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.MFR_Diff_Coeff[k]+cell[domain_cellnumber-1].mixture.MFR_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].Density_old+cell[domain_cellnumber-1].Density_old)/4.0;
                B2 = (cell[i].mixture.MFR_Diff_Coeff[k]*cell[i+1].size_old+cell[i+1].mixture.MFR_Diff_Coeff[k]*dy_co)*(cell[i].Density_old*cell[i+1].size_old+cell[i+1].Density_old*dy_co)/pow((dy_co+cell[i+1].size_old),3);
                B0 = B1+B2;

                // coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.MFR_Diff_Coeff[k]+cell_new[domain_cellnumber-1].mixture.MFR_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[domain_cellnumber-1].mixture.Density)/4.0;
                B2_1 = (cell_new[i].mixture.MFR_Diff_Coeff[k]*cell_new[i+1].size+cell_new[i+1].mixture.MFR_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell[i+1].size+cell[i+1].mixture.Density*dy_co)/pow((cell_new[i+1].size+dy_co),3);
                B0_1 = B1_1+B2_1;

                a[i] = 0.0;
                b[i] = (mass_fr+A0*B0_1);
                b[i] += source[i].mass;
                c[i] = -A0*B2_1;
                r[i] = 2*A0*B1*cell[domain_cellnumber-1].MFR_old[k]+(mass_fr-A0*B0)*cell[i].MFR_old[k]+A0*B2*cell[i+1].MFR_old[k];  //ghost point for cell[domain_cellnumber-1]
                r[i] += 0.0;  // to consider additional diffusion due to other facts: thermal diffusion et al.
                if(k<3)
                    r[i] += source[i].mixture_fr[k];  // to consider reaction source term. Here, the reaction will be considered separately, but particle source term should be considered in this part.
            }
            else if(i==domain_cellnumber-1)
            {
                mass_fr = cell[i].u_old*dy_co*cell[i].Density_old;  // cell.u has changed, can't be used here!
                A0 = deta_x;
                // coefficients based on old nth value!
                B1 = (cell[i].mixture.MFR_Diff_Coeff[k]*cell[i-1].size_old+cell[i-1].mixture.MFR_Diff_Coeff[k]*dy_co)*(cell[i].Density_old*cell[i-1].size_old+cell[i-1].Density_old*dy_co)/pow((cell[i-1].size_old+dy_co),3);
                B2 = (cell[i].mixture.MFR_Diff_Coeff[k]+cell[0].mixture.MFR_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].Density_old+cell[0].Density_old)/4.0;
                B0 = B1+B2;
            
                // coefficients based on new nth (or n+1th) value!
                B1_1 = (cell_new[i].mixture.MFR_Diff_Coeff[k]*cell_new[i-1].size+cell_new[i-1].mixture.MFR_Diff_Coeff[k]*dy_co)*(cell[i].mixture.Density*cell_new[i-1].size+cell[i-1].mixture.Density*dy_co)/pow((cell_new[i-1].size+dy_co),3);
                B2_1 = (cell_new[i].mixture.MFR_Diff_Coeff[k]+cell_new[0].mixture.MFR_Diff_Coeff[k])/(dy_co+dy_co)*(cell[i].mixture.Density+cell[0].mixture.Density)/4.0;
                B0_1 = B1_1+B2_1;

                // coefficients of the algebratic equations
                a[i] = -A0*B1_1;
                b[i] = (mass_fr+A0*B0_1);
                b[i] += source[i].mass;
                c[i] = 0.0;
                r[i] = A0*B1*cell[i-1].MFR_old[k]+(mass_fr-A0*B0)*cell[i].MFR_old[k]+2*A0*B2*cell[0].MFR_old[k];  //ghost point for cell[0]
                r[i] += 0.0;  // to consider addition diffusion 
                if(k<3)               
                    r[i] += source[i].mixture_fr[k];  // to consider reaction source term, including particle reaction source
            }
            else
            {
                mass_fr = cell[i].u_old*cell[i].size_old*cell[i].Density_old; // the mass flow rate is the old mass flow rate! so .u, .size can't be used here!
                A0 = deta_x;
             //    if((cell[i].u_old*cell[i].size_old-cell[i].u*cell[i].size)/(cell[i].u*cell[i].size)*100>0.001)
//                     printf("u*y wierd!%d,  %10.7f =  %10.7f\n",i, cell[i].u_old*cell[i].size_old*1e5,cell[i].u*cell[i].size*1e5);
                // coefficients based on old nth value!
                B1 = B2;
                //B1 = 2*(cell[i].mixture.MFR_Diff_Coeff[k]*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].mixture.MFR_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==1?dy_co:cell[i-1].size_old)+cell[i-1].Density_old*cell[i].size_old)/pow(((i==1?dy_co:cell[i-1].size_old)+cell[i].size_old),3);
                B2 = (cell[i].mixture.MFR_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].mixture.MFR_Diff_Coeff[k]*cell[i].size_old)*(cell[i].Density_old*(i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i+1].Density_old*cell[i].size_old)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size_old)+cell[i].size_old),3);
                B0 = B1+B2;

                //coefficients based on new nth (or n+1th) value!
                B1_1 = B2_1;
                //B1_1 = 2*(cell_new[i].mixture.MFR_Diff_Coeff[k]*(i==1?dy_co:cell_new[i-1].size)+cell_new[i-1].mixture.MFR_Diff_Coeff[k]*cell_new[i].size)*(cell_new[i].mixture.Density*(i==1?dy_co:cell_new[i-1].size)+cell_new[i-1].mixture.Density*cell_new[i].size)/pow(((i==1?dy_co:cell_new[i-1].size)+cell_new[i].size),3);
                B2_1 = (cell[i].mixture.MFR_Diff_Coeff[k]*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.MFR_Diff_Coeff[k]*cell[i].size)*(cell[i].mixture.Density*(i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i+1].mixture.Density*cell[i].size)/pow(((i==domain_cellnumber-2?dy_co:cell[i+1].size)+cell[i].size),3);
                B0_1 = B1_1+B2_1;


                a[i] = -A0*B1_1;
                b[i] = (mass_fr+A0*B0_1);
                b[i] += source[i].mass;
                c[i] = -A0*B2_1;
                r[i] = A0*B1*cell[i-1].MFR_old[k]+(mass_fr-A0*B0)*cell[i].MFR_old[k]+A0*B2*cell[i+1].MFR_old[k];
             //    if((cell[i].Density_old-cell[i].mixture.Density)/cell[i].Density_old*100>0.001)
//                     printf("Density wierd!%d,  %10.7f === %10.7f\n",i, cell[i].Density_old, cell[i].mixture.Density);

                r[i] += 0.0;  // to consider additional diffusion
                if(k<3)
                    r[i] += source[i].mixture_fr[k];
            }
        }
        //    printf("0. %d %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.MixtureFr_Coal[He],cell_new[0].mixture.MixtureFr_Coal[He]);

        //to calculate the candidate massfraction of kth species velocity on (n+1)th odt line
        TRIDIAG(a,b,c,r,Mixture_Fr,domain_cellnumber);

        //    printf("0_1. %d %8.5f  %8.5f  %8.5f  %8.5f\n", E_S_number, cell_new[int(domain_cellnumber/2)].mixture.MixtureFr_Coal[He],cell_new[0].mixture.MixtureFr_Coal[He],Mixture_Fr[int(domain_cellnumber/2)],Mixture_Fr[0]);

        // Mixture_Fr = rho_mix*Yi, rho_mix and Yi should be calculated out separately here
        for(i=0; i<domain_cellnumber; i++)
        {
            //to restore the calculated cell[i].mixture.MixtureFr_Coal[k]*rho_mix
            cell_new[i].mixture.MixtureFr_Coal[k] = Mixture_Fr[i];  
            // to calculate the new mole fraction according to Xi = Yi*rho_mix/rho_i


            cell_new[i].mixture.MixtureFr_Coal[k] = (cell_new[i].mixture.MixtureFr_Coal[k]<0 && cell_new[i].mixture.MixtureFr_Coal[k]>-1e-7) ? 0.0 : cell_new[i].mixture.MixtureFr_Coal[k];
            cell_new[i].mixture.MixtureFr_Coal[k] = (cell_new[i].mixture.MixtureFr_Coal[k]>1 && cell_new[i].mixture.MixtureFr_Coal[k]<1+1e-7) ? 1.0 : cell_new[i].mixture.MixtureFr_Coal[k];

        

            if(!(cell_new[i].mixture.MixtureFr_Coal[k]>=0 && cell_new[i].mixture.MixtureFr_Coal[k]<=1))
            {
                *MFRDiff_Flag = 0;
                printf("%dth step: error occurs during %dth MFR(%s) in 1st circulation in MFR diffusion process!cells=%d, cell number=%d, deta_x=%f\n",E_S_number,k,Species_Name[k],i,domain_cellnumber,deta_x);
                printf("number  cell_size       cell_size_old   cell.u         cell.MixtureFr_Coal       cellnew.MixtureFr_Coal    cell.spedif     cell.density    cell.Tem\n");
                for(i1=0; i1<domain_cellnumber; i1++)
                {
                    if(i1<10 || i1>domain_cellnumber-10)
                        printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.MixtureFr_Coal[k],cell_new[i1].mixture.MixtureFr_Coal[k],cell[i1].mixture.MFR_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
                    if(i1>i-20 && i1<i+20)
                        printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.MixtureFr_Coal[k],cell_new[i1].mixture.MixtureFr_Coal[k],cell[i1].mixture.MFR_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);
                    if(i1%200==0)
                        printf("%6d  %8.5e\t%8.5e\t%8.5f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\t%10.7f\n",i1, cell[i1].size,cell[i1].size_old,cell[i1].u,cell[i1].mixture.MixtureFr_Coal[k],cell_new[i1].mixture.MixtureFr_Coal[k],cell[i1].mixture.MFR_Diff_Coeff[k],cell[i1].mixture.Density,cell[i1].mixture.Tem);        
                }
     
//                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%10.7f\t%10.7f\n",E_S_number,cell[i].mixture.MixtureFr_Coal[k],i==domain_cellnumber-1?cell[0].mixture.MixtureFr_Coal[k]:cell[i+1].mixture.MixtureFr_Coal[k],i==0?cell[domain_cellnumber-1].mixture.MixtureFr_Coal[k]:cell[i-1].mixture.MixtureFr_Coal[k],cell[i].size,i==domain_cellnumber-1?cell[0].size:cell[i+1].size,i==0?cell[domain_cellnumber-1].size:cell[i-1].size);
//                printf("%d\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",E_S_number,cell_new[i].mixture.MixtureFr_Coal[k],cell_new[i+1].mixture.MixtureFr_Coal[k],cell_new[i-1].mixture.MixtureFr_Coal[k],cell_new[i].size,cell[i+1].size,cell_new[i-1].size);
                printf("%8.5f\t%8.5f\t%8.5f\n",cell[i].mixture.Density,cell[i+1].mixture.Density,cell[i-1].mixture.Density);
                printf("cell[0].mixture.MixtureFr_Coal[k]=%8.5f, cell[end].mixture.MixtureFr_Coal[k]=%8.5f\n",cell[0].mixture.MixtureFr_Coal[k],cell[domain_cellnumber-1].mixture.MixtureFr_Coal[k]);
                goto abnormal2;
            }
        }
    }



//     // to calculate new mole fraction of 0th species in the mixture for each cell
//     for(i=0; i<domain_cellnumber; i++)
//     {
//         sum_massfr = 0.0;
//         for(k=1; k<5; k++)
//             sum_massfr += cell_new[i].mixture.MixtureFr_Coal[k];
//         cell_new[i].mixture.MixtureFr_Coal[0] = 1.0-sum_massfr;

//         if(cell_new[i].mixture.MixtureFr_Coal[0] > 1.0)
//         {
//             printf("0th species molefr in %dth cell too large! modification should be done\n",i);
            
//         }
//         else if(cell_new[i].mixture.MixtureFr_Coal[0]<0.0)
//         {
//             printf("0th species molefr in %dth cell too small(%f, %f, %f)! modification should be done\n",i,sum_massfr, cell_new[i].mixture.MixtureFr_Coal[0],cell[i].mixture.MixtureFr_Coal[0]);
//             sum_massfr += cell[i].mixture.MixtureFr_Coal[0];
//             cell_new[i].mixture.MixtureFr_Coal[0] = cell[i].mixture.MixtureFr_Coal[0]/sum_massfr;
//             for(k=1; k<5; k++)
//                 cell_new[i].mixture.MixtureFr_Coal[k] /= sum_massfr;
//         }
//     }

// uniform the mass fraction
    for(i=0; i<domain_cellnumber; i++)
    {
        sum_massfr = 0.0;
        for(k=0; k<5; k++)
            sum_massfr += cell_new[i].mixture.MixtureFr_Coal[k];
        for(k=0; k<5; k++)
            cell_new[i].mixture.MixtureFr_Coal[k] /= sum_massfr;
    }


    // to restore the changed value (mass fraction and mixture density)  
    for(i=0; i<domain_cellnumber; i++)
    {
//         cell_new[i].size_old = cell[i].size_old;
//         cell_new[i].u_old = cell[i].u_old;
//         cell_new[i].Density_old = cell[i].Density_old;
//         for(k=0; k<5; k++)
//             cell_new[i].MFR_old[k] = cell[i].MFR_old[k];
        cell[i] = cell_new[i];
    }

 abnormal2:    ;
}
