#include "header.h"


/***********************************************************************************************************
This sub function is used to do the triplet map process. The meaning of the variables defined in the function
are as follows:
eddy_cell: cell number that eddy occupies
eddy_start: cell index that eddy begins from left
trip_n: 1/3 of eddy_cell
eddy_length: selected length of a candidate eddy, which is not the true eddy length
eddy_position: position of eddy left start point,but from (0 domain_y),which is not the true eddy position
eddy_pdfl_c: constant c in eddy length pdf function
eddy_position1: really position of eddy left start point. eddy_position1=eddy_position-domain_y/2
ranum: random number that used to judge if a candidate eddy will occur
eddy_l: the real eddy length based on eddy_length and eddy_position
eddy_l1/l2/l3: eddy length of each 1/3 eddy
energy_u/v/w: vij, which indicates the stretch rate of that position
energy_u1/v1/w1 et al: the same meaning with energy_u/v/w, but for 1/3 eddy.
energy_uvw: terms in the square root of the equation calculating lemda
energy_uvw1/uvw2/uvw3: each terms of 1/3 eddy
lemda: eddy possibility of an eddy occurrence. if eddy_pdf>eddy_rand, the eddy will occur
dt_stirate distribution
eddy_pdf: r: time step for eddy judgement in poisson process
eddy_u_mean: favor average u velocity of the eddy;
eddy_v_mean:
eddy_w_mena:
pdf_fl / pdf_gy: pdf value of eddy length and position
u_trip/v_trip/w_trip:u/v/w velocity after the triplet map
dy_trip: cell size after triplet map according mass flow rate conservation. dy_trip=dy*u/u_trip;
c_u/v/w: index of pressure scrambling: u_trip=u[f(y0)]+c_u*K(y)
eddy_timescale: eddy time scale
eddy_max: eddy max length in each ODT step.
************************************************************************************************************/

void TRIPLET_MAP(int E_S_number, int E_S_old, int domain_cellnumber, int *core_bdcell,  double *grid_y, struct cell_information *cell, struct eddy_information *eddy_info, struct source_information *source, struct cell_average cell_ave[][ysample_number], int *eddy_occur, int *eddy_totalnumber, int *Trip_Flag, double *eddy_lp)
{
    // FLAG is a sign that shows if this triplet map process is valid.
    int eddy_cell, eddy_start, eddy_subcell[2], i, trip_n, FLAG;				//number of cells that an eddy occupis, and the number index that an eddy starts 
    double odt_height;
    double bd_coe = 1.0;
    double core_bd[2], core_length;  //length of the core region
    double eddy_bd[2]; // to define the boundary of the eddy appearance region
    double eddy_max, eddy_min,eddy_l_frequent;
    double eddy_length, eddy_length1,eddy_position,eddy_pdfl_c=0.0,eddy_position1,ranum;
    double Energy_eddy, Energy_eddy1, Energy_eddy2, Energy_eddy3;   // this term is eddy turn_up energy. if Energy_eddy<0, the eddy will be suppressed!!!!!!!!
    double u_var_max[3];    // the maximum u difference for three small eddies in one large candidate eddy.

    double lemda, eddy_pdf, eddy_rand;        // lemda: eddy distribution rate. eddy_pdf:posion stream possiblity
    double dt_stir,eddy_mass_mean=0.0, eddy_u_mean=0.0, eddy_vlate_mean=0.0, eddy_w_mean=0.0, eddy_v_mean=0.0;  //eddy velocity
    double pdf_fl, pdf_gy;					// pdf of eddy length and eddy position
    double c_u,c_v,c_w;  //ci in triplet map equ. u(y)->u(f(y))+ci*(y-f(y))
    double eddy_timescale;  //eddy time scale

    odt_height = E_S_number*dx_stir;
    //	struct cell_information cell_trip[totalnumber/3];
    FLAG = 1;  // default flag of triplet map process is valid.
    eddy_min = 6.0*dy_cen;  //the minimum eddy length
    core_bd[0] = grid_y[core_bdcell[0]];
    //core_bd[0] = core_bd[0]<(grid_y[0]+0.005)?grid_y[0]+0.005:core_bd[0];
    core_bd[1] = grid_y[core_bdcell[1]+1];

    core_length = grid_y[core_bdcell[1]+1]-grid_y[core_bdcell[0]];
    eddy_bd[0] = (grid_y[core_bdcell[0]]-bd_coe*core_length)<(grid_y[0]+0.01)?(grid_y[0]+0.01):(grid_y[core_bdcell[0]]-bd_coe*core_length);
    eddy_bd[1] = (grid_y[core_bdcell[1]+1]+bd_coe*core_length)>(grid_y[domain_cellnumber]-0.01)?(grid_y[domain_cellnumber]-0.01):(core_bd[1]+bd_coe*core_length);

    //to set maximum u difference for 3 small eddies a large value
    u_var_max[0] = 5.0;
    u_var_max[1] = 5.0;
    u_var_max[2] = 5.0;

    // to sample an eddy position and the corresponding candidate cell number of left boundary of the eddy
    //in order to increase eddy sampling method, domain_factor should also be a function of jet height
    //this function should be tested later. anyway, actually, in my opinion, domain_factor has only 'a little' effect
    //on eddy selection procedure when stirnumber is not very high. when stirnumber>100 in this simulation
    //domain_factor won't be a problem whether it is a function or a constant.
    ranum = (double)rand()/RAND_MAX;
    eddy_position = eddy_bd[0]+ranum*(eddy_bd[1]-eddy_bd[0]-eddy_min);		
    eddy_position1 = eddy_position;
    eddy_start = (int)((eddy_position1+domain_y/2.0)/domain_y*domain_cellnumber);


    // to judge which cell is the exact left boundary of the eddy at the given eddy position
    while((grid_y[eddy_start]>eddy_position1 || grid_y[eddy_start+3]<eddy_position1) && eddy_start+3<domain_cellnumber)
    {
        if(grid_y[eddy_start]>eddy_position1)
            eddy_start -= 3;
        else if(grid_y[eddy_start+1]<eddy_position1)
            eddy_start += 3;
    }
    while(grid_y[eddy_start]>eddy_position1)
        eddy_start --;
    while(grid_y[eddy_start]<eddy_position1)
        eddy_start ++;
    eddy_start = fabs(eddy_position1-grid_y[eddy_start])>fabs(grid_y[eddy_start-1]-eddy_position1)?eddy_start-1:eddy_start;

    /*     if(eddy_start+eddy_cell>domain_cellnumber) */
    /*         printf("eddy_start weird! %d, %8.5f\n", eddy_start, eddy_position1); */
	
    //printf("eddy_cell=%d\t", eddy_cell);	

  /********to set the maximum eddy size as approximate 2 times of spreading width.******/ 
    /*in this model, eddy_frequent (where maximum pdf_eddy appears) is also a function of height****/
    // eddy_max = (eddy_bd[1]-eddy_bd[0]);    // to get the maximium eddy length
    eddy_max = (0.2*odt_height)<(jet_width_OS+0.1*odt_height)?(jet_width_OS+0.1*odt_height):(0.2*odt_height);
    // notes: eddy_l_frequent should be a variable!
    //eddy_l_frequent = 0.006+0.005*dx*xstep;
    eddy_l_frequent = 0.003;
	

    /* to calculate constant c in pdf of eddy length f(l)=c*pow(eddy_length,-2)*exp(-2*eddy_length_frequent/eddy_length).
       c was calculated by principle that integral of f(l) through lmin and lmax is unit */
    eddy_pdfl_c=2*eddy_l_frequent/(exp(-2*eddy_l_frequent/eddy_max)-exp(-2*eddy_l_frequent/eddy_min));

    // to sample an eddy length according to pdf(eddy_length) and to calculate a candidate cell numbers which is occupied by the eddy
    ranum = (double) rand()/RAND_MAX;
    eddy_length = -2*eddy_l_frequent/log(2*ranum*eddy_l_frequent/eddy_pdfl_c+exp(-2*eddy_l_frequent/eddy_min));


    //if meet the requirement of eddy length and eddy position, to calculate the exact eddy length
    if(grid_y[eddy_start]>eddy_bd[0] && grid_y[eddy_start]+eddy_length<eddy_bd[1])
    {
        eddy_cell = (int)(eddy_length/domain_y*domain_cellnumber);

        if(eddy_cell+eddy_start >= domain_cellnumber)
            eddy_cell = domain_cellnumber-eddy_start-100;

        while((grid_y[eddy_cell+eddy_start]-grid_y[eddy_start]<eddy_length) && (eddy_start+eddy_cell<domain_cellnumber-3))
            eddy_cell += 3;
        while(grid_y[eddy_cell+eddy_start]-grid_y[eddy_start]>eddy_length)
            eddy_cell -= 3;
        while((grid_y[eddy_cell+eddy_start]-grid_y[eddy_start]<eddy_length) && (eddy_start+eddy_cell<domain_cellnumber-1))
            eddy_cell ++ ;

        //printf("eddy_cell after = %d\n", eddy_cell);

        // to adjust eddy cells so that it was integer times of 3.
        eddy_cell= (int)((eddy_cell+1)/3)*3;

        eddy_cell = (eddy_start+eddy_cell>domain_cellnumber)?eddy_cell-3:eddy_cell;

        if(grid_y[eddy_start+eddy_cell]-grid_y[eddy_start]>eddy_length)
        {
            if(fabs(grid_y[eddy_start+eddy_cell]-grid_y[eddy_start]-eddy_length)>fabs(grid_y[eddy_start+eddy_cell-3]-grid_y[eddy_start]-eddy_length))
                eddy_cell -= 3;
        }
        else
        {
            if((eddy_start+eddy_cell+3<domain_cellnumber)&& (fabs(grid_y[eddy_start+eddy_cell]-grid_y[eddy_start]-eddy_length)>fabs(grid_y[eddy_start+eddy_cell+3]-grid_y[eddy_start]-eddy_length)))
                eddy_cell += 3;
        }

        // 
        eddy_cell = eddy_cell<6?6:eddy_cell;
        //eddy_cell = (eddy_start+eddy_cell>domain_cellnumber)?eddy_cell-3:eddy_cell;

        trip_n=(eddy_cell/3);							// 1/3 cells of an eddy cell number

        // to record the selected random eddy length (in order to figue out the potential mistakes)
        eddy_length1 = eddy_length;
        // to identify the final exact eddy length
        eddy_length = grid_y[eddy_start+eddy_cell]-grid_y[eddy_start];
        /* 	} */

        // if right boundary of the eddy exceed domain right boundary, this sampling is abandoned
        if(grid_y[eddy_start+eddy_cell]<eddy_bd[1])
        {
 //            //if the height is less than origin,no eddy occurs
//             if(xstep*dx<origin)
//             {
//                 Energy_eddy1 = 0.0;
//                 Energy_eddy2 = 0.0;
//                 Energy_eddy3 = 0.0;
//             }
//             else
//             {
//                 //for small eddies(small cell numbers), the eddy energy was calculated based on splitting each cell in the eddy into 3 pieces.
//                 if(eddy_cell<18)
//                     SMALL_EDDY_ENERGY(eddy_start, eddy_cell, cell, &Energy_eddy1, &Energy_eddy2, &Energy_eddy3);

//                 //to calculate the eddy energy of each 1/3 length of the eddy
//                 //for large eddies, just using original cells
//                 else
//                 {
//                     //to split a large candidate eddy into three equal pieces
//                     eddy_subcell[0] = trip_n;
//                     while(grid_y[eddy_start+eddy_subcell[0]]-grid_y[eddy_start]<(eddy_length/3.0))
//                         eddy_subcell[0] +=3 ;
//                     while(grid_y[eddy_start+eddy_subcell[0]]-grid_y[eddy_start]>(eddy_length/3.0))
//                         eddy_subcell[0] -= 3;
//                     eddy_subcell[0] =((int) ((eddy_subcell[0]+1)/3))*3;
//                     if(grid_y[eddy_subcell[0]+eddy_start]-grid_y[eddy_start]>eddy_length/3.0)
//                         eddy_subcell[0] = fabs((grid_y[eddy_start+eddy_subcell[0]]-grid_y[eddy_start])-eddy_length/3.0)>fabs((grid_y[eddy_start+eddy_subcell[0]-3]-grid_y[eddy_start])-eddy_length/3.0)?eddy_subcell[0]-3:eddy_subcell[0];
//                     else
//                         eddy_subcell[0] = fabs((grid_y[eddy_start+eddy_subcell[0]]-grid_y[eddy_start])-eddy_length/3.0)>fabs((grid_y[eddy_start+eddy_subcell[0]+3]-grid_y[eddy_start])-eddy_length/3.0)?eddy_subcell[0]+3:eddy_subcell[0];

//                     if(eddy_subcell[0]<6)
//                     {
//                         //     printf("too small cell number for the first piece!\n");
//                         eddy_subcell[0] = 6;
//                     }

//                     eddy_subcell[1] = (int)((eddy_cell-eddy_subcell[0])/2)+eddy_subcell[0];
//                     while(grid_y[eddy_start+eddy_subcell[1]]-grid_y[eddy_start]<(eddy_length*2.0/3.0))
//                         eddy_subcell[1] += 3;
//                     while(grid_y[eddy_start+eddy_subcell[1]]-grid_y[eddy_start]>(eddy_length*2.0/3.0))
//                         eddy_subcell[1] -= 3;
//                     eddy_subcell[1] = ((int)((eddy_subcell[1]+1)/3))*3;
//                     if(grid_y[eddy_subcell[1]+eddy_start]-grid_y[eddy_start]>eddy_length*2.0/3.0)
//                         eddy_subcell[1] = fabs((grid_y[eddy_start+eddy_subcell[1]]-grid_y[eddy_start])-eddy_length*2.0/3.0)>fabs((grid_y[eddy_start+eddy_subcell[1]-3]-grid_y[eddy_start])-eddy_length*2.0/3.0)?eddy_subcell[1]-3:eddy_subcell[1];
//                     else
//                         eddy_subcell[1] = fabs((grid_y[eddy_start+eddy_subcell[1]]-grid_y[eddy_start])-eddy_length*2.0/3.0)>fabs((grid_y[eddy_start+eddy_subcell[1]+3]-grid_y[eddy_start])-eddy_length*2.0/3.0)?eddy_subcell[1]+3:eddy_subcell[1];
//                     if(eddy_subcell[1]-eddy_subcell[0]<6)
//                     {
//                         //    printf("too small cellnumber for the second piece!\n");
//                         eddy_subcell[1] = eddy_subcell[0]+6;
//                     }

//                     //Since for a small Z value (Z<100, eg.), a too large eddy couldn't be supressed because the energy_eddy>Z, but not energy_eddy>>Z,
//                     //an additional step is used to supressed the flat profile of u velocity of the eddy.
//                     //first, to calculate a candidate maximum u difference (u_var_max[]) for each of small eddies.
//                     //if any of these 3 u_var_max[] be less than a small u value, for eg. 0.1m/s, then, one of small eddies was regarded as having a flat profile of u velocity, and the candidate large eddy was suspended.
//                     if(grid_y[eddy_start+eddy_cell]-grid_y[eddy_start]>jet_width+xstep*dx*0.05)  //very large eddy: eddy_length>half_widht
//                     {
//                         u_var_max[0] = fabs(cell[eddy_start].u-cell[eddy_start+(int)(eddy_subcell[0]/2)].u)>fabs(cell[eddy_start].u-cell[eddy_start+eddy_subcell[0]].u)?fabs(cell[eddy_start].u-cell[eddy_start+(int)(eddy_subcell[0]/2)].u):fabs(cell[eddy_start].u-cell[eddy_start+eddy_subcell[0]].u);
//                         u_var_max[0] = u_var_max[0]>fabs(cell[eddy_start+(int)(eddy_subcell[0]/2)].u-cell[eddy_start+eddy_subcell[0]].u)?u_var_max[0]:fabs(cell[eddy_start+(int)(eddy_subcell[0]/2)].u-cell[eddy_start+eddy_subcell[0]].u);

//                         u_var_max[1] = fabs(cell[eddy_start+eddy_subcell[0]].u-cell[eddy_start+(int)((eddy_subcell[0]+eddy_subcell[1])/2)].u)>fabs(cell[eddy_start+eddy_subcell[0]].u-cell[eddy_start+eddy_subcell[1]].u)?fabs(cell[eddy_start+eddy_subcell[0]].u-cell[eddy_start+(int)((eddy_subcell[0]+eddy_subcell[1])/2)].u):fabs(cell[eddy_start+eddy_subcell[0]].u-cell[eddy_start+eddy_subcell[1]].u);
//                         u_var_max[1] = u_var_max[1]>fabs(cell[eddy_start+(int)((eddy_subcell[0]+eddy_subcell[1])/2)].u-cell[eddy_start+eddy_subcell[1]].u)?u_var_max[1]:fabs(cell[eddy_start+(int)((eddy_subcell[0]+eddy_subcell[1])/2)].u-cell[eddy_start+eddy_subcell[1]].u);

//                         u_var_max[2] = fabs(cell[eddy_start+eddy_subcell[1]].u-cell[eddy_start+(int)((eddy_cell+eddy_subcell[1])/2)].u)>fabs(cell[eddy_start+eddy_subcell[1]].u-cell[eddy_start+eddy_cell-1].u)?fabs(cell[eddy_start+eddy_subcell[1]].u-cell[eddy_start+(int)((eddy_cell+eddy_subcell[1])/2)].u):fabs(cell[eddy_start+eddy_subcell[1]].u-cell[eddy_start+eddy_cell-1].u);
//                         u_var_max[2] = u_var_max[2]>fabs(cell[eddy_start+(int)((eddy_cell+eddy_subcell[1])/2)].u-cell[eddy_start+eddy_cell-1].u)?u_var_max[2]:fabs(cell[eddy_start+(int)((eddy_cell+eddy_subcell[1])/2)].u-cell[eddy_start+eddy_cell-1].u);

//                         //if no more than 2 small cells have a flat u profile
//                         if((u_var_max[0]>eddy_large_umin && u_var_max[1]>eddy_large_umin) || (u_var_max[0]>eddy_large_umin && u_var_max[2]>eddy_large_umin) || (u_var_max[1]>eddy_large_umin && u_var_max[2]>eddy_large_umin))
//                         {

//                             // to calculate eddy energy for each small eddy, which is 1/3 of the orginal eddy

//                             /*         Energy_eddy = LARGE_EDDY_SUP(eddy_start, eddy_cell, cell); */
//                             Energy_eddy1 = CAL_LARGEEDDY_ENERGY(eddy_start, eddy_subcell[0], cell);
//                             Energy_eddy2 = CAL_LARGEEDDY_ENERGY(eddy_start+eddy_subcell[0], eddy_subcell[1]-eddy_subcell[0], cell);
//                             Energy_eddy3 = CAL_LARGEEDDY_ENERGY(eddy_start+eddy_subcell[1], eddy_cell-eddy_subcell[1],cell);
//                         }
//                         //flat profile
//                         else
//                         {                    
//                             Energy_eddy1 = -0.0;
//                             Energy_eddy2 = -0.0;
//                             Energy_eddy3 = -0.0;
//                         }
//                     }
//                     //not large eddy
//                     else
//                     {
//                         Energy_eddy1 = CAL_LARGEEDDY_ENERGY(eddy_start, eddy_subcell[0], cell);
//                         Energy_eddy2 = CAL_LARGEEDDY_ENERGY(eddy_start+eddy_subcell[0], eddy_subcell[1]-eddy_subcell[0], cell);
//                         Energy_eddy3 = CAL_LARGEEDDY_ENERGY(eddy_start+eddy_subcell[1], eddy_cell-eddy_subcell[1],cell);
//                     }
//                 }
//             }

            // large eddy suspension
//            if(Energy_eddy1<=0.0 || Energy_eddy2<=0.0 || Energy_eddy3<=0)

// large eddy suppression method. if the extanded length out of core region is larger than 1/3 of eddy length, this eddy should be suspended
            if(core_bd[0]-grid_y[eddy_start]>eddy_length/3.0 || grid_y[eddy_start+eddy_cell]-core_bd[1]>eddy_length/3.0)
            {
                /*         lemda = CAL_LEMDA(eddy_start, eddy_cell, cell); */
                /*         pdf_fl = eddy_pdfl_c*pow(eddy_length,-2)*exp(-2*eddy_l_frequent/eddy_length); */
                /*         pdf_gy = 1/(domain_y*domain_factor-eddy_length); */
                /*         for(i=eddy_start; i<eddy_start+eddy_cell;i++) */
                /*         { */
                /*             eddy_u_mean += cell[i].u*cell[i].size*cell[i].mixture.Density; */
                /*             eddy_mass_mean += cell[i].mixture.Density*cell[i].size; */
                /*         } */
                /*         eddy_u_mean /= eddy_mass_mean; */
                /*         dt_stir = dx_stir/eddy_u_mean; */
                /*         eddy_pdf = lemda*dt_stir/(pdf_fl*pdf_gy);  */
                /*         eddy_lp[2] += 1.0; */
                /*         eddy_lp[3] += eddy_pdf; */
//                printf("too large eddies! eddy_start=%8.5f, core_0=%8.5f,core_1=%8.5f,eddy_end=%8.5f,eddy_length=%8.5f\n",grid_y[eddy_start],core_bd[0],core_bd[1],grid_y[eddy_start+eddy_cell],eddy_length);
                eddy_pdf = -0.0; 
            }

            //if the height is less than origin,no eddy occurs
            else if(odt_height<origin)
            {
                eddy_pdf = -0.0;
            }

            // neither too large eddy nor small eddy
            else
            {
                lemda = CAL_LEMDA(eddy_start, eddy_cell, cell);
                pdf_fl = eddy_pdfl_c*pow(eddy_length,-2)*exp(-2*eddy_l_frequent/eddy_length);
                pdf_gy = 1/(eddy_bd[1]-eddy_bd[0]-eddy_min);
                for(i=eddy_start; i<eddy_start+eddy_cell;i++)
                {
                    eddy_u_mean += cell[i].u*cell[i].size*cell[i].mixture.Density;
                    eddy_mass_mean += cell[i].mixture.Density*cell[i].size;
                }

                eddy_u_mean /= eddy_mass_mean;
                dt_stir = dx_stir/eddy_u_mean;
                eddy_pdf = lemda*dt_stir/(pdf_fl*pdf_gy);
                if(lemda<0 || pdf_fl<0)
                    printf("impossible!!!!!!!!!!!!!!!!!!!!!!!lemda=%8.5e\tpdf_fl=%8.5e\n",lemda,pdf_fl);
                eddy_lp[2] += 1.0;
                eddy_lp[3] += eddy_pdf;
                //            printf("lemda=%8.5e\t eddy_pdf=%8.5e\n", lemda, eddy_pdf);
            }

            //to suspend the small eddy
            if(eddy_pdf>2 || eddy_cell<=6)
                eddy_pdf=-1.0;

            eddy_rand = (double)rand()/RAND_MAX;

            //eddy occurs
            if(eddy_pdf > eddy_rand)
            {
                eddy_timescale = C_odt/(eddy_length*eddy_length*lemda);
			
                if(fabs(cell[eddy_start].y) > 0.35)
                    printf("too far from center line %8.5f\t%8.5f\t%8.5f\t%8.5f\n",cell[eddy_start].y,cell[eddy_start+eddy_cell].y,eddy_length,eddy_pdf);
            
                Energy_eddy = CAL_SMALLEDDY_ENERGY(eddy_start, eddy_cell, cell);

                if(eddy_start<0 || eddy_start+eddy_cell>domain_cellnumber)
                    printf("eddy selection weird!!!!! eddy_start=%d, eddy_trip=%d",eddy_start, eddy_cell);

                /*********************************************************************************************/
                //  to do the triplet map process, after this step, struct "cell" will be changed by triplet!!!!!!!
                EDDY_TRIPLET(E_S_number, &FLAG, eddy_start, eddy_cell, cell, source, cell_ave);

                if(FLAG == 1)  // if this triplet map process is valid
                {
                    (*eddy_occur)++;      //eddy_occur is the serial number for the struct array eddy_info[]
                    (*eddy_totalnumber)++;     // eddy_number is the total number of the eddies
                    // to record the summation of eddy length and its possibility
                    eddy_lp[0] += eddy_length;
                    eddy_lp[1] += eddy_pdf;
                    *Trip_Flag = 1;  //eddy occurs

                    // to get the new coordinate
                    RENEW_Y(domain_cellnumber, grid_y, cell);

                    core_bdcell[0] = core_bdcell[0]>eddy_start?eddy_start:core_bdcell[0];
                    core_bdcell[0] = core_bdcell[0]<99?99:core_bdcell[0];
                    core_bdcell[1] = eddy_start+eddy_cell-1>core_bdcell[1]?eddy_start+eddy_cell-1:core_bdcell[1];
                    core_bdcell[1] = core_bdcell[1]>domain_cellnumber-100?domain_cellnumber-100:core_bdcell[1];
                    core_bd[0] = grid_y[core_bdcell[0]];
                    core_bd[1] = grid_y[core_bdcell[1]+1];
                    eddy_bd[0] = (core_bd[0]-bd_coe*(core_bd[1]-core_bd[0]))<(-domain_y/2.0+0.01)?(-domain_y/2.0+0.01):(core_bd[0]-bd_coe*(core_bd[1]-core_bd[0]));
                    eddy_bd[1] = (core_bd[1]+bd_coe*(core_bd[1]-core_bd[0]))>(domain_y/2.0-0.01)?(domain_y/2.0-0.01):(core_bd[1]+bd_coe*(core_bd[1]-core_bd[0]));

                    // printf("eddy1=%8.5f l1(%8.5f<%d>)=%8.5f  eddy2=%8.5f  l2(%8.5f<%d>)=%8.5f,   eddy3=%8.5f  l3=%8.5f\n", Energy_eddy1,grid_y[eddy_start+eddy_subcell[0]], eddy_subcell[0],  grid_y[eddy_start+eddy_subcell[0]]-grid_y[eddy_start],  Energy_eddy2, grid_y[eddy_start+eddy_subcell[1]], eddy_subcell[1],  grid_y[eddy_start+eddy_subcell[1]]-grid_y[eddy_start+eddy_subcell[0]],Energy_eddy3, grid_y[eddy_start+eddy_cell]-grid_y[eddy_start+eddy_subcell[1]]);

                    /*	//record tripletmap when: 50 < eddy_cell < 70
                        if(eddy_cell<70 && eddy_cell>50)
                        {
                        printf("the velocity of the eddy before the triplet map is:\n");
                        printf("  cell_po         velo_u          velo_v          velo_w\n");
                        for(i=eddy_start;i<eddy_start+eddy_cell;i++)
                        printf("%8.4f\t%8.4f\t%8.4f\t%8.4f\n",cell[i].y,cell[i].u,cell[i].v_trip,cell[i].w);
                        }*/

                    eddy_u_mean = 0.0;
                    eddy_v_mean = 0.0;
                    eddy_w_mean = 0.0;
                    eddy_vlate_mean = 0.0;
                    eddy_mass_mean = 0.0;

                    //to calculate eddy average velocity			
                    for(i=eddy_start; i<eddy_start+eddy_cell;i++)
                    {
                        eddy_u_mean += cell[i].mixture.Density*cell[i].u*cell[i].u*cell[i].size;
                        eddy_v_mean += cell[i].mixture.Density*cell[i].u*(v_tripcoe*cell[i].v_trip+cell[i].v_lateral*v_latcoe)*cell[i].size;
                        eddy_w_mean += cell[i].mixture.Density*cell[i].u*cell[i].w*cell[i].size;
                        eddy_vlate_mean += cell[i].mixture.Density*cell[i].u*cell[i].v_lateral*cell[i].size;
                        eddy_mass_mean += cell[i].mixture.Density*cell[i].size*cell[i].u;			
                    }

                    eddy_u_mean /= eddy_mass_mean;		
                    eddy_v_mean /= eddy_mass_mean;
                    eddy_w_mean /= eddy_mass_mean;	
                    eddy_vlate_mean /= eddy_mass_mean;

                    // to record the detailed information for each eddy
                    eddy_info[*eddy_occur-1].ser_number = *eddy_totalnumber;
                    eddy_info[*eddy_occur-1].xcellstart = E_S_old;
                    //                    eddy_info[*eddy_occur-1].xcellstart1 = *xstir;
                    eddy_info[*eddy_occur-1].ycellstart = eddy_start;
                    eddy_info[*eddy_occur-1].ycellend = eddy_start+eddy_cell-1;
                    eddy_info[*eddy_occur-1].ystart = grid_y[eddy_start];
                    eddy_info[*eddy_occur-1].yend = grid_y[eddy_start+eddy_cell];
                    eddy_info[*eddy_occur-1].lifetime = beta*eddy_timescale;
                    eddy_info[*eddy_occur-1].length = eddy_length;
                    eddy_info[*eddy_occur-1].height_judge = eddy_timescale*eddy_u_mean*beta;
                    eddy_info[*eddy_occur-1].height = eddy_length*eddy_factor;
                    eddy_info[*eddy_occur-1].xcellend = E_S_old+(int)(eddy_info[*eddy_occur-1].height_judge/dx_stir);  //according to height_judge, or eddy_timescale...
                    eddy_info[*eddy_occur-1].u = eddy_u_mean;
                    eddy_info[*eddy_occur-1].v = eddy_v_mean; //the equavalent lateral velocity
                    eddy_info[*eddy_occur-1].w = eddy_w_mean;
                    eddy_info[*eddy_occur-1].vlateral = eddy_vlate_mean;

                      if(*eddy_totalnumber%100 == 0)
                      {
                            printf("%4.1d  %6.4f  %8.5fm  %8.5fs  %8.5fm %8.5f(%4.1d)   %8.5f(%4.1d)  %8.5fmm %8.5fmm  %d  %8.5f  %8.5f\n",*eddy_totalnumber,eddy_pdf,E_S_number*dx_stir,eddy_timescale,eddy_position1,grid_y[eddy_start],eddy_start,grid_y[eddy_start+eddy_cell],eddy_start+eddy_cell,eddy_length*1000,eddy_length1*1000,eddy_cell, fabs(cell[eddy_start].y),Energy_eddy);

                            //                         printf("%6.4e\t%6.4e\t%6.4e\t%6.4e\n",lemda, Energy_eddy1, Energy_eddy2, Energy_eddy3);
                            //                            printf("eddy_height=%f, eddy_length=%f, ratio=%f, ratio1=%f\n",eddy_info[*eddy_occur-1].height,eddy_length,eddy_info[*eddy_occur-1].height/eddy_length,eddy_timescale/eddy_length);
                             printf("bd[0]=%8.5f,core[0](%d)=%8.5f,core[1](%d)=%8.5f,bd[1]=%8.5f, core_length=%8.5f\n",eddy_bd[0],core_bdcell[0],core_bd[0],core_bdcell[1],core_bd[1],eddy_bd[1],core_bd[1]-core_bd[0]);
                       }
                }
            }			
            /*             else */
            /*                 //printf("eddy doesn't occur\n"); */
            /*                 ; */
        }
//         else
//         {
// //            *xstir--;
//             printf("too large eddy! case 1!!! eddy_bd[0]=%8.5f, grid_y[eddy_end]=%8.5f, grid_y[1]=%8.5f, eddy_bd[1]=%8.5f\n",eddy_bd[0],grid_y[eddy_start+eddy_cell],grid_y[eddy_start+eddy_cell-3], eddy_bd[1]);
//         }
    }
    // if eddy_position+eddy_length > domain_y, to suspend this sampling process
//     else
//     { 
// //        *xstir--;
//          printf("too large sample!! case0!! bd[0]=%8.5f, grid_y[eddy_start]=%8.5f,grid_y[eddy_end]=%8.5f, bd[1]=%8.5f\n",eddy_bd[0],grid_y[eddy_start],grid_y[eddy_start]+eddy_length,eddy_bd[1]);
//         // printf("can be here!!!%8.5f,%8.5f,%8.5f\n",grid_y[eddy_start],eddy_position1,-domain_y*domain_factor/2.0);
//     }
}





/*****************************************************************************************/
// This function is used to suppress large eddy.
// In this function, the eddy flux energy is calculated
/*****************************************************************************************/
void SMALL_EDDY_ENERGY(int eddy_start, int eddy_cell, struct cell_information *cell, double *Energy_eddy1, double *Energy_eddy2, double *Energy_eddy3)
{

    int i,j,k;
    int piece_start1, piece_start2, piece_start3;
    int cells = eddy_cell*3;
    double eddy_length, eddy_length1, length, length1;     //eddy length and 1/3 of eddy length
    struct cell_information cell_pieces[eddy_cell*3];

    //    printf("begin\t");
    //    printf("eddy_cell=%d\n", eddy_cell);

    //to calculate eddy length and 1/3 of eddy length
    eddy_length = (cell[eddy_start+eddy_cell-1].y-cell[eddy_start].y)+(cell[eddy_start].size+cell[eddy_start+eddy_cell-1].size)/2.0;
    eddy_length1 = eddy_length/3.0;

    //to regenerate the cell series so that each cells in cell[] was divided into 3 pieces
    // and copy these pieces to cell_pieces[]
    for(i=0; i<cells; i++)
    {
        j = i/3;
        cell_pieces[i] = cell[eddy_start+j];
        cell_pieces[i].size = cell[eddy_start+j].size/3.0;
        if(i==0)
            cell_pieces[i].y = cell[eddy_start].y-cell[eddy_start].size/2+cell_pieces[i].size/2.0;
        else
            cell_pieces[i].y = cell_pieces[i-1].y+(cell_pieces[i-1].size+cell_pieces[i].size)/2.0;
    }

    // printf("cells=%d, cells_1=%d, eddy_l=%8.5f, eddy_l_1=%8.5f\n", eddy_cell, cells, eddy_length, cell_pieces[cells-1].y-cell_pieces[0].y+(cell_pieces[cells-1].size+cell_pieces[0].size)/2);

    //to identify how many cells in each 1/3 of original eddy
    piece_start1 = 0;
    piece_start2 = eddy_cell;
    piece_start3 = 2*eddy_cell;

    // to identify the second small eddy's start point
    length = cell_pieces[piece_start2-1].y-cell_pieces[piece_start1].y+(cell_pieces[piece_start2-1].size+cell_pieces[piece_start1].size)/2;

    if(length < eddy_length1)
    {
        while(length < eddy_length1)
        {
            piece_start2 += 3;
            length = cell_pieces[piece_start2-1].y-cell_pieces[piece_start1].y+(cell_pieces[piece_start2-1].size+cell_pieces[piece_start1].size)/2;
        }
    }
    else
    {
        //        printf("piece_start2 = %d\t", piece_start2);
        while(length>eddy_length1 && piece_start2>3)
        {
            //printf("piece_start2 = %d\t", piece_start2);
            piece_start2 -= 3;
            length = cell_pieces[piece_start2-1].y-cell_pieces[piece_start1].y+(cell_pieces[piece_start2-1].size+cell_pieces[piece_start1].size)/2;
        }
    }

    if(length > eddy_length1)
    {
        length1 = cell_pieces[piece_start2-1-3].y-cell_pieces[piece_start1].y+(cell_pieces[piece_start2-1-3].size+cell_pieces[piece_start1].size)/2;
        if(fabs(length-eddy_length1)>fabs(length1-eddy_length1))
            piece_start2 -= 3;
        if(piece_start2 <=3)
        {
           printf("weird! too small cells for the first piece! pie_start2=%d,cells=%d,length=%8.5f,eddy_l=%8.5f\n",piece_start2,eddy_cell,length,eddy_length);
           for(i=0; i<eddy_cell; i++)
               printf("cell[%d].size=%8.5f,cell[%d].u=%8.5f\n",i,cell[i+eddy_start].size,i+eddy_start,cell[i+eddy_start].u);
        }
    }
    else
    {
        length1 = cell_pieces[piece_start2-1+3].y-cell_pieces[piece_start1].y+(cell_pieces[piece_start2-1+3].size+cell_pieces[piece_start1].size)/2;
        if(fabs(length-eddy_length1)>fabs(length1-eddy_length1))
            piece_start2 += 3;
    }

    //to adjust piece_start2 in order to avoid stacksize(memory) overflow! in this case, this eddy will be suppressed finally (since an eddy which is 1/3 of original eddy is composed of 3 cells, thus energy_eddy=0 for this eddy)
    if(piece_start2 == 0)
        piece_start2 = 3;
    else if(piece_start2 >=cells-6)
        piece_start2 = cells-6;

    piece_start3 = piece_start3>piece_start2?piece_start3:piece_start2+3;
    // to identify the third small eddy's start point
    // here, the length is the 2/3 length of the original eddy
    length = cell_pieces[piece_start3-1].y-cell_pieces[piece_start1].y+(cell_pieces[piece_start3-1].size+cell_pieces[piece_start1].size)/2;
    if(length < 2*eddy_length1)
    {
        while(length < 2*eddy_length1)
        {
            piece_start3 += 3;
            length = cell_pieces[piece_start3-1].y-cell_pieces[piece_start1].y+(cell_pieces[piece_start3-1].size+cell_pieces[piece_start1].size)/2;
        }
    }
    else
    {
        while(length > 2*eddy_length1)
        {
            piece_start3 -= 3;
            length = cell_pieces[piece_start3-1].y-cell_pieces[piece_start1].y+(cell_pieces[piece_start3-1].size+cell_pieces[piece_start1].size)/2;
        }
    }
    if(length > 2*eddy_length1)
    {
        length1 = cell_pieces[piece_start3-1-3].y-cell_pieces[piece_start1].y+(cell_pieces[piece_start3-1-3].size+cell_pieces[piece_start1].size)/2;
        if(fabs(length-2*eddy_length1)>fabs(length1-2*eddy_length1))
            piece_start3 -= 3;
        if(piece_start3-piece_start2<=3)
        {
            printf("weird! too small cells for the second piece! pie_start2=%d, pie_start3=%d,cell=%d\n",piece_start2, piece_start3,eddy_cell);
            for(i=0; i<eddy_cell; i++)
                printf("cell[%d].size=%8.5f,cell[%d].u=%8.5f\n",i,cell[i+eddy_start].size,i+eddy_start,cell[i+eddy_start].u);
        }
    }
    else
    {
        length1 = cell_pieces[piece_start3-1+3].y-cell_pieces[piece_start1].y+(cell_pieces[piece_start3-1+3].size+cell_pieces[piece_start1].size)/2;
        if(fabs(length-eddy_length1)>fabs(length1-eddy_length1))
            piece_start3 += 3;
    }    

    if((piece_start2-piece_start1)%3 != 0)
    {
        printf("split small eddy weird1! p_s1=%d, p_s2=%d, %8.5f, %8.5f\n", piece_start1, piece_start2,eddy_length, eddy_length1);
        for(i=0; i<eddy_cell; i++)
            printf("cell[%d].size=%8.5f,cell[%d].u=%8.5f\n",i,cell[i+eddy_start].size,i+eddy_start,cell[i+eddy_start].u);
    }
    
    //to adjust piece_start2 in order to avoid stacksize(memory) overflow! in this case, this eddy will be suppressed finally (since an eddy which is 1/3 of original eddy is composed of 3 cells, thus energy_eddy=0 for this eddy)
    if(piece_start3==cells)
    {
        printf("error split_3, %d\n", piece_start3);
        piece_start3 -= 3;
    }
    else if(piece_start3 == piece_start2)
        piece_start3 += 3;
    
    *Energy_eddy1 = CAL_LARGEEDDY_ENERGY(piece_start1, piece_start2-piece_start1, cell_pieces);
    *Energy_eddy2 = CAL_LARGEEDDY_ENERGY(piece_start2, piece_start3-piece_start2, cell_pieces);
    *Energy_eddy3 = CAL_LARGEEDDY_ENERGY(piece_start3, cells-piece_start3, cell_pieces);
    //    printf("end!\n");
}







/*****************************************************************************************/
// This function is used to suppress large eddy.
// In this function, the eddy flux energy of each 1/3 piece of the eddy is calculated, Z_large is used in this function
/*****************************************************************************************/
double CAL_LARGEEDDY_ENERGY(int eddy_start, int eddy_cell, struct cell_information *cell)
{

    int i, j, k, trip_n;
    // density should be considered in pressure scrambling model
    double eddy_l, mean_visc=0.0;
    double r_uK=0.0, r_uJ=0.0, r_uKK=0.0, r_uJK=0.0, A, S;    //only one A and on S!!!
    double r_uuK=0.0, r_uuJ=0.0,r_uvK=0.0, r_uvJ=0.0, r_uwK=0.0, r_uwJ=0.0;
    double P_u, P_v, P_w;  
    double Ky[eddy_cell], Jy[eddy_cell];  // to record K and J value for each cell;
    double sgn;
    double Energy_eddy;

    double standard_eddylength = 0.0016;  // since the larger an eddy length is, the larger eddy energy eddy is acquired. This means that a very large eddy is easier to be judged to happen if a constant Z_large was used. In order to avoid such a case, a standard eddy length is introduced. A modified Z is calculated by Z_m = Z*eddy_length/standard_eddylength (if eddy_length > standard_eddylength)
    double Z_large_modified;
   
    // cell_trip is for triplet map
    struct cell_information cell_trip[eddy_cell];


    trip_n=eddy_cell/3;


    // triplet mapping process     struct species_information species;
    for(i=0; i<trip_n; i++)
    {
        cell_trip[i] = cell[3*i+eddy_start];
        cell_trip[i+trip_n]=cell[3*(trip_n-i)-2+eddy_start];
        cell_trip[2*trip_n+i]=cell[3*i+2+eddy_start];
    }
    // to calculate the new coordinates.
    cell_trip[0].y = cell[eddy_start].y;
    for(i=1; i<eddy_cell; i++)
        cell_trip[i].y = cell_trip[i-1].y+(cell_trip[i].size+cell_trip[i-1].size)/2;

    if(fabs(cell_trip[eddy_cell-1].y-cell[eddy_cell-1+eddy_start].y)>1e-6)
    {
        printf("trip cell weird!%d,%d, cell_trip[final].y=%8.5f, cell[final].y=%8.5f\n",eddy_cell,eddy_start,cell_trip[eddy_cell-1].y,cell[eddy_start+eddy_cell-1].y);
        for(i=0; i<eddy_cell; i++)
            printf("cell[%d].size=%8.5f, cell[%d].y=%8.5f, cell_trip[%d].size=%8.5f, cell_trip[%d].y=%8.5f\n",i,cell[i].size,i,cell[i].y,i,cell_trip[i].size,i,cell_trip[i].y);
    }

    // to calculate Ky and Jy
    for(i=0; i<trip_n; i++)
    {
        Ky[i] = cell_trip[i].y-cell[3*i+eddy_start].y;
        Ky[i+trip_n] = cell_trip[i+trip_n].y-cell[3*(trip_n-i)-2+eddy_start].y;
        Ky[i+2*trip_n] = cell_trip[i+2*trip_n].y-cell[3*i+2+eddy_start].y;
        Jy[i] = fabs(Ky[i]);
        Jy[i+trip_n] = fabs(Ky[i+trip_n]);
        Jy[i+2*trip_n] = fabs(Ky[i+2*trip_n]);
    }
    /*         for(i=0; i<eddy_cell; i++) */
    /*             printf("%8.5f  %8.5f\n", Ky[i], Jy[i]); */

    // to calculate r_uK, r_uJ, et al.
    for(i=0; i<eddy_cell; i++)
    {
        r_uK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uJ += cell_trip[i].u*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uKK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*Ky[i]*cell_trip[i].size;
        r_uJK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*Jy[i]*cell_trip[i].size;

        r_uuK += cell_trip[i].u*cell_trip[i].mixture.Density*cell_trip[i].u*Ky[i]*cell_trip[i].size;
        r_uuJ += cell_trip[i].u*cell_trip[i].u*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uvK += cell_trip[i].u*(0.*cell_trip[i].v_lateral+cell_trip[i].v_trip)*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uvJ += cell_trip[i].u*(0.*cell_trip[i].v_lateral+cell_trip[i].v_trip)*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uwK += cell_trip[i].u*cell_trip[i].w*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uwJ += cell_trip[i].u*cell_trip[i].w*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;

    }
    A = r_uJ==0?0:-r_uK/r_uJ;

    /*           printf("r_uK=%8.5f,\tr_uJ=%8.5f,\tr_uKK=%8.5f,\tr_uJK=%8.5f,\tA=%8.5f\n", r_uK, r_uJ,r_uKK, r_uJK, A); */
    /*           printf("r_uuK=%8.5f,\tr_uuJ=%8.5f\n", r_uuK, r_uuJ); */
    /*           printf("r_uvK=%8.5f,\tr_vvJ=%8.5f\n", r_uvK, r_uvJ); */
    /*           printf("r_uwK=%8.5f,\tr_uwJ=%8.5f\n", r_uwK, r_uwJ); */
    
    S = (1+A*A)*r_uKK/2+A*r_uJK;
    P_u = r_uuK+A*r_uuJ;
    P_v = r_uvK+A*r_uvJ;
    P_w = r_uwK+A*r_uwJ;
    //to calculate eddy length
    eddy_l = cell[eddy_start+eddy_cell-1].y-cell[eddy_start].y+(cell[eddy_start].size+cell[eddy_start+eddy_cell-1].size)/2.0;
    //to calculate mean kinematic viscosity of the eddy
    for(i=0; i<eddy_cell; i++)
        mean_visc += cell[i+eddy_start].mixture.Density/cell[i+eddy_start].mixture.Kinetic_Viscosity_Coeff*cell[i+eddy_start].size;
    mean_visc /= eddy_l;
    mean_visc = 1.0/mean_visc;
    
    //     printf("mean_visc=%8.5e\n",mean_visc);

    /*     Energy_eddy = ((1-alfa)*P_u*P_u+alfa/2*(P_v*P_v+P_w*P_w))*pow((eddy_l*eddy_l/mean_visc),2)/(2*S*r_uKK)-Z; */


    
    Z_large_modified = eddy_l>standard_eddylength?(Z_large*eddy_l/standard_eddylength):Z_large;

    if(S==0.0 || r_uKK==0.0)
        Energy_eddy = TINY-Z_large_modified;
    else
        Energy_eddy = (P_u*P_u+P_v*P_v+P_w*P_w)*pow((eddy_l*eddy_l/mean_visc),2)/(2*S*r_uKK)-Z_large_modified;
    

    return(Energy_eddy);
    
}






/*****************************************************************************************/
// This function is used to suppress small eddy.
// In this function, the eddy flux energy of the whole small eddy is calculated, Z_small is used in this function
/*****************************************************************************************/
double CAL_SMALLEDDY_ENERGY(int eddy_start, int eddy_cell, struct cell_information *cell)
{

    int i, j, k, trip_n;
    // density should be considered in pressure scrambling model
    double eddy_l, mean_visc=0.0;
    double r_uK=0.0, r_uJ=0.0, r_uKK=0.0, r_uJK=0.0, A, S;    //only one A and on S!!!
    double r_uuK=0.0, r_uuJ=0.0,r_uvK=0.0, r_uvJ=0.0, r_uwK=0.0, r_uwJ=0.0;
    double P_u, P_v, P_w;  
    double Ky[eddy_cell], Jy[eddy_cell];  // to record K and J value for each cell;
    double sgn;
    double Energy_eddy;
   
    double standard_eddylength = 0.0048;  // since the larger an eddy length is, the larger eddy energy eddy is acquired. This means that a very large eddy is easier to be judged to happen if a constant Z_large was used. In order to avoid such a case, a standard eddy length is introduced. A modified Z is calculated by Z_m = Z*eddy_length/standard_eddylength (if eddy_length > standard_eddylength)
    double Z_small_modified;


    // cell_trip is for triplet map
    struct cell_information cell_trip[eddy_cell];


    trip_n=eddy_cell/3;


    // triplet mapping process     struct species_information species;
    for(i=0; i<trip_n; i++)
    {
        cell_trip[i] = cell[3*i+eddy_start];
        cell_trip[i+trip_n]=cell[3*(trip_n-i)-2+eddy_start];
        cell_trip[2*trip_n+i]=cell[3*i+2+eddy_start];
    }

    // to calculate the new coordinates.
    cell_trip[0].y = cell[eddy_start].y;
    for(i=1; i<eddy_cell; i++)
        cell_trip[i].y = cell_trip[i-1].y+(cell_trip[i].size+cell_trip[i-1].size)/2;
       
    // to calculate Ky and Jy
    for(i=0; i<trip_n; i++)
    {
        Ky[i]=cell_trip[i].y-cell[3*i+eddy_start].y;
        Ky[i+trip_n]=cell_trip[i+trip_n].y-cell[3*(trip_n-i)-2+eddy_start].y;
        Ky[i+2*trip_n]=cell_trip[i+2*trip_n].y-cell[3*i+2+eddy_start].y;
        Jy[i]=fabs(Ky[i]);
        Jy[i+trip_n]=fabs(Ky[i+trip_n]);
        Jy[i+2*trip_n]=fabs(Ky[i+2*trip_n]);
    }
    /*         for(i=0; i<eddy_cell; i++) */
    /*             printf("%8.5f  %8.5f\n", Ky[i], Jy[i]); */

    // to calculate r_uK, r_uJ, et al.
    for(i=0; i<eddy_cell; i++)
    {
        r_uK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uJ += cell_trip[i].u*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uKK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*Ky[i]*cell_trip[i].size;
        r_uJK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*Jy[i]*cell_trip[i].size;

        r_uuK += cell_trip[i].u*cell_trip[i].mixture.Density*cell_trip[i].u*Ky[i]*cell_trip[i].size;
        r_uuJ += cell_trip[i].u*cell_trip[i].u*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uvK += cell_trip[i].u*(0.*cell_trip[i].v_lateral+cell_trip[i].v_trip)*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uvJ += cell_trip[i].u*(0.*cell_trip[i].v_lateral+cell_trip[i].v_trip)*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uwK += cell_trip[i].u*cell_trip[i].w*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uwJ += cell_trip[i].u*cell_trip[i].w*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;

    }
    A = r_uJ==0?0:-r_uK/r_uJ;

    /*           printf("r_uK=%8.5f,\tr_uJ=%8.5f,\tr_uKK=%8.5f,\tr_uJK=%8.5f,\tA=%8.5f\n", r_uK, r_uJ,r_uKK, r_uJK, A); */
    /*           printf("r_uuK=%8.5f,\tr_uuJ=%8.5f\n", r_uuK, r_uuJ); */
    /*           printf("r_uvK=%8.5f,\tr_vvJ=%8.5f\n", r_uvK, r_uvJ); */
    /*           printf("r_uwK=%8.5f,\tr_uwJ=%8.5f\n", r_uwK, r_uwJ); */
    
    S = (1+A*A)*r_uKK/2+A*r_uJK;
    P_u = r_uuK+A*r_uuJ;
    P_v = r_uvK+A*r_uvJ;
    P_w = r_uwK+A*r_uwJ;
    //to calculate eddy length
    eddy_l = cell[eddy_start+eddy_cell-1].y-cell[eddy_start].y+(cell[eddy_start].size+cell[eddy_start+eddy_cell-1].size)/2.0;
    //to calculate mean kinematic viscosity of the eddy
    for(i=0; i<eddy_cell; i++)
        mean_visc += cell[i+eddy_start].mixture.Density/cell[i+eddy_start].mixture.Kinetic_Viscosity_Coeff*cell[i+eddy_start].size;
    mean_visc /= eddy_l;
    mean_visc = 1.0/mean_visc;
    
    //     printf("mean_visc=%8.5e\n",mean_visc);

    /*     Energy_eddy = ((1-alfa)*P_u*P_u+alfa/2*(P_v*P_v+P_w*P_w))*pow((eddy_l*eddy_l/mean_visc),2)/(2*S*r_uKK)-Z; */
    Z_small_modified = eddy_l>standard_eddylength?Z_small*eddy_l/standard_eddylength:Z_small;

    if(S==0.0 || r_uKK==0.0)
        Energy_eddy = TINY-Z_small_modified;
    else
        Energy_eddy = (P_u*P_u+P_v*P_v+P_w*P_w)*pow((eddy_l*eddy_l/mean_visc),2)/(2*S*r_uKK)-Z_small_modified;
    

    return(Energy_eddy);
    
}










// This function is used to calculate eddy distribution rate for the candidate eddy.
double CAL_LEMDA(int eddy_start, int eddy_cell, struct cell_information *cell)
{

    int i, j, k, trip_n;
    // density should be considered in pressure scrambling model
    double eddy_l, mean_visc=0.0;
    double r_uK=0.0, r_uJ=0.0, r_uKK=0.0, r_uJK=0.0, A, S;
    double r_uuK=0.0, r_uuJ=0.0,r_uvK=0.0, r_uvJ=0.0, r_uwK=0.0, r_uwJ=0.0;
    double P_u, P_v, P_w;  //only one A and on S!!!
    double Ky[eddy_cell], Jy[eddy_cell];  // to record K and J value for each cell;
    double sgn;
    double Energy_eddy,Eddy_massflux=0.0;
    double lemda;
   
    // cell_trip is for triplet map
    struct cell_information cell_trip[eddy_cell];

    trip_n=eddy_cell/3;

    // triplet mapping process     struct species_information species;
    for(i=0; i<trip_n; i++)
    {
        cell_trip[i] = cell[3*i+eddy_start];
        cell_trip[i+trip_n]=cell[3*(trip_n-i)-2+eddy_start];
        cell_trip[2*trip_n+i]=cell[3*i+2+eddy_start];
    }
    // to calculate the new coordinates.
    for(i=0; i<eddy_cell; i++)
        cell_trip[i].y = i==0?cell[i+eddy_start].y:cell_trip[i-1].y+(cell_trip[i].size+cell_trip[i-1].size)/2;
       
    // to calculate Ky and Jy
    for(i=0; i<trip_n; i++)
    {
        Ky[i]=cell_trip[i].y-cell[3*i+eddy_start].y;
        Ky[i+trip_n]=cell_trip[i+trip_n].y-cell[3*(trip_n-i)-2+eddy_start].y;
        Ky[i+2*trip_n]=cell_trip[i+2*trip_n].y-cell[3*i+2+eddy_start].y;
        Jy[i]=fabs(Ky[i]);
        Jy[i+trip_n]=fabs(Ky[i+trip_n]);
        Jy[i+2*trip_n]=fabs(Ky[i+2*trip_n]);
    }
    /*         for(i=0; i<eddy_cell; i++) */
    /*             printf("%8.5f  %8.5f\n", Ky[i], Jy[i]); */


    // to calculate r_uK, r_uJ, et al.
    for(i=0; i<eddy_cell; i++)
    {
        r_uK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uJ += cell_trip[i].u*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uKK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*Ky[i]*cell_trip[i].size;
        r_uJK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*Jy[i]*cell_trip[i].size;

        r_uuK += cell_trip[i].u*cell_trip[i].mixture.Density*cell_trip[i].u*Ky[i]*cell_trip[i].size;
        r_uuJ += cell_trip[i].u*cell_trip[i].u*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uvK += cell_trip[i].u*(0.*cell_trip[i].v_lateral+cell_trip[i].v_trip)*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uvJ += cell_trip[i].u*(0.*cell_trip[i].v_lateral+cell_trip[i].v_trip)*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uwK += cell_trip[i].u*cell_trip[i].w*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uwJ += cell_trip[i].u*cell_trip[i].w*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;

    }
    A = r_uJ==0?0:-r_uK/r_uJ;

    /*           printf("r_uK=%8.5f,\tr_uJ=%8.5f,\tr_uKK=%8.5f,\tr_uJK=%8.5f,\tA=%8.5f\n", r_uK, r_uJ,r_uKK, r_uJK, A); */
    /*           printf("r_uuK=%8.5f,\tr_uuJ=%8.5f\n", r_uuK, r_uuJ); */
    /*           printf("r_uvK=%8.5f,\tr_vvJ=%8.5f\n", r_uvK, r_uvJ); */
    /*           printf("r_uwK=%8.5f,\tr_uwJ=%8.5f\n", r_uwK, r_uwJ); */
    
    S = (1+A*A)*r_uKK/2+A*r_uJK;
    P_u = r_uuK+A*r_uuJ;
    P_v = r_uvK+A*r_uvJ;
    P_w = r_uwK+A*r_uwJ;
    //to calculate eddy length
    eddy_l = cell[eddy_start+eddy_cell-1].y-cell[eddy_start].y+(cell[eddy_start].size+cell[eddy_start+eddy_cell-1].size)/2;
    //to calculate mean kinematic viscosity of the eddy
    for(i=0; i<eddy_cell; i++)
        mean_visc += cell[i+eddy_start].mixture.Density/cell[i+eddy_start].mixture.Kinetic_Viscosity_Coeff*cell[i+eddy_start].size;
    mean_visc /= eddy_l;
    mean_visc = 1.0/mean_visc;

    /* Energy_eddy = ((1-alfa)*P_u*P_u+alfa/2*(P_v*P_v+P_w*P_w))*pow((eddy_l*eddy_l/mean_visc),2)/(2*S*r_uKK)-Z; */
    if(S==0.0 || r_uKK==0.0)
        Energy_eddy = TINY-Z_small;
    else
        Energy_eddy = (P_u*P_u+P_v*P_v+P_w*P_w)*pow((eddy_l*eddy_l/mean_visc),2)/(2*S*r_uKK)-Z_small;

    for(i=eddy_start; i<eddy_start+eddy_cell; i++)
        Eddy_massflux += cell[i].u*cell[i].size*cell[i].mixture.Density;


    // suppress the eddy if eddy energy is less than 0
    if(Energy_eddy<0)
        lemda = 0.0;
    else
        lemda = C_odt*mean_visc/pow(eddy_l,4)*sqrt(r_uKK*Energy_eddy/(pow(eddy_l,2)*Eddy_massflux));
    return(lemda);
}





// This function is used to calculate eddy distribution rate for the candidate eddy.
void EDDY_TRIPLET(int E_S_number, int *FLAG, int eddy_start, int eddy_cell, struct cell_information *cell, struct source_information *source, struct cell_average cell_ave[][ysample_number])
{

    int i, j, k, trip_n, ave_step;

    // density should be considered in pressure scrambling model
    double eddy_l, mean_kinvisc=0.0;
    double r_uK=0.0, r_uJ=0.0, r_uKK=0.0, r_uJK=0.0, A, S;
    double r_uuK=0.0, r_uuJ=0.0,r_uvK=0.0, r_uvJ=0.0, r_uwK=0.0, r_uwJ=0.0;
    double c_u, c_v, c_w, b_u, b_v, b_w, P_u, P_v, P_w;  //only one A and on S!!!
    double Ky[eddy_cell], Jy[eddy_cell];  // to record K and J value for each cell;
    double sgn;
    double ave_halfwidth;
   
    // cell_trip is for triplet map
    struct cell_information cell_trip[eddy_cell], cell_trip_backup[eddy_cell];
    struct source_information source_trip[eddy_cell], source_trip_backup[eddy_cell];

    trip_n= (int)(eddy_cell/3);


    // triplet mapping process     struct species_information species;
    for(i=0; i<trip_n; i++)
    {
        cell_trip[i] = cell[3*i+eddy_start];
        cell_trip[i+trip_n]=cell[3*(trip_n-i)-2+eddy_start];
        cell_trip[2*trip_n+i]=cell[3*i+2+eddy_start];
        cell_trip_backup[i] = cell_trip[i];
        cell_trip_backup[i+trip_n] = cell_trip[i+trip_n];
        cell_trip_backup[i+2*trip_n] = cell_trip[i+2*trip_n];

        source_trip[i] = source[3*i+eddy_start];
        source_trip[i+trip_n]=source[3*(trip_n-i)-2+eddy_start];
        source_trip[2*trip_n+i]=source[3*i+2+eddy_start];
        source_trip_backup[i] = source_trip[i];
        source_trip_backup[i+trip_n] = source_trip[i+trip_n];
        source_trip_backup[i+2*trip_n] = source_trip[i+2*trip_n];
    }

    // to calculate the new coordinates.
    for(i=0; i<eddy_cell; i++)
        cell_trip[i].y = i==0?cell[i+eddy_start].y:cell_trip[i-1].y+(cell_trip[i].size+cell_trip[i-1].size)/2;
       
    // to calculate Ky and Jy
    for(i=0; i<trip_n; i++)
    {
        Ky[i]=cell_trip[i].y-cell[3*i+eddy_start].y;
        Ky[i+trip_n]=cell_trip[i+trip_n].y-cell[3*(trip_n-i)-2+eddy_start].y;
        Ky[i+2*trip_n]=cell_trip[i+2*trip_n].y-cell[3*i+2+eddy_start].y;
        Jy[i]=fabs(Ky[i]);
        Jy[i+trip_n]=fabs(Ky[i+trip_n]);
        Jy[i+2*trip_n]=fabs(Ky[i+2*trip_n]);
    }
    /*         for(i=0; i<eddy_cell; i++) */
    /*             printf("%8.5f  %8.5f\n", Ky[i], Jy[i]); */


    // to calculate r_uK, r_uJ, et al.
    for(i=0; i<eddy_cell; i++)
    {
        r_uK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uJ += cell_trip[i].u*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uKK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*Ky[i]*cell_trip[i].size;
        r_uJK += cell_trip[i].u*cell_trip[i].mixture.Density*Ky[i]*Jy[i]*cell_trip[i].size;

        r_uuK += cell_trip[i].u*cell_trip[i].mixture.Density*cell_trip[i].u*Ky[i]*cell_trip[i].size;
        r_uuJ += cell_trip[i].u*cell_trip[i].u*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uvK += cell_trip[i].u*(0.*cell_trip[i].v_lateral+cell_trip[i].v_trip)*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uvJ += cell_trip[i].u*(0.*cell_trip[i].v_lateral+cell_trip[i].v_trip)*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;
        r_uwK += cell_trip[i].u*cell_trip[i].w*cell_trip[i].mixture.Density*Ky[i]*cell_trip[i].size;
        r_uwJ += cell_trip[i].u*cell_trip[i].w*cell_trip[i].mixture.Density*Jy[i]*cell_trip[i].size;

    }
    A = r_uJ==0?0:-r_uK/r_uJ;

    /*           printf("r_uK=%8.5f,\tr_uJ=%8.5f,\tr_uKK=%8.5f,\tr_uJK=%8.5f,\tA=%8.5f\n", r_uK, r_uJ,r_uKK, r_uJK, A); */
    /*           printf("r_uuK=%8.5f,\tr_uuJ=%8.5f\n", r_uuK, r_uuJ); */
    /*           printf("r_uvK=%8.5f,\tr_vvJ=%8.5f\n", r_uvK, r_uvJ); */
    /*           printf("r_uwK=%8.5f,\tr_uwJ=%8.5f\n", r_uwK, r_uwJ); */
    
    S = (1+A*A)*r_uKK/2+A*r_uJK;
    P_u = r_uuK+A*r_uuJ;
    P_v = r_uvK+A*r_uvJ;
    P_w = r_uwK+A*r_uwJ;

    sgn = P_u>0?1.0:-1.0;
    c_u = S==0?0:(-P_u+sgn*sqrt(P_u*P_u*(1-alfa)+alfa*(P_v*P_v+P_w*P_w)/2))/(2*S);
    b_u = c_u*A;
    sgn = P_v>0?1.0:-1.0;
    c_v = S==0?0:(-P_v+sgn*sqrt(P_v*P_v*(1-alfa)+alfa*(P_u*P_u+P_w*P_w)/2))/(2*S);
    b_v = c_v*A;
    sgn = P_w>0?1.0:-1.0;
    c_w = S==0?0:(-P_w+sgn*sqrt(P_w*P_w*(1-alfa)+alfa*(P_v*P_v+P_u*P_u)/2))/(2*S);
    b_w = c_w*A;

    for(i=0; i<eddy_cell; i++)
    {
        cell_trip[i].u += c_u*Ky[i]+b_u*Jy[i];
        cell_trip[i].v_trip += c_v*Ky[i]+b_v*Jy[i];
        cell_trip[i].w += c_w*Ky[i]+b_w*Jy[i];
        cell_trip[i].size = cell_trip_backup[i].u*cell_trip_backup[i].size/cell_trip[i].u;
    
        /*	if(cell_trip[i].size<0)
                printf("error!!!!! %f\t%d\n", cell_trip[i].size, i+eddy_start);
                if(cell_trip[i+trip_n].size<0)
                printf("error!!!!! %f\t%d\n", cell_trip[i+trip_n].size, i+eddy_start+trip_n);
                if(cell_trip[i+2*trip_n].size<0)
                printf("error!!!!! %f\t%d\n", cell_trip[i+2*trip_n].size, i+eddy_start+2*trip_n);
        */
        // to avoid too large cell size changes
        if( cell_trip[i].u<umin_eddy)
        {  
            printf("errors occurs! this eddy is abandoned!\n");
            printf(" %8.5f\t%8.5f\t%8.5f\t%8.5f\t%8.5f\n",cell_trip[i].u, v_latcoe*cell_trip[i].v_lateral+cell_trip[i].v_trip, cell_trip_backup[i].u, cell_trip_backup[i].v_trip+v_latcoe*cell_trip_backup[i].v_lateral, c_u*Ky[i]+b_u*Jy[i]); 
            *FLAG = 0;
            goto abnormal1;
        }
    }


    if(*FLAG == 1)
    {
        ave_step = int((E_S_number+1)/record_xstep);
        //These code is used to calculate the uv Reynolds stress
        if(((ave_step+1)*record_xstep*dx_stir-(E_S_number+1)*dx_stir)<=(record_xstep*dx_stir))
        {

            //halfwidth of the average region
            ave_halfwidth = domain_y/2.0;

            // at least a piece of eddy locates in the average region
            if(cell[eddy_start+eddy_cell].y+cell[eddy_start+eddy_cell].size/2>-ave_halfwidth && cell[eddy_start].y-cell[eddy_start].size/2<ave_halfwidth)
            {
                // k is used to identify the serial number of the cells of cell_ave
                if(cell[eddy_start].y-cell[eddy_start].size/2<=-ave_halfwidth)
                    k = 0;
                else
                    k = (cell[eddy_start].y-cell[eddy_start].size/2)<0.0?(ysample_number/2-1)-(int)(fabs(cell[eddy_start].y-cell[eddy_start].size/2)/ave_halfwidth*(ysample_number/2)):(int)(fabs(cell[eddy_start].y-cell[eddy_start].size/2)/ave_halfwidth*(ysample_number/2))+(ysample_number/2);
            
                if(cell_ave[ave_step][k].y<cell[eddy_start].y-cell[eddy_start].size/2)
                    k++;
                if(cell_ave[ave_step][k].y<cell[eddy_start].y-cell[eddy_start].size/2)
                    printf("ave position judging weird! ave=%8.5f, eddy=%8.5f", cell_ave[ave_step][k].y, cell[eddy_start].y-cell[eddy_start].size/2);

                j = 0;  //j for cell_triplet.

                for(i=eddy_start; i<eddy_start+eddy_cell; i++)
                {
                    if(cell[i].y-cell[i].size/2<=cell_ave[ave_step][k].y && cell[i].y+cell[i].size/2>cell_ave[ave_step][k].y)
                    {
                        while(cell_trip[j].y+cell_trip[j].size/2<cell_ave[ave_step][k].y)
                            j++;

                        if(j>=eddy_cell)
                            printf("weird! for trip: %d", j);

                        // this term is <M+T> which means change of vertical velocity due to eddy event during an ODT step
                        cell_ave[ave_step][k].deta_u += (cell_trip[j].u-cell[i].u)*(cell_trip[j].u+cell[i].u)/2.0/(record_xstep*dx_stir);
                        //   printf("cell_ave[%d].detau=%8.5f,y=%8.5f\n", k, cell_ave[(E_S_number+1)/record_xstep][k].deta_u, cell_ave[(E_S_number+1)/record_xstep][k].y);

                        k++;

                        // in case that cell[i].size or cell_trip[j].size > cell_ave[].size
                        if(cell[i].y+cell[i].size/2>cell_ave[ave_step][k].y)
                            i--;
                        if(cell_trip[j].y+cell_trip[j].size/2>cell_ave[ave_step][k].y)
                            j--;
                        if(k>=ysample_number) // exceeds the right boundary of the average region
                            i = eddy_start+eddy_cell+1;
                    }
                }
            }
        }

        for(i=0; i<eddy_cell; i++)
        {
            cell[i+eddy_start] = cell_trip[i];
            source[i+eddy_start] = source_trip[i];
        }
    }
 abnormal1:	;
}









// initialize the eddy state for each step. to judge which cell is occured by which eddy.
void EDDY_JUDGESTATE(int E_S_number, int E_S_old, int *eddy_occur, int domain_cellnumber, double *grid_y, struct cell_information *cell_info, struct eddy_information *eddy_info)
{
    int i, y_min, y_max, k,i1;
    int eddy_decrease=0; 
    double effect_height_old, effect_height_new;
    double effect_length_old, effect_length_new;

    struct eddy_information eddy_reduce[*eddy_occur];
    //initialize eddy occurrence state for each cell
    for(i=0; i<domain_cellnumber; i++)
    {
        cell_info[i].eddystate = 0;
        cell_info[i].eddynumber = eddy_number;
        cell_info[i].eddystate_instant = 0;
        cell_info[i].abseddynumber = 0;
    }

    for(i=0; i<*eddy_occur; i++)
    {
        if(eddy_info[i].xcellstart > E_S_number)
            printf("error happned to %dth eddy! eddy_start=%d",i,eddy_info[i].xcellstart);
        else if(eddy_info[i].xcellend < E_S_old)
            eddy_decrease++;
        else if(eddy_info[i].ystart<-domain_y/2 || eddy_info[i].yend>domain_y/2)
            eddy_decrease++;
        else
            eddy_reduce[i-eddy_decrease] = eddy_info[i];

        if((i+1)<eddy_decrease)
            printf("i weird!!! i=%d, eddy_decrease=%d, *eddy_occur=%d\n", i, eddy_decrease, *eddy_occur);
    }
    *eddy_occur -= eddy_decrease;

    //    if(E_S_number%print_step == 0)
    //    printf("decrease=%d\t eddy_occur=%d\t",eddy_decrease, *eddy_occur);
	
    //identify which cell is occupied by which eddy
    for(i=0; i<*eddy_occur; i++)
    {
        //eddy exists at this ODT step
        if(eddy_reduce[i].xcellstart<=E_S_old && eddy_reduce[i].xcellend>=E_S_old)
        {
            eddy_info[i] = eddy_reduce[i];

            if(eddy_info[i].ystart<-domain_y/2 || eddy_info[i].yend>domain_y/2)
            {
                printf("eddy[%d].ystart weird!in JUDGESTATE .start=%f, .yend=%f,.length=%f\n", i, eddy_info[i].ystart, eddy_info[i].yend,eddy_info[i].length);
                printf("eddy_decrease=%d, eddy_number=%d\n",eddy_decrease, *eddy_occur);
                printf("E_S_number=%d, E_S_old=%d\n",E_S_number, E_S_old);
                printf("xcellend=%d\n",eddy_info[i].xcellend);
                for(i1=i-10; i1<i+10; i1++)
                    printf("%d,.ystart=%f,.yend=%f,.xcellend=%d,%d, ycellstart=%d, ycellend=%d,length=%f,%f\n",i1,eddy_info[i1].ystart,eddy_info[i1].yend,eddy_info[i1].xcellstart,eddy_info[i1].xcellend,eddy_info[i1].ycellstart, eddy_info[i1].ycellend,eddy_info[i1].length,eddy_info[i1].length1);
            }

            //to calculate a candidate y_min and y_max
            y_min = (int)((eddy_info[i].ystart+domain_y/2)/domain_y*domain_cellnumber);
            y_max = (int)((eddy_info[i].yend+domain_y/2)/domain_y*domain_cellnumber);
            /*           y_min = (int)(2*eddy_info[i].ystart*domain_cellnumber/cellnumber/(dy_co+dy_cen))+domain_cellnumber/2; */
            /*             y_max = (int)(2*eddy_info[i].yend*domain_cellnumber/cellnumber/(dy_co+dy_cen))+domain_cellnumber/2; */

            //to calculate the exact y_min and y_max.
            while(grid_y[y_min]>eddy_info[i].ystart || grid_y[y_min+1]<eddy_info[i].ystart)
            {
                if(grid_y[y_min] > eddy_info[i].ystart)
                    y_min--;
                else if(grid_y[y_min+1] < eddy_info[i].ystart)
                    y_min++;
            }
			
            while(grid_y[y_max-1]>eddy_info[i].yend || grid_y[y_max]<eddy_info[i].yend)
            {
                if(grid_y[y_max-1] > eddy_info[i].yend)
                    y_max--;
                else if(grid_y[y_max] < eddy_info[i].yend)
                    y_max++;
            }
            
            // to choose the closest point as the eddy boundary
            eddy_info[i].ycellstart = (fabs(grid_y[y_min]-eddy_info[i].ystart)>fabs(grid_y[y_min+1]-eddy_info[i].ystart))?y_min+1:y_min;

            eddy_info[i].ycellend = (fabs(grid_y[y_max]-eddy_info[i].yend)>fabs(eddy_info[i].yend-grid_y[y_max-1]))?y_max-1:y_max;

            if(!(eddy_info[i].ycellstart>0 && eddy_info[i].ycellend<domain_cellnumber))
                printf("eddy[%d] error in EDDYJUDGE! .ycellstart=%d, .ycellend=%d, .ystart=%f, .yend=%f\n", i, eddy_info[i].ycellstart, eddy_info[i].ycellend, eddy_info[i].ystart, eddy_info[i].yend);

            // to adjust the eddy start point and end point
            eddy_info[i].ystart = grid_y[y_min];

            eddy_info[i].yend = grid_y[y_max];

            //to renew the eddy information for the cells occupied by the eddy
            for(k=eddy_info[i].ycellstart; k<=eddy_info[i].ycellend; k++)
            {
                // to record the eddy instant value.
                if(eddy_info[i].xcellstart>E_S_number-50 && eddy_info[i].xcellstart < E_S_number)
                    cell_info[k].eddystate_instant = 1;

                if(cell_info[k].eddystate == 0)
                {
                    cell_info[k].eddystate = 1;
                    cell_info[k].eddynumber = i;
                    cell_info[k].abseddynumber = eddy_info[i].ser_number;
                }

                else   // if this cell has been occupied by an eddy before, the cell will be occupied by the larger eddy according to their effective length.
                {

                    //                    if(eddy_info[cell_info[k].eddynumber].length*0.7<eddy_info[i].length)
                    //                    {
                    effect_height_old = (eddy_info[cell_info[k].eddynumber].xcellend-E_S_old)*dx_stir;  //the effective height of the old eddy that occupying this cell is the length from the odt line to the eddy top;
                    effect_height_new = (eddy_info[i].xcellend-E_S_old)*dx_stir;  //the effective height of the new eddy that will occupy this cell;
                    effect_length_old = eddy_info[cell_info[k].eddynumber].length*pow(effect_height_old/eddy_info[cell_info[k].eddynumber].height_judge,2);
                    effect_length_new = eddy_info[i].length*pow(effect_height_new/eddy_info[i].height_judge,2);
                    if(effect_length_new >= effect_length_old)
                    {
                        cell_info[k].eddynumber = i;
                        cell_info[k].abseddynumber = eddy_info[i].ser_number;
                    }
                }
            }
        }
        else
            printf("eddy judge weird!!!cS=%d, E_S_number=%d, ce=%d, E_S_old=%d\n",eddy_reduce[i].xcellstart, E_S_number, eddy_reduce[i].xcellend, E_S_old );
    }
}












// to identify .cellstart/.cellend according to .ystart .yend
void EDDY_JUDGECELL(int eddy_occur, int domain_cellnumber, double *grid_y, struct eddy_information *eddy_info)
{
    int i, y_min, y_max;

    //identify which cell is occupied by which eddy
    for(i=0; i<eddy_occur; i++)
    {
        //eddy exists at this ODT step
        if(eddy_info[i].ystart<-domain_y/2 || eddy_info[i].yend>domain_y/2)
        {
            printf("eddy[%d].ystart weird!in JUDGECELL .start=%f, .yend=%f,.length=%f\n", i, eddy_info[i].ystart, eddy_info[i].yend,eddy_info[i].length);
            printf("xcellend=%d\n",eddy_info[i].xcellend);
        }

        //to calculate a candidate y_min and y_max
        y_min = (int)((eddy_info[i].ystart+domain_y/2)/domain_y*domain_cellnumber);
        y_max = (int)((eddy_info[i].yend+domain_y/2)/domain_y*domain_cellnumber);

        //to calculate the exact y_min and y_max.
        while(grid_y[y_min]>eddy_info[i].ystart || grid_y[y_min+1]<eddy_info[i].ystart)
        {
            if(grid_y[y_min] > eddy_info[i].ystart)
                y_min--;
            else if(grid_y[y_min+1] < eddy_info[i].ystart)
                y_min++;
        }
        
        while(grid_y[y_max-1]>eddy_info[i].yend || grid_y[y_max]<eddy_info[i].yend)
        {
            if(grid_y[y_max-1] > eddy_info[i].yend)
                y_max--;
            else if(grid_y[y_max] < eddy_info[i].yend)
                y_max++;
        }
            
        // to choose the closest point as the eddy boundary
        eddy_info[i].ycellstart = (fabs(grid_y[y_min]-eddy_info[i].ystart)>fabs(grid_y[y_min+1]-eddy_info[i].ystart))?y_min+1:y_min;
        
        eddy_info[i].ycellend = (fabs(grid_y[y_max]-eddy_info[i].yend)>fabs(eddy_info[i].yend-grid_y[y_max-1]))?y_max-1:y_max;
        
        if(!(eddy_info[i].ycellstart>0 && eddy_info[i].ycellend<domain_cellnumber))
            printf("eddy[%d] error in JUDGE_CELL! .ycellstart=%d, .ycellend=%d, .ystart=%f, .yend=%f\n", i, eddy_info[i].ycellstart, eddy_info[i].ycellend, eddy_info[i].ystart, eddy_info[i].yend);

        // to adjust the eddy start point and end point
        eddy_info[i].ystart = grid_y[y_min];
        
        eddy_info[i].yend = grid_y[y_max];
    }
}


